###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package AbstractCategoryGizmoHack;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);

use vars qw(@ISA);
@ISA=qw(GizmoBuilder Gizmo::Category);
my $version = "4.5";

sub get_version {
    return $version;
}

sub get_default_permissions {
    my $self = shift;
    
    my $AC;
    if (ref($self) && $self->{_default_permissions}) {
        $AC = $self->{_default_permissions};
    } else {
        $AC = $self->SUPER::get_default_permissions();    
    
        $AC->add_op('MOD', 'add_child');
        $AC->add_op('MOD', 'create_child');
        
        # save these permissions to self if possible
        if (ref($self)) {
            $self->{_default_permissions} = $AC;
        }
    }
    
    return $AC;           
}


sub gizmo_list_header {
    my $class = shift;

    return "";
}

sub gizmo_list_decorator {
	return 'HTML::Decorator::NullDecorator';
}

sub www_create_child {
  my $self = shift;
  my $child_type = shift;

  $self->debug_msg(3, "Gizmo::www_create_child()");

  my $child_isa = $FORM{child_type} || $child_type;

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  my $child = $child_isa->new({
			 uid => $USER->uid(),
			 parent_id => $parentid,
			});

  # this method should ALWAYS be overridden in derived classes.
  $child->process_edit_form();
}

sub _show_create_success {
    my $self = shift;
    my $isa = $self->is_a();

    my $parentid = $self->parent_id();

    my $parent_obj = Gizmo->new($parentid);

    my $grandparent = $parent_obj->parent_id();

    $self->two_choices("<Big>New $isa added!</big><br /><br />What do you want to do?",
		       "confirm",
		       "Add another $isa",
		       "index.pl?child_type=$isa&amp;isa=Category&amp;iid=$grandparent&amp;op=add_child&amp;app=",
		       "Enough for now!",
		       "index.pl?id=$grandparent");
}

sub show_summary {
    my $self = shift;
    my @args = @_;

    my $buttons     = $self->get_buttons();
    my $id = $self->id();
    my $isa = $self->is_a();

    $self->{is_inline} = 1;
    my $text = $self->show();
   
    return $buttons.$text;
}

sub www_save {
	my $self = shift;

	if (!$self->get_iid()) {
		use Gizmo::Category;
		my $cat = Gizmo->new($self->parent_id());
		$cat->name($FORM{name});
		$cat->uid($self->uid());
		$cat->parent_id($self->parent_id);
		$cat->is_a("Category");
		$cat->keywords("");
		$cat->display_look("");
		$cat->created_by("");
		$cat->t1("");
		$cat->t2("");
		$cat->t3("");
		my $newcatid = $cat->add();

		$self->{parent_id} = $newcatid;
	}

	$self->SUPER::www_save();

}




1;

__END__

=head1 NAME

AbstractCategoryGizmoHack: an abstract class for creating Gizmo-Category combinations, such as FAQ

=head1 SYNOPSIS    
    
=head1 DESCRIPTION

=head2 Methods


=cut


