###########################################################################
# <LICENSEINFO>
# Metadot SmartSite Server                                                                      
# Copyright (C) 2001 Metadot Corporation                                                        
#                                                                                               
# For more information go to                                                                    
# http://www.metadot.com                                                                        
# or contact us at info@metadot.com                                                             
#                                                                                               
#</LICENSEINFO>
############################################################################


package DBAccess::DBSchema::Table;
use strict;

use MetadotObject;

use vars qw(@ISA);
@ISA = qw(MetadotObject);

my %CONSTRUCTOR_PROPERTIES = (    
      table_name    => {value=>undef},
      columns       => {value=>undef},
      table_info    => {value=>undef},
);

sub get_constructor_properties {
    return \%CONSTRUCTOR_PROPERTIES;
}


sub _initialize {
    my ($self, %args) = @_;
    $self->SUPER::_initialize(%args);

    unless ($self->{columns}) {
        $self->{columns} = {};
    }
}


# returns a list of the column names
sub get_column_names {
    my $self = shift;
    my $column_ref = $self->get_columns();
    
    my $result = keys(%$column_ref);
    return $result;
}

# returns a column by name
sub get_column {
    my $self = shift;
    my $column_name  = uc(shift) || die ('Must specify column name');
    my $result      = $self->get_columns()->{$column_name};
    
    #print STDERR "$self\->set_column($column_name) returns $result\n";    
    return $result;
}

# adds or replaces a column definition
sub set_column {
    my $self = shift;
    my $column_obj       = shift || die('Must specify a column object');

    my $name            = uc($column_obj->get_name());
    
    #print STDERR "$self\->set_column($column_obj) $name\n";

    my $column_ref       = $self->get_columns();
    $column_ref->{$name} = $column_obj;
}


# generic accessors / mutators

# name is a synonym for table_name
sub get_name {
    my $self = shift;
    return $self->get_table_name(@_);
}

sub set_name {
    my $self = shift;
    return $self->set_table_name(@_);
}

sub get_table_name {
    my $self = shift;
    return $self->access_property('table_name', @_);
}

sub set_table_name {
    my $self = shift;
    return $self->mutate_property('table_name', @_);
}

sub get_columns {
    my $self = shift;
    return $self->access_property('columns', @_);
}

sub set_columns {
    my $self = shift;
    return $self->mutate_property('columns', @_);
}



1;
