###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
#                                                                #
#  THIS IS DEPRECATED. USE Metadot::ObjectAccess::ObjectAccessor #
#                                                                #
##################################################################                             
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################
##################################################################







package DBAccess::ObjectAccessor;
use strict;
use DBAccess;

sub new {
   my $proto = shift;
   my $class = ref($proto) || $proto;
   my $self = {
      properties => [],
      tables     => [],
      key_fields => [],
      joins      => [],
      field_map  => {},
   };

   bless ($self, $class);

   my %args = @_;
   if (%args) {
      $self->initialize(%args);
   }

   return $self;
}

# initializes the object based on constructor's arguments
sub initialize {
   my $self = shift;
   my %args = @_;

   foreach my $property ('properties','tables','key_fields','joins') {
      if (defined($args{$property})) {
         $self->{$property} = $self->_force_to_array_ref($args{$property});
      }
   }

   foreach my $property ('field_map') {
      if (defined($args{$property})) {
         $self->{$property} = $self->_force_to_hash_ref($args{$property});
      }
   }

}

# loads an object's properties from the database. The object must
# be uniquely identified by a key value
sub retrieve {
   my $self = shift;
   my $key = shift || die("Must provide a key value");
   my $object = shift || {};

   my @fields = ();
   my @properties = $self->get_properties();

   # construct the sql statement
   foreach my $property (@properties) {
      push @fields, $self->get_field_for_property($property);
   }
   
   my $select_fields = join(',', @fields);
   my $select_from   = join(',', $self->get_tables());
   my @wheres = ($self->get_key_clause($key));
   if ($self->get_join_clause()) {
      push @wheres, $self->get_join_clause();
   }
   my $where_clause  = join(' AND ', @wheres);

   print STDERR "ObjectAccessor->retrieve using: $select_fields\n$select_from\n$where_clause\n\n";

   my @values = DBAccess->sqlSelect( $select_fields, $select_from, $where_clause );
   
   # populate the object from the results
   my %properties_hash = ();
   foreach my $property (@properties) {
      $properties_hash{$property} = shift @values;
   }
   $object = $self->initialize_object($object, %properties_hash);

   return $object;
}

sub retrieve_where {
   my $self = shift;
   my $where = shift || die("Must supply a where clause");
   my $class = shift;

   my @results = ();
   my @fields = ();
   my @properties = $self->get_properties();

   # construct the sql statement
   foreach my $property (@properties) {
      push @fields, $self->get_field_for_property($property);
   }
   
   my $select_fields = join(',', @fields);
   my $select_from   = join(',', $self->get_tables());
   my @wheres = ($where);
   if ($self->get_join_clause()) {
      push @wheres, $self->get_join_clause();
   }
   my $where_clause  = join(' AND ', @wheres);

   print STDERR "ObjectAccessor->retrieve using: $select_fields\n$select_from\n$where_clause\n\n";

   my $sth = DBAccess->sqlSelectMany( $select_fields, $select_from, $where_clause );
   
   while (my @values = $sth->fetchrow()) {
      # populate the object from the results
      my %properties_hash=();
      foreach my $property (@properties) {
         $properties_hash{$property} = shift @values;
      }
      my $object = $self->initialize_object($class, %properties_hash);
      push @results, $object;
   }
   return @results;
}

sub get_field_for_property {
   my ($self, $property) = @_;
   my $result;
   if (defined($self->{field_map}->{$property})) {
      $result = $self->{field_map}->{$property};
   } else {
      $result = $property;
   }
   return $result;
}

sub get_key_clause {
   my $self = shift;
   my $key = shift || die("Must provide the key value");
   my $result;

   my @wheres = ();
   foreach my $key_field ($self->get_key_fields()) {
      push @wheres, "$key_field = $key";
   }

   $result = join(' AND ', @wheres);
   return $result;
}

sub get_join_clause {
   my $self = shift;
   my $result;

   my @wheres = ();
   foreach my $join ($self->get_joins()) {
      push @wheres, $join;
   }
   
   $result = join(' AND ', @wheres);
   return $result;
}

# accessors & mutators
sub get_properties {
   my $self = shift;
   return $self->_force_to_array($self->{properties});
}

sub get_tables {
   my $self = shift;
   return $self->_force_to_array($self->{tables});
}

sub get_key_fields {
   my $self = shift;
   return $self->_force_to_array($self->{key_fields});
}

sub get_joins {
   my $self = shift;
   return $self->_force_to_array($self->{joins});
}

sub set_properties {
   my $self = shift;
   my @values = @_;
   $self->{properties} = $self->_force_to_array_ref(@values);
}

sub set_tables {
   my $self = shift;
   my @values = @_;
   $self->{tables} = $self->_force_to_array_ref(@values);
}

sub set_key_fields {
   my $self = shift;
   my @values = @_;
   $self->{key_fields} = $self->_force_to_array_ref(@values);
}

sub set_joins {
   my $self = shift;
   my @values = @_;
   $self->{joins} = $self->_force_to_array_ref(@values);
}


# private methods
sub _force_to_array {
   my $class = shift;
   my @input = @_;
   my @result;

   if (@input && scalar(@input) == 1 && ref($input[0]) eq 'ARRAY') {
      @result = @{ $input[0] };
   } else {
      @result = @input;
   }
   return @result;
}

sub _force_to_array_ref {
   my $class = shift;
   my @input = @_;
   my $result;
   
   if (@input && scalar(@input) == 1 && ref($input[0]) eq 'ARRAY') {
      $result = $input[0];
   } else {
      $result = [ @input ];
   }
   return $result;
}

sub _force_to_hash {
   my $class = shift;
   my @input = @_;
   my %result;

   if (@input && scalar(@input) == 1 && ref($input[0]) eq 'HASH') {
      %result = %{ $input[0] };
   } else {
      %result = @input;
   }
   return %result;
}

sub _force_to_hash_ref {
   my $class = shift;
   my @input = @_;
   my $result;
   
   if (@input && scalar(@input) == 1 && ref($input[0]) eq 'HASH') {
      $result = $input[0];
   } else {
      $result = { @input };
   }
   return $result;
}


sub initialize_object {
    my $self = shift;
    my $object = shift;
    my %properties = @_;

    # if object was a class name, try to instantiate it
    unless (ref($object)) {
        # try first to instantiate
        eval {
            $object = $object->new();
        };
        # catch by creating an empty hash
        if ($@) {
            $object = {};
        }
    }

# Commenting out, because this tends to call Portal->initialize()
#    if (ref($object) ne 'HASH' && $object->can('initialize')) {
#        $object->initialize(%properties);
#    } elsif (ref($object) ne 'HASH' && $object->can('set_properties')){
    if (ref($object) ne 'HASH' && $object->can('set_properties')){
        $object->set_properties(%properties);
    } else {
        foreach my $key ( keys(%properties) ) {
            $object->{$key} = $properties{$key};
        }
    }
    return $object;
}



1;
