###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name$
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package FormMaker;
use strict;
# use Data::Dumper;
use Sendmail;
# use Text::CSV;
use Text::CSV_XS;
use CGI qw(escapeHTML);


use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $DISPLAY $PARAMS $CACHE);
use Gizmo::FormMaker::FormMakerElement;
use Gizmo::FormMaker::FieldElement;
use Metadot::Controllable;

use vars qw(@ISA);
@ISA=qw(GizmoBuilder Metadot::Controllable);
my $version = "1.1";

sub get_version {
    return $version;
}


my $AC = 
  {
   BUNDLES => {		     		      
	       MOD => { DESC => 'Edit',
            #CKIM-SHORTCUTS: added a new op 'create_shortcut'
			OPS => [qw(add_child create_child modify save up down paste create_shortcut)],  
			DEFAULT_LEVEL => 8,
			MIN_LEVEL => 2,
		      },
	       DEL => { DESC => 'Delete and Cut',
			OPS => [qw(delete delete_ok cut)],  
			DEFAULT_LEVEL => 8,
			MIN_LEVEL => 8,
			PARENT_DEP => {cut => 'MOD'},
		      },
	       DISP => { DESC => 'View',
			 OPS => [qw(show show_summary process_form)],  
			 DEFAULT_LEVEL => 0,
			 MIN_LEVEL => 0,
			 DEFAULTS_TO => 'on',
		       },
	       EDITP => { DESC => 'Change<br>Permissions',
			  OPS => [qw(edit_permissions set_permissions change_owner)],  
			  DEFAULT_LEVEL => 8,
			  MIN_LEVEL => 8,
			},
	      },
   DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
  };


###
### ACCESS CONTROL METHODS
###
sub get_AC {
  my $self = shift;  
  my $class = ref $self || $self;
  
  die "&get_AC should be overridden in class $class" 
    unless ($class eq __PACKAGE__);

  return $AC;
}




####################################
sub new {
  my $proto = shift;
  my $id = shift;

  my $class = ref($proto) || $proto;
  my $self;

#    print STDERR "in FormMaker::new\n";
  if (defined ($id)){
      $self = $class->SUPER::new($id);
  } else {
      $self = $class->SUPER::new();
  }

  $self->{is_a}       = __PACKAGE__;

  $self->_form_initialize();
  $self->set_field_info (
    "name", "<b>Form Title</b>", 1,
    "description", "<b>Description</b>", 0,
		"t1", "<b>Email To</b>", 1,
        "t2", "<b>Subject Template</b>", 0,
        "t3", "<b>Body Template</b>", 0,
		"i3", "<b>Email Format</b>", 0,
    );
		
	$self->set_field_type('i3', 'select');
  
  $self->{display_wrapped} = { show => 1,
                             };
      
  bless ($self, $class);
  # reconsecrate
  return $self;
}

sub get_field_names {
    my $self = shift;
#     print STDERR "in get_field_names\n";
    #return the list of set field used by the gizmo object
    my @flist = ();
    foreach my $field (keys (%{$self->{fields}} ) ) {
	push @flist, $field;
    }
    return @flist;
}

## will replace the html produced by GizmoBuilder for given instance variable
sub _get_alt_text {
      my $self = shift;
      my $field = shift;
      my $id = $self->id();

      return unless defined $field;

      my $value = $FORM{$field} || $self->{$field};
      my $info = "<input type=text size=\"20\" maxlength=\"30\" name=\"$field\" value=\"$value\" />";
      return $info;
}

sub get_gizmo_name {
   my $self = shift;
   return "FormMaker";
}

#override one of the default form types.
sub _form_initialize {
    my $self = shift;
		
	$self->{fields}->{i3}->{formtype} = "select";
	$self->{fields}->{i3}->{selectsize} = "1";
    $self->{fields}->{i3}->{selectoptions} = "0,1,2";
    my $ts = "
		
";
    $self->set_intro_text($ts);

}


sub get_email_to {
   my $self = shift;
   return $self->{t1};
}

sub get_format{
   my $self = shift;
   return $self->{i3};
}

sub get_subject_template {
    my $self = shift;
    return $self->{t2};
}

sub get_body_template {
    my $self = shift;
    return $self->{t3};
}


sub get_success_page {
  my $self = shift;
  $DISPLAY->print_choices( 
      "Thank you for your submission.",
	  'readme', "Return to the portal", "index.pl?iid=".($FORM{caller_iid} || 0)
     );
}


sub show_edit_form {
    my $self  = shift;
    my $title = shift;
    $self->debug_msg(4, $self->{is_a}."::show_edit_form($title)");
		my $id   = $self->id;
		my $name = $self->name || $FORM{name} ;
		$name = escapeHTML($name);
		my $description = $self->description  || $FORM{description} ;
		my $email_to = $self->get_email_to()  || $FORM{t1} ;
		my ($txt_format, $html_format, $csv_format, $custom_format);
		my $format = $self->get_format();
		my $childs_list = '<table>';
		if ($format==1) { 
			 $html_format="checked"; 
		} elsif ($format==2){
			 $csv_format="checked"; 
		} elsif ($format==3){
			 $custom_format="checked"; 
		} else {
			 $txt_format="checked";
		}
		
		if ($id) {
			 $childs_list .= $self->show_childs_list( type=>'FormMakerElement', decorator=>'HTML::Decorator::NullDecorator');
		}
    $childs_list .= '</table>';
		
		my $output = "\n<h1>$title</h1>";
		$output .=  "
		<p>The form Name will be used as its title when the form is displayed in a page.<br>
		The optional Description will be displayed under the name and may
		contain additional instructions for filling out your Form.<br>
     The information in the submitted form will be emailed to the indicated address.
     The 'Email format' is the format that will be used when emailing the submitted form information.
     Comma-separated 'csv' format will allow you to import data to MS Excel or similar programs.<br>
     Click 'Add new field' to start defining your form fields, once you have filled out
     the required information <br>
		
		<p><b>Name</b> (*)<br>
	  <input type='text' name='name' size='55' maxlength='255' value=\"$name\" /><br>
	  <p><b>Description</b><br>
	  <textarea name='description' rows=2 cols=46 WRAP='VIRTUAL'>$description</textarea>
	  <br>
		<p><b>Mail form submissions to this email address</b> (*)<br>
	  <input type='text' name='t1' size='55' maxlength='255' value=\"$email_to\" />
	  <br>
          Add a comma between email addresses.
		<p><b>Custom subject line for emails</b><br>
	  <input type='text' name='t2' size='80' maxlength='255' value=\"".$self->get_t2()."\" />
	  <br>
          Optionally specify a custom subject line for emails. If you do not
          specify the subject line, a standard subject line will be used.".
            $self->get_template_field_explanation()."</p>".
		"<p><b>Email Format</b><br>
	  <input type='radio' name='i3' value='0' $txt_format /> Human-readable format plain-text <br>" .
#		"<input type='radio' name='i3' value='1' $html_format /> Human-readable format html <br>" .
		"<input type='radio' name='i3' value='2' $csv_format /> CSV (comma-separated) format<br>
		 <input type='radio' name='i3' value='3' $custom_format /> Custom format (define the template below)<br>
	  <br>
		<p><b>Custom body for emails</b> (choose Custom format above)<br>
	  <textarea name='t3' cols='80' rows='8' wrap='virtual'>".escapeHTML($self->get_t3())."</textarea>
	  <br>
          Optionally specify a custom body for emails. This is required if you
          selected 'Custom format' above. If you do not specify the custom body, 
          then a standard plain-text email will be sent.".
            $self->get_template_field_explanation()."</p>".
		"<p><b>Form Fields</b><br>
		  $childs_list<br>
	  <input type='submit' name='new_field'  value='Add New Field' />
	  <p><i>(*) Mandatory field(s)</i><br>
		
		";
    
    return $output;
}

## called before saving or updating
sub process_edit_form {
    my $self  = shift;
    my $error = shift || undef;
		$FORM{name} = escapeHTML( $FORM{name} );

    ## check for required fields
    foreach my $field (keys (%{$self->{fields}})) {

	  # upload fields do their own "is_required" checking
	  next if ( $self->{fields}{$field}{formtype} =~ /_upload$/ );

        if ($self->{fields}->{$field}->{required} &&
            (!$FORM{$field} || !($FORM{$field} =~ /\S/)) # field is not just whitespace
            ) {
            $error .= "<br>".$self->{fields}->{$field}->{label} . 
                " is a required field.\n";
        }
    }

    # if email format 3 is selected, then a custom body is required
    if ($FORM{i3} == 3 && !$FORM{t3}) {        
        $error .= "<br>You have selected a custom email format, but did not specify the custom body.\n";
    }

    my $iid = $self->get_iid();

    if (defined($error) && $iid){ ## if error and updating

        $self->www_modify($error);
    }
    elsif (defined($error)) {   ## if error and saving new
        my $parentid = $self->parent_id();
        my $parent = Gizmo->restore($parentid);
#          print STDERR "in gizmo::process_edit_form, calling parent add child on error\n";
        $parent->www_add_child($error, $self->is_a); 
    }
    else {                        ## no errors
        my $checked_reqs = 1;
        $self->www_save($checked_reqs);
				$FORM{child_type} = 'FormMakerElement';
				    
    }
}

# this method handles what to do when you are finished successfully creating a new gizmo
sub _show_create_success {
	my $self = shift;	
	 if ($FORM{new_field}) { 
	   $FORM{child_type} = 'FormMakerElement';
		 $FORM{name} = '';
		 $FORM{description} = '';
		 $self->www_add_child('', 'FormMakerElement'); 
	}
	else {	
		 #$self->www_modify();
		 my $parentid = $self->parent_id();
		 my $url = "index.pl?id=$parentid";
		 $self->md_redirect($url);
	}
	
}

sub _show_edit_success {
	my $self = shift;
  if ($FORM{new_field}) {
		 $FORM{name} = '';
		 $FORM{description} = '';
		 $self->www_add_child('', 'FormMakerElement'); 
	}
	else {	
		 $self->md_redirect("index.pl?iid=".$self->parent_id);
	}

}



#########################################
sub show_childs_list {

	my $self = shift;
	my %args = @_;

	my $id = $self->{id};
	$self->debug_msg(3, "Category::show_childs_list(id=$id)");

			my $row_color_on = "#FFFFFF";
			my $row_color_off = "#EEEEEE";
			my $row_color = $row_color_on;

	#print STDERR "Category->show_childs_list\n",Dumper \%args;

    # deleted a bunch of code that was created by someone that
    # didn't know what he was doing
	
    my $str = "";
	my @childs_list = @{$self->build_gizmo_list(%args)};
    foreach my $gizmo (@childs_list) {
        $str .= $gizmo->show_summary();
    }

	return $str;

}

##########################################
### show
### return one newsitem (page format)
##########################################
sub show {
    my $self        = shift;
    my $isa         = $self->is_a();
    my $iid         = $self->get_iid();
    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $buttons     = $self->get_buttons();
    my $show_border = $self->get_c5();
		$self->debug_msg(1, "FormMaker::show()");
		
		my $form = "<form name='form$iid' action='index.pl' method='post'>";

    #just do inline alignment for the 'bullet' view.
    my $default_bullet_align = "top"; # default alignment for bullet list is 'top'.
				      # The actual set alignment is used in
				      # the more detailed view.
		my $childs_list .= $self->show_childs_list( type=>'FormMakerElement', decorator=>'HTML::Decorator::NullDecorator');
    my $html        = "$buttons";
  
    $html .= "
		<b>$name</b></td></tr>$form
		<tr><td></td><td>&nbsp;</td></tr>
		<tr><td></td><td>$description</td></tr>
		<tr><td></td><td>&nbsp;</td></tr>
    <tr><td></td><td><table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">$childs_list</table></td></tr>
		<tr><td></td><td align='center' height='60' valign='middle'>
		<input type='reset' value='Clear' /> 
		<input type='submit' name='Submit' value='Submit' />
		<input type='hidden' name='isa' value='$isa' />
		<input type='hidden' name='iid' value='$iid' />
		<input type='hidden' name='caller_iid' value='".($FORM{iid} || $FORM{id})."' />
		<input type='hidden' name='op' value='process_form' />
		</form>
		";

    return $html;
}


##########################################
### show_in_list
### return one newsitem (list format)
##########################################
# aju: do I need this method?
sub show_in_list {
    my $self        = shift;
    my $isa         = $self->is_a();
    my $iid         = $self->get_iid();
    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $buttons     = $self->get_buttons();
    my $show_border = $self->get_c5();
		$self->debug_msg(1, "FormMaker::show_in_list()");
		
		my $form = "<form name='form$iid' action='index.pl' method='post'>";

    #just do inline alignment for the 'bullet' view.
    my $default_bullet_align = "top"; # default alignment for bullet list is 'top'.
				      # The actual set alignment is used in
				      # the more detailed view.
		my $childs_list .= $self->show_childs_list( type=>'FormMakerElement', decorator=>'HTML::Decorator::NullDecorator');
    my $html        = "$buttons";
  
    $html .= "
		<b>$name</b></td></tr>$form
		<tr><td></td><td>&nbsp;</td></tr>
		<tr><td></td><td>$description</td></tr>
		<tr><td></td><td>&nbsp;</td></tr>
    <tr><td></td><td><table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">$childs_list</table></td></tr>
		<tr><td></td><td align='center' height='60' valign='middle'>
		<input type='reset' value='Clear' /> 
		<input type='submit' name='Submit' value='Submit' />
		<input type='hidden' name='isa' value='$isa' />
		<input type='hidden' name='iid' value='$iid' />
		<input type='hidden' name='caller_iid' value='".($FORM{iid} || $FORM{id})."' />
		<input type='hidden' name='op' value='process_form' />
		</form>
		";

    return $html;
}


sub show_summary {
    my $self        = shift;
    my $isa         = $self->is_a();
    my $iid         = $self->get_iid();
    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $buttons     = $self->get_buttons();
    my $show_border = $self->get_c5();
		$self->debug_msg(1, "FormMaker::show_summary()");
		
		my $form = "<form name='form$iid' action='index.pl' method='post'>";
		my $send_user_info='';
# 		my $logged_in = $USER->is_logged_in() || 0;
# 		if ( $logged_in >= 1) {  # if logged in
# 			 $send_user_info = "
# 			 <tr><td></td><td colspan='2'><input type='checkbox' name='send_user_info' value='1'>
# 			  Submit my name and email with this form.
# 			 </td></tr>
# 			 ";
# 		}

    #just do inline alignment for the 'bullet' view.
    my $default_bullet_align = "top"; # default alignment for bullet list is 'top'.
				      # The actual set alignment is used in
				      # the more detailed view.
		my $childs_list .= $self->show_childs_list( type=>'FormMakerElement', decorator=>'HTML::Decorator::NullDecorator');
    my $html        = "$buttons";

#   Whoever coded this mess didn't use correct HTML!!
#
#     $html .= "
#         <b>$name</b></td></tr>$form
#         <tr><td></td><td>&nbsp;</td></tr>
#         <tr><td></td><td>$description</td></tr>
#         <tr><td></td><td>&nbsp;</td></tr>
#     <tr><td></td><td><table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
#         $childs_list
#         $send_user_info
#         </table></td></tr>
#         <tr><td></td><td align='center' height='60' valign='middle'>
#         <input type='reset' value='Clear'>
#         <input type='submit' name='Submit' value='Submit'>
#         <input type='hidden' name='isa' value='$isa'>
#         <input type='hidden' name='iid' value='$iid'>
#         <input type='hidden' name='op' value='process_form'>
#         </form>
#         ";

     $html .= "
         $form
         <b>$name</b><br/>         
         $description<br/>
         <table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
         $childs_list
          <tr>
            <td colspan='2'></td>
            <td align=\"left\">
                 <input type='reset' value='Clear' />
                 <input type='submit' name='Submit' value='Submit' />
                 <input type='hidden' name='isa' value='$isa' />
                 <input type='hidden' name='iid' value='$iid' />
           		 <input type='hidden' name='caller_iid' value='".($FORM{iid} || $FORM{id})."' />
                 <input type='hidden' name='op' value='process_form' />
             </td>
           </tr>
         </table>
         </form>
         $send_user_info
         ";


    return $html;
}


sub _removeH {
    my ($self, $value) = @_;
    return $value unless defined($value);
    $value =~ s/<(?:[^>'"]*|(['"]).*?\1)*>//gs;
    return $value;
}

####################################################
######                                        #######
######            FRONT-END METHODS           #######
######                                        #######
#####################################################
sub www_process_form {
    my $self = shift;
    
    $self->debug_msg(3, "In $self\->www_process_form");
    
    my $name = $self->_removeH($self->get_name());
    my $btitle = (uc $name) . " FORM SUBMISSION";
    my $btitle_text = "######\n#\n# $btitle\n#\n######\n\n";
    my $format = $self->get_format();
    my $email_to= $self->get_email_to();
    my $cc;
    my ($field_name,$field_type,$field_text,$field_value);
    my (@childs_list, @allowed_list);
    my $gizmo;
    my $subj;
    my $email_from;
    my $msg;
    my $csv = Text::CSV_XS->new({binary => 1});
    my $str;
    my $basename = $PARAMS->{basename} ;
    my $parentid = $self->parent_id();
    my $link = $basename . "/index.pl?iid=$parentid";

    $self->debug_msg(5, "In $self\->www_process_form, about to process subject");
    
    if ($self->get_subject_template()) {
        $subj = $self->process_template($self->get_subject_template());
    } else {
        $subj = "[$name] - Form Submission";
    }

    # get user info
    my $intro = "This is an automatically-generated email sent from $basename" . ".\n  Please do not reply.\n";
    my $user_info ='';
    my $footer_text = "\n--\nEmail generated by Metadot FormMaker v" . $self->get_version() . " on " . localtime(time) . "\n" . 
	"From: $link\n" . 
# 		    "IP# of submitter: " .
	"\n";
    my $footer_html = "--<br>\nEmail generated by Metadot FormMaker v" . $self->get_version() . " on " . localtime(time) . "<br>\n" .  
	"From: $link<br>\n" . 
# 		    "IP# of submitter: " .
	"<br>\n";
# 		if (( $FORM{send_user_info} ) && ($USER->is_logged_in() >= 1)){
# 		    my $uid = $USER->{uid} ;
# 		    my $user_name  = $USER->fullname();
# 		    my $user_email = $USER->get_email_from_uid ($uid);
#  		    $user_info = "User Info: Name: $user_name Email: $user_email";
# 		}

    $self->debug_msg(5, "In $self\->www_process_form, about to process body using format '$format'");
    $self->debug_msg(5, "In $self\->www_process_form, my body template is: ".$self->get_body_template());
    
    if ($format == 3 && $self->get_body_template()) {
        $msg = $self->process_template($self->get_body_template());
    } else {
        # standard msg processing

        if ($format == 1) {
            $msg .= "<html><body>$btitle<br /><br /><table border=\"0\">";
            $msg .= "<tr><td width='30%'></td><td width='70%'></td></tr>\n";
        } elsif ($format == 2) {
            $msg .= "$btitle_text\n\nField Name,Field Value\n";
        } else {
        # 	$msg .= "$btitle\n\n* Field Label\nValue\n\n";
            # per daniel..
            $msg .= "$btitle_text\n\n";
        }

        $self->debug_msg(5, "In $self\->www_process_form, about to build gizmo list");
        
        @childs_list = @{$self->build_gizmo_list(type=>'FormMakerElement', decorator=>'HTML::Decorator::NullDecorator')};
        # first determine which ones I'm allowed to display
        #print @childs_list;

        foreach $gizmo (@childs_list) {
            if ($gizmo->is_allowed_to_do('show', $USER) ) {	push @allowed_list, $gizmo;	}
        }

        foreach $gizmo (@allowed_list) {
            $field_text = $self->_removeH($gizmo->get_description());
            $field_name = $gizmo->get_name();
            $field_type  = $gizmo->get_type();
            $field_value = $self->_format_from_FORM($field_name, $format);
            if ($field_type eq 'TextBlock') { next;}
            if ($field_type eq 'Checkbox') { if ($field_value) {$field_value='Yes';} }
            if ($format == 1) {
                $msg .= "<tr><td>$field_text</td><td>$field_value</td></tr>\n";
            } elsif ($format == 2) {
                my @cols = ($field_name, $field_value);
                my $status = $csv->combine(@cols);    # combine columns into a string
                my $str = $csv->string();               # get the combined string
                print STDERR "got csv string $str from $field_value\n";
                $msg .= $str."\n";
            } else {
                $msg .= "### $field_text\n$field_value\n\n";
            }
        }
    }

    if ($format == 1) {
        $msg .= "</table>$footer_html</body></html>";
    } 
    else {
        $msg .= $footer_text;
    }

    #print $msg; # debug

    $self->debug_msg(5, "In $self\->www_process_form, about to send email with subject: $subj<br>message: $msg");
    
    # sending mail:	
    my $sm = Sendmail->new($email_to, $cc, "", $subj, $msg);
    $sm->set_from($email_from);
    
    if ($format == 1) {
        $sm->set_encoding('html'); 
    }

    if ($sm->sendMail()) {
    	my $content = $self->get_success_page(); # show success page
    }
}


sub _format_from_FORM {
    my ($self, $field_name, $format) = @_;
    my $fvalue = $FORM{$field_name};
    if (!$fvalue) {
	return $fvalue;
    }
    my $vlist;
    if( ref( $fvalue ) eq 'ARRAY' ) { #  then unpack..
	$vlist = $fvalue;
    }
    else {
# 	print STDERR "fvalue is $fvalue\n";
	$vlist = [$fvalue];
    }
#     print STDERR "in _format_from_FORM, got vlist ", Dumper($vlist), " from $fvalue\n";
    my $vstring = '';
    if ($format == 1) {
	$vstring = join("<br>", @$vlist);
    }
    elsif ($format == 2) {
	$vstring = join("\n", @$vlist); 
    }
    else {
	$vstring = join("\n", @$vlist);
    }
#     print STDERR "got vstring: $vstring\n";
    return $vstring;
}

sub www_create_child {
  my $self = shift;
  my $child_type = shift;

  $self->debug_msg(3, "Gizmo::www_create_child()");

  my $child_isa = $FORM{child_type} || $child_type;

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  my $child = $child_isa->new({
			 uid => $USER->uid(),
			 parent_id => $parentid,
			});

  # this method should ALWAYS be overridden in derived classes.
  $child->process_edit_form();
}



sub www_add_child {
    my $self = shift;
    my $errormsg = shift;
    my $child_type = shift;
    $self->debug_msg(3, $self->{is_a}."::www_add_child()");
    
    my $child_isa = $FORM{child_type} || $child_type;
    my $gizmo_name = $child_isa->get_gizmo_name();
    
    my $parentid = $self->defined_or_exit($self->get_iid(), "parent id is missing.");
    # we re-instanciate the object because we want a brand new one
    # unless there is an error message (we want to keep its data)
    #my $child = $child_isa->new({ parent_id => $parentid }) unless ($errormsg);
    my $child = $child_isa->new({ parent_id => $parentid });
    
    $errormsg = $errormsg ?
    "<p><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p>\n" :
    "";
    
    my $form = $child->show_edit_form("Adding new $gizmo_name");
    my $content =
    Components->color_palette_code(). 
    "
    <form name=fm action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">
    <table align='right'><tr><td><input type='submit' value='Create'></td></tr></table>
    <input type='hidden' name='isa' value='".$self->is_a()."'>\n
    <input type='hidden' name='iid' value='$parentid'>\n
    <input type='hidden' name='op' value='create_child'>\n
    <input type='hidden' name='child_type' value='$child_isa'>\n
    $errormsg
    $form
    <table align='right'><tr><td><input type='submit' value='Create'></td></tr></table>
    <br>&nbsp;
    </form>
    ";
    
    $child->print_form($content, "Adding new $gizmo_name");
}


##############

sub delete {
  my $self = shift;

  my $id = $self->{id};
  $self->debug_msg(3, "Gizmo::delete(id=$id)");

  if(!$id){
    $self->debug_msg(3, "Error Gizmo::delete(id=$id): I don't know what id to delete");
    return 0;
  }


  # Delete childs
  my @childs_list = @{$self->build_gizmo_list(type=>'FormMakerElement', decorator=>'HTML::Decorator::NullDecorator')};
  foreach my $gizmo (@childs_list) {
  	$gizmo->delete();
  }

  return $self->SUPER::delete();
}


sub get_template_field_explanation {
    return <<EOM
        You can include form fields by enclosing the corresponding field name
        in double brackets, such as: &lt;&lt;requester_name&gt;&gt;                
EOM
;
}


# expands <<field_name>>
sub process_template {
    my $self = shift;
    my $text = shift;

    my @tags = ($text =~ m/<<\s*.*?\s*>>/g);
    foreach my $field_tag (@tags)  {
        my $field_name;
        my $replace_text;

        if ($field_tag =~ m/<<\s*(.*?)\s*>>/) {
            $field_name = $1;
            $replace_text = $self->_format_from_FORM($field_name, 3);
        } else {
            $replace_text = '';
        }
        
        $text =~ s/\Q$field_tag\E/$replace_text/g;

        $self->debug_msg(5, "Replaced tag '$field_tag', field '$field_name' with '$replace_text'");
    }

    return $text;
}


1;

__END__

