###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
#
#
# How to enable GizmoShortcuts:
# - Your Gizmo class must descend from GizmoBuilder
# - Make sure your class has the following op: 'create_shortcut' (this is
#   included with GizmoBuilder)
# - If you have any ops that need to be handled natively by the shortcut
#   rather than by the gizmo itself, add that op to %native_ops
# - If you override the delete() method, make sure to call SUPER::delete()  
#
############################################################################
package GizmoShortcut;


use strict;
use Portal;
use GizmoBuilder;
use Metadot::Controllable;

use vars qw(@ISA);
my $version = "4.5";

sub get_version {
    return $version;
}

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);

use Data::Dumper;

@ISA=qw(GizmoBuilder Metadot::Controllable);
my $is_public = "no";

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $args = $class->normalize_constructor_args(shift);
    my $id = $args->{id};
    
    my $self;
    
    if (defined ($id)){
        return $class->restore($id);
    } else {
        $self  = $class->SUPER::new($args);
    }
    
    my $source_obj = $args->{source_obj} || shift;

    $self->set_field_info (
        "name", "<b>Link Title</b> <i><font size=-2>Only if using Link Display Format.</font></i>", 0,
        "c2",   "<b>Display Format</b>", 0,
    );
    #$self->set_field_type('c2', 'radio');
    $self->set_field_form_text('c2', \'_get_radio_button_html');
    #$self->set_field_form_text('c2', $self->_get_radio_button_html());

    bless ($self, $class);      
    $self->set_is_a('GizmoShortcut');
    $self->_initialize_source( target=>$source_obj );

    return $self;
}


sub restore {
    my $class = shift;
    my @args = @_;
    my $self = $class->SUPER::restore(@args);

    $self->_initialize_source();

    return $self;
}


sub _initialize_source {
    my $self = shift || die ("Can't initialize NULL object");
    my %args = @_;
    
    my $source_obj = $args{target};
    # if this shortcut already has an object assigned, initialize the object    
    if (defined($self->get_source_id()) && $self->get_source_class()){
        eval {
            $self->set_source_object($self->get_source_class()->new($self->get_source_id()));
        };
    }
    # alternately, if a source object was provided, initialize with that object
    elsif ($source_obj) {
        $self->set_source_object($source_obj);   
    }
}
    

sub delete_shortcuts_for {
    my $self = shift;
    my $source_id = shift || die ('Must specify a source id');
    
    # CKIM-SHORTCUTS collect all shortcuts of this item
    my @ids;  
    my $source_id_field = $self->get_source_id_field_name();
    $self->get_object_accessor()->delete_where(where=>"isa='GizmoShortcut' and $source_id_field=$source_id");
    #DBAccess->sqlDelete("instance","isa='GizmoShortcut' and $source_id_field=$source_id");    
}

sub show_summary {
    my ($self, @params) = @_;
    my $buttons     = $self->get_buttons();
    
    my $result = $buttons;

    my $gizmo = $self->get_source_object();
    #print STDERR "show_summary: my gizmo is $gizmo\n";
    if ($gizmo) {
        # try to pick up the gizmo's name
        my $name ||= $self->get_name(target=>$gizmo);
        if ($self->get_full_text_option() eq 'summ') { ## if summary

            my $edit_mode_was_on = 0; 
            if ($SESSION->admin_mode()) {
                ## do this step before turning admin mode off
                $result .= $self->create_link_to_original($self->show_shortcut_link_image() );
                $result .= "<br>";
                $edit_mode_was_on = 1; 
                ## turn off temporarily - this is done so that
                ##  edit buttons will not display for gizmo
                $SESSION->edit_off();
            }

            my @show_op_list = qw(show_summary show);
            foreach my $show_op (@show_op_list) {
                if ($gizmo->can($show_op)) {
                    $result .= $gizmo->$show_op();
                    last;
                }
            }

            if ($edit_mode_was_on) {
                $SESSION->edit_on();## turn back on
            }
        } else { ## show link {
            $result .= $self->create_link_to_original();
        }
    } else {
        $result .= $self->get_name();
    }

    $result .= "<br>\n";
    return $result;
}


sub get_name {
    my $self = shift;
    my %args = @_;
    my $default = $args{default};
    my $target = $args{target} || $self->get_source_object();

    # try to get the name from the Shortcut instance
    my $name = $self->name();
    if (!$name && $target && ref($target)) {
        # try to get the gizmo's name
        $name = $target->get_name();
        unless ($name) {
            $name = $target->get_description();
            my $max_len = 50;
            if (length($name) > $max_len) {
                $name = Utility->trim_clean($name, $max_len).'...';
            }
        }
        $name ||= $default;
        $name ||= 'Shortcut to '.($target->get_gizmo_name());
    }
    $name ||= $default || 'Unnamed Shortcut';

    return $name;
}


sub show_shortcut_link_image {
    # generates html to display a shortcut image.
    my $self = shift;
    my $alt_text = shift || 'Go to original';
    
    return '' unless($SESSION->admin_mode());

    my $imageslocation = $PARAMS->{imageslocation};    
    my $str = "<img class=\"icon\" src=\"$imageslocation/obj/shortcut_link.gif\" width=\"11\" height=\"11\" border=\"0\" alt=\"".$alt_text."\">";
    return $str;
}

sub create_link_to_original {
    # generates an href to the original item
    my $self = shift;
    my $link_text = shift || $self->get_name().$self->show_shortcut_link_image();
    
    
    my $result;
    
    my $id = $self->get_source_id();
    my $isa = $self->get_source_class();
    my $op = 'show';
    $result  = "<a href='".$self->make_url("index.pl?id=$id&amp;isa=$isa&amp;op=$op")."'>$link_text</a>\n";
    
    return $result;
}


#============ accessor and mutator functions =============================


sub get_gizmo_name {
    my $self = shift;
    my $class = ref($self) || $self;
    
    if ($self eq $class) {
        return "Shortcut";   
    } else {
        return $self->get_source_object()->get_gizmo_name();
    }
}

sub get_is_public {
    return $is_public;   
}

sub get_source_object {
    my $self = shift;
    return $self->{source};    
}

sub get_source_id_field_name {
    return 'i1';   
}

sub get_source_class_field_name {
    return 'c1';   
}

sub get_show_full_text_option_name {
    return 'c2';
}

sub get_source_id {
    my $self = shift;
    return $self->{$self->get_source_id_field_name()};        
}

sub get_full_text_option {
    my $self = shift;
    return $self->{$self->get_show_full_text_option_name()};        
}

sub get_source_class {
    my $self = shift;
    return $self->{$self->get_source_class_field_name()};        
}   

sub set_source_id {
    my ($self, $value) = @_;
    $self->{$self->get_source_id_field_name()} = $value;
}

sub set_source_class {
    my ($self, $value) = @_;
    $self->{$self->get_source_class_field_name()} = $value;
}

sub set_source_object {
    my ($self, $source_obj) = @_;

    my $source_id = $source_obj->get_iid();
    my $source_class = $source_obj->get_is_a();
    
    # dereference GizmoShortcut, in case they are trying to use one as the source
    while ($source_class eq "GizmoShortcut") {
        $source_id = $source_obj->get_source_id();
        $source_class = $source_obj->get_source_class();
        $source_obj = $source_class->new($source_id);
    }
    
    $self->set_source_id($source_id);
    $self->set_source_class($source_class);
    $self->set_show_from($source_obj->get_show_from());
    $self->{source} = $source_obj;    

    # no longer need to determine the name. it is now optional
}


# we override this method to make it not show the edit button, which is not a
# valid button for shortcut gizmos
sub show_panel{
  my $self = shift;
  my $id   = shift || $self->id();
  my $isa  = shift || $self->is_a();
  $self->debug_msg(5, "Gizmo::show_panel(id=$id,isa=$isa)");

  return '' unless($SESSION->admin_mode());

  my $str = "";  

  $str .= $self->show_edit_button();

  $str .= $self->show_ac_button();
  
  $str .= $self->show_cut_button();
  $str .= $self->show_del_button();

  $str .= $self->show_shortcut_button();

  return $str;
}

sub _get_radio_button_html {
  my $self = shift;

  my $radio_name = $self->get_show_full_text_option_name();
  my $display_option = $FORM{$radio_name} || $self->get_full_text_option();
  $self->debug_msg(5, "$self\->_get_radio_button_html has radio name '$radio_name', form value of which is '$FORM{$radio_name}', full_text_option is".$self->get_full_text_option()."; DISPLAY OPTION is $display_option");
  my $link_checked = '';
  my $summ_checked = '';
  if ($display_option eq 'summ') {
      $summ_checked = 'checked';
  } else {
      $link_checked = 'checked';
  }

  my $str = <<EOM
<input type="radio" name="$radio_name" value="link" $link_checked> Show as link&nbsp;&nbsp;&nbsp;&nbsp;
<input type="radio" name="$radio_name" value="summ" $summ_checked> Show content
EOM
;

  return $str;
}

1;
