###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Revision$
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package NewsArticleImage;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);
use Metadot::Controllable;

use vars qw(@ISA);
@ISA=qw(GizmoBuilder Metadot::Controllable);
my $version = "4.5";

sub get_version {
    return $version;
}

####################################
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $args = $class->normalize_constructor_args(shift);
    my $id = $args->{id};
    
    my $self;
    
    if (defined ($id)){
        return $class->restore($id);
    } else {
        $self  = $class->SUPER::new($args);
    }

  $self->{is_a}       = __PACKAGE__;

  bless ($self, $class);
  $self->_initialize();
  return $self;
}

sub _initialize {
  my $self = shift;
  my @args = @_;
  $self->SUPER::_initialize(@args);

  $self->set_field_info (
    "name", "<b>Title</b>", 0,
    "t1", "<b>Sub Title</b>", 0,
    "description", "<b>Description</b>", 0,
    "item_attachment_file",       "<b>Select Image</b>",  0,
#    "long_description", "<b>Long Description</b>", 0,
#    "show_from", "<b>Begin showing on the following date:</b> - [YYYY-MM-DD]", 0,
    "url", "<b>Story location URL</b> <i>e.g. http://www.yourcompany.com</i>", 0,
    "c2", "<b>Choose Layout</b>", 0,
    "keywords", "<b>Keywords and Synonyms</b>", 0,
    );

#  $self->set_field_type( 'show_from', 'show_from' );
  $self->set_field_type( 't1', 'text');
  $self->set_field_type( 'item_attachment_file', 'user_selected_upload' );
  
  $self->set_field_type('c2', 'radio');
  $self->set_field_form_text('c2', $self->_get_radio_button_html());


  $self->{display_wrapped} = { show => 1,
                             };
}

sub _get_radio_button_html {
    my $self = shift;

    my $layout = $FORM{c2} || $self->get_layout() || 'layout1';

    my $layout1_checked = $layout eq 'layout1' ? 'checked' : '';
    my $layout2_checked = $layout eq 'layout2' ? 'checked' : '';

my $html = "
<table border=\"0\" border=\"0\" cellspacing=\"2\" cellpadding=\"2\">
  <tr>
    <td>
      <input type=\"radio\" name=\"c2\" value=\"layout1\" $layout1_checked>
    </td>
    <td>
      <table border=\"1\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">
        <tr>
          <td rowspan=2 align='center' " . Utility->get_spacer(30, 60, 1) . ">IMG</td>
          <td>" . Utility->get_spacer(30, 20) . "</td>
        </tr>
        <tr>
          <td>" . Utility->get_spacer(30, 40) . "</td>
        </tr>
      </table>
    </td>
    <td>
      <input type=\"radio\" name=\"c2\" value=\"layout2\" $layout2_checked>
    </td>
    <td>
      <table border=\"1\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">
        <tr>
          <td align='center' " . Utility->get_spacer(30, 30, 1) . ">IMG</td>
          <td>" . Utility->get_spacer(30, 30) . "</td>
        </tr>
        <tr>
          <td colspan='2'>" . Utility->get_spacer(60, 30) . "</td>
        </tr>
      </table>
    </td>
  </tr>
</table>\n";

    return $html;
}

my %FORM_FIELDS;
sub get_form_fields {
    my $class = shift;
    unless (%FORM_FIELDS) {
        %FORM_FIELDS = $class->SUPER::get_form_fields();
    }
}

sub get_gizmo_name {
   my $self = shift;
   return "News Article Image";
}

sub gizmo_list_decorator {
	return 'HTML::Decorator::NullDecorator';
}

##########################################
### show
### return one newsitem (page format)
##########################################
sub showX {
  my $self = shift;
  $self->debug_msg(4, "NewsArticleImage::show()");
  my $s="";

  my $name            = $self->get_name();
  my $description     = $self->get_description();
  my $longdescription = $self->get_long_description();
  my $url             = $self->get_url();
  my $showfrom        = Utility->format_datetime($self->get_show_from(), 'M d, Y');

  # must only show the date part of the field
  #( $showfrom ) = split( /\s+/, $showfrom );

  $s.= "<b>$name</b><i>\n$showfrom</i>\n<br /><br />$description\n";
  $s.= "<br /><br />$longdescription\n" if ($longdescription);
#    $s.= "<br /><br />See story at <a target=_news href=\"".$self->make_url($url)."\">$url</a>\n" if ($url && ($url ne "http://"));
  $s.= "<br /><br />See story at <a target=_news href=\"". $url ."\">$url</a>\n" if ($url && ($url ne "http://"));
  $s.= "<br>\n";

  return $s;
}

# similar to show_in_column, but shows ordering arrows and does not include a 'dash' in front of
# the name of the news item.
sub show_summaryX {
    my $self = shift;

    my $iid             = $self->get_iid();
    my $name            = $self->get_name();
    my $description     = $self->get_description();
    my $url             = $self->get_url();
    my $longdescription = $self->get_long_description();
    my $buttons           = $self->get_buttons();
    my $s               = "";

    if($longdescription || $url){
	$name = "<a class='newsitemtitle' href='".$self->make_url("index.pl?id=$iid&amp;isa=NewsArticleImage&amp;op=show")."'>$name</a>";
    }

    ($buttons) && ($buttons .= "&nbsp;\n");
    $s.= "<table cellpadding=\"0\" cellspacing=\"0\"><tr><td class='newsitemtitle'>$buttons $name\n</td></tr>\n";
    $s.= "<tr><td class='newsitem'>$description<br>&nbsp;</td></tr></table>\n\n";

    $s = <<EOM
        <table width="160" border="0" cellspacing="1" cellpadding="0" bgcolor="#CCCCCC"><tr><td>
        <table width="100%" border="0" cellspacing="0" cellpadding="5" bgcolor=white><tr><td>
        $s
        </td></tr></table>
        </td></tr></table>
EOM
        ;

    return $s;
}

##########################################
### show_in_list
### return one newsitem (list format)
##########################################
sub show_in_listX {
    my $self = shift;
    my $bgcolor = shift ;
    my $ext_showfrom = shift ;
    # aju 07-24-02: if not time to show item, just return empty string.
    if (!$self->is_show_time()) {
	return '';
    }
    $bgcolor = defined($bgcolor) ? " bgcolor=\"$bgcolor\"" : "" ;
    $self->debug_msg(4, "NewsArticleImage::show_in_list()");
    
    my $iid         = $self->get_iid();
    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $url         = $self->get_url();
    my $showfrom    = $ext_showfrom || $self->get_show_from();
#      print STDERR "in show_in_list, with showfrom $showfrom for $name\n";
    my $panel       = $self->get_panel();

    # must only show the date part of the field
    #( $showfrom ) = split( /\s+/, $showfrom );
    
# aju 04-4-02: don't want the link to lead straight to the msg url -- then
# can't see local content.  Always use the op=show url.
#     if ($url){
#         $url = $self->make_url($url) ;
#     }
#     else {
        $url = $self->make_url("index.pl?iid=$iid&amp;isa=NewsArticleImage&amp;op=show") ;
#     }
    
    my $my_style_tag = "" ;
    my $my_class_tag = "" ;
    my $image_html = "&nbsp;" ;

    my $s = "" ;
    $s .= "<tr $bgcolor>" ;
    $s .= "<td align='center' valign='top' nowrap=\"nowrap\" class='DATE' width=\"20\"><div style=\"white-space: nowrap\">[$showfrom]</div></td>" ;
    $s .= "<td valign='top' $my_style_tag>" ;
    $s .= "<a href=\"$url\" $my_class_tag><b $my_class_tag>$name</b></a>" ;
#    $s .= "<a href=\"$message_url\" $my_class_tag>$name</a>" ;
#    $s .= "<br>from&nbsp;<b $my_class_tag>$authorname</b>" ;
    $s .= "</td>" ;
    $s .= "<td align='center' valign='top'>$image_html</td>" ;
    $s .= "</tr>" ;
    
    if ($description && 0) {
        $s .= "<tr $bgcolor>" ;
        $s .= "<td>&nbsp;</td>" ;
        $s .= "<td colspan='2'  $my_class_tag>" ;
        $s .= "<i>$description</i>" ;
    #    $s .= "<br>" ;
    #    $s .= "($read_more_link $thread_link)" ;
        $s .= "</td>" ;
        $s .= "</tr>" ;
    }
    
    return $s ;
#    return "<b>$name</b> - <i>$showfrom<i> $panel<br>\n$description<br>\n";
}

##########################################
### show_in_column
### return one newsitem (news column format)
##########################################
sub show_in_columnX {
  my $self = shift;
  $self->debug_msg(4, "NewsArticleImage::show_in_column()");

  my $iid             = $self->get_iid();
  my $name            = $self->get_name();
  my $description     = $self->get_description();
  my $url             = $self->get_url();
  my $longdescription = $self->get_long_description();
  my $panel           = $self->get_panel();
  my $s               = "";

  if($longdescription || $url){
    $name = "<a class='newsitemtitle' href='".$self->make_url("index.pl?id=$iid&amp;isa=NewsArticleImage&amp;op=show")."'>$name</a>";
  }

  ($panel) && ($panel .= "&nbsp;\n");
  #$s.= $panel;
  $s.= "<table cellpadding=\"0\" cellspacing=\"0\"><tr><td class='newsitemtitle'>$panel- $name\n</td></tr>\n";
  $s.= "<tr><td class='newsitem'>$description<br>&nbsp;</td></tr></table>\n\n";

  return $s;
}


sub _XXX_process_edit_form {
    my $self  = shift;
    my $error = shift || undef;

    ## check for required fields
    foreach my $field (keys (%{$self->{fields}})) {

	# upload fields do their own "is_required" checking
	next if ( $self->{fields}{$field}{formtype} =~ /_upload$/ );

        if ($self->{fields}->{$field}->{required} &&
            (!$FORM{$field} || !($FORM{$field} =~ /\S/)) # field is not just whitespace
            ) {
            $error .= "<br>".$self->{fields}->{$field}->{label} . 
                " is a required field.\n";
        }
    }

    my $iid = $self->get_iid();

    if (defined($error) && $iid){ ## if error and updating

        $self->www_modify($error);
    }
    elsif (defined($error)) {   ## if error and saving new
        my $parentid = $self->parent_id();
        my $parent = Gizmo->restore($parentid);
#          print STDERR "in gizmo::process_edit_form, calling parent add child on error\n";
        $parent->www_add_child($error, $self->is_a); 
    }
    else {                        ## no errors
        my $checked_reqs = 1;
        $self->www_save($checked_reqs);
    }
}

sub get_title {
    my $self = shift;
    return $self->name();
}

sub get_subtitle {
    my $self = shift;
    return $self->t1();
}

sub get_layout {
    my $self = shift;
    return $self->c2();
}

sub get_image_url {
    my $self = shift;
    my ( $file, $file_icon, $file_url ) = $self->SUPER::get_upload_filename_icon_and_url( {
        id => $self->id(),
        isa => $self->is_a(),
        field_name => 'item_attachment_file',
    });
    return $file_url;
}

1;

__END__

=head1 NAME

Gizmo::NewsArticleImage - supports the creation and display of news items in an 
internal news channel.

=head1 SYNOPSIS

    use Gizmo::NewsArticleImage;
    $newsitem = NewsArticleImage->new ($id);

    $name = $newsitem->get_gizmo_name ();
    $html = $newsitem->show ();
    $html = $newsitem->show_in_list ();
    $html = $newsitem->show_in_column ();
    $version = NewsArticleImage->get_version();


=head1 DESCRIPTION

Supports the creation of a new NewsArticleImage (returning a pointer to the
new object), and controls the way existing NewsArticleImages are displayed
(returning the formatted HTML).

Calling the new function generates a new NewsArticleImage. GizmoBuilder acquires the
next available id and assigns it to the new NewsArticleImage and new returns a pointer
to the new NewsArticleImage.

    get_gizmo_name()

tells anyone who calls it that this gizmo is a "News Item".

Class Method to retrieve the version of the gizmo

    $version = NewsArticleImage->get_version();




The remaining functions control how a NewsArticleImage is displayed. 
  $newsitem->show()

shows one NewsArticleImage. 

  $newsitem->show_in_list() 

shows one NewsArticleImage in a list format. 

  $newsitem->show_in_column() 

shows one NewsArticleImage in a table
    (news column)format.

=cut


