###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package SubmissionSystem;

use strict;

use Metadot qw($PARAMS $SESSION %FORM $USER $DISPLAY);

use Gizmo;
use GizmoBuilder;
use Metadot::GizmoContainer;
use Metadot::Controllable;
use Utility;
use vars qw(@ISA);

use Gizmo::SubmissionSystem::SubmissionItem;
my $CHILD_TYPE = 'SubmissionItem';

@ISA=qw(Metadot::GizmoContainer Metadot::Report Metadot::Controllable);

my $version = "4.5";


sub get_AC {
    my $self = shift;
    return $self->get_default_permissions();
}

sub get_default_permissions {
    my $self = shift;

    my $ac = SubmissionSystem->SUPER::get_default_permissions();
    $ac->add_bundle('APPROVE', 'Can approve', 8, 2);
    $ac->add_op('APPROVE', 'approve');
    $ac->remove_op('MOD', 'add_child');
    $ac->remove_op('MOD', 'create_child');
    $ac->add_op('DISP', 'add_child');
    $ac->add_op('DISP', 'create_child');
    $ac->add_op('DISP', 'show_approvers');
    return $ac;
}

sub get_version {
    return $version;
}

#sub gizmo_list_decorator {
#	return 'HTML::Decorator::NullDecorator';
#}

####################################
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self;

	my $args = $class->normalize_constructor_args(shift);

    if (defined ($args->{id})){
        #$self  = $class->SUPER::new($args->{id});
        # avoid additional instantiation cost
        return $class->restore($args->{id});
    } else {
      $self  = $class->SUPER::new($args);
    }

    $self->{is_a} = __PACKAGE__;

    my $approval_field_name = $self->get_approval_option_field_name();
    my $show_approvers_field_name = $self->get_show_approvers_field_name();
    my $link_field_name = $self->get_submission_link_title_field_name();
    $self->set_field_info (
        "name",         "<b>Name</b>", 1,
        "description",  "<b>Description</b>",   0,
        $approval_field_name, "<b>Choose approval option</b>", 0,
        $show_approvers_field_name, "<b>Show list of approvers</b> <i>(Only applies if approval required.)</i>", 0,
        $link_field_name, "<b>Submission link title</b>", 1,
        "keywords",     "<b>Keywords</b>", 0
	);

    $self->set_field_type($approval_field_name, 'radio');
    $self->set_field_form_text($approval_field_name, $self->_get_radio_button_html());
    $self->set_field_type($show_approvers_field_name, 'radio');

    $self->set_field_type( 'description', 'htmlarea' );
    $self->{display_wrapped} = { show => 1 };

    bless ($self, $class);        # reconsecrate
    return $self;
}

sub show {
    my $self = shift;
    my %params = @_;

    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $show_only_userid = $params{user_id};

    my $html = "<b>$name</b>";
    $html .= "<b>:</b> $description\n" if ($description);

    my $bgcolor = 'bgcolor="#FFFFFF"';
    my $col_count = 0;

    # generate the headers that allow me to sort each
    $self->_process_sort_input();

    $html .= <<EOM
    <p>
    <table cellspacing="0" cellpadding="0" border="0" bgcolor="#777777" width="100%"><tr><td>
    <table cellspacing="1" cellpadding="3" border="0" width="100%">
      <tr valign='bottom'>
EOM
;
    foreach my $column (@{$self->get_columns()}) {
        $html .= '<th bgcolor=white nowrap="nowrap">'.$self->_generate_sort_header($column)."</th>\n";
    }

    $html .= "</tr>\n";
    $col_count += 7;

    my $data = $self->get_data(%params);
    my $rs = $data->{row_set};
    while (my ($child_iid) = $rs->fetchrow()) {
        my $child = $CHILD_TYPE->restore($child_iid);
        next unless $child->is_allowed_to_do('show', $USER);
        next if defined($show_only_userid) && $child->{uid} != $show_only_userid;
        $html .= $self->_generate_table_row(
            object      => $child,
            bgcolor     => $bgcolor,
        )."\n";
    }
    $rs->finish();

    # if no childs, display a message
    unless ($data->{row_count} > 0) {
        $html .= "<td colspan=$col_count align='center' $bgcolor><br><i>- There are no items pending -</i><br /><br /></td>\n";
    }

    $html .= <<EOM
    </table>
    </td></tr></table>
EOM
;

    my $submit_link = $self->build_submit_link();
    my $link_title  = $self->get_submission_link_title();
    $html .= "<div><a href=\"$submit_link\" class='text_button'>$link_title</a></div>\n";

    #return $self->print_portal($html);
    return $html;
}

sub get_columns {
    my $self = shift;
    my @columns = (
        {
            name            => 'Submission Info',
        },
        {
            name            => 'Date Submitted',
            field           => 'creationdate',
            sort_order      => 1,
            sort_priority   => 0,
        },
        {
            name            => 'Status',
            field           => $CHILD_TYPE->get_status_field_name(),
            sort_order      => 1,
            sort_priority   => 0,
        },
        {
            name            => 'Action',
        },
    );
    return \@columns;
}

sub get_data {
    my $self = shift;
    my %args = @_;

    my $id = $self->id();

    my $extra_where = $self->get_non_approved_where(%args);
    my $where_clause = "parentiid=$id and $extra_where";
    if ($args{where_clause}) {
        $where_clause .= " and $args{where_clause}";
    }

    my $order_by = '';
    if ($self->sort_field()) {
       $order_by = $self->sort_field();
       if ($self->sort_order() >= 0) {
           $order_by .= ' asc';
       } else {
           $order_by .= ' desc';
       }
    } else {
        # define the default
        $order_by =  'CreationDate DESC,'
                     . 'name ASC';
    }

    my $rs = DBAccess->sqlSelectMany("iid", "instance", $where_clause, "order by $order_by");
    my ($count) = DBAccess->sqlSelect('count(*)', "instance", $where_clause);

    my $result = {
        row_set     => $rs,
        row_count   => $count,
    };

    #use Data::Dumper;
    #print STDERR "get_data with $where_clause returns\n",Dumper $result;
    return $result;
}

sub _process_sort_input {
    my $self = shift;
    if (defined $FORM{sort_field}) {
        $self->sort_field($FORM{sort_field});
    }
    if (defined $FORM{sort_order}) {
        $self->sort_order($FORM{sort_order});
    }
}

sub _generate_sort_header {
    my $self = shift;
    my $column = shift;
    my $text = $column->{name};
    my $sort_field = $column->{field};
    my $sort_order = $column->{sort_order} || 1;
    my $result;

    unless ($sort_field) {
        return $text;
    }

    my $iid = $FORM{iid} || $self->id();
    my $isa = $FORM{isa} || $self->is_a();
    my $op  = $FORM{op} || 'show';

    # determine what image to use for indicate sort order
    my $sort_image;
    if ($self->sort_field() eq $sort_field) {
        if ($self->sort_order() >= 0) {
            $sort_image = "<img class=\"icon\" src=\"$PARAMS->{imageslocation}/obj/triangle_up.gif\" border=\"0\">";
        } else {
            $sort_image = "<img class=\"icon\" src=\"$PARAMS->{imageslocation}/obj/triangle_down.gif\" border=\"0\">";
        }
        $sort_order = -1 * $self->sort_order(); # toggle the sort
    } else {
        $sort_image = '';
    }

    $result = "<a href=\"index.pl?iid=$iid&amp;isa=$isa&amp;op=$op&amp;sort_order=$sort_order&amp;sort_field=$sort_field\">$text</a> $sort_image";
    return $result;
}

sub sort_field {
    my $self = shift;
    my $input = shift;

    if (defined $input) {
        $SESSION->set_value('submission_item_sort_field', $input);
    }
    return $input || $SESSION->get_value('submission_item_sort_field') || 'creationdate';
}

sub sort_order {
    my $self = shift;
    my $input = shift;

    if (defined $input) {
        $SESSION->set_value('submission_item_sort_order', $input);
    }
    return $input || $SESSION->get_value('submission_item_sort_order') || -1;
}

sub _generate_table_row {
    my $self = shift;
    my %args = @_;
    my $html = '';

    my $object = $args{object};
    my $bgcolor = $args{bgcolor} || 'bgcolor="#FFFFFF"';

    my $fullname_field_name = $CHILD_TYPE->get_fullname_field_name();
    my $email_field_name = $CHILD_TYPE->get_email_field_name();

    my $fullname    = $object->$fullname_field_name();
    my $email       = $object->$email_field_name();
    my $name        = $object->get_name();
    my $description = $object->get_description();
    my $submitdate  = Utility->format_datetime($object->get_creation_date(), 'm/e/y T');
    my $status_label = $object->get_status_field_label();
    ## highlight label if pending
    if ($object->get_status() eq $CHILD_TYPE->get_pending_status_code()) {
        $status_label = "<b>$status_label</b>";
    }
    my $last_comment = $object->get_last_comments();

    my $isa         = $object->get_is_a();
    my $id          = $object->id();
    my $action_links= $object->get_action_links();

    my $not_registered = '';

    unless ($object->is_submission_user_registered()) {
        $not_registered .= "&nbsp;<font color=\"red\">(Not registered)</font>";
    }
    my $submission_info = "<br><b>$name</b>";
    $submission_info .= "<b>:</b> $description\n" if defined($description);
    $submission_info .= "\n<br><b>Last Comment:</b><br>$last_comment\n" if $last_comment;
    $html .= <<EOM
        <tr valign=baseline>
          <td align='left' $bgcolor>
            $fullname - <a href="mailto:$email">$email</a> $not_registered
            $submission_info
          </td>
          <td align='left' $bgcolor>$submitdate</td>
          <td align='left' $bgcolor>$status_label</td>
          <td align='left' $bgcolor nowrap='nowrap'>$action_links</td>
        </tr>
EOM
;
}


sub build_submit_link {
    my $self = shift;
    my $isa         = $self->get_is_a();
    my $iid         = $self->get_iid(); 
    
    my $submit_link = "index.pl?isa=$isa&amp;iid=$iid&amp;op=add_child&amp;child_type=$CHILD_TYPE";
    return $submit_link;
}

###################################
sub show_summary {
    my $self        = shift;

    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $buttons     = $self->get_buttons();
    my $isa         = $self->get_is_a();
    my $iid         = $self->get_iid(); 
    my $link_title  = $self->get_submission_link_title();
    my $show_approvers = $self->show_approvers();

    my $submit_link = $self->build_submit_link();

    my $pending_text = '';

    if ($self->is_allowed_to_do('approve', $USER)) {
        my $extra_pending_where  = $self->get_pending_where();
        my $extra_rejected_where = $self->get_rejected_where();
        my ($count_pending) = DBAccess->sqlSelect("count(*)", "instance", "parentiid = $iid and $extra_pending_where");
        my ($count_rejected) = DBAccess->sqlSelect("count(*)", "instance", "parentiid = $iid and $extra_rejected_where");
        if ($count_pending > 0) {
            $pending_text .= "$count_pending " . $CHILD_TYPE->get_status_label( $CHILD_TYPE->get_pending_status_code() );
        }
        if ($count_rejected > 0) {
            $pending_text .= ", " if $pending_text ne '';
            $pending_text .= "$count_rejected " . $CHILD_TYPE->get_status_label( $CHILD_TYPE->get_rejected_status_code() );
        }
        $pending_text = "[Approver: there are <a href=\"index.pl?$isa=$isa&amp;iid=$iid&amp;op=show\">$pending_text</a> submission(s)]" if $pending_text ne '';
    }

    my $html = "";
    $html .= "\n$buttons " if $buttons;
    $html .= "<b>$name</b> - <a href=\"$submit_link\" class='text_button'>$link_title</a>\n";
    $html .= "<br>$pending_text\n" if $pending_text;
    $html .= "<br>$description\n" if $description;
    $html .= "<br><a href=\"index.pl?$isa=$isa&amp;iid=$iid&amp;op=show_approvers\">View approvers</a>\n" if $show_approvers;

    return $html;
}

sub _show_create_success {
    my $self = shift;

    my $isa = $self->is_a();
    my $parentid = $self->parent_id();
    my $iid         = $self->get_iid(); 

    $self->two_choices("<Big>New " . $self->get_gizmo_name() .
                       " added!</big><br /><br />What do you want to do?",
                       "confirm",
                       "Modify the Approvers",
                       "index.pl?isa=$isa&amp;iid=$iid&amp;op=edit_permissions",
                       "Add another " . $self->get_gizmo_name(),
                       "index.pl?child_type=$isa&amp;isa=".($FORM{isa} || 'Category')."&amp;iid=$parentid&amp;op=add_child&amp;app=",
                       "Enough for now!",
                       "index.pl?id=".$self->id());

}

sub get_name {
    my $self = shift;
    return $self->name();
}

sub get_gizmo_name {
    return 'Submission System';
}

sub get_is_public {
    return 'yes';
}

sub get_approval_option_field_name {
    return 'c1';
}

sub get_submission_link_title_field_name {
    return 'c2';
}

sub get_show_approvers_field_name {
    return 'c3';
}

sub get_approval_option {
    my $self = shift;
    my $field_name = $self->get_approval_option_field_name();
    return $self->$field_name();
}

sub get_submission_link_title {
    my $self = shift;
    my $field_name = $self->get_submission_link_title_field_name();
    return $self->$field_name();
}

sub show_approvers {
    my $self = shift;
    my $field_name = $self->get_show_approvers_field_name();
    return lc($self->$field_name()) eq 'yes' && $self->is_approval_required() ? 1 : 0;
}

sub _get_radio_button_html {
    my $self = shift;

    my $field_name = $self->get_approval_option_field_name();
    my $approval_option = $FORM{$field_name} || $self->get_approval_option();

    my $approval_required_selected = '';
    my $no_approval_required_selected = '';
    if ($approval_option eq 'no_approval_required') {
        $approval_required_selected = '';
        $no_approval_required_selected = 'checked';
    } else {
        $approval_required_selected = 'checked';
        $no_approval_required_selected = '';
    }

    my $html = <<EOM
    <input type="radio" name="$field_name" value="approval_required" $approval_required_selected>Require approval before submissions are published<br>
    <input type="radio" name="$field_name" value="no_approval_required" $no_approval_required_selected>Straight publishing, no approval needed
EOM
;
    return $html;
}

sub is_approval_required {
    my $self = shift;

    return $self->get_approval_option() eq 'no_approval_required' ? 0 : 1;
}

sub show_edit_form {
    my $self = shift;
    my $link_field_name = $self->get_submission_link_title_field_name();

    if ($FORM{$link_field_name} eq '') {
        $FORM{$link_field_name} = 'submit your file now';
    }

    return $self->SUPER::show_edit_form(@_); 
}

sub www_show_approvers {
  my $self = shift;
  $self->debug_msg(3, "Gizmo::SubmissionSystem::www_show_approvers()");

  my $iid = $self->defined_or_exit($self->id,"id is missing.");

  unless ($self->show_approvers()) {
      $DISPLAY->print_click_back('Operation not allowed');
      exit;
  }

  $self->print_portal($self->get_approver_display());
}

sub get_approver_display {
  my $self = shift;
  $self->debug_msg(3, "Gizmo::SubmissionSystem::get_approver_display()");

  ## get all users who can approve
  my $uids = Utility->uniquify_array($self->get_all_who_can('APPROVE'));
  my @names = ();

  my $html = "<b>List of Approvers</b><p>\n";
  foreach my $uid (@$uids) {
      next if $uid == MetadotConfig->get_reserved_uid('ADMIN');
      push @names, $USER->get_fullname_from_uid($uid);
  }

  foreach my $name (sort @names) {
      $html .= "<li>$name<br>\n";
  }

  return $html;
}

sub www_delete {
  my $self = shift;
  my $name = $self->get_gizmo_name();
  my $isa = $self->get_is_a();
  $self->debug_msg(3, "Gizmo::SubmissionSystem::www_delete()");

  my $iid = $self->defined_or_exit($self->id,"id is missing.");

  $self->three_choices("Delete This $name:".$self->name()."?",
                       "",
                       "Yes",
                       "index.pl?iid=$iid&amp;isa=$isa&amp;op=delete_ok",
                       "No!", $SESSION->{referer});
}

sub delete {
    my $self = shift;

    my $id = $self->id();

    my $rs = DBAccess->sqlSelectMany("iid", "instance", "parentiid=$id");
    while (my ($child_iid) = $rs->fetchrow()) {
        $self->get_object_accessor()->delete( key=>$child_iid );
    }

    $self->SUPER::delete();
}

## Class method that will return a list of all submission systems objects
##  in which the given user has a submission pending
sub get_submission_systems_with_submissions {
    my $class = shift;
    my %args  = @_;

    my $user_id = $args{user_id} || $USER->uid();
    $user_id = undef unless Utility->is_valid_integer($user_id); # secu_200401
    
    #my $status_field_name = $CHILD_TYPE->get_status_field_name();
    #my $pending_status_code = $CHILD_TYPE->get_pending_status_code();
    my $extra_where = $class->get_non_approved_where();
  
    #my $rs = DBAccess->sqlSelectMany('distinct(parentiid)', 'instance', "isa='$CHILD_TYPE' and userid = $user_id and $status_field_name = $pending_status_code");
    my $rs = DBAccess->sqlSelectMany('distinct(parentiid)', 'instance', "isa='$CHILD_TYPE' and userid = $user_id and $extra_where");

    my @ss_list = ();
    while (my($parent_iid) = $rs->fetchrow) {
        my $ss = SubmissionSystem->restore($parent_iid);
        next unless $ss;
        push (@ss_list, $ss);
    }
    return \@ss_list;
}

## Class method that will return a list of all submission systems objects
##  in which the given user has approve privilege
sub get_submission_systems_for_review {
    my $class = shift;
    my %args  = @_;

    my $user_id = $args{user_id} || $USER->uid();
    $user_id = undef unless Utility->is_valid_integer($user_id); # secu_200401
    
    my $extra_where = $class->get_non_approved_where();
  
    my $rs = DBAccess->sqlSelectMany('distinct(parentiid)', 'instance', "isa='$CHILD_TYPE' and $extra_where");

    my @ss_list = ();
    while (my($parent_iid) = $rs->fetchrow) {
        my $ss = SubmissionSystem->restore($parent_iid);
        next unless $ss && $ss->is_allowed_to_do('approve', $USER);
        push (@ss_list, $ss);
    }
    return \@ss_list;
}

sub get_non_approved_where {
    my $self = shift;
    my %args = @_;

    my $show_pending_only = $args{show_pending_only} || 0;
    my $status_field_name = $CHILD_TYPE->get_status_field_name();

    my $where = "(" . $self->get_pending_where(%args);

    unless ($show_pending_only) {
        $where .= " OR " . $self->get_rejected_where(%args);
    }

    $where .= ")";

    return $where;
}

sub get_pending_where {
    my $self = shift;
    my %args = @_;

    my $status_field_name = $CHILD_TYPE->get_status_field_name();
    my $pending_status_code = $CHILD_TYPE->get_pending_status_code();

    return "$status_field_name = $pending_status_code";
}

sub get_rejected_where {
    my $self = shift;
    my %args = @_;

    my $status_field_name = $CHILD_TYPE->get_status_field_name();
    my $rejected_status_code = $CHILD_TYPE->get_rejected_status_code();

    return "$status_field_name = $rejected_status_code";
}

1;

__END__
