###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package GizmoBuilder;
use strict;
use Data::Dumper;
use vars '$VERSION';

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CACHE $ACCESS_BROKER $DISPLAY );

use Metadot::JSFormFactory ;

use Metadot::UploadsManager;

$VERSION=1.0;

use CGI qw(:all);
use Gizmo;
use Utility;
use Operations;
use vars qw(@ISA);
@ISA=qw( Gizmo Metadot::UploadsManager );

###
### ACCESS CONTROL METHODS
###


sub get_AC {
    my $self = shift;
    return $self->get_default_permissions();
}


# storing default permissions in $AC to prevent thousands of calls to add_op
sub get_default_permissions {
    my $self = shift;
    my $AC;
    if (ref($self) && $self->{_default_permissions}) {
        $AC = $self->{_default_permissions};
    } else {
        #print STDERR "GizmoBuilder->get_default_permissions with self=$self\n";    
        $AC = Operations->new;
    
        $AC->add_bundle('DISP', 'View', '0', '0');
        $AC->modify_bundle('DISP', 'DEFAULTS_TO', 'on' );
        $AC->add_op('DISP','show');
        $AC->add_op('DISP','show_summary');
        $AC->add_op('DISP','show_in_column');
        $AC->add_op('DISP','download');
    
        $AC->add_bundle('MOD', 'Edit', '8', '2');
        $AC->add_op('MOD', 'create');
        $AC->add_op('MOD', 'modify');
        $AC->add_op('MOD', 'save');
        $AC->add_op('MOD', 'up');
        $AC->add_op('MOD', 'down');
        $AC->add_op('MOD', 'paste');
        $AC->add_op('MOD', 'delfile');
        $AC->add_op('MOD', 'delfileok');
        $AC->add_op('MOD', 'modify_properties');
        $AC->add_op('MOD', 'save_properties');
    
        # the following three are UploadsManager requirements
        $AC->add_op('MOD', 'confirm_upload_deletion');
        $AC->add_op('MOD', 'delete_upload');
        $AC->add_op('DISP', 'download_file');
    
        #CKIM-SHORTCUTS: add this op to allow creation of shortcuts
        $AC->add_op('DISP', 'create_shortcut');
    
        $AC->add_bundle('DEL', 'Delete and Cut', '8', '8');
        $AC->add_op('DEL', 'delete');
        $AC->add_op('DEL', 'delete_ok');
        $AC->add_op('DEL', 'cut');
        $AC->add_op_parent_dependency('DEL', 'cut', 'MOD');
    
        $AC->add_bundle('EDITP', 'Change<br>Permissions', '8', '8');
        $AC->add_op('EDITP', 'edit_permissions');
        $AC->add_op('EDITP', 'set_permissions');
        $AC->add_op('EDITP', 'change_owner');
        
        # save these permissions to self if possible
        if (ref($self)) {
            $self->{_default_permissions} = $AC;
        }

    }
    #if set, this text will be displayed at the top of the gizmo edit form.
    
    return $AC;    
}


sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    
    # Some gizmos pass only an $id to the constructor, while
    # others send a hashref of arguments. This causes the
    # constructor to always see a hashref of arguments, no
    # matter what the actual parameters were.
    my $args = $class->normalize_constructor_args(shift);
    my $id = $args->{id};
    
    my $self;
    
    if (defined ($id)){
        # I'm retrieving an instance from the db. Gizmo->restore() takes
        # care of all instantiation duties by calling new() if needed.
        # Therefore, we can just return the result.
        return $class->restore($id);
    } else {
        $self  = $class->SUPER::new($args);
    }
    
    $self->debug_msg(4, "Instantiating new $self");
    $self->_initialize();

    # ckim: bless the class with the lower-level if possible
    if (defined($self->{isa}) && ($self->{isa} ne '')) {
        $class = $self->{isa};
    }
    bless ($self, $class);  
    
    # reconsecrate
    return $self;
}

sub _initialize {
    my $self = shift;

    $self->{_intro_text_string} = "";

    ## set the form type for each field
    $self->{fields}->{name}->{formtype}             = "text";
    $self->{fields}->{description}->{formtype}      = "mediumtext";
    $self->{fields}->{long_description}->{formtype} = "longtext";
    $self->{fields}->{cool}->{formtype}             = "radio";
    #$self->{fields}->{show_childs}->{formtype}     = "";
    #$self->{fields}->{display_look}->{formtype}    = "";
    $self->{fields}->{url}->{formtype}              = "text";
    $self->{fields}->{keywords}->{formtype}         = "mediumtext";
    #$self->{fields}->{instance_order}->{formtype}  = "";
    $self->{fields}->{show_from}->{formtype}        = "smalltext";
    #$self->{fields}->{creation_date}->{formtype}   = "";
    #$self->{fields}->{update_date}->{formtype}     = "";
    #$self->{fields}->{nb_items}->{formtype}        = "";
    #$self->{fields}->{nb_reads}->{formtype}        = "";
    #$self->{fields}->{updated_by}->{formtype}      = "";
    #$self->{fields}->{workflow_id}->{formtype}     = "";
    #$self->{fields}->{hidden_comment}->{formtype}  = "";
    #$self->{fields}->{last_post}->{formtype}       = "";
    $self->{fields}->{file1}->{formtype}            = "file";
    #$self->{fields}->{created_by}->{formtype}      = "";
    $self->{fields}->{t1}->{formtype}               = "longtext";
    $self->{fields}->{t2}->{formtype}               = "longtext";
    $self->{fields}->{t3}->{formtype}               = "longtext";
    $self->{fields}->{t4}->{formtype}               = "longtext";
    $self->{fields}->{t5}->{formtype}               = "longtext";
    $self->{fields}->{t6}->{formtype}               = "longtext";
    $self->{fields}->{t7}->{formtype}               = "longtext";
    $self->{fields}->{t8}->{formtype}               = "longtext";
    $self->{fields}->{t9}->{formtype}               = "longtext";
    $self->{fields}->{t10}->{formtype}              = "longtext";
    $self->{fields}->{c1}->{formtype}               = "smalltext";
    $self->{fields}->{c2}->{formtype}               = "smalltext";
    $self->{fields}->{c3}->{formtype}               = "smalltext";
    $self->{fields}->{c4}->{formtype}               = "smalltext";
    $self->{fields}->{c5}->{formtype}               = "smalltext";
    $self->{fields}->{d1}->{formtype}               = "smalltext";
    $self->{fields}->{d2}->{formtype}               = "smalltext";
    $self->{fields}->{d3}->{formtype}               = "smalltext";
    $self->{fields}->{d4}->{formtype}               = "smalltext";
    $self->{fields}->{d5}->{formtype}               = "smalltext";
    $self->{fields}->{i1}->{formtype}               = "smalltext";
    $self->{fields}->{i2}->{formtype}               = "smalltext";
    $self->{fields}->{i3}->{formtype}               = "smalltext";
    $self->{fields}->{i4}->{formtype}               = "smalltext";
    $self->{fields}->{i5}->{formtype}               = "smalltext";
    $self->{jsform} = Metadot::JSFormFactory->get_jsform() ;

    ## initialize other fields
    foreach my $field (keys (%{$self->{fields}})) {
        $self->{fields}->{$field}->{label}    = '';
        $self->{fields}->{$field}->{required} = 0;
        $self->{fields}->{$field}->{order}    = 0;
        $self->{fields}->{$field}->{form_text}     = '';
    }

}


#sub get_gizmo_name {
#   my $self = shift;
#   return $self->{gizmo_name} || $self->{is_a};
#}

sub display_in_category {
    my $self = shift;
    my $op = shift;

    if ($self->{display_wrapped}->{$op}) {
        return 1;
    } else {
        return 0;
    }
}

# This class method builds the header category uses for listing children of a gizmo
# type. If you want something other than the default header, overload this function
# and handle the header in your gizmo.
sub gizmo_list_header {
	return '';
}

sub legacy_list_header {
	my $class = shift;
	my $cat_name = shift;
	my $output;
	
	my $gizmo_name = $class->get_gizmo_name();
 
	my @namewords = split(/\ /, $cat_name);
	my $lastword = $namewords[$#namewords];
	if (lc $lastword eq lc($gizmo_name . "s")) {
		$output = "<h3>$cat_name</h3>";
	} elsif (lc $lastword eq lc $gizmo_name) {
		$output = "<h3>$cat_name"."s</h3>";
	} else {
		$output = "<h3>$cat_name ".$gizmo_name."s</h3>";
	}

	return $output;
}

# This class method builds the layout category uses for listing children of a gizmo
# type. If you want something other than the default header, overload this function
# and handle the header in your gizmo.
sub gizmo_list_layout {
	my $class = shift;
	my $head = shift;
	my $childs_list = shift;
	my $output;

	my $gizmo_name = $class->get_gizmo_name();

	$output .= "<!-- $gizmo_name List -->\n<br /><br /><br />\n<div class='gizmolist'> $head</div>"
			.  "\n$childs_list\n<!-- End of $gizmo_name List -->\n\n";

	return $output;
}

# This class method sets the decorator that the category uses when listing children
# of this gizmo type. The default is BulletDecorator. This can be overridden. A
# popular choice is NullDecorator
sub gizmo_list_decorator {
	return 'HTML::Decorator::BulletDecorator';
	#return 'HTML::Decorator::NullDecorator';
}

## called before saving or updating
sub process_edit_form {
    my $self  = shift;
    my $error = shift || undef;

    ## check for required fields
    foreach my $field (keys (%{$self->{fields}})) {

	# upload fields do their own "is_required" checking
	next if ( $self->{fields}{$field}{formtype} =~ /_upload$/ );

        if ($self->{fields}->{$field}->{required} &&
            (!$FORM{$field} || !($FORM{$field} =~ /\S/)) # field is not just whitespace
            ) {
            $error .= "<br>".$self->{fields}->{$field}->{label} . 
                " is a required field.\n";
        }
    }

    my $iid = $self->get_iid();

    if (defined($error) && $iid){ ## if error and updating

        $self->www_modify($error);
    }
    elsif (defined($error)) {   ## if error and saving new
        my $parentid = $self->parent_id();
        my $parent = Gizmo->restore($parentid);
#          print STDERR "in gizmo::process_edit_form, calling parent add child on error\n";
        $parent->www_add_child($error, $self->is_a); 
    }
    else {                        ## no errors
        my $checked_reqs = 1;
        $self->www_save($checked_reqs);
    }
}


## called for saving and updating
sub www_save {
    my $self  = shift;
    my $checked_reqs = shift; #optional
    my $order = 0;
    my $uid = $USER->uid();
    my $error = undef;

#    print STDERR "AT WWW_SAVE $FORM{show_from}\n";

    foreach my $field (keys (%{$self->{fields}})) {

        if ( $self->{fields}{$field}{formtype} =~ /_upload$/ ||
	     $self->{fields}{$field}{formtype} =~ /text$/ ) {

#	    print STDERR "APPLYING ARRAY FIX TO FIELD $field\n";

	    # added here for the mess IE 5.0 does when submitted form
	    # contains special characters
	    if( ref( $FORM{$field} ) eq 'ARRAY' ) {
		$FORM{$field} = $FORM{$field}->[0];
	    }	    
	}
    }

    # XXX remove?
    if(ord substr($FORM{file1},0,1) < 32) {
        $FORM{file1} = "" ;
    }

    if (!$checked_reqs) {
        ## check for required fields
        foreach my $field (keys (%{$self->{fields}})) {

	    # upload fields do their own "is_required" checking
	    next if ($self->{fields}{$field}{formtype} =~ /_upload$/ );

            if ($self->{fields}->{$field}->{required} &&
                (!$FORM{$field} || !($FORM{$field} =~ /\S/)) # field is not just whitespace
	       ) {
                $error .= "<br>".$self->{fields}->{$field}->{label} . 
		  " is a required field.\n";
            }
        }
    }

    my $iid = $self->get_iid();
    
    if (defined($error) && $iid){ ## if error and updating
	
        $self->www_modify($error);
	
        exit;
	
    } elsif (defined($error)) {   ## if error and saving new
	
        my $parentid = $self->parent_id();
        my $parent = Gizmo->restore($parentid);
        $parent->www_add_child($error, $self->is_a);
        exit;
    }
    
    foreach my $field (keys (%{$self->{fields}})) {
        $order = $self->{fields}->{$field}->{order};
        next if !defined($order) || $order <= 0;
        if ('mediumtext' eq $self->{fields}->{$field}->{formtype} || 'longtext' eq $self->{fields}->{$field}->{formtype}) {

            my $text = $FORM{$field};
            if ($text && ('text' eq $FORM{"type$field"})) {
                $text = escapeHTML($text);
                $text = $self->translate_white_space($text);
                $text = "<!-- form type is text -->" . $text;
                $FORM{$field} = $text; 
            }
        }
	
	if ( $self->{fields}{$field}{formtype} eq 'show_from' ) {
	    if( $FORM{$field} ) { 
            $FORM{$field} = $FORM{$field} . ' 00:00:01';
        } else {
            # MySQL saves blank string to a '0000-00-00', which is no good
            $FORM{$field} = undef;
        }
	}

	# we make an exception with file1 because we won't save it unless the
	# upload is successful
	unless($field eq 'file1') {
	    $self->{$field} = $FORM{$field};
	}
    }

    if (!$iid){ ## we are creating new
	
        $iid = $self->add();
	
        if ($iid) {
            my $isa = $self->is_a();
            my $parentid = $self->parent_id();
	    
	    ## handle situation when file upload is required
	    if($FORM{file1}){
		
		my $filename;

		$filename = $self->upload_file($FORM{file1}, $iid, $uid );

		if($filename eq 'USER_QUOTA_EXCEEDED' ||
		   $filename eq 'GLOBAL_QUOTA_EXCEEDED' ) {
		    
		    my $parentid = $self->parent_id();
		    my $parent = Gizmo->restore($parentid);
		    
		    # backtrack object creation
		    $self->delete();

		    if( $filename eq 'USER_QUOTA_EXCEEDED' ) {
		    
			$parent->www_add_child('File upload has exceeded allowed user quota, please request a larger quota to the site administrator or delete some of your files to free up space', $self->is_a);
		    } else {

			$parent->www_add_child('File upload has exceeded allowed site-wide quota, please request a larger quota to the site administrator or delete some of your files to free up space', $self->is_a);
			
		    }
		    exit;
		    
		} elsif($filename){
		    
		    $self->file1($filename);
		    $self->SUPER::save();
		}
	    }
	    
	    my $err = $self->process_uploads( $iid );

	    if( $err ) {
		my $parentid = $self->parent_id();
		my $parent = Gizmo->restore($parentid);
		
		# backtrack object creation
		$self->delete();
		
		$parent->www_add_child( $err, $self->is_a );
		exit;	
	    }

	    if(uc($FORM{createchannel}) eq "YES") {
		
		    my $isa = lc($self->is_a);
		    my $type;
		    if ($isa eq "discussion") {
		        $type = "Discussion";
		    } else {
		        $type = "Internal";
		    }
		    
		    my $newchan = Channel->new();
		    my $cid = $newchan->create($iid, $type, $FORM{name}, "$FORM{description}");
		    
		    if(!$cid){
		        $self->two_choices("Could not create the channel for this $isa",
					   "error",
					   "Enough for now!", "index.pl?id=$iid");
		    }
	    }
	    
	    $self->_show_create_success();    
        }
    } else {			## we are updating
	
	if(uc($FORM{createchannel}) eq "YES") {
	    
	    my $isa = lc($self->is_a);
	    my $type;
	    if ($isa eq "discussion") {
		$type = "Discussion";
	    } else {
		$type = "Internal";
	    }
	    
	    my $newchan = Channel->new();
	    my $cid = $newchan->create($iid, $type, $FORM{name}, "$FORM{description}");
	    
	    if(!$cid){
		$self->two_choices("Could not create the channel for this $isa",
				   "error",
				   "Enough for now!", "index.pl?id=$iid");
	    }
	}
	

	my $err = $self->process_uploads( $iid );

	if( $err ) {
	    
	    $self->www_modify( $err );
	    exit;
	}

	if($FORM{file1}){	    
	    
	    my $filename;

	    $filename = $self->upload_file($FORM{file1}, $iid, $uid );
		
	    if($filename eq 'USER_QUOTA_EXCEEDED' ||
	       $filename eq 'GLOBAL_QUOTA_EXCEEDED' ) {

		$self->file1('');
		$self->save();		
		$FORM{file1} = '';

		if( $filename eq 'USER_QUOTA_EXCEEDED' ) {

		    $self->www_modify('File upload has exceeded allowed user quota, please request a larger quota to the site administrator or delete some of your files to free up space');
		} else {
		    
		    $self->www_modify('File upload has exceeded allowed site-wide quota, please request a larger quota to the site administrator or delete some of your files to free up space');
		}

	    } elsif ($filename){

		$self->file1($filename);
		$self->save();
	    }
	} else {

	    $self->save();	    
	}
	
	$self->_show_edit_success();
        #$self->md_redirect("index.pl?iid=".$self->parent_id);
    }
    
}

# this method handles what to do when you are finished successfully creating a new gizmo
sub _show_create_success {
	my $self = shift;

#  	my $isa = lc($self->is_a());
	my $isa = $self->is_a();
	my $parentid = $self->parent_id();

    $self->two_choices("<Big>New " . $self->get_gizmo_name() .
                       " added!</big><br /><br />What do you want to do?",
                       "confirm",
                       "Add another " . $self->get_gizmo_name(),
                       "index.pl?child_type=$isa&amp;isa=".($FORM{isa} || 'Category')."&amp;iid=$parentid&amp;op=add_child&amp;app=",
                       "Enough for now!",
                       "index.pl?id=$parentid");

}

# this method handles what to do when you are finished successfully editing a gizmo
sub _show_edit_success {
	my $self = shift;

	$self->md_redirect("index.pl?iid=".$self->parent_id);

}


sub translate_white_space {
     my ($self, $text) = @_;
  
     $text =~ s/\r//g;
     $text =~ s/\n/<br>/g;
# aju: 04-22-02 removed 'enforced' spacing, upon request.
#      $text =~ s/ /&nbsp;/g;
  
     return $text;
}

sub untranslate_white_space {
    my ($self, $text) = @_;
    $text =~ s/&nbsp;/ /g;
    $text =~ s/<br>/\n/g;

    return $text;
}

sub www_delfile {
  my $self = shift;
  $self->debug_msg(3, "Gizmo::www_delfile()");
  my $iid   = $self->id();
  my $isa   = $self->is_a();
  my $db_rec = $FORM{name};
  my $file = $self->$db_rec() || $self->file1();
#  my $file1 = $self->file1();
  my $name  = $self->name();


  $self->two_choices("Do you want to delete the file $file attached to the $isa: <br>$name?",
                     "caution",
                     "Yes please", $self->make_url("index.pl?id=$iid&amp;isa=$isa&amp;op=delfileok&amp;name=$db_rec"),
                     "No!", $self->make_url("index.pl?id=$iid&amp;isa=$isa&amp;op=modify"));
}

sub www_delfileok {
  my $self = shift;
  my $iid   = $self->id();
  my $isa = $self->is_a();
  $self->debug_msg(3, "Gizmo::www_delfileok()");
  $self->delete_attachment;
  $self->save();

  $self->md_redirect($self->make_url("index.pl?id=$iid&amp;isa=$isa&amp;op=modify"));
}


sub www_create_shortcut {
    #CKIM-SHORTCUTS: this op creates a shortcut of the $self
    my $self = shift;
    my $iid   = $self->id();
    my $isa = $self->is_a();
    my $parentid = $self->parent_id();
    
    # check for any possible error conditions
    my $error = "";
    
    require "Gizmo.pm"; 
    require "GizmoTool.pm"; 
    my $shortcut = GizmoShortcut->new({ parent_id => $parentid });
    $shortcut->set_source_object($self);
    $shortcut->set_uid($USER->uid());
    my $newid = $shortcut->add();
    if ($newid != 0) {        
        $shortcut->_cut();            
    } else {
        $error .= "There was a problem creating the shortcut. Please check the log for errors.\n";
    }

    if (!$error) {
        # still no errors? redirect the user
        # $self->md_redirect("index.pl?iid=$parentid");
        
        # instead of redirecting, send a message
        my $return_id;
        if ($isa eq 'Category') {
            $return_id = $iid;   
        } else {
            $return_id = $parentid;
        }
        $self->print_choices("<big>Shortcut named \"" . $shortcut->get_name(). "\" created!</big><br /><br />The shortcut will appear in your clipboard, which you can paste into another category.",
                       "confirm",
                       "Click here to continue",
                       "index.pl?id=$return_id");
        exit;
    } else {
        # error handling 
        die ("Could not create a GizmoShortcut due to the following errors:\n$error");
    }
}


## build form for adding and updating
sub show_edit_form {
    my $self  = shift;
    my $title = shift;

    $self->debug_msg(3, "Gizmo::show_edit_form for $self");
    $self->debug_msg(5, "Gizmo::show_edit_form has fields:<br>".Dumper($self->{fields}));
    my $s = "";
    my $order = 0;

    $s .= "<br /><br /><h1>" . $title . "</h1>\n";

    # if there is any intro text to be displayed at the top, add it.
    $s .= "<p>" . $self->get_intro_text() . "</p>";

    foreach my $field (sort {$self->{fields}->{$a}->{order} <=> $self->{fields}->{$b}->{order}} keys (%{$self->{fields}})) {
        $self->debug_msg(5, "Gizmo::show_edit_form checking $field");
    
        $order = $self->{fields}->{$field}->{order};
        next if !defined($order) || $order <= 0;
        
        $self->debug_msg(5, "Gizmo::show_edit_form getting form box for $field");

        $s .= $self->get_form_box($field);
     
    }
   
    $s .= "<br /><br /><br /><i>(*) Mandatory field(s)</i><br>";
    # add form elements to add the gizmo as a channel if appropriate
    my $channel = "";
    my $isa = lc($self->{is_a});
#      print STDERR "in show_edit_form with form op: " . $FORM{op} . "\n";

    # add the hooks to create a channel where appropriate.  Actually, I believe that since Category
    # overrides this method, then 'discussion' objects are the only relevant type here.
    # uncomment this code to add back in the capability to give the user a choice as to
    # whether or not the discussion is exported to be a channel.
    if (($isa eq "discussion") || ($isa eq "category") ) {
        if ($FORM{op} =~ m/(add|create)/)  { #if creating object
	    if ($isa eq "discussion") {        # if a discussion, automatically create channel.  
		$s .= "\n<input type=hidden name='createchannel' value='YES'>\n";
	    }
#              $channel .= "<br /><br /><b>Create a news channel for this $isa?</b><br>\n";
#              $channel .= "Yes&nbsp;<input type='radio' name='createchannel' value='YES'>&nbsp;&nbsp;\n";
#              $channel .= "No&nbsp;<input type='radio' name='createchannel' value='NO' checked><br>\n";
        }
        else {			#modifying existing obj
	    if ( $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) {
		# if user is admin, show 'create channel' form elements if channel does not exist
		my $iid = $self->{id};
		# does channel for this item already exist?
		my ($cid) = DBAccess->sqlSelect("cid", "channel", "iid=$iid");
		if (!$cid) {	# if not...
		    $channel .= "<br /><br /><b>Create a channel for this $isa?</b><br>\n";
		    $channel .= "Yes&nbsp;<input type='radio' name='createchannel' value='YES'>&nbsp;&nbsp;\n";
		    $channel .= "No&nbsp;<input type='radio' name='createchannel' value='NO' checked><br>\n";
		}
	    }
	}
    }
#      else {
#          $channel .= "<br /><br /><b>Category Channel</b><br>\n";
#          $channel .= "<a href='".$self->make_url("index.pl?cid=".$self->cid."&amp;isa=Channel&amp;op=modify")."'>Edit news channel information</a><br>\n";
#          $channel .= "<a href='".$self->make_url("index.pl?cid=".$self->cid."&amp;isa=Channel&amp;op=delete")."'>Delete news channel</a><br>\n";
#      }

    $s .= $channel;
    return $s;

}

## build individual form box for field
sub get_form_box {
    my $self        = shift;
    my $field_name  = shift;

    my $field_value = $FORM{$field_name} || $self->{$field_name};
    my $label       = $self->{fields}->{$field_name}->{label};
    my $required    = $self->{fields}->{$field_name}->{required};
    my $formtype    = $self->{fields}->{$field_name}->{formtype};
    my $options     = $self->{fields}->{$field_name}->{options} || {};
    my $isa         = $self->{is_a};
    my $iid         = $self->{id};
    
    # New, special handling for form_text. You can now pass a reference
    # to a string, and it will call the method by that name to generate the form text.
    my $form_text   = $self->{fields}->{$field_name}->{form_text};
    if (ref($form_text)) {
        $form_text = $self->$$form_text();
    }
    
#    $field_value    = escapeHTML($field_value);
    
    # Substitute <<VALUE>> keyword in label and form_text
    $self->debug_msg(5, "get_form_box($self,$field_name) has type='$formtype'; value='$field_value'; text='$form_text'");
    $label     =~ s/\<\<VALUE\>\>/$field_value/s;
    $form_text =~ s/\<\<VALUE\>\>/$field_value/s;

    my $form_box = '';

    $form_box .= "<br /><br />$label" if $label;
    if ($required) {        

        $form_box .= " (*)" ;
        # JSForm
        $self->{jsform}->add_check({
                name => $field_name,
                label_text => $label,
                check_type => "notempty",
            }) 
        if (defined($self->{jsform}));
        # JSForm
    }
    $form_box .= "<br>\n" if $label;
    
    if ($formtype eq 'smalltext') {
    	
        # must convert double quotes to html entities to preserve syntax
    	$field_value = escapeHTML( $field_value );           
        $form_box .= "<input type=\"text\" name=\"$field_name\" size=\"25\" maxlength=\"100\" value=\"$field_value\">\n";
    	if ($field_name =~ m/(^d[0-9]+)/) {
                $form_box .= "<br>(format: YYYY-MM-DD)";
    	}
    } 
    elsif ($formtype eq 'hidden') {

    	# must convert double quotes to html entities to preserve syntax
    	$field_value =~ s/\"/\&\#34\;/g;    
        $form_box .= "<input type=\"hidden\" name=\"$field_name\" value=\"$field_value\">\n";
    }
    # add hook for Check Boxes
    elsif ($formtype eq 'checkbox'){    

        $form_box .= "<input type = 'checkbox' name='$field_name' value='on'";
    	if ($field_value){
    	    $form_box .= " checked ";
    	}
        $form_box .= ">";                  

    } elsif ($formtype eq 'text') {

    	# must convert double quotes to html entities to preserve syntax
    	$field_value = escapeHTML( $field_value );    
        $form_box .= "<input type=\"text\" name=\"$field_name\" size=\"55\" maxlength=\"255\" value=\"$field_value\">\n";

    } elsif ($formtype eq 'mediumtext' && $field_name eq 'keywords'){

        $form_box .= "<textarea name=\"$field_name\" rows=1 cols=80 WRAP=\"VIRTUAL\">$field_value</textarea>\n";        

    } elsif ($formtype eq 'mediumhtml' || $formtype eq 'mediumtext') {

        my ($html_type, $text_type) = ("","");
        if ($field_value eq '' || $field_value =~ s/\A<!-- form type is text -->//) {
            $field_value = $self->untranslate_white_space($field_value);
            $text_type = 'checked';
        } else {
            $field_value = escapeHTML($field_value);
            $html_type = 'checked';
        }

        $form_box .= "<textarea name=\"$field_name\" rows=4 cols=80 WRAP=\"VIRTUAL\">$field_value</textarea>\n";
        unless ($formtype eq 'mediumhtml') {
            $form_box .= "<br><input type = radio name = type$field_name value = html $html_type>html&nbsp;<input type = radio name = type$field_name value = text $text_type>text";
        }

    } elsif ($formtype eq 'longtext') {

        my ($html_type, $text_type) = ("","");
        if ($field_value eq '' || $field_value =~ s/\A<!-- form type is text -->//) {
            $field_value = $self->untranslate_white_space($field_value);
            $text_type = 'checked';
        } else {
            $field_value = escapeHTML($field_value);
            $html_type = 'checked';
        }
        $form_box .= "<textarea name=\"$field_name\" rows=10 cols=80 WRAP=\"VIRTUAL\">$field_value</textarea>\n";
        $form_box .= "<br><input type = radio name = type$field_name value = html $html_type>html&nbsp;<input type = radio name = type$field_name value = text $text_type>text";

    } elsif ($formtype eq 'variabletext') {
        my $cols = $options->{cols} || 80;
        my $rows = $options->{rows} || 5;

        my ($html_type, $text_type) = ("","");
        if ($field_value eq '' || $field_value =~ s/\A<!-- form type is text -->//) {
            $field_value = $self->untranslate_white_space($field_value);
            $text_type = 'checked';
        } else {
            $field_value = escapeHTML($field_value);
            $html_type = 'checked';
        }
        
        $form_box .= "<textarea name=\"$field_name\" rows='$rows' cols='$cols' WRAP=\"VIRTUAL\">$field_value</textarea>\n";
        $form_box .= "<br><input type = radio name = type$field_name value = html $html_type>html&nbsp;<input type = radio name = type$field_name value = text $text_type>text";

    } elsif ($formtype eq 'htmlarea') {

        $form_box .= $DISPLAY->display_htmlarea_field({
                             field_name  => $field_name,
                             field_value => $field_value });
        #my $field_id = "htmlarea_$field_name";
        #$field_value = escapeHTML($field_value);
        #
        #$form_box .= $DISPLAY->show_site_library_link()."<textarea id=\"$field_id\" class=\"htmlarea\" name=\"$field_name\" style=\"width:100%\" rows='20' cols='80' WRAP=\"VIRTUAL\">$field_value</textarea>\n";
        #$form_box .= $DISPLAY->convert_field_to_htmlarea($field_id);

    } elsif ($formtype eq 'radio') {

        $form_box .= "No <input type='radio' name='$field_name' value='No'";
        $form_box .= " checked " if ($field_value ne "Yes") ;
        $form_box .= ">\nYes <input type='radio' name='$field_name' value='Yes'";
        $form_box .= " checked " if ($field_value eq "Yes") ;
        $form_box .= ">\n";

    # aju 08-21-01: add 'select' form field handling. This requires adding some
    # additional attributes to the field-- selectsize and selectoptions.  Given
    # this information, the pulldown or list menu can be created.
    } elsif ($formtype eq 'select') {
        
        my $selectsize    = $self->{fields}->{$field_name}->{selectsize};
        my $selectoptions    = $self->{fields}->{$field_name}->{selectoptions};
        my @selectoplist = split (',', $selectoptions);
        $form_box .= "<select name='$field_name'";
        if ($selectsize > 1) {
            $form_box .= " size='$selectsize'";
        }
        $form_box .= ">";
	    # CKIM 04-10-02: added ability to use separate label and value
	    foreach my $soption (@selectoplist) {
	    	($soption, my $slabel) = split ($self->get_value_label_separator(), $soption);
	    	$slabel ||= $soption;
	        if ($soption eq $field_value) {
	            $form_box .= "<option value='$soption' selected>$slabel\n";
	        }
	        else {
	            $form_box .= "<option value='$soption'>$slabel\n";
	        }
	    }
	    $form_box .= "</select>";

    } elsif ($formtype eq 'file') {

        if ($field_value && (defined $iid)) {
          $form_box .= "$field_value&nbsp;&nbsp;&nbsp;&nbsp;<a class=\"text_button\" href=".$self->make_url("index.pl?id=$iid&amp;isa=$isa&amp;op=delfile&amp;name=$field_name").">delete attachment</a>";
        } else {
          $form_box .= "<INPUT TYPE=\"file\" NAME=\"$field_name\" SIZE=45>";
        }

    } elsif ( $formtype =~ /_upload$/ ) {

    	my $op = $FORM{op};
    	my $return_url = $self->make_url( "index.pl?id=$iid&amp;isa=$isa&amp;op=$op" );
    	
    	$form_box .= $self->get_upload_input_form_line( {
    							 isa => $isa,
    							 id => $iid,
    							 field_name => $field_name,
    							 field_desc => $label,
    							 is_required => $required,
    							 type => $formtype,
    							 return_url => $return_url,
    							} );
    } elsif ($formtype eq 'show_from') {

    	# must only show the date part of the field
    	( $field_value ) = split(/\s+/, $field_value);
    
    	if( $field_value eq '0000-00-00' ) { $field_value = '' }
    
    	# must convert double quotes to html entities to preserve syntax
    	$field_value = escapeHTML( $field_value );       
	
        $form_box .= "<input type=\"text\" name=\"$field_name\" size=\"13\" maxlength=\"10\" value=\"$field_value\">\n";

    } elsif ($formtype eq 'other') {

        $self->debug_msg(5, "Using formtype 'other'; will add '$form_text'");
        $form_box .= $form_text;

    }

    return $form_box;
}

sub get_value_label_separator {
	return 	'<!-- label -->';
}

###################################
sub show {
    my $self = shift;

    my $name            = $self->get_name();
    my $description     = $self->get_description();
    my $keywords        = $self->get_keywords();
    my $longdescription = $self->get_long_description();
    my $html            = '';

    $html .= "<b>$name</b>\n";
    $html .= "<br>$description\n" if ($description);
    $html .= "<br /><br />$longdescription\n" if ($longdescription);
    $html .= "<br /><br />Synonyms: $keywords\n" if ($keywords);

    return $html;
}

sub show_as_bullet {
    my $self        = shift;
    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $buttons     = $self->get_buttons();

    return "<LI>$buttons $name\n$description\n</LI>\n\n";

}

# should be overridden in most cases
sub show_summary {
    my $self        = shift;
	return $self->show();
}


# should be overridden in most cases
sub show_in_column {
    my $self        = shift;
	return $self->show();
}


sub show_postscript {
	return '';
}

sub show_as_roll_over {
    my $self        = shift;
    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $buttons     = $self->get_buttons();

    return "<LI>$buttons $name\n$description\n</LI>\n\n";

}

sub attachment_icon {
    my $self = shift;
    my ($file1) = shift;
    my $iid = $self->id;
    my $uid = $self->uid;
    my $isa = $self->is_a;

    $self->debug_msg(4, "attachmenticon($iid, $file1)");
    return "" if (!$file1);
    return "" if (!$iid);

    my $icon_src = Metadot::Utils::Display->get_icon_src($file1) ;

    my $attach;

    $attach = "&nbsp;" . "<a href='".$self->make_url("index.pl?id=$iid&amp;isa=$isa&amp;op=download")."'>" . "<img align='bottom' src=\"".$icon_src."\" border=\"0\" align='center' alt='Download Me!' width=\"16\" height=\"16\" /></a>" . "&nbsp;" . "<a href='".$self->make_url("index.pl?id=$iid&amp;isa=$isa&amp;op=download")."'><span class=\"centerNewsLink\">download</span></a>" . "&nbsp;" ;

    return $attach;
}

#### ACCESSOR METHODS ####

sub get_f1_attachment_url {
    my $self = shift;

    my $iid = $self->id;
    my $isa = $self->is_a;
    my $file1 = $self->file1;

    return "" if (!$file1);
    return "" if (!$iid);

    return "index.pl?id=$iid&amp;isa=$isa&amp;op=download";
	    
}


sub get_panel {
    my $self = shift;
    
    return $self->show_panel();
}

sub get_buttons {
    my $self = shift;
    my $args = shift || {};

    my $first_panel = $self->show_panel();
    $first_panel .= $self->show_up_down_buttons() . $self->show_scheduling_buttons();
    my $second_panel = $self->show_switch_area_panel();
    my $add_panel = $args->{add_panel};

    my $result = ($first_panel . $second_panel ne '') ? "<span style='white-space: nowrap;'>$first_panel $second_panel</span>" : "";
    $result .= $add_panel ? " <span style='white-space: nowrap;'>$add_panel</span>" : '';
    $result &&= " <span style='white-space: nowrap;'>$result</span>";

    return $result;
}

sub get_file_1_icon {
    my $self = shift;
    my $attach = $self->file1 ?  $self->attachment_icon($self->file1) : "";
    #print STDERR "'" . $self->file1 ."'\n" ;
    return $attach;
}

sub get_cool_icon {
    my $self = shift;
    my $coolicon;

    if (uc($self->cool) eq "YES"){


        my $imageslocation = $PARAMS->{'imageslocation'};
        my $star = "<img class=\"icon\" src='$imageslocation/star.gif' width='15' height='16' border='0' alt='Cool' />";

        $coolicon = "$star\n";
    } else {
      $coolicon="";
    }

    return $coolicon;
}

sub get_uid {
        my $self = shift;

        return $self->{uid};
}

sub get_iid {
        my $self = shift;
        return $self->{id};
}

sub get_parent_iid {
        my $self = shift;

        return $self->{parent_id};
}

sub get_is_a {
        my $self = shift;

        return $self->{is_a};
}

sub get_name {
        my $self = shift;

        return $self->{name};
}

sub get_description {
        my $self = shift;

        return $self->{description};
}

sub get_long_description {
        my $self = shift;

        return $self->{long_description};
}

sub get_cool {
        my $self = shift;

        return $self->{cool};
}

sub get_show_childs {
        my $self = shift;

        return $self->{show_childs};
}

sub get_display_look {
        my $self = shift;

        return $self->{display_look};
}

sub get_url {
        my $self = shift;

        return $self->{url};
}

sub get_keywords {
        my $self = shift;

        return $self->{keywords};
}

sub get_instance_order {
        my $self = shift;

        return $self->{instance_order};
}

sub get_show_from {
        my $self = shift;

        return $self->{show_from};
}

sub get_creation_date {
        my $self = shift;

        return $self->{creation_date};
}

sub get_update_date {
        my $self = shift;

        return $self->{update_date};
}

sub get_nb_items {
        my $self = shift;

        return $self->{nb_items};
}

sub get_nb_reads {
        my $self = shift;

        return $self->{nb_reads};
}

sub get_updated_by {
        my $self = shift;

        return $self->{updated_by};
}

sub get_workflow_id {
        my $self = shift;

        return $self->{workflow_id};
}

sub get_hidden_comment {
        my $self = shift;

        return $self->{hidden_comment};
}

sub get_last_post {
        my $self = shift;

        return $self->{last_post};
}

sub get_file_1 {
        my $self = shift;

        return $self->{file1};
}

sub get_created_by {
        my $self = shift;

        return $self->{created_by};
}

sub get_t1 {
        my $self = shift;

        return $self->{t1};
}

sub get_t2 {
        my $self = shift;

        return $self->{t2};
}


sub get_t3 {
        my $self = shift;

        return $self->{t3};
}

sub get_t4 {
        my $self = shift;

        return $self->{t4};
}

sub get_t5 {
        my $self = shift;

        return $self->{t5};
}

sub get_t6 {
        my $self = shift;

        return $self->{t6};
}

sub get_t7 {
        my $self = shift;

        return $self->{t7};
}

sub get_t8 {
        my $self = shift;

        return $self->{t8};
}

sub get_t9 {
        my $self = shift;

        return $self->{t9};
}

sub get_t10 {
        my $self = shift;

        return $self->{t10};
}

sub get_c1 {
        my $self = shift;

        return $self->{c1};
}

sub get_c2 {
        my $self = shift;

        return $self->{c2};
}

sub get_c3 {
        my $self = shift;

        return $self->{c3};
}

sub get_c4 {
        my $self = shift;

        return $self->{c4};
}

sub get_c5 {
        my $self = shift;

        return $self->{c5};
}

sub get_d1 {
        my $self = shift;

        return $self->{d1};
}

sub get_d2 {
        my $self = shift;

        return $self->{d2};
}

sub get_d3 {
        my $self = shift;

        return $self->{d3};
}

sub get_d4 {
        my $self = shift;

        return $self->{d4};
}

sub get_d5 {
        my $self = shift;

        return $self->{d5};
}

sub get_i1 {
        my $self = shift;

        return $self->{i1};
}

sub get_i2 {
        my $self = shift;

        return $self->{i2};
}

sub get_i3 {
        my $self = shift;

        return $self->{i3};
}

sub get_i4 {
        my $self = shift;

        return $self->{i4};
}

sub get_i5 {
        my $self = shift;

        return $self->{i5};
}


##### MUTATOR METHODS #####

## updates hash with appropriate field values
sub set_field_info {
    my $self = shift;

    my $field;
    my $order = 1;

    while ($#_ > 0) {
        $field    = shift;

        $self->{fields}->{$field}->{label}    = shift;
        $self->{fields}->{$field}->{required} = shift;
        $self->{fields}->{$field}->{order}    = $order++;
    }
    
}


## will override default text for an individual field in form display
sub set_field_form_text {
    my $self = shift;
    my $field = shift;
    my $text  = shift;

    return unless (defined($field) && defined($text) && defined($self->{fields}->{$field}));

    $self->{fields}->{$field}->{formtype}   = 'other';
    $self->{fields}->{$field}->{form_text}  = $text;
}

## will override default field type
sub set_field_type {
    my $self = shift;
    my $field = shift;
    my $type  = shift;
    my $args = shift || {};

    return unless (defined($field) && defined($type) && defined($self->{fields}->{$field}));

    $self->{fields}->{$field}->{formtype} = $type;
    $self->{fields}->{$field}->{options} = $args;
}

sub set_uid {
        my $self = shift;
        my $value = shift;

        $self->{uid} = $value;
}

sub set_iid {
        my $self = shift;
        my $value = shift;

        $self->{id} = $value;
}

sub set_parent_iid {
        my $self = shift;
        my $value = shift;

        $self->{parent_id} = $value;
}

sub set_is_a {
        my $self = shift;
        my $value = shift;

        $self->{is_a} = $value;
}

sub set_name {
        my $self = shift;
        my $value = shift;

        $self->{name} = $value;
}

sub set_description {
        my $self = shift;
        my $value = shift;

        $self->{description} = $value;
}

sub set_long_description {
        my $self = shift;
        my $value = shift;

        $self->{long_description} = $value;
}

sub set_cool {
        my $self = shift;
        my $value = shift;

        $self->{cool} = $value;
}

sub set_show_childs {
        my $self = shift;
        my $value = shift;

        $self->{show_childs} = $value;
}

sub set_display_look {
        my $self = shift;
        my $value = shift;

        $self->{display_look} = $value;
}

sub set_url {
        my $self = shift;
        my $value = shift;

        $self->{url} = $value;
}

sub set_keywords {
        my $self = shift;
        my $value = shift;

        $self->{keywords} = $value;
}

sub set_instance_order {
        my $self = shift;
        my $value = shift;

        $self->{instance_order} = $value;
}

sub set_show_from {
        my $self = shift;
        my $value = shift;

        $self->{show_from} = $value;
}

sub set_creation_date {
        my $self = shift;
        my $value = shift;

        $self->{creation_date} = $value;
}

sub set_update_date {
        my $self = shift;
        my $value = shift;

        $self->{update_date} = $value;
}

sub set_nb_items {
        my $self = shift;
        my $value = shift;

        $self->{nb_items} = $value;
}

sub set_nb_reads {
        my $self = shift;
        my $value = shift;

        $self->{nb_reads} = $value;
}

sub set_updated_by {
        my $self = shift;
        my $value = shift;

        $self->{updated_by} = $value;
}

sub set_workflow_id {
        my $self = shift;
        my $value = shift;

        $self->{workflow_id} = $value;
}

sub set_hidden_comment {
        my $self = shift;
        my $value = shift;

        $self->{hidden_comment} = $value;
}

sub set_last_post {
        my $self = shift;
        my $value = shift;

        $self->{last_post} = $value;
}

sub set_file_1 {
        my $self = shift;
        my $value = shift;

        $self->{file1} = $value;
}

sub set_created_by {
        my $self = shift;
        my $value = shift;

        $self->{created_by} = $value;
}

sub set_t1 {
        my $self = shift;
        my $value = shift;

        $self->{t1} = $value;
}

sub set_t2 {
        my $self = shift;
        my $value = shift;

        $self->{t2} = $value;
}


sub set_t3 {
        my $self = shift;
        my $value = shift;

        $self->{t3} = $value;
}

sub set_t4 {
        my $self = shift;
        my $value = shift;

        $self->{t4} = $value;
}

sub set_t5 {
        my $self = shift;
        my $value = shift;

        $self->{t5} = $value;
}

sub set_t6 {
        my $self = shift;
        my $value = shift;

        $self->{t6} = $value;
}

sub set_t7 {
        my $self = shift;
        my $value = shift;

        $self->{t7} = $value;
}

sub set_t8 {
        my $self = shift;
        my $value = shift;

        $self->{t8} = $value;
}

sub set_t9 {
        my $self = shift;
        my $value = shift;

        $self->{t9} = $value;
}

sub set_t10 {
        my $self = shift;
        my $value = shift;

        $self->{t10} = $value;
}

sub set_c1 {
        my $self = shift;
        my $value = shift;

        $self->{c1} = $value;
}

sub set_c2 {
        my $self = shift;
        my $value = shift;

        $self->{c2} = $value;
}

sub set_c3 {
        my $self = shift;
        my $value = shift;

        $self->{c3} = $value;
}

sub set_c4 {
        my $self = shift;
        my $value = shift;

        $self->{c4} = $value;
}

sub set_c5 {
        my $self = shift;
        my $value = shift;

        $self->{c5} = $value;
}

sub set_d1 {
        my $self = shift;
        my $value = shift;

        $self->{d1} = $value;
}

sub set_d2 {
        my $self = shift;
        my $value = shift;

        $self->{d2} = $value;
}

sub set_d3 {
        my $self = shift;
        my $value = shift;

        $self->{d3} = $value;
}

sub set_d4 {
        my $self = shift;
        my $value = shift;

        $self->{d4} = $value;
}

sub set_d5 {
        my $self = shift;
        my $value = shift;

        $self->{d5} = $value;
}

sub set_i1 {
        my $self = shift;
        my $value = shift;

        $self->{i1} = $value = $value;
}

sub set_i2 {
        my $self = shift;
        my $value = shift;

        $self->{i2} = $value;
}

sub set_i3 {
        my $self = shift;
        my $value = shift;

        $self->{i3} = $value;
}

sub set_i4 {
        my $self = shift;
        my $value = shift;

        $self->{i4} = $value;
}

sub set_i5 {
        my $self = shift;
        my $value = shift;

        $self->{i5} = $value;
}

sub get_intro_text  {
    my $self = shift;
    return $self->{_intro_text_string};
}

sub set_intro_text {
    my $self = shift;
    my $ts = shift;
    $self->{_intro_text_string} = $ts;
}


# Overrides Gizmo::delete so that UploadManager files can be deleted
sub delete {
    my $self = shift;

    my $isa = $self->is_a();
    my $id = $self->id();
    
    $self->delete_all_uploads( 
			      {
			       id => $id,
			       isa => $isa,
			      }
			     );
    
    return $self->SUPER::delete();
}


sub process_uploads {
    my $self = shift;
    my $iid = shift;

    ( defined $iid ) || die "must provide iid param";

    my $err;

    foreach my $field (sort {$self->{fields}->{$a}->{order} <=> $self->{fields}->{$b}->{order}} keys (%{$self->{fields}})) {
	
        my $order = $self->{fields}->{$field}->{order};

        next if !defined($order) || $order <= 0;

	if( $self->{fields}->{$field}->{formtype} =~ /_upload$/ ) {
	    
	    my $label       = $self->{fields}->{$field}->{label};
	    my $required    = $self->{fields}->{$field}->{required};
	    my $isa         = $self->{is_a};
	    my $iid         = $self->{id};
	    my $type = $self->{fields}->{$field}->{formtype};

	    my $result = $self->process_upload( {
						 isa => $isa,
						 id => $iid,
						 field_name => $field,
						 field_desc => $label,
						 type => $type,
						 is_required => $required, } );
	    
	    if($result) { $err .= "$result<br>" }	    
	}
    }

    return $err;
}


# overrides Gizmo::www_download to provide backward compatibiliy for UploadsManager uploads
sub www_download {
    my $self = shift;

    # find the first uploads manager upload in this GB and use that
    foreach my $field (sort {$self->{fields}->{$a}->{order} <=> $self->{fields}->{$b}->{order}} keys (%{$self->{fields}})) {
	
        my $order = $self->{fields}->{$field}->{order};
	
        next if !defined($order) || $order <= 0;

	if( $self->{fields}->{$field}->{formtype} =~ /_upload$/ ) {
	    
	    my $isa         = $self->{is_a};
	    my $iid         = $self->{id};

	    return $self->www_download_file( { id => $iid,
					       isa => $isa,
					       field_name => $field, } );
	}
    }

    # if it didn't find any, use the legacy method

    return $self->SUPER::www_download();
}



1;


__END__

=head1 NAME

GizmoBuilder 

=head1 SYNOPSIS

    use GizmoBuilder;

    $gizmobuilder = GizmoBuilder->new ($id);

    $AC = $gizmobuilder->get_AC ();
    GIZMO->get_default_permissions()
    $boolean = $gizmobuilder->display_in_category();
    $gizmobuilder->process_edit_form ();
    $gizmobuilder->www_save ();
    $gizmobuilder->www_delfile ();
    $gizmobuilder->www_delfileok ();
    $html = $gizmobuilder->show_edit_form ($title);
    $html = $gizmobuilder->get_form_box ($field_name);
    $html = $gizmobuilder->show ();
    $html = $gizmobuilder->show_summary ();
    $html = $gizmobuilder->show_postscript ();
    $html = $gizmobuilder->show_as_bullet ();
    $html = $gizmobuilder->attachment_icon ($file1);
    $url = $gizmobuilder->get_f1_attachment_url ();
    $html = $gizmobuilder->get_panel ();
    $html = $gizmobuilder->get_buttons ();
    $html = $gizmobuilder->get_file_1_icon ();
    $html = $gizmobuilder->get_cool_icon ();
    $uid = $gizmobuilder->get_uid ();
    $iid = $gizmobuilder->get_iid ();
    $parent_iid = $gizmobuilder->get_parent_iid ();
    $isa = $gizmobuilder->get_is_a ();
    $name = $gizmobuilder->get_name ();
    $description = $gizmobuilder->get_description ();
    $long_description = $gizmobuilder->get_long_description ();
    $cool = $gizmobuilder->get_cool ();
    $show = $gizmobuilder->get_show_childs ();
    $display_look = $gizmobuilder->get_display_look ();
    $url = $gizmobuilder->get_url ();
    $keywords = $gizmobuilder->get_keywords ();
    $instance_order = $gizmobuilder->get_instance_order ();
    $show_from = $gizmobuilder->get_show_from ();
    $creation_date = $gizmobuilder->get_creation_date ();
    $update_date = $gizmobuilder->get_update_date ();
    $nb_items = $gizmobuilder->get_nb_items ();
    $nb_reads = $gizmobuilder->get_nb_reads ();
    $updated_by = $gizmobuilder->get_updated_by ();
    $workflow_id = $gizmobuilder->get_workflow_id ();
    $hidden_comment = $gizmobuilder->get_hidden_comment ();
    $last_post = $gizmobuilder->get_last_post ();
    $file1 = $gizmobuilder->get_file_1 ();
    $created_by = $gizmobuilder->get_created_by ();
    $t1 = $gizmobuilder->get_t1 ();
    $t2 = $gizmobuilder->get_t1 ();
    $t3 = $gizmobuilder->get_t1 ();
    $t4 = $gizmobuilder->get_t1 ();
    $t5 = $gizmobuilder->get_t1 ();
    $t6 = $gizmobuilder->get_t1 ();
    $t7 = $gizmobuilder->get_t1 ();
    $t8 = $gizmobuilder->get_t1 ();
    $t9 = $gizmobuilder->get_t1 ();
    $t10 = $gizmobuilder->get_t10 ();
    $c1 = $gizmobuilder->get_c1 ();
    $c2 = $gizmobuilder->get_c2 ();
    $c3 = $gizmobuilder->get_c3 ();
    $c4 = $gizmobuilder->get_c4 ();
    $c5 = $gizmobuilder->get_c5 ();
    $d1 = $gizmobuilder->get_d1 ();
    $d2 = $gizmobuilder->get_d2 ();
    $d3 = $gizmobuilder->get_d3 ();
    $d4 = $gizmobuilder->get_d4 ();
    $d5 = $gizmobuilder->get_d5 ();
    $i1 = $gizmobuilder->get_i1 ();
    $i2 = $gizmobuilder->get_i2 ();
    $i3 = $gizmobuilder->get_i3 ();
    $i4 = $gizmobuilder->get_i4 ();
    $i5 = $gizmobuilder->get_i5 ();
    $gizmobuilder->set_field_info ();
    $gizmobuilder->set_uid ($value);
    $gizmobuilder->set_iid ($value);
    $gizmobuilder->set_parent_iid ($value);
    $gizmobuilder->set_is_a ($value);
    $gizmobuilder->set_name ($value);
    $gizmobuilder->set_description ($value);
    $gizmobuilder->set_long_description ($value);
    $gizmobuilder->set_cool ($value);
    $gizmobuilder->set_show_childs ($value);
    $gizmobuilder->set_display_look ($value);
    $gizmobuilder->set_url ($value);
    $gizmobuilder->set_keywords ($value);
    $gizmobuilder->set_instance_order ($value);
    $gizmobuilder->set_show_from ($value);
    $gizmobuilder->set_creation_date ($value);
    $gizmobuilder->set_update_date ($value);
    $gizmobuilder->set_nb_items ($value);
    $gizmobuilder->set_nb_reads ($value);
    $gizmobuilder->set_updated_by ($value);
    $gizmobuilder->set_workflow_id ($value);
    $gizmobuilder->set_hidden_comment ($value);
    $gizmobuilder->set_last_post ($value);
    $gizmobuilder->set_file_1 ($value);
    $gizmobuilder->set_created_by ($value);
    $gizmobuilder->set_t1 ($value);
    $gizmobuilder->set_t2 ($value);
    $gizmobuilder->set_t3 ($value);
    $gizmobuilder->set_t4 ($value);
    $gizmobuilder->set_t5 ($value);
    $gizmobuilder->set_t6 ($value);
    $gizmobuilder->set_t7 ($value);
    $gizmobuilder->set_t8 ($value);
    $gizmobuilder->set_t9 ($value);
    $gizmobuilder->set_t10 ($value);
    $gizmobuilder->set_c1 ($value);
    $gizmobuilder->set_c2 ($value);
    $gizmobuilder->set_c3 ($value);
    $gizmobuilder->set_c4 ($value);
    $gizmobuilder->set_c5 ($value);
    $gizmobuilder->set_d1 ($value);
    $gizmobuilder->set_d2 ($value);
    $gizmobuilder->set_d3 ($value);
    $gizmobuilder->set_d4 ($value);
    $gizmobuilder->set_d5 ($value);
    $gizmobuilder->set_i1 ($value);
    $gizmobuilder->set_i2 ($value);
    $gizmobuilder->set_i3 ($value);
    $gizmobuilder->set_i4 ($value);
    $gizmobuilder->set_i5 ($value);


=head1 DESCRIPTION

GizmoBuilder is a class that provides for an easy way to create Gizmos.


Using GizmoBuilder, a new gizmo can be created in 3 simple steps.  There are also some optional steps that can be taken to modify
Gizmos that wish to change the default settings provided to them by GizmoBuilder.

1.  Write a constructor.  (an example of how to write the proper constructor needed is given below)

2.  Gizmos can use all the fields of the instance table.  So, in the constructor, specify which of those fields you
will be using in your gizmo.  At the same time, specify labels for those fields, and specify which fields are required.
By doing so, you allow the metadot framework to automatically generate a form for your gizmo that asks people to fill in
the appropriate fields, and ensures that required fields are actually filled in.

Here is an example of a constructor:  (note that the line numbers would not be used in actual code)

  1.  sub new {
  2.      my $proto = shift;
  3.      my $id = shift;
  4.
  5.      my $class = ref($proto) || $proto;
  6.      my $self;
  7.
  8.      if (defined ($id)){
  9.       $self  = $class->SUPER::new($id);
  10.     } else {
  11.       $self  = $class->SUPER::new();
  13.       $self->{cool}='No';
  14.     }
  15.
  16.     $self->{is_a} = __PACKAGE__;

  17.     $self->set_field_info (
  18.         "name",        "<b>Item Name</b>",                                  1,
  19.         "url",         "<b>URL</b> <i>e.g. http://www.yourcompany.com</i>", 0,
  20.         "description", "<b>Description</b>",                                0,
  21.         "file1",       "<b>Attachment file</b>",                            0,
  22.         "keywords",    "<b>Keywords and Synonyms</b>",                      0,
  23.         "cool",        "<b>Star this Item</b>",                             0,
  24.         );

  25.     bless ($self, $class);        # reconsecrate
  26.     return $self;
  27. }

Here is the line by line explanation:

1.  This line names the subroutine.  The new subroutine will be called by the metadot framework when it instantiates your gizmo

2.  This line assigns the name of the class to the $proto variable

3.  This line assigns the id (an optional parameter) to the $id variable

5.  This line ensures that if the class name passed in was actually a reference to a class, we use the actual class name.

8-14.  This line is important:  if an id is passed in, that means we are instantiating a gizmo which has previously been created
and then stored into the database.  If an id is not passed in, we are instead instatiating a new gizmo

13.  This line means that gizmos are not starred by default

16.  This line takes the package that the gizmo is in (such as Item, or Table), and assigens it to the is_a instance variable

17-24.  These lines specify which fields, out of all of the possible fields that one can use with gizmos, are actually going
to be used with this particular gizmo.  The first parameter is the name of the field.  The second parameter is the label, 
used when displaying forms when the item is created or edited.  The third parameter is set to 1 if it is required.  Otherwise,
it is set to zero.

26.  This line makes $self into a perl object.


3.  Write the following methods:

    show
    show_summary
    show_as_dash

In these methods, you can use getter methods to retreive information
stored in the gizmo.  A list of all getter and setter methods is given
in the synopsis of this documentation.

Here is an example of a show method:

    
     1.    sub show {
     2.    my $self = shift;
     3.
     4.    my $name            = $self->get_name();
     5.    my $description     = $self->get_description();
     6.    my $url             = $self->get_url();
     7.    my $keywords        = $self->get_keywords();
     8.    my $file            = $self->get_file_1();
     9.    my $file_icon       = $self->get_file_1_icon();
     10.    my $cool_icon       = $self->get_cool_icon();
     11.    my $longdescription = $self->get_long_description();
     12.    my $html            = '';
     13.
     14.    $html .= ($url) ? "<a href='$url'>$name</a>\n" : "<b>$name</b>\n";
     15.    $html .= $cool_icon;
     16.    $html .= "<br>$description\n" if ($description);
     17.    $html .= "<br /><br />$longdescription\n" if ($longdescription);
     18.    $html .= "<br /><br />File attached: $file $file_icon\n" if ($file);
     19.    $html .= "<br /><br />Synonyms: $keywords\n" if ($keywords);
     20.
     21.    return $html;
     22. }

Line-by-line explanation:

1.  Names the subroutine

2.  assigns the object to the $self variable.  $self is analagous to the 'this' object in Java    

4-11.  Retreives values from the object into variable

14-19.  Uses the variables previously retreived to generate html that contains those variables.

In addition to the above, there are optional methods that can be
written.  For example you could write a method called
www_show_details, which would provide a way to show additional fields
that you have chosed not to display in your show method.  Note
carefully the presense of www_.  Metadot has a www_show method, which
calls the show method of your gizmo.  It does not know about any
methods that you may write.  Thus, you need to provide the www_
version of your methods.  You can then do all of the processing in
your www_show_details method, or your www_show_details method can call
a show_details method that you would write.

When adding new www_ methods, there is an additional detail that must
be taken care of.  As noted in the documentation for Auditable and
Operations, metadot security is based on the idea of controlling
access to the www_ methods.  These methods are grouped into bundles,
and permissions are assigned to bundles.  So, your www_show_details
method would need to be added to a bundle, or a new bundle would need
to be created and your www_show_details method would need to be added
to that bundle.

The Operations class is the one that stores the association between
the operations (or subroutines) and bundles.  To get an operations
instance, call the get default permissions as a class method on the
super class of your gizmo (the super class of your gizmo would be
GizmoBuilder).  That call looks like this:

  my $default_permissions = Discussion->SUPER::get_default_permissions;

Once you have the object, you can modify it as described in the
documentation of the Operations class.

Note that both the retrieval and the modification of the operations
object take place as class operations.

Once you are finished modifying the object, you need to make it
available via as a class method call.  This class method will override
the class method you used to get the first operations instance.  Here
is how that method would look:

  sub get_default_permissions {
      return $default_permissions;
  }

In addition to writing your own new methods, such as www_show_details,
you can override existing www_ methods which are inherited from
GizmoBuilder.  Documentation on these methods is found in the
documentation of Gizmo.pm

These methods are:

    www_save();
    www_delfile();
    www_delfileok ();
    www_show ;
    www_change_owner ;
    www_edit_permissions ;
    www_modify ([$errormsg]);
    www_save ;
    www_delete ;
    www_delete_ok ;
    www_up ;
    www_down ;
    www_cut ;
    www_paste ;
    www_download ;


=head2 METHODS

Constructor: Called by concrete classes which inherit from
GizmoBuilder.  Complete documentation of how to write a concrete
constructor that then calls the GizmoBuilder constructor is given in
the Description of this documentation.

    $gizmobuilder = GizmoBuilder->new ($id);

Returns an operations object, which can be modified by concrete
classes, as documented in the Description section of this
documentation and in the documentation of the Operations class.

    GizmoBuilder->get_default_permissions();

Called by the metadot framework, this method does not need to be
called or overridden by people using the GizmoBuilder class 

    $AC = $gizmobuilder->get_AC ();

Called by the metadot framework, this method should not be overridden by people
using the GizmoBuilder class. This function returns a boolean value from a hash 
inside the gizmo. This value is used in the App architecture to determine wether
the output of the method being called should be wrapped in the application layout. 

    $boolean = $gizmobuilder->display_in_category ();

Processes the information in an edit form.  This function may be
overridden in concrete gizmos.

    $gizmobuilder->process_edit_form ();

Saves information that has been entered in a web form.  This function
may be overridden in concrete gizmos.

  $gizmobuilder->www_save ();

Asks for confirmation to delete a file attached to a gizmo.  This
function may be overridden in concrete gizmos.

    $gizmobuilder->www_delfile ();

Deletes a file attached to a gizmo.  This function may be overridden in concrete gizmos.    

    $gizmobuilder->www_delfileok ();

Provides the html used by www_show_edit_form.  May be overridden in concrete gizmos.

    $html = $gizmobuilder->show_edit_form ($title);

Provides the html for form boxes.  May be overridden in concrete gizmos.

    $html = $gizmobuilder->get_form_box ($field_name);

Provides the html for the www_show method.  May be overridden in concrete gizmos.

    $html = $gizmobuilder->show ();

Provides the html for the www_show_as_bullet_method.  May be overridden in concrete gizmos.

    $html = $gizmobuilder->show_as_bullet ();

Provides the html needed to display a link to a url that, when
clicked, downloads a file attached to a gizmo. 

    $html = $gizmobuilder->attachment_icon ($file1);

Provides the url that is clicked on to download a file attached to a gizmo    

    $url = $gizmobuilder->get_f1_attachment_url ();

Provides the html to display the edit buttons, and the edit
permissions button, if a user has permission to see those buttons 

    $html = $gizmobuilder->get_panel ();

Provides the html to display the panel (using the get_panel method) as well as the up/down buttons

    $html = $gizmobuilder->get_buttons ();

Provides the html to display the image that will be associated with
the link used to download an attached file

    $html = $gizmobuilder->get_file_1_icon ();

Provides the html needed to display the cool icon, displayed next to
gizmos which have been designated as cool

    $html = $gizmobuilder->get_cool_icon ();

Returns the uid of the person who owns the gizmo

    $uid = $gizmobuilder->get_uid ();

Returns the iid of the gizmo

    $iid = $gizmobuilder->get_iid ();

Returns the parent_iid of the gizmo

    $parent_iid = $gizmobuilder->get_parent_iid ();

Returns what the gizmo is.  For example, it would return Item, or Table, or Discussion.

    $isa = $gizmobuilder->get_is_a ();

Returns the name of the gizmo instance

    $name = $gizmobuilder->get_name ();

Returns the description of the gizmo instance

    $description = $gizmobuilder->get_description ();

Returns the long description of the gizmo instance

    $long_description = $gizmobuilder->get_long_description ();

Returns true if the gizmo has been set to cool status

    $cool = $gizmobuilder->get_cool ();

Returns true if the children of the gizmo should be shown

    $show = $gizmobuilder->get_show_childs ();

Returns which look the gizmo should have.  Used for the Category Gizmo

    $display_look = $gizmobuilder->get_display_look ();

Returns a url that a person may choose to associate with a gizmo

    $url = $gizmobuilder->get_url ();

Returns the keywords associated with a gizmo

    $keywords = $gizmobuilder->get_keywords ();

Returns the order in which a gizmo should display on the page,
relative to all the other gizmos of the same type 

    $instance_order = $gizmobuilder->get_instance_order ();

Returns the date on which a gizmo should appear.

    $show_from = $gizmobuilder->get_show_from ();

Returns the date on which a gizmo was created

    $creation_date = $gizmobuilder->get_creation_date ();

Returns the date on which a gizmo was updated

    $update_date = $gizmobuilder->get_update_date ();

Returns nb_items, an integer field which can be used for any purpose.

    $nb_items = $gizmobuilder->get_nb_items ();

Returns nb_reads, an integer field which can be used for any purpose.   

    $nb_reads = $gizmobuilder->get_nb_reads ();

Returns the uid of the person who updated the Gizmo

    $updated_by = $gizmobuilder->get_updated_by ();

Not used.  Reserved for future use.    

    $workflow_id = $gizmobuilder->get_workflow_id ();

Returns a text field which may be used for any purpose

    $hidden_comment = $gizmobuilder->get_hidden_comment ();

Returns a datetime field whch may be used for any purpose

    $last_post = $gizmobuilder->get_last_post ();

Returns a varchar which is used by metadot to store information about upload files

    $file1 = $gizmobuilder->get_file_1 ();

Returns the uid of the person who created the instance of the gizmo

    $created_by = $gizmobuilder->get_created_by ();

Returns a text field which may be used for any purpose

    $t1 = $gizmobuilder->get_t1 ();

Returns a text field which may be used for any purpose

    $t2 = $gizmobuilder->get_t1 ();

Returns a text field which may be used for any purpose

    $t3 = $gizmobuilder->get_t1 ();

Returns a text field which may be used for any purpose

    $t4 = $gizmobuilder->get_t1 ();

Returns a text field which may be used for any purpose

    $t5 = $gizmobuilder->get_t1 ();

Returns a text field which may be used for any purpose

    $t6 = $gizmobuilder->get_t1 ();

Returns a text field which may be used for any purpose

    $t7 = $gizmobuilder->get_t1 ();

Returns a text field which may be used for any purpose

    $t8 = $gizmobuilder->get_t1 ();

Returns a text field which may be used for any purpose

    $t9 = $gizmobuilder->get_t1 ();

Returns a text field which may be used for any purpose

    $t10 = $gizmobuilder->get_t10 ();

Returns a character field which may be used for any purpose

    $c1 = $gizmobuilder->get_c1 ();

Returns a character field which may be used for any purpose

    $c2 = $gizmobuilder->get_c2 ();

Returns a character field which may be used for any purpose

    $c3 = $gizmobuilder->get_c3 ();

Returns a character field which may be used for any purpose

    $c4 = $gizmobuilder->get_c4 ();

Returns a character field which may be used for any purpose

    $c5 = $gizmobuilder->get_c5 ();

Returns a date field which may be used for any purpose.  The format returned is YYYY-MM-DD

    $d1 = $gizmobuilder->get_d1 ();

Returns a date field which may be used for any purpose.  The format returned is YYYY-MM-DD

    $d2 = $gizmobuilder->get_d2 ();

Returns a date field which may be used for any purpose.  The format returned is YYYY-MM-DD

    $d3 = $gizmobuilder->get_d3 ();

Returns a date field which may be used for any purpose.  The format returned is YYYY-MM-DD

    $d4 = $gizmobuilder->get_d4 ();

Returns a date field which may be used for any purpose.  The format returned is YYYY-MM-DD

    $d5 = $gizmobuilder->get_d5 ();

Returns an integer field which may be used for any purpose.    

    $i1 = $gizmobuilder->get_i1 ();

Returns an integer field which may be used for any purpose.    

    $i2 = $gizmobuilder->get_i2 ();

Returns an integer field which may be used for any purpose.    

    $i3 = $gizmobuilder->get_i3 ();

Returns an integer field which may be used for any purpose.    

    $i4 = $gizmobuilder->get_i4 ();

Returns an integer field which may be used for any purpose.    

    $i5 = $gizmobuilder->get_i5 ();

Allows the fields of the instance table that will be used by the
gizmo, and the order of those fields, to be reset. 

    $gizmobuilder->set_field_info ();

Sets the uid of the owner of the instance.

    $gizmobuilder->set_uid ($value);

Sets the iid of the instance    

    $gizmobuilder->set_iid ($value);

Sets the parent iid of the instance

    $gizmobuilder->set_parent_iid ($value);

Sets what type of Gizmo this is, such as Item, or Table.  The maximum number of characters is 20

    $gizmobuilder->set_is_a ($value);

Sets the name of the Gizmo.  The maximum number of characters is 80

    $gizmobuilder->set_name ($value);

Sets the description of the Gizmo.  The maximum number of characters is 65535

    $gizmobuilder->set_description ($value);

Sets the long description of the Gizmo.  The maximum number of characters is 65535    

    $gizmobuilder->set_long_description ($value);

Sets whether or not an item is marked cool.  

    $gizmobuilder->set_cool ($value);

Sets whether or not the children of the object are shown

    $gizmobuilder->set_show_childs ($value);

Sets the type of look used for the category (used in the Category gizmo)

    $gizmobuilder->set_display_look ($value);

Sets the url associated with a gizmo.  The maximum number of characters is 222

    $gizmobuilder->set_url ($value);

Sets the keywords associated with a gizmo.  The maximum number of characters is 255

    $gizmobuilder->set_keywords ($value);

Sets the order in which a gizmo appears, relative to other gizmos of
that same type.  Max value is 2e32 -1 

    $gizmobuilder->set_instance_order ($value);

Sets the date on which a gizmo should be displayed

    $gizmobuilder->set_show_from ($value);

Sets the date on which a gimzo was created

    $gizmobuilder->set_creation_date ($value);

Sets the date on which a gizmo was updated

    $gizmobuilder->set_update_date ($value);

An integer that may be used for any purpose.  Minimum value is -2e32.  Max value is 2e32 -1    

    $gizmobuilder->set_nb_items ($value);

An integer that may be used for any purpose.  Minimum value is -2e32.  Max value is 2e32 -1    

    $gizmobuilder->set_nb_reads ($value);

Sets the uid of the person updating the gizmo

    $gizmobuilder->set_updated_by ($value);

Not currently used.  Reserved

    $gizmobuilder->set_workflow_id ($value);

Sets a text field which may be used for any purpose.  Maximum number of characters is 65535

    $gizmobuilder->set_hidden_comment ($value);

Sets a datetime field which may be used for any purpose

    $gizmobuilder->set_last_post ($value);

Used by metadot to store information about attached files    

    $gizmobuilder->set_file_1 ($value);

Sets the uid of the person who created the object.    

    $gizmobuilder->set_created_by ($value);

Sets a text field which may be used for any purpose.  The maximum number of characters is 65535    

    $gizmobuilder->set_t1 ($value);

Sets a text field which may be used for any purpose.  The maximum number of characters is 65535    

    $gizmobuilder->set_t2 ($value);

Sets a text field which may be used for any purpose.  The maximum number of characters is 65535    

    $gizmobuilder->set_t3 ($value);

Sets a text field which may be used for any purpose.  The maximum number of characters is 65535    

    $gizmobuilder->set_t4 ($value);

Sets a text field which may be used for any purpose.  The maximum number of characters is 65535    

    $gizmobuilder->set_t5 ($value);

Sets a text field which may be used for any purpose.  The maximum number of characters is 65535    

    $gizmobuilder->set_t6 ($value);

Sets a text field which may be used for any purpose.  The maximum number of characters is 65535    

    $gizmobuilder->set_t7 ($value);

Sets a text field which may be used for any purpose.  The maximum number of characters is 65535    

    $gizmobuilder->set_t8 ($value);

Sets a text field which may be used for any purpose.  The maximum number of characters is 65535    

    $gizmobuilder->set_t9 ($value);

Sets a text field which may be used for any purpose.  The maximum number of characters is 65535    

    $gizmobuilder->set_t10 ($value);

Sets a character field which may be used for any purpose.  The maximum
number of characters is 111     

    $gizmobuilder->set_c1 ($value);

Sets a character field which may be used for any purpose.  The maximum
number of characters is 111     

    $gizmobuilder->set_c2 ($value);

Sets a character field which may be used for any purpose.  The maximum
number of characters is 111     

    $gizmobuilder->set_c3 ($value);

Sets a character field which may be used for any purpose.  The maximum
number of characters is 111     

    $gizmobuilder->set_c4 ($value);

Sets a character field which may be used for any purpose.  The maximum
number of characters is 111     

    $gizmobuilder->set_c5 ($value);

Sets a date field which may be used for any purpose.  The format is YYYY-MM-DD

    $gizmobuilder->set_d1 ($value);

Sets a date field which may be used for any purpose.  The format is YYYY-MM-DD

    $gizmobuilder->set_d2 ($value);

Sets a date field which may be used for any purpose.  The format is YYYY-MM-DD

    $gizmobuilder->set_d3 ($value);

Sets a date field which may be used for any purpose.  The format is YYYY-MM-DD

    $gizmobuilder->set_d4 ($value);

Sets a date field which may be used for any purpose.  The format is YYYY-MM-DD

    $gizmobuilder->set_d5 ($value);

Sets an integer that may be used for any purpose.  Minimum value is
-2e32.  Max value is 2e32 -1     

    $gizmobuilder->set_i1 ($value);

Sets an integer that may be used for any purpose.  Minimum value is
-2e32.  Max value is 2e32 -1     

    $gizmobuilder->set_i2 ($value);

Sets an integer that may be used for any purpose.  Minimum value is
-2e32.  Max value is 2e32 -1     

    $gizmobuilder->set_i3 ($value);

Sets an integer that may be used for any purpose.  Minimum value is
-2e32.  Max value is 2e32 -1     

    $gizmobuilder->set_i4 ($value);

Sets an integer that may be used for any purpose.  Minimum value is
-2e32.  Max value is 2e32 -1     

    $gizmobuilder->set_i5 ($value);

=cut
