###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Message;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CGI);

use Carp;
use vars qw(@ISA $AUTOLOAD);

use HTML::Parser;
use DBAccess;
use Text::Wrap;
use Portal;
use Metadot::Utils::Theme;
use Gizmo;
use Gizmo::Discussion;
use Metadot::UserFactory;
use Metadot::Controllable;
use Utility;

# CKIM: testing to see where this breaks
#@ISA = qw(Discussion Portal);
@ISA = qw(Portal Metadot::Controllable);


sub get_owner_uid {
    my $self = shift;
   
    my $mid = $self->{mid};

    # the ownwer of a message is the owner of
    # the parent discussion, not the poster
    my $discussionid = $self->get_iid($mid) || $FORM{discussion_id};

    my $parent = Discussion->new($discussionid);
    
    return $parent->get_owner_uid();
}

sub get_uid {
    my $self = shift;
    return $self->{postuid};
}

sub uid {
    my $self = shift;
    return $self->get_uid();
}

sub new {
    my $proto = shift;
    my $mid = shift;
    my $class = ref($proto) || $proto;
    
    my $self;

    $mid = undef unless Utility->is_valid_integer($mid); # secu_200401
    
    # CKIM: removed the line that passes IID, because
    #       that corresponds to the discussion, not the message
    #$self = $class->SUPER::new();

    $self->{mid}             = undef;
    $self->{iid}             = "";
    $self->{parent_mid}      = "";
    $self->{postuid}             = "";
    $self->{name}            = "";
    $self->{description}     = "";
    $self->{creation_date}   = "";
    $self->{update_date}     = "";
    $self->{updated_by}      = "";
    $self->{status}          = "";
    $self->{priority}        = "";
    $self->{email}           = "";
    $self->{author_name}     = "";
    $self->{author_email}    = "";
    
    # [vadim] runtime flag
    $self->{editing}    = undef ;

    if(defined ($mid)){

      ($self->{mid}            ,
       $self->{postuid}            ,
       $self->{parent_mid}     ,
       $self->{iid}            ,
       $self->{name}           ,
       $self->{description}    ,
       $self->{creation_date}  ,
       $self->{update_date}    ,
       $self->{updated_by}     ,
       $self->{status}         ,
       $self->{priority}       ,
       $self->{email}          ,
       $self->{author_name}    ,
       $self->{author_email}   ,
      )
	= DBAccess->sqlSelect(
		    "mid           ,
                    userid         ,
		    parentmid      ,
		    iid            ,
		    name           ,
		    description    ,
		    creationdate ,
		    updatedate ,
		    updatedby      ,
		    status         ,
		    priority       ,
		    email          ,
		    authorname     ,
		    authoremail",
		    "message",
		    "mid=$mid"
		   );
    
    }
    $self->{creation_date} = Utility->format_datetime($self->{creation_date}, 'b e, Y, r');
    $self->{update_date}   = Utility->format_datetime($self->{update_date}, 'b e, Y, r');
    
    bless ($self, $class);
    $self->initialize();

    return $self;
}


sub initialize {
    my $self = shift;
    
    my $iid = $self->{iid} || $FORM{discussion_id};
    if (defined($iid) && $iid ne '') {
        $self->set_container(Gizmo->restore($iid));    
    }
    $self->debug_msg(4, "initialized container($iid) as ".$self->get_container());
}

# CKIM: trying to remove the iid confusion
sub get_iid {
    my $self = shift;
    return $self->get_container_id();
}
sub get_container_id {
   my $class = shift;
   my $mid   = shift;

   $mid = undef unless Utility->is_valid_integer($mid); # secu_200401
   
   # check if i'm a fully instantiated object
   if (ref($class)) {
       $mid = $class->{mid};
   }
   unless( defined($mid) && $mid ne '') { return undef }

   my $iid;
   # if i'm a fully instantiated object, i may have a container object
   if (ref($class)) {
       my $container = $class->get_container();
       if ($container) {
           $iid = $container->get_iid();
       }
   }
   if (!defined $iid) {
       ($iid) = DBAccess->sqlSelect("iid","message","mid=$mid") if $mid;
   }

   return $iid;
}

# aju 03-9-02
sub get_id {
    my $self = shift;
    return $self->get_mid();
}

sub id {
    my $self = shift;
    return $self->get_mid();
}

sub get_mid {
    my $self = shift;
    return $self->{mid};
}

sub _get_permissions {
  my $self = shift;

  unless ($self->{permissions}) {

    my $mid = $self->{mid};

    ## get iid of parent
    my $discussionid = $self->get_iid($mid) || $FORM{discussion_id};

    return unless defined $discussionid;

    my $parent = Discussion->new($discussionid);

    my $id = $parent->id();
    my $isa = $parent->is_a();

   # retrieve a frozen perms object from the perms table
    my $frozen_perms;
    if($isa and defined $id) {
        # XXX this is very inefficient

        ($frozen_perms) = DBAccess->sqlSelect("permissions", "permissions", "id=$id AND isa='$isa'");
    }

    # if it finds a row then unfreeze the perms object
    if($frozen_perms) {

      $self->_set_permissions(Metadot::Permissions->unfreeze($frozen_perms));

    } else {
        # if the object doesn't have a perms object then just go with the default

        $self->_set_permissions($parent->get_default_permissions_object());

    }
  }

  die "perms are not there" unless ($self->{permissions});

  return $self->{permissions};

}
#####################################################
# WWW FUNCTIONS

sub www_show {
    my $self = shift;
    $self->debug_msg(3, "Message::www_show()");

    if (!$self->mid){
	$self->click_back ("Add Form Error:", "mid is missing.");
	exit(0);
    }

#DG
    my ($global_look) = $PARAMS->{globallook};
    $global_look ||= "old";
    $self->debug_msg(4, "Message::www_show() has global_look $global_look.");

   # fix -- move so that template mode gets the header stuff too
   $self->print_http_header();

    if ($global_look eq "usetemplate") {
        my $results = $self->show();
        #print STDERR "self->show() returns:\n$results\n\n\n\n\n";
        
        # the "-2" param corresponds to utility template 2 (arghh)
        $self->debug_msg(4, "Message::www_show() running simple_expand.");
        
        # GizmoTags assume the caller is a Gizmo instance. My container should be one.
        my $templated = $self->show_template(-2);
        #print STDERR "$self\->show processed the template and got\n$templated\n\n\n\n";
        
        my $expanded = $DISPLAY->simple_expand($self->get_container_id(), $self->get_container(), $templated , $results);
        $self->debug_msg(4, "Message::www_show() FINISHED running simple_expand.");
        #print STDERR "$self\->show expanded the text as\n$expanded\n\n\n\n";

        print $expanded;

    } else {

        $self->debug_msg(4, "Message::www_show() running show_top.");
        print $self->show_top($self->iid);
        $self->debug_msg(4, "Message::www_show() running show_title.");
        print $self->show_title("Message");
        $self->debug_msg(4, "Message::www_show() running show.");
        print $self->show(),
        "\n\n<!-- End of Message -->\n\n";
        $self->debug_msg(4, "Message::www_show() running show_bottom.");
        print $self->show_bottom();
    }
    $self->debug_msg(4, "Message::www_show() FINISHED.");
}

sub www_modify_own_message {
    my $self = shift;
    my %args = @_;
    if ($self->is_user_modifying_own_message()) {
        return $self->www_modify_message(%args);
    } else {
        $self->print_click_back('Error','You cannot modify a message posted by another user.');
        exit;
    }
}

# returns true if the message belongs to the user, and self-editing messages are permitted
sub is_user_modifying_own_message {
    my $self = shift;
    return ($self->message_belongs_to_user() && $PARAMS->{allow_discussion_message_editing});
}

sub www_modify_message {
  my $self = shift;
  my %args = @_;
  my $errormsg = $args{errormsg};
  my $review = $args{review};
  my $mid = $self->mid();
  $self->debug_msg(3, "Message::www_modify_message()");

  my $discussionid = $self->get_container_id();
  if (!$discussionid){
    $self->click_back ("Modify Form Error:", "discussion id is missing.");
    exit(0);
  }

  if ($errormsg) {
    $errormsg = "<p><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p>\n";
  }

  # [vadim] set editing flag
  $self->{editing} = 1 ;
  
    #aju
    my $formbgcolor;
    if (Metadot::Utils::Theme->use_theme_info()) {
	$formbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
    }
    else {
	$formbgcolor = $PARAMS->{'formbgcolor'};
    }
  
    my $form_op;
    if ($self->is_user_modifying_own_message()) {
        $form_op = 'save_own_message';
    } else {
        $form_op = 'save_message';
    }
    my $formstring =   "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\"><tr><td>" .
        "<form action='index.pl'  method='post' ENCTYPE=\"multipart/form-data\">
           <input type='hidden' name='op' value='$form_op'>
           <input type='hidden' name='isa' value='Message'>
           <input type='hidden' name='mid' value='$mid'>
           ".$self->make_hidden()."
           $errormsg\n  $review\n" .
          $self->show_compose_form("Editing message:<p>") .
          "<table align='center'><tr><td>
           <input type='submit' name='action' value='Review'>
           <input type='submit' name='action' value='Save Message'>
           </td></tr></table></form></td></tr></table>" ;

    $formstring = "<tr bgcolor='#$formbgcolor'><td>&nbsp;</td><td align='center'>$formstring</td><td>&nbsp;</td></tr>";
  
  $DISPLAY->print_http_header();  
  my $look = $PARAMS->{globallook};
  if ($look eq "usetemplate") {
      #print  $DISPLAY->simple_expand($discussionid, $self, $self->show_template(-1), $formstring);
      print  $DISPLAY->simple_expand($self->get_container_id(), $self->get_container(), $self->show_template(-1), $formstring);
  } else {
      # this is compatible with the new template mode
      $DISPLAY->print_form($formstring,"Editing message",$discussionid);
  }  
}


sub www_compose {
  my $self = shift;
  my %args = @_;
  my $errormsg = $args{errormsg} || "";
  my $review   = $args{review} || "";
  $self->debug_msg(3, "Message::www_compose()");

  my $parentmid = $self->mid() || $FORM{parent_mid} || "";
  my $discussionid = $self->iid() || $FORM{discussion_id};
  
  if (!$discussionid){
    $self->click_back ("Add Form Error:", "discussion id is missing.");
    exit(0);
  }

  if ($errormsg) {
    $errormsg = "<p><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p>\n";
  }

  my $form = "" ;

  $form .= "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\"><tr><td>
         <form action='index.pl'  method='post' ENCTYPE=\"multipart/form-data\">
         <input type='hidden' name='op' value='send'>
         <input type='hidden' name='isa' value='Message'>
         <input type='hidden' name='discussion_id' value='$discussionid'>
         <input type='hidden' name='parent_mid' value='$parentmid'>
         ".$self->make_hidden();
           
  $form .= "
         $errormsg\n
         $review\n".
        $self->show_compose_form("Composing a new message:").
        "<table align='center'><tr><td>
         <input type='submit' name='action' value='Review'>
         <input type='submit' name='action' value='Send Message'>
         </td></tr></table></form></td></tr></table>" ;
        
    #CKIM: switched to using print_form
    $self->print_form($form, 'Composing message', $discussionid);
    
#     #aju
#     my $formbgcolor;
#     if (Metadot::Utils::Theme->use_theme_info()) {
#         $formbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
#     }
#     else {
#         $formbgcolor = $PARAMS->{'formbgcolor'};
#     }
#
#     $DISPLAY->print_http_header();
#     print $DISPLAY->show_page_top;
#     print $DISPLAY->simple_header() . "<body>\n";
#     print "<center>" . $DISPLAY->form_top();
#     print "<tr bgcolor='#$formbgcolor'><td>&nbsp;</td><td align='center'>$form</td><td>&nbsp;</td></tr>";
#     print $DISPLAY->form_bottom() . "</center>";
#     print $DISPLAY->close_html();
}

sub www_reply {
  my $self = shift;
  $self->debug_msg(3, "Message::www_reply()");

  if (!$self->mid){
    $self->click_back ("Add Form Error:", "mid is missing.");
    exit(0);
  }

  if (!$self->can_reply()){
    $self->click_back ("Message Error:", "Can't reply - not approved.");
    exit(0);
  }
  $self->prepare_reply();
  $self->www_compose;
}

sub can_reply {
  my $self = shift;
  
  return $self->is_approved();
}

sub is_approved {
    my $self = shift;
    return (return $self->status() eq 'OK');
}

sub www_delete_message {
        my $self = shift;
        my $mid = $self->mid();
        my $referrer = $SESSION->{referer};
        # [vadim] do not forget to escape the URL !
         my $target = $CGI->escape($referrer);
        #
        
        $self->debug_msg(3, "Message::www_delete()");
        $self->print_http_header;
        $self->two_choices("Do you want to delete this Message?",
                "caution",
                "Yes please", 
                $self->make_url("index.pl?mid=$mid&amp;isa=Message&amp;op=delete_message_ok&amp;target=$target"), "No!", $referrer);
}

##############

sub www_delete_message_ok {

 
    my $self = shift;
    # [vadim] we should avoid to display a just deleted message, so redirect to parent instead:
    my $target = "index.pl?isa=Discussion&amp;iid=" . $self->iid ;
#    if ($FORM{mid} ne $self->mid() && defined($FORM{target})) {
#        $target = $FORM{target} ;
#    }
    $self->debug_msg(3, "Message::www_delete_ok()");
    $self->_delete_message();
    $self->md_redirect($self->make_url($target));
}

sub www_approve {
  my $self = shift;

  $self->_approve();

  $self->md_redirect($SESSION->{referer});
}

sub www_send {
  my $self = shift;
  $self->debug_msg(3, "Message::www_send()");

  $self->name($FORM{name});
  $self->description($FORM{description});

  my $errormsg = "";
  #$errormsg .= "<LI>Please specify a title" if (!$self->name()) ;
  
  my ($status, @errors) = DBAccess->validate_data(
      table         => 'message',
      parameters    => \%FORM,
  );

  if ($status) {
      foreach my $error (@errors) {
          $errormsg .= "<LI>$error\n";
      }
  }

  if ($FORM{action} && $FORM{action} eq 'Review') {
    $self->www_compose(errormsg=>$errormsg,  review=>$self->show_review());
  } elsif ($FORM{action} eq 'Send Message') {
    if ($errormsg) {
      $self->www_compose(errormsg=>$errormsg);
    } else {
      $self->send();


      _update_parent($FORM{parent_mid});
      $self->two_choices ("Message sent!", 
                          "confirm",
                          "Continue", 
                          $self->make_url("index.pl?iid=$FORM{discussion_id}&amp;isa=Discussion&amp;op=show"));
    } 
  } else {
      $self->click_back ("Send Error:", "no action specified.");
  }
}

sub www_save_own_message {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Message::www_save_own_message()");
    if ($self->message_belongs_to_user()) {
        return $self->www_save_message(%args);
    } else {
        $self->print_click_back('Error','You cannot save a message posted by another user.');
    }
}

sub message_belongs_to_user {
    my $self = shift;
    my $uid = shift || $USER->uid();
    my $result = ($self->{postuid} == $uid);
    return $result;
}

sub www_save_message {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Message::www_save()");
    
    $self->name($FORM{name});
    $self->description($FORM{description});
    my $parent_mid = $self->parent_mid();
    my $discussion_id = $self->iid();
    
    my $errormsg = "";
    #$errormsg .= "<li>Too bad! Title field cannot be empty..." if (!$self->name()) ;

    my ($status, @errors) = DBAccess->validate_data(
        table         => 'message',
        parameters    => $self,
    );
    
    if ($status) {
        foreach my $error (@errors) {
            $errormsg .= "<LI>$error\n";
        }
    }
    
    if ($FORM{action} eq 'Review') {
        $self->www_modify_message(errormsg=>$errormsg,  review=>$self->show_review());
    } elsif ($FORM{action} eq 'Save Message') {
        if ($errormsg) {
          $self->www_modify_message(errormsg=>$errormsg);
        } else {
          
            my $moderated = Discussion->is_moderated($discussion_id);
            $self->set_moderated_status($discussion_id);
          
            $self->save();
            if( $moderated && $PARAMS->{send_moderated_posting_notifications} ) {
                my $disc = Discussion->restore( $discussion_id );
                $self->_send_moderate_notification( $disc );
            }        
            
            _update_parent($parent_mid);
            $self->two_choices ("Message saved!", 
                "confirm",
                "Continue", 
                $self->make_url("index.pl?iid=$discussion_id&amp;isa=Discussion&amp;op=show"));
        } 
    } else {
      $self->click_back ("Save Error:", "no action specified.");
    }
}

#####################################################
# BACKEND FUNCTIONS

sub tag_open_sub{

	my $not_ok;
	my ($tagname, $attr) = @_;

	if(defined($tagname) && $tagname eq 'img') {

		# will remove the img tag altogether if its src attribute 
		# contains any & and ? characters
		#      use this line instead if you want to go all the way to 
		#      prohibiting all image types but the ones mentioned in 
		#      the regexp:
		#	      if( $attr->{src} && ( ($attr->{src} =~ /\&|\?/) || 
		#           ($attr->{src} !~ /(gif|jpg|png|jpeg)$/i) ) ) {
		if( $attr->{src} &&  ($attr->{src} =~ /\&|\?/) ) {

			$not_ok = 1;

		}

	}

};


# show message content
sub show {

    my $self = shift;
    $self->debug_msg(4, "Message::show()");

    my $container = $self->get_container();
    my $moderated = defined($container->c1()) ? "Yes" : "No";
    my $status = $self->status();
    return '' if ($moderated eq 'Yes' && $status ne 'OK' && !$self->is_allowed_to_do('moderate', $USER));

    my $mid = $self->mid;
    my $iid = $self->iid;
    my $name = $self->name;
    my $parentmid = $self->parent_mid;
    my $uname = $self->author_name;
    my ($image) = DBAccess->sqlSelect("file1","instance","iid=$iid"); 
    my $creationdate = $self->creation_date;
    my $updatedate = $self->{update_date};
    my $updatedby   = $self->{updated_by};
    my $modified_message = (defined($updatedby) && $updatedby ne '') ? "<font color=\"red\">Modifed by $updatedby on $updatedate.</font>" : "";
     
    $self->debug_msg(5, "Message::show() about to call parser");
    #use HTML:Parser to strip the excess characters
    my $description = $self->description;
    #aju 08-21-01 put in the user's line breaks. (Is this the right thing to do?)
    $description =~ s/\n/<br>/g;
    my ($parsedDescription) = new HTML::Parser (
    				 api_version   => 3,
    				 start_h => [ tag_open_sub,  'tagname,attr,offset,length' ],
    				);
    $parsedDescription->parse($description);
    $self->debug_msg(5, "Message::show() finished parsing");
    

    my $inresponse;
    $inresponse = " (in response to <a href='".$self->make_url("index.pl?mid=$parentmid&amp;isa=Message&amp;op=show")."'>$parentmid</a>)\n" if($parentmid); 
    
    $self->debug_msg(5, "Message::show() looking for root");
    my $root_mid = $self->find_root();
    $self->debug_msg(5, "Message::show() looking for found root $root_mid");
    my $PNthread = $self->prev_next_thread($root_mid) ;

    my $thread = $self->show_thread($root_mid);
   
    
    my $header = "<table border='0' cellspacing='0' cellpadding='1'>\n";
    $header .=  "<tr><td valign='top'><b>Author:</b></td>\n";
    $header .= "<td valign='top'><div style=\"white-space: nowrap\">$uname </div></td></tr>\n";
    $header .= "<tr><td valign='top'><b>Date:</b></td><td valign='top'>$creationdate</td></tr>\n";
    $header .= "<tr><td valign='top'><b>Subject:</b></td><td valign='top'>$name</td></tr>\n";
    $header .= "<tr><td valign='top'><b>Modified:</b></td><td valign='top'>$modified_message</td></tr>\n" if ($modified_message ne '');
    $header .= "</table>\n\n\n";

    my $s= "
<table width=\"100%\" border='0' cellspacing='0' cellpadding='2'>
<tr><td rowspan='2'>$header</td>
<td align='right'><a href='".$self->make_url("index.pl?mid=$mid&amp;isa=Message&amp;op=reply")."'>Reply</a></td></tr>
<tr><td align='right'><div style=\"white-space: nowrap\"><a href='".$self->make_url("index.pl?iid=$iid&amp;isa=Discussion&amp;op=show")."'>Back to list</a></div></td></tr></table>";

    if ($image) {
        # XXX [vadim] 4.5 last-minute solution for the Messages in default Discussions
        # to avoid ugly 'bad images'
        # SHOULD BE REMOVED AS SOON AS POSSIBLE IN THE NEXT RELEASE
        require "Gizmo.pm"; 
        require "GizmoTool.pm";         
        my $dis = Discussion->new($iid) ;
        if ($dis->check_download()) {
	       $s .= "<table align='right'><tr><td><img src=\"index.pl?op=download&amp;iid=$iid&amp;isa=Discussion\"></td></tr></table>";
        }
    }
    $header= "<table border='0' cellspacing='0' cellpadding='1'>
<tr><td valign='top'><b>Author:</b></td><td valign='top'>$uname </td></tr>
<tr><td valign='top'><b>Date:</b></td><td valign='top'>$creationdate</td></tr>
<tr><td valign='top'><b>Subject:</b></td><td valign='top'>$name</td></tr>
</table><br>\n";

$s .= "<br /><br />
$description
<br /><br /><br />
<table width=\"100%\" cellspacing=\"1\">
<tr>
<td><div style=\"white-space: nowrap\"><b>Messages in thread:</b></div></td>
<td colspan='2' align='right'>$PNthread</td>
</tr>
<tr class='discussionheader'>
<td><font class='discussionheader'><b>Subject</b></font></td>\n
<td><font class='discussionheader'><b>Author</b></font></td>\n
<td><font class='discussionheader'><b>Date</b></font></td>\n
</tr>\n
$thread \n
</table>\n
<br /><br />\n
";
    
    $self->debug_msg(5, "Message::show() finished");
    return $s;
}

sub get_previous_thread_root {
    my $self = shift;
    my $rootmid = shift || $self->find_root();
    my $iid =  $self->iid();
    my ($result) = DBAccess->sqlSelect("max(mid)", "message", "mid<$rootmid and parentmid=0 and iid=$iid");
    return $result;
}

sub get_next_thread_root {
    my $self = shift;
    my $rootmid = shift || $self->find_root();
    my $iid =  $self->iid();
    my ($result) = DBAccess->sqlSelect("min(mid)", "message", "mid>$rootmid and parentmid=0 and iid=$iid");
    return $result;
}

# prints navigation to previous and next thread
# print(current_thread_root, iid)
# returns prev next nav
sub prev_next_thread {
  my $self = shift ;
  my $rootmid = shift ;
  my $iid =  $self->iid();

  $rootmid = undef unless Utility->is_valid_integer($rootmid); # secu_200401
  
  ## find previous thread root
  my ($previous_thread_root) = $self->get_previous_thread_root($rootmid);
  my $previous = "Previous Thread";
  $previous = "<a class='prevnextbar' href='".$self->make_url("index.pl?mid=$previous_thread_root&amp;isa=Message&amp;op=show")."'>$previous</a>" if ($previous_thread_root);
  
  ## find next thread root
  my ($next_thread_root) = $self->get_next_thread_root($rootmid);
  my $next = "Next Thread";
  $next = "<a class='prevnextbar' href='".$self->make_url("index.pl?mid=$next_thread_root&amp;isa=Message&amp;op=show")."'>$next</a>" if ($next_thread_root);

  return "&lt; $previous | $next &gt;";
}

# find the root of this thread
# findroot(pid)
# returns mid
sub find_root {
  my $self = shift ;
  my $mid = shift || $self->mid();
  
  my ($pid) = DBAccess->sqlSelect("parentmid", "message", "mid=$mid", " ORDER BY message.updatedate");
  return $mid if ($pid == 0);
  return $self->find_root($pid);
}

# display the thread
# show_thread(thread_root_mid)
# returns prev next nav
sub show_thread {
  my $self = shift ;
  my $root_mid = shift ;
  my $mid = $self->mid();
  my $iid = $self->iid();
 
  $root_mid = undef unless Utility->is_valid_integer($root_mid); # secu_200401
  
  my $highlight = ($mid == $root_mid) ? 1 : 0 ;
  my @root_row = DBAccess->sqlSelect("mid, name, authorname, userid, creationdate, status",
			   "message", "mid=$root_mid");
  my $discussion = Discussion->new($iid);
  my $html = $discussion->_show_one_row(\@root_row,0,$highlight);
  my $thread = $discussion->_show_thread_child($root_mid,1,$mid);
  $html .= join("",@$thread);
  return $html;
}

sub set_moderated_status {
    my $self = shift;
    my $did = shift || $self->iid() || $FORM{discussion_id};
    
    $self->{status} = $self->determine_moderated_status($did);
    return $self->{status};
}

sub determine_moderated_status {
    my $class = shift;
    my $did = shift || die("Must supply discussion id");
    
    my $moderated = Discussion->is_moderated($did);
    my $status = $moderated ? "MD" : "OK";

    return $status;
}

sub add_message {
    my $class = shift;
    my %args = @_;

    my $did = $args{discussion_id} || $args{iid};
    $did = undef unless Utility->is_valid_integer($did); # secu_200401
    defined($did) || die("Must supply discussion_id");   
    
    my $moderated = Discussion->is_moderated($did);
    my $status = $args{status} || $class->determine_moderated_status($did);
    
    my $db_now = Utility->db_now();

    my %fields = (
                  userid        => $args{userid}        || $USER->uid(), 
                  authorname 	=> $args{authorname}    || $USER->fullname(),
                  authoremail	=> $args{authoremail}   || $USER->{email},
                  iid       	=> $did,
                  parentmid     => $args{parentmid}     || 0,
                  name       	=> $args{name},
                  description	=> $args{description},
                  priority   	=> $args{priority},
                  status    	=> $status,
                  creationdate  => $args{creationdate}  || $db_now,
                  updatedate    => $args{updatedate}    || $db_now,
    );
    
    #print STDERR "Inserting new message with args\n",Dumper(\%args),"\nInsert fields are:\n",Dumper(\%fields);
    DBAccess->sqlInsert("message", %fields);
    #print STDERR "Message was inserted\n";

    # update if not moderated
    DBAccess->sqlSimple("update instance set nbitems=nbitems+1 where iid=$did") unless $moderated;

    if( $moderated && $PARAMS->{send_moderated_posting_notifications} ) {
        my $disc = Discussion->restore( $did );
        $class->_send_moderate_notification( $disc );
    }

}

# send new message to discussion = add to DB
sub send {
  my $self = shift;

  $self->debug_msg(4, "Message::send()");
  my $did = $FORM{discussion_id};
  my $status = $self->set_moderated_status($did);

  my $authorname = $USER->get_display_name();
 
  $self->add_message( 
	    userid           => $USER->{uid}, 
	    authorname 	     => $authorname,
	    authoremail	     => $USER->{email},
	    iid       	     => $did,
	    parentmid        => $FORM{parent_mid} || 0,
	    name       	     => $FORM{name},
	    description	     => $FORM{description},
	    priority   	     => $FORM{priority},
	    status    	     => $status,
	     );
  
}

sub _send_moderate_notification {
    my $class = shift;
    my $discussion = shift;

    my $users = $discussion->get_all_who_can('MODERATE');

    my @moderator_emails;

    foreach my $usr ( @$users ) {
        my $usr_obj = Metadot::UserFactory->get_user($usr);
        push( @moderator_emails, $usr_obj->email() );
    }
    
    if( scalar( @moderator_emails ) > 0 ) {
		@moderator_emails = @{Utility->filter_valid_emails( @moderator_emails )};
    } 

    unless( scalar( @moderator_emails ) > 0 ) { return }

    my $to = join ',', @moderator_emails;
    my $cc = "";
    my $bcc = "";
            
    my $disc_name = $discussion->name();

    my $subject = "[$disc_name] New message awaits moderation";
    my $urgent = 0;
    my $mesg = $PARAMS->{'posting_notification_message'};
        
    my $basename = $PARAMS->{basename};
    my $link_text = "$basename/index.pl?iid=0&amp;isa=Portal&amp;op=show_pending_messages";
    $mesg =~ s/<<posting_approval_link>>/$link_text/g;
    $mesg =~ s/<<discussion_name>>/$disc_name/g;
            
    #print STDERR "CKIM: Register-> sending new user notifications to $to\n";
    if ($to ne '') {

	#print STDERR "SENDING MAIL TO: $to\n MESSAGE: $subject -- $mesg\n";
	my $sm = Sendmail->new($to, $cc, $bcc, $subject, $mesg);
	$sm->sendMail();
    }    
}


# update message
sub save {
  my $self = shift;
  $self->debug_msg(4, "Message::send()");
  my $mid = $self->mid();

  my $db_now = Utility->db_now();
  DBAccess->sqlUpdate(
      "message", 
      "mid=$mid",
      (
            name       	     => $self->name(),
            description	     => $self->description(),
            updatedby        => $USER->{fullname},
            updatedate       => $db_now,
            status           => $self->status(),
            )
	     ) if $mid;
}

# show HTML form to compose new message
sub show_compose_form {
  my $self = shift;
  my $title = shift;
  $self->debug_msg(4, "Message::show_compose_form($title)");

  my $name = $self->name;
  #aju: substitute any single quotes in the name, for placing in the form text field.
  $name =~ s/\'/&#146;/g;
  $name =~ s/&/&amp;/g;
  my $description = $self->description;
  $description =~ s/&/&amp;/g;
  my $did = $FORM{discussion_id} || $self->iid();
  my $is_editing = $self->{editing} ? 1 : 0; ## editing or inserting new

  my $moderated_message = !$is_editing && Discussion->is_moderated($did) ?
                          "<p><font color=\"red\">This discussion is moderated. After sending a message, you must wait until a moderator approves the message to see it posted.</font><p>" : ""; 
  my $edit_message = $is_editing ? 
  "<p><font color=\"red\">If you modify this message, the message will be marked modifed with your name and time.</font><p>" : ""; 

  my $s  = $moderated_message . $edit_message . $self->title_bar($title)."<center>\n";
  $s .= "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\" width='100%'>";
  $s .= "<tr align='left'>\n";
  $s .= "<td align='left'>\n";
  $s .= "<b>Message Title</b>:&nbsp\n";
  $s .= "<input size='33' name=name maxlength='100' value='$name'>\n";
  $s .= "<p><b>Message:</b><br>";
  $s .= "<textarea name=description rows=22 cols=44 WRAP='VIRTUAL'>$description</textarea>\n";
  $s .= "</td></tr></table>";

  return $s;
}

# show the review of message
sub show_review {
  my $self = shift;
  $self->debug_msg(4, "Message::show_review()");


  #use HTML::Parse to parse the name
  my ($parsedName) = new HTML::Parser (
      				 api_version   => 3,
      				 start_h => [ tag_open_sub,  'tagname,attr,offset,length' ],
      				);
  my $name = $FORM{name};
  $parsedName->parse($name);

  
  #instead use HTML::Parser to parse the HTML out
  my ($parsedMsg) = new HTML::Parser (
      				 api_version   => 3,
      				 start_h => [ tag_open_sub,  'tagname,attr,offset,length' ],
      				);
      				
  my $msg = $FORM{description};
  $parsedMsg->parse($msg);



  my $s= $self->title_bar("Reviewing your message");
  $msg =~ s/\n/<br>/g;

  $s.= "<b>$name</b><br>$msg<br /><br />\n\n";
  
  return $s;
}

# modify object attribute for reply
sub prepare_reply {
  my $self = shift;
  $self->debug_msg(4, "Message::prepare_reply()");
  
  ### prepare the Re: message
  $self->name("Re: ".$self->name) if ($self->name !~ m/^Re: /);

  ### Copy the orginal msg and add > in front of each line.
  $Text::Wrap::columns=65;
  my $pre1="> ";
  my $pre2="> ";
  $self->description(wrap($pre1, $pre2, $self->description));
  
  ### [vadim] Set the new flag (to recognize the difference between editing message and new one)
  $self->{editing} = undef ;
}

# compute recursively (bottom-up) updatedate
# to display message by threads (order by update)
sub _update_parent {
  my $mid = shift;
  $mid = undef unless Utility->is_valid_integer($mid); # secu_200401
  return unless ($mid);
  my $db_now = Utility->db_now();
  DBAccess->sqlUpdate ("message", "mid = $mid", updatedate => $db_now);
  my ($parentmid) = DBAccess->sqlSelect ("parentmid", "message", "mid = '$mid'");   
  _update_parent($parentmid);
  # updating parent 'updatedate'recursively
}


sub _delete_message {
  my $self = shift;
  my $mid  = shift || $self->mid();
  $mid = undef unless Utility->is_valid_integer($mid); # secu_200401
  my $iid  = $self->iid();

  # Selecting all messages that mid is parentmid
  my ($c)=DBAccess->sqlSelectMany("mid", "message", "iid=$iid AND parentmid=$mid");

  while(my ($childmid) =$c->fetchrow()) {
      $self->_delete_message($childmid);
  }
  DBAccess->sqlDelete("message", "mid=$mid") if defined($mid);
}

sub _approve() {
  my $self = shift;

  my $mid  = $self->mid();
  my $did  = $self->iid();

  # set creationdate and updatedate to db_now. Note, you must set creationdate so that 
  # the daily digest, monthly digests will send based on the approval date.
  my $db_now = Utility->db_now();

  DBAccess->sqlUpdate("message","mid=$mid",(status=>'OK', creationdate=>$db_now, updatedate=>$db_now)) if defined($mid);

  # the following form of sqlUpdate is not compatible w/ Oracle
  #DBAccess->sqlUpdate("instance", "iid=$did", (-nbitems=>'nbitems+1')) if defined $did;
  DBAccess->sqlSimple("update instance set nbitems=nbitems+1 where iid=$did") if defined $did;
}

# gets the container of this message, usually a discussion
sub get_container {
    my $self = shift;
    return $self->{container};
}

sub set_container {
    my $self = shift;
    my $container = shift;
    $self->{container} = $container;
}

# alias for get_container. Prevents TemplateMgr for "looping like crazzy"
sub get_parent {
    my $self = shift;
    return $self->get_container();
}

# inherit permission from the container
sub is_allowed_to_do {
    my $self = shift;
    my @args = @_;

    my $result = $self->SUPER::is_allowed_to_do(@args);
    $self->debug_msg(4, "$self\->is_allowed_to_do(@args) gets '$result'");

    unless ($result) {
        my $container = $self->get_container();
        if ($container) {
            $result = $container->is_allowed_to_do(@args);
            $self->debug_msg(4, "$self\->is_allowed_to_do(@args) checks parent $container and gets '$result'");
        }
    }
    return $result;
}


# inherit from container
sub is_moderated {
    my $self = shift;
    return $self->get_container()->is_moderated(@_);
}


##########################################
# exec methods service MetadotController
##########################################
use constant _RESPONSE_SHOW_FULL_ => 'show_full';

sub exec_show {
    my $self    = shift;
    my $context = shift;

    return _RESPONSE_SHOW_FULL_;
}


#####################################################

1;


__END__

=head1 NAME

Message - Descended from L<Discussion> and L<Portal>; controls message
display and content

=head1 SYNOPSIS

    use Message;
    $msg = Message->new ([$mid]);

    $iid = $msg->get_iid ($mid);
    $msg->www_show ;
    $msg->www_modify_message ([errormsg=>$errormsg, review=>$review]);
    $msg->www_compose ([errormsg=>$errormsg="", review=>$review=""]);
    $msg->www_reply ;
    $boolean = $msg->can_reply ;
    $msg->www_delete_message ;
    $msg->www_delete_message_ok ;
    $msg->www_approve ;
    $msg->www_send ;
    $msg->www_save_message ;
    $html = $msg->show ;
    $html = $msg->prev_next_thread ($root_mid);
    $root_mid = $msg->find_root ([$mid]);
    $html = $msg->show_thread ($root_mid) ;
    $msg->send ;
    $msg->save ;
    $html = $msg->show_compose_form ($title);
    $html = $msg->show_review ;
    $msg->prepare_reply ;


=head1 DESCRIPTION

B<Message> controls the display and content of discussion messages. Messages
can be reviewed, moderated, and posted.

Constructor; can instantiate a message object with $mid

    $msg = Message->new ([$mid]);

Returns $iid or the id of the discussion the message belongs to.

    $iid = $msg->get_iid ($mid);

Web method that prints content of message.

    $msg->www_show ;

Prints form to modify a message. Will display $errormsg if passed. Will 
display message in review format if $review html passed.

    $msg->www_modify_message ([$errormsg, $review]);

Prints form that allows composing of new message. Will display $errormsg 
if passed. Will display message in review format if $review html passed.

    $msg->www_compose ([errormsg=>$errormsg="", review=>$review=""]);

Prints form that allows a message to be replied to.

    $msg->www_reply ;

Web method that asks user if they are sure they want to delete message.

    $msg->www_delete_message ;

Deletes message.

    $msg->www_delete_message_ok ;

Changes message status to allow it to be viewed.

    $msg->www_approve ;

Posts message to a discussion.

    $msg->www_send ;

Updates a message's content.

    $msg->www_save_message ;

Returns html of message content.

    $html = $msg->show ;

Returns html of navigation to previous and next thread of $root_mid.

    $html = $msg->prev_next_thread ($root_mid);

Finds $root_mid or root mid of thread that $mid belongs; if $mid not
passed then use's objects $mid

    $root_mid = $msg->find_root ([$mid]);

Returns html that is the list of all messages for thread that begins
at $root_mid

    $html = $msg->show_thread ($root_mid) ;

Saves new message to database.

    $msg->send ;

Updates a current message in database.

    $msg->save ;

Returns html of form for editing or adding a message.

    $html = $msg->show_compose_form ($title);

Returns html of message in review format.

    $html = $msg->show_review ;

Formats current message for replying.

    $msg->prepare_reply ;

=cut
