###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Cache;
use strict;
use MetadotConfig;
use DBAccess;


my %instance_cache;
srand();

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    bless ($self, $class); 
    return $self;
}

sub get_cache {
    my $class = shift;

    my $namespace = MetadotConfig->get_dbname();

    my $instance = $instance_cache{$namespace};
    unless (defined $instance) {
        $instance = Metadot::Cache->new();
        $instance_cache{$namespace} = $instance;
    }
    return $instance;
}
    
sub set_value {
    my $self = shift;
    my $name = shift;
    my $value = shift;
    $self->{$name}{'VALUE'} = $value;
}

sub get_value{
    my $self = shift;
    my $name = shift;
    return $self->{$name}{'VALUE'};
}

sub get_timestamp{
    my $self = shift;
    my $name = shift;
    return $self->{$name}{'TIMESTAMP'};
}

sub set_as_dirty {
    my $self = shift;
    my $timestamp_name = shift;
    
    my $timestamp = time . "-" . rand() . rand();
    if (DBAccess->sqlSelect("name", "timestamps", "name = ".DBAccess->quote($timestamp_name))) {   #secu_200401
        DBAccess->sqlUpdate("timestamps", "name = ".DBAccess->quote($timestamp_name), value => $timestamp);
    } else {
        DBAccess->sqlInsert("timestamps", name => $timestamp_name, value => $timestamp);
    }
}

sub is_dirty {
    my $self = shift;
    my $name = shift;
    my $cache_timestamp = $self->{$name}{'TIMESTAMP'};
    my ($db_timestamp) = DBAccess->sqlSelect("value", "timestamps", "name = ".DBAccess->quote($name)); #secu_200401
    if (!$db_timestamp && ($cache_timestamp == 1)) {
        return undef;
    } elsif (!$db_timestamp){
        $self->{$name}{'TIMESTAMP'} = 1;
        return 1;
    } elsif ($db_timestamp ne $cache_timestamp) {
        return $db_timestamp;
    } else {
        return undef;
    }
}

sub set_timestamp{
    my $self = shift;
    my $name = shift;
    my $timestamp = shift;
    $self->{$name}{'TIMESTAMP'} = $timestamp;
}

1;


__END__


