#!/usr/bin/perl -w
#
###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::EmailToDiscussion;
use strict;

use Portal;
use Metadot qw( $PARAMS );
use MetadotConfig;
use Metadot::Config;
use Message;
use Data::Dumper;

# CKIM: commenting this out, because it requires a high version of perl,
#       and its intended effect is not working
# use Encode;

use constant _LOCALHOST_ => 'localhost';

my $instance;

sub new {
    my ($proto, %args) = @_;
    my $class = ref($proto) || $proto;
    
    unless (defined $instance) {
        $instance = {};
        bless ($instance, $class);
    }


    $instance->{'message'} = "";
    $instance->{server_names} = $args{server_names} || [MetadotConfig->get_all_server_names()];
    $instance->{receivers} = {};
    $instance->{unique_alias} = {};
    
    return $instance;
}

sub message {
    my $self = shift;
    return $self->{'message'};
}

sub execute {
    my $self = shift;

    my $result = 1;
    
    # initialize params
    my $mdot = Portal->new(1);
    
    my $popuser = $PARAMS->{'subscriptionsender'};
    my $pophost = $PARAMS->{'subscriptionhost'};
    my $poppwd  = $PARAMS->{'subscriptionpwd'};
    my $poppwd_display = defined $poppwd ? "NOT SHOWN" : "UNDEFINED";
    
    unless ($popuser && $pophost && $poppwd) {
        $self->{'message'} = "One or several required parameters are not defined: POPUSER = ($popuser), POPHOST = ($pophost), POPPWD = ($poppwd_display)";
        return undef;
    }
    
    require Mail::POP3Client;
    my $pop = new Mail::POP3Client (
				    USER     => $popuser,
				    PASSWORD => $poppwd,
				    HOST     => $pophost
				   );
    
    unless (defined $pop) {
        $self->{'message'} = "Unable to connect to server ($pophost), using account ($popuser). May be password is incorrect ?";
        return undef;
    }
    
    # moved these to the inside of the loop
    # my ($from, $subject, $head, $body, $iid, $mid, $date);
    
    print STDERR "\n\nFound ".$pop->Count()." messages in queue\n";
    
    for (my $i = 1; $i <= $pop->Count(); $i++) {
        my ($from, $subject, $iid, $mid, $date, $previous_header, $header_key);
        my $head = $pop->Head( $i );
        my %header_hash = (); ## used to store contents of header
        $self->{receivers} = {}; ## stores To and/or Cc accounts
        $self->{unique_alias} = {}; ## keeps track of alias
        print STDERR "\n\n$head\n";
        foreach ( $pop->Head( $i )) {## populate hash with header values
            if ($_ =~ m/^(\S+):\s*(.*)$/) {
                $header_key = lc($1);
                $header_hash{$header_key} = $2;
                $previous_header = lc($header_key);
            } else {
                $header_hash{$previous_header} .= $_;
            }
        }

        print STDERR "header_hash:\n" . Dumper(\%header_hash);
        
        # default name of the server to process messages for
        my $server_name_to_process = _LOCALHOST_;
        my $alias = undef;

        $from = $header_hash{from};
        $date = $header_hash{date};
        $subject = $header_hash{subject};
        $self->process_receivers($header_hash{to});
        $self->process_receivers($header_hash{cc});
        print STDERR "Finished processing receivers, got:\n",Dumper $self->{receivers};


#        foreach ( $pop->Head( $i )) {
#                if ($_ =~ m/^From/) {
#                    s/From:\s*//;
#                    $from = $_;
#                } elsif ($_ =~ m/^Date/) {
#                    s/Date:\s*//;
#                    $date = $_;
#                } elsif ($_ =~ m/^Subject/) {
#                    s/Subject:\s*//;
#                    $subject = $_;
#                } elsif ($_ =~ m/^(To|CC):/) {
#                    $self->process_receivers($_);
#                }
#    	}
        
        my $head_and_body = $pop->HeadAndBody( $i );
        
        # CKIM: this was an attempt to decode foreign-language characters,
        #       but Encode::decode() doesn't do the job, and this module
        #       requires a high version of perl

#         # detect the charset and decode
#         my $content_type_hashkey = 'content-type';
#         if ($header_hash{$content_type_hashkey}) {
#             $header_hash{$content_type_hashkey} =~ m/charset="(.*)"/;
#             my $charset = $1;
#             print STDERR "Detected charset '$charset'\n";
#             if ($charset) {
#                 $subject = decode($charset, $subject);
#                 $head_and_body = decode($charset, $head_and_body);
#
#                 #print STDERR "Decoded subject as '$subject'\n\n";
#                 #print STDERR "Decoded body as '$head_and_body'\n\n";
#             }
#         }
                
        foreach $server_name_to_process (keys(%{$self->{receivers}})) {
            ## array ref of records to process
            my $receivers = $self->{receivers}->{$server_name_to_process};

            $ENV{'SERVER_NAME'} = $server_name_to_process;
            MetadotConfig->set_server_name($server_name_to_process);

            print STDERR "\n\nServer: $server_name_to_process, Message: $subject\n";

            # this is needed to update the groups hash with the corresponding server groups table            
            # some implementations (SPE) require skipping this step, because they are running only
            # one server and it has a very large user base, therefore it is very expensive to reload it.
            unless (MetadotConfig->get_param_value('do_not_reload_portal_when_fetching_email')) {
                my $mdot = Portal->new(1);
            }

            foreach my $receiver (@$receivers) {
                $iid   = $receiver->{iid};
                $mid   = $receiver->{mid};
                $alias = $receiver->{alias};
                
                # converted the arguments to named parameters.
                # also, removed the && filter here. it causes the system
                # to stop processing email even though it can continue
                $result = $self->post_message(
                    from            => $from,
                    subject         => $subject,
                    date            => $date,
                    head_and_body   => $head_and_body,
                    iid             => $iid,
                    mid             => $mid,
                    alias           => $alias,
                    );
            }

            # close connection to the site's database
            DBAccess->close_db_object();
        }
        
    	$pop->Delete($i);
    }
    
    $pop->Close();
    
    return $result;
}


sub post_message {
    my ($self, %args) = @_;
    my $from            = $args{from};
    my $subject         = $args{subject};
    my $date            = $args{date};
    my $head_and_body   = $args{head_and_body} || $args{body};
    my $iid             = $args{iid};
    my $mid             = $args{mid};
    my $alias           = $args{alias};
    
    
    $iid = undef unless Utility->is_valid_integer($iid); # secu_200401
    $mid = undef unless Utility->is_valid_integer($mid); # secu_200401
    
    print STDERR "post_message received iid='$iid'; mid='$mid', alias='$alias', PARAM: '$PARAMS->{allow_discussion_email_alias}'\n";

    # !$iid is now allowed, as long as there is an alias
    if (!$iid && $alias && $PARAMS->{allow_discussion_email_alias}) {
        # search for the discussion corresponding to the alias
        $iid = Discussion->get_iid_for_alias($alias);
        $mid = 0;
    }
    
    print STDERR "post_message using iid='$iid' and mid '$mid'\n";

    ## comment out will handle this case with a rejection letter
    #unless ($iid) {
    #    $self->{'message'} = "Could not extract iid and mid from 'to' field of message with subject $subject from $from; skipping this insertion";
    #    return undef;
    #}

    require MIME::Parser;
    my $parser = MIME::Parser->new();
    $parser->output_to_core(1);
    
    my $entity = $parser->parse_data($head_and_body);
    
    my $content;
    
    my $find_content = sub {
        my $myself_sub = shift;
        my $entity = shift;
        $content && return;
        my @parts = $entity->parts;
    
        FOUND_CONTENT:
        foreach my $sought_type ( 'text/plain', 'text/html' ) {
            my $effective_type = lc $entity->effective_type();
            print STDERR "\nMAIN ENTITY EFFECTIVE TYPE: $effective_type";
            if ($effective_type eq $sought_type) {	    
                $content = $entity->bodyhandle->as_string; 
                $content && last FOUND_CONTENT;
            }
            
            if( scalar (@parts) ) {
                foreach my $part (@parts) {	
                    my $effective_type = lc $part->effective_type();
                    print STDERR "\nSUB PART EFFECTIVE TYPE: $effective_type";
                    if ($effective_type eq $sought_type) {
                        $content = $part->bodyhandle->as_string;
                        $content && last FOUND_CONTENT;
                    }
                }
            }
        }
        
        unless($content) {
            foreach my $part (@parts) {	
                my $effective_type = lc $part->effective_type();		
                if( $effective_type =~ /multipart/) {
                    &$myself_sub( $myself_sub, $part );
                }
            }
        }
    };
    
    &$find_content( $find_content, $entity );
    
    # remove metadot added lines
    $content =~ s/--------(.*)--------//gis;
    
    print STDERR "\nCONTENT: $content";
    print STDERR "\n******* $iid and $mid **************";

    my ($exists) = DBAccess->sqlSelect("count(*)", "message", "mid='$mid'");
    my ($exists1) = DBAccess->sqlSelect("count(*)", "instance", "iid='$iid'");
    
    my $moderated = $exists1 && $iid && Discussion->is_moderated($iid);
    my $status = $moderated ? "MD" : "OK";
    
    my $email;
    my $sender_email = "";
    
    # use the 'discussion account' email if available, otherwise just use the default.
    my ($sender_login) = $PARAMS->{'subscriptionsender'};
    my ($sender_host) = $PARAMS->{'subscriptionhost'};
    if ($sender_login && $sender_host) {
        $sender_email = "$sender_login\@$sender_host";
    } else {
        $sender_email = $PARAMS->{'mailsenderaddress'}
    }
    
    if ($from =~ m/(.*?)<(.*?)>/i) {
        $email = $2;
    } elsif ($from =~ m/\s*(.*)\s*\(/) {
        $email = $1;
    } else {
        $email = $from;
    }
    
    print STDERR "\nin storeMessages, got email: $email\n";
    
    my ($uid) = DBAccess->sqlSelect("userid", "users", "UPPER(email)='".uc($email)."'");
    if(!defined ($iid) || !$iid) {
        my ($subject1, $message);
        if ($alias) {
            $self->{'message'} = "Alias $alias not found in system with subject $subject from $from; skipping this insertion";
            $subject1 = "[Discussion alias not found] $subject";
            $message = "Your posting below has been rejected because we could not identify the alias $alias.\n";
        } else {
            $self->{'message'} = "Could not extract iid and mid from 'to' field of message with subject $subject from $from; skipping this insertion";
            $subject1 = "[Discussion not found] $subject";
            $message = "Your posting below has been rejected because we could not find a corresponding discussion.\n";
        }
        $message .= "Your message will not be posted on the web.\n";
        $message .= "Thank you,\n\nthe Administration\n\n-------------------------------------------------\n";
        $message .= $content;
        # can only send a bounce message if we have an email address
        if ($email) {
            my $message_id2 = $iid."+".$mid;
            my $sm = Sendmail->new($email, '', '',$subject1, $message);
            $sm->set_from($sender_email);
            $sm->set_message_id($message_id2);
            my ($result) = $sm->sendMail();
        }
        return undef;
    }

    if(!defined ($uid) || $email eq '' || $mid eq '') {
        $self->{'message'} = "Was not able to identify user from email $email";

        # can only send a bounce message if we have an email address
        if ($email) {
            # send "could not identify you" message
            my $subject1 = "[Discussion posting rejected] $subject";
            my $message = "Your posting below has been rejected because we could not identify your \nemail address as belonging to a registered user of this site. This \ncan happen when you reply from an email account to which your discussion \nmessages are forwarded.\n";
            $message .= "Your message will not be posted on the web.\n";
            $message .= "Thank you,\n\nthe Administration\n\n-------------------------------------------------\n";
            $message .= $content;
            my $message_id2 = $iid."+".$mid;
            my $sm = Sendmail->new($email, '', '',$subject1, $message);
            $sm->set_from($sender_email);
            $sm->set_message_id($message_id2);
            my ($result) = $sm->sendMail();
        }
        return undef;
    }

    use Metadot::UserFactory;
    my $user = Metadot::UserFactory->get_user($uid);
    
    my $disc;
    if( $exists1 ) {
	# restore the discussion so that we can verify posting rights to it
	$disc = Gizmo->restore( $iid );
	
#	use Data::Dumper;
#	logMessage( "USER IS:" . Dumper $user );
#	logMessage( "DISCUSSION IS:" . Dumper $disc );

	unless( $disc->is_allowed_to_do( 'send', $user ) ) {
	    
	    # send email rejecting posting
	    my $subject1 = "[Discussion posting rejected] $subject";
	    my $message = "Your posting below has been rejected because you don't have posting\nrights to the discussion you replied to.\n\nPlease ask the discussion owner to grant you posting rights.\n";
	    $message .= "Your message will not be posted on the web.\n";
	    $message .= "Thank you,\n\nthe Administration\n\n-------------------------------------------------\n";
	    $message .= $content;
	    my $message_id2 = $iid."+".$mid;
	    my $sm = Sendmail->new($email, '', '',$subject1, $message);
	    $sm->set_from($sender_email);
	    $sm->set_message_id($message_id2);
	    my ($result) = $sm->sendMail();
	    
	    return;
	}	  
    }

    if(($exists > 0 || $mid==0) && ($exists1 > 0)) {

        my $db_now = Utility->db_now();
        my $channelname = $disc->get_channel_name();
        $subject =~ s/\[$channelname\]\s*//;## remove channel name from subject
        
        Message->add_message(
            userid           => $uid,
            authorname 	     => $user->get_display_name(),
            authoremail	     => $email,
            iid       	     => $iid,
            parentmid        => $mid,
            name       	     => $subject,
            description	     => $content,
            );
        
        #create channel for new replied message
        my ($cid, $version) = DBAccess->sqlSelect("cid, version", "channel", "iid=$iid");
    
        DBAccess->sqlUpdate ("channel", "cid=$cid", (updatedate => $db_now));
    
        #aju 08-23-01: not clear why the following was needed-- normal channels provide notifications.
        #               &sendMailToGrpMembers($from, $subject, $date, $content, $iid, $new_mid, $email);
    } else { ##iid does not exist
        my $subject1 = "Original message has been removed!";
        my $message = "The message \"".$subject."\" you replied to has been removed.\n";
        $message .= "Your message will not be posted on the web.\n";
        $message .= "Thank you!.\n";
        my $message_id2 = $iid."+".$mid;
        my $sm = Sendmail->new($email, '', '',$subject1, $message);
        $sm->set_from($sender_email);
        $sm->set_message_id($message_id2);
        my ($result) = $sm->sendMail();
    }
}


# specifies the domain that Discussion email aliases belong to. This setting is defined in PARAMS or the 
# default is defined in another method, get_default_alias_domain
sub get_alias_domain {
    my $self = shift;
    my $result = $PARAMS->{email_alias_base_domain} || $self->get_default_alias_domain();
    return $result;
}


# specifies the domain that Discussion email aliases belong to. The default can be specified by MetadotConfig,
# otherwise it is generated from the server name
sub get_default_alias_domain {
    my $self = shift;
    my $result = MetadotConfig->get_param_value('email_alias_base_domain') || 'discussions.'.MetadotConfig->get_server_name();
    return $result;
}


# parses a fully-qualified email address, and returns the server name and account name
sub parse_full_alias {
    my $self = shift;
    my $full_alias = shift || die("Must specify a fully-qualified email address");
    my ($alias, $domain) = split('@', $full_alias);

    my @server_names = @{$self->get_server_names()};

    print STDERR "parse_full_alias of '$full_alias', against server names:\n@server_names\n";
    # try to find the best tail-end match. the best match is the longest server name that
    # actually does have a tail-end match.
    my $server_name = '';

    # if only one server name exists, don't bother matching. Assume this is the correct server.
    if ((scalar(@server_names) == 1) && $server_names[0] == _LOCALHOST_ ) {
        $server_name = $server_names[0];
    } else {
        foreach my $server (@server_names) {
            if (($domain =~ m/$server$/i) && (length($server) > length($server_name))) {
                $server_name = $server;
            }
        }
    }

    print STDERR "Parser discovered server name '$server_name'\n";
    return ($alias, $server_name);
}



sub get_server_names {
    my $self = shift;
    return $self->{server_names};
}

sub process_receivers {
    my $self = shift;
    my $receiver_string = shift;

    print STDERR "\nprocess_receivers() receiver_string: '$receiver_string'\n";
    for (split /,/, $receiver_string) {
        my ($iid, $mid, $server_name_to_process, $alias);
        if (($_ =~ m/^.*\+(.*)\+(\d+)\+(\d+)/)) {
            $server_name_to_process = $1;
            $iid = $2;
            $mid = $3;                    
        } elsif (($_ =~ m/^\s*(.*)\s*$/)) {
            # look for an alias
            $alias = $1;
            # strip away < and >
            $alias =~ s/^.*<//;
            $alias =~ s/>.*$//;
            ($alias, $server_name_to_process) = $self->parse_full_alias($alias);
            print STDERR "\nprocess_receivers() found alias: '$alias', and discovered server '$server_name_to_process'\n";
        }
        if ($server_name_to_process) {
            ## check if alias unique - alias already processed
            next if ($alias && $self->{unique_alias}->{$server_name_to_process}->{$alias});
            $self->{receivers}->{$server_name_to_process} ||= [];
            push @{$self->{receivers}->{$server_name_to_process}}, { iid => $iid, mid => $mid, alias => $alias };
            $alias && $self->{unique_alias}->{$server_name_to_process}->{$alias}++;
        }
    }
}



1;
