package Metadot::MetadotController;
use strict;
use Data::Dumper;

use Metadotclass;
use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CACHE $ACCESS_BROKER $DISPLAY %REQUEST_VARS );
use Portal;
use Metadot::SkinManager;
use Metadot::RequestContext;
use Template;

my $SINGLETON = undef;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = $SINGLETON;
    unless ($self) {
        $self = {};
        bless $self, $class;
        $SINGLETON = $self;
    }
    return $self;
}

sub test {
    return "This is the MetadotController";
}

use constant _FAILSAFE_  => '_failsafe';


my %MAPPING = (
    _FAILSAFE_ => 'generic.tt2',
    Calendar => {
        mappings => {
            show_full       => 'calendar.tt2',
            show_summary    => 'calendar_small.tt2',
        },
        default_op => 'show',
    },
    Category => {
        mappings => {
            show_home               => 'portal_home.tt2',
            show_sub                => 'portal_sub.tt2',
            show_full               => 'portal_chooser.tt2',
            show_summary            => 'category_summary.tt2',
            show_skin_layout        => 'layout_chooser.tt2',
            show_printer_friendly   => 'category.tt2',
            show_printer_friendly_layout  => 'layout_chooser.tt2',
        },
        default_op => 'show',
    },
    CMSDocument => {
        mappings => {
            show_full       => 'document.tt2',
            show_summary    => 'document_summary.tt2',
        },
        default_op => 'show',
    },
    Contractor => {
        mappings => {
            show_full       => 'organization.tt2',
            show_summary    => 'organization.tt2',
        },
        default_op => 'show',
    },
    ContactUs => {
        mappings => {
            show_full       => 'contact_us.tt2',
            show_summary    => 'contact_us.tt2',
        },
        default_op => 'show',
    },
    Discussion => {
        mappings => {
            show_full       => 'discussion.tt2',
            show_summary    => 'discussion_summary.tt2',
        },
        default_op => 'show',
    },
    Event => {
        mappings => {
            show_full       => 'event.tt2',
        },
        default_op => 'show',
    },
    Faq => {
        mappings => {
            show_full       => 'faq.tt2',
            show_summary    => 'faq.tt2',
        },
        default_op => 'show',
    },
    FormMaker => {
        mappings => {
            show_full       => 'form_maker.tt2',
            show_summary    => 'form_maker.tt2',
        },
        default_op => 'show',
    },
    GizmoShortcut => {
        mappings => {
            show_full       => 'gizmoshortcut.tt2',
            show_summary    => 'gizmoshortcut.tt2',
        },
        default_op => 'show',
    },
    ImageItem => {
        mappings => {
            show_full       => 'image_item.tt2',
            show_summary    => 'image_item.tt2',
        },
        default_op => 'show',
    },
    Item => {
        mappings => {
            show_full       => 'item_full.tt2',
            show_summary    => 'item.tt2',
        },
        default_op => 'show',
    },
    JobRequest => {
        mappings => {
            show_full       => 'job_request.tt2',
            show_summary    => 'job_request.tt2',
        },
        default_op => 'show',
    },
    LayoutArea => {
        mappings => {
            show_full       => 'layout_area.tt2',
            show_summary    => 'layout_area.tt2',
        },
        default_op => 'show',
    },
    Message => {
        mappings => {
            show_full       => 'message.tt2',
        },
        default_op => 'show',
    },
    Newsitem => {
        mappings => {
            show_full       => 'newsitem_full.tt2',
            show_summary    => 'newsitem.tt2',
        },
        default_op => 'show',
    },
    NewsArticleImage => {
        mappings => {
            show_full       => 'newsarticleimage_full.tt2',
            show_summary    => 'newsarticleimage.tt2',
        },
        default_op => 'show',
    },
    NewsPaperView => {
        mappings => {
            show_full       => 'scrollview.tt2',
            show_summary    => 'scrollview.tt2',
        },
        default_op => 'show',
    },
    Poll => {
        mappings => {
            show_full       => 'poll.tt2',
            show_summary    => 'poll.tt2',
        },
        default_op => 'show',
    },
    Separator => {
        mappings => {
            show_full       => 'section.tt2',
            show_summary    => 'section.tt2',
        },
        default_op => 'show',
    },
    Session => {
        mappings => {
            login           => 'login.tt2',
        },
        default_op => 'login',
    },
    SiteLibrary => {
        mappings => {
            show_full       => 'site_library.tt2',
            show_summary    => 'site_library.tt2',
        },
        default_op => 'show',
    },
    SiteMap => {
        mappings => {
            show_full       => 'sitemap.tt2',
            show_summary    => 'sitemap.tt2',
        },
        default_op => 'show',
    },
    SubmissionSystem => {
        mappings => {
            show_full       => 'submission_system.tt2',
            show_summary    => 'submission_system_summary.tt2',
        },
        default_op => 'show',
    },
    SubmissionItem=> {
        mappings => {
            show_full       => 'submission_item.tt2',
            show_summary    => 'submission_item_summary.tt2',
        },
        default_op => 'show',
    },
    Table => {
        mappings => {
            show_full       => 'table.tt2',
            show_summary    => 'table_summary.tt2',
        },
        default_op => 'show',
    },
    Task => {
        mappings => {
            show_full       => 'task.tt2',
        },
        default_op => 'show',
    },
    TaskList => {
        mappings => {
            show_full       => 'task_list.tt2',
            show_summary    => 'task_list_summary.tt2',
        },
        default_op => 'show',
    },
);



# checks a form parameter 'disable_skins', and returns true if present
sub is_disabled {
    my $class = shift;
    my $session_key = 'disable_skins';
    
    my $result = $FORM{$session_key};
    if (defined $result) {
        $SESSION->set_value($session_key, $result);
    } else {
        $result = $SESSION->get_value($session_key);
    }
    return $result;
}


# checks a form parameter 'force_default_skin', and returns true if present
sub force_default_skin {
    my $class = shift;
    my $session_key = 'force_default_skin';
    
    my $result = $FORM{$session_key};
    if (defined $result) {
        $SESSION->set_value($session_key, $result);
    } else {
        $result = $SESSION->get_value($session_key);
    }
    #print STDERR "force_default_skin? '$result'\n";
    return $result;
}


sub do {
    my $class = shift;
    my $context = shift || die("Must supply the application context");

    if ($class->is_disabled()) {
        return undef;
    }

    my $subject = $context->get_subject();
    return undef unless (defined $subject);

    return if $class->detect_deep_recursion($subject);

    my $op      = $context->get_op();
    my $isa     = $subject->is_a();
    my $user    = $context->get_user();
    my $result;

    Metadotclass->debug_msg(4,"MetadotController\->do with subject: $subject ($subject->{id}), op: $op, isa: $isa, user: $user\n");

    # check permissions
    if ($subject->is_allowed_to_do($op, $user, 1)) {
        Metadotclass->debug_msg(5, "Metadotcontroller confirmed permission.");
        my $subject_class = ref($subject);
        my $default_op  = $MAPPING{$subject_class}{default_op};
        my $exec_base   = 'exec_';
    
        # if no op was defined, use the default op
        if (!defined $op) {
            $op = $default_op;
        }
    
        my $exec_method = $exec_base.$op;
        my $exec_response;
        my $mapping;
        
        Metadotclass->debug_msg(5, "MetadotController tries to exec '$exec_method'");

        # now try to run the exec method
        if ($subject->can($exec_method)) {
            $exec_response = $subject->$exec_method($context);
            Metadotclass->debug_msg(5, "MetadotController got exec response '$exec_response'");
            
            if (defined $exec_response) {
                # see if the response is a Context object
                if (ref($exec_response) && $exec_response->isa('Metadot::RequestContext')) {
                    Metadotclass->debug_msg(5, "Will execute $exec_response");
                    $result = $class->do($exec_response);
                } elsif ($exec_response =~ /\.tt2$/) {
                    # the response is a direct reference to a tt2 file
                    $mapping = $exec_response;
                } else {
                    $mapping = $MAPPING{$subject_class}{mappings}{$exec_response};
                }
            }
                
            Metadotclass->debug_msg(5, "got mapping $mapping");
    
            # finally, map to the generic one if the mapping still is not found.
            # some classes may actually send direct HTML as a response, so check
            # for that first.
            unless ($result || $mapping || 
                      ( $subject_class->can('response_may_contain_content') 
                        && $subject_class->response_may_contain_content()
                       )
                    ) {
                $mapping = $MAPPING{_FAILSAFE_};
            }
        } else {
            #BACKWARDS COMPATIBILITY WITH NON-TEMPLATE ENABLED GIZMOS
            if ($subject->isa('Gizmo')) {
                Metadotclass->debug_msg(4, "Attempted backward compatibility mode");
                if($subject->handle($op, 1)) {
                    $result = "<!-- MetadotController passed $subject/$op to handler -->";
                    Metadotclass->debug_msg(4, "Attempted $subject\->handle($op), result: ".($result ? 1 : 0));
                } elsif (!$result && $subject->can($op)) {
                    $result = $subject->$op();
                    Metadotclass->debug_msg(4, "Attempted $subject\->$op(), result: ".($result ? 1 : 0));
                }
            }
        }
    
        Metadotclass->debug_msg(3, "Using method  '$subject_class\->$exec_method', got response '$exec_response' and mapping '$mapping'\n");
    
        # process the mapping
        unless ($result) {
            if ($mapping) {
                # currently, only .tt2 mapping is specifically supported
                $result = $class->view({context=>$context, template=>$mapping});
            } elsif ($exec_response) {
                # all other return values assume that the template source was provided
                $result = $class->view({context=>$context, template_source=>$exec_response});
            }
        }
    } else {
        print STDERR "Operation not allowed\n";
    }    
    return $result;
}


# stores template objects in a cache. all this will be moved to a Viewer class
#
# send the args:
#   Metadot::RequestContext  context            : specifies the context of the request
#   filename                 template           : specifies the template filename to process
#   string                   template_source    : specifies the template source to process

my $TEMPLATE_CACHE = {};
sub view {
    my $class = shift;
    my $args  = shift;
    
    if ($class->is_disabled()) {
        return undef;
    }

    my $context = $args->{context} || Metadot::RequestContext->new();
    my $subject = $context->get_subject();
    
    Metadotclass->debug_msg(4, "MetadotController view has template: '$args->{template}'; template_source: '$args->{template_source}'; context: '$context'; subject: '$subject'");

    return if $class->detect_deep_recursion($subject);
    
    # either send argument template, to refer to an existing template
    # or send string template_source, to provide the source code of the template
    my $template_source;
    if ($args->{template}) {
        $template_source = $args->{template};
    } elsif ($args->{template_source}) {
        $template_source = \$args->{template_source};
    } else {
        return '';
    }

    my @INCLUDE_PATH = Metadot::SkinManager->get_template_include_path();

    Metadotclass->debug_msg(4, "Template include path='@INCLUDE_PATH'");

    # check if the template object can be found in cache.
    my $template = $TEMPLATE_CACHE->{"@INCLUDE_PATH"};
    #print STDERR "Got from template from cache? $template\n";

    unless ($template) {
        $template = Template->new({
                      INCLUDE_PATH  => \@INCLUDE_PATH,
                      PRE_CHOMP     => 1,
                      POST_CHOMP    => 1,
                      RECURSION     => 1,
                     }) || die $Template::ERROR, "\n";
        $TEMPLATE_CACHE->{"@INCLUDE_PATH"} = $template;
        #print STDERR "Created template instance $template\n";
    }

    my $data = {
        self            => $subject,
        portal_context  => $context,
        portal_factory  => Metadot::PortalFactory->new(),
        portal_params   => $PARAMS,
        display         => $DISPLAY,
        user            => $context->get_user(),
        form            => $context->get_form(),
        session         => $context->get_session(),
        skin_manager    => new Metadot::SkinManager,
        request_vars    => \%REQUEST_VARS,
    };

    #print STDERR Dumper $subject;
    #print STDERR Dumper $data;

    my $out;
    my $template_process;
    if ($template->process($template_source,  $data, \$template_process)) {
        # for Category instances, do post_processing of user templates, unless the op is 'modify*'
        my $op = $context->get_op();
        my $subject = $context->get_subject();
        
        if ($subject && $subject->can('processes_user_directives') && $subject->processes_user_directives() && ($op !~ '^(modify|save)')) {
            $class->process_user_tags(
                input   => \$template_process,
                output  => \$out,
                context => $context,
            );
        } else {
            $out = $template_process;
        }
        return $out."\n";
    } elsif ($template->error()) {
        $class->handle_view_error($template);
    }
}


# process user tags
my $USER_TEMPLATE_PROCESSOR = Template->new({}) || die("Could not instantiate user template processor:\n", $Template::ERROR);
sub process_user_tags {
    my $class = shift;
    my %args = @_;

    my $input   = $args{input}      || die("Must specify input reference");
    my $output  = $args{output}     || die("Must specify output reference");
    my $context = $args{context};

    # if there are no tags, then don't bother.
    if ($$input =~ /\[%/) {
        
        # unescape text within template tags
        my @tags = ($$input =~ /\[%.*?%\]/g);
        foreach my $tag (@tags) {
            my $replace = $tag;
            $replace =~ s/&quot;/"/g;
            $replace =~ s/&lt;/</g;
            $replace =~ s/&gt;/>/g;

            $$input =~ s/\Q$tag\E/$replace/g;
        }

        # build some data structures for users to access
        my %data = ();

        # create some read-only representations (data transfer hashes) of context objects
        if ($context && ref($context)) {
            my (%user_dth, %form_dth, %self_dth);

            if ($context->get_user()) {
                %user_dth    = %{$context->get_user()};
                # password is a public field, but we don't want people to 
                # have access to it. Even though it is hashed, it can
                # be broken by brute force.
                delete $user_dth{password};
            }
            if ($context->get_form()) {
                %form_dth    = %{$context->get_form()};
            }
            if ($context->get_subject && ref($context->get_subject())) {
                %self_dth    = %{$context->get_subject()};
            }

            $data{user}     = \%user_dth;
            $data{form}     = \%form_dth;
            $data{self}     = \%self_dth;
        }    
        $data{display}       = $DISPLAY;

        unless ($USER_TEMPLATE_PROCESSOR->process($input, \%data, $output)) {
            $class->handle_view_error($USER_TEMPLATE_PROCESSOR  );
            $$output = $$input;
        }
    } else {
        # just pass the output
        $$output = $$input;
    }

}




# this will eventually be moved to Viewer->handle_error
sub handle_view_error {
    my $class = shift;
    my $template = shift;

    my $error = "An error has occurred while processing the template:<br/>\n".$template->error();

    Metadotclass->debug_msg(0, $error);

    return undef;
}


sub detect_deep_recursion {
    my $class = shift;
    my $subject = shift || return();
    my $limit = shift || 25;

    # avoid deep recursion
    my $run_counter = "$class\_RUNTIME_COUNTER";
    $REQUEST_VARS{$run_counter}->{$subject} = $REQUEST_VARS{$run_counter}->{$subject} + 1;
    return ($REQUEST_VARS{$run_counter}->{$subject} > $limit);
}


1;
