package Metadot::PortalFactory;
use strict;
use Data::Dumper;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CACHE $ACCESS_BROKER $DISPLAY);
use Portal;
use Metadot::MetadotController;
use Metadot::RequestContext;
use Metadot::AuthenticatorFactory;
use MetadotConfig;
use Metadot::UserFactory;
use Gizmo;


my $SINGLETON = undef;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = $SINGLETON;
    unless ($self) {
        $self = {};
        bless $self, $class;
        $SINGLETON = $self;
    }
    return $self;
}

sub test {
    return "This is a PortalFactory";
}

sub get_user {
    my $class  = shift;
    my $userid = shift;

    my $result;
    if (defined $userid) {
        $result = $USER->new($userid);
    } else {
        $result = $USER;
    }

    #print STDERR "PortalFactory gets user with uid '$userid' $result (".Dumper($result).")\n";
    return $result;
}

sub get_display {
    return $DISPLAY;
}

sub get_controller {
    my $controller_class = MetadotConfig->get_param_value('controller_type') || 'Metadot::MetadotController';
    return $controller_class->new();
}

sub build_context {
    my $class = shift;
    my $args = shift;

    my $context = Metadot::RequestContext->new( properties => $args);
    return $context;
}

sub get_home_object {
    my $class = shift;
    my $iid = MetadotConfig->get_reserved_iid('ROOT_CATEGORY');
    Metadotclass->debug_msg(5, "$class\->get_home_object seeks object using iid '$iid'");
    my $object = Gizmo->restore($iid);
    Metadotclass->debug_msg(5, "get_home_object finds $object with iid '".$object->id()."'");

    return $object;
}

sub get_hidden_category {
    my $class = shift;
    my $object = Gizmo->restore(MetadotConfig->get_reserved_iid('HIDDEN_CATEGORY'));
    return $object;
}

sub get_parent_of_root {
    my $class = shift;
    my $object = Gizmo->restore(MetadotConfig->get_reserved_iid('PARENT_OF_ROOT'));
    return $object;
}

sub get_portal_parent {
    my $class = shift;
    my $object = Gizmo->restore(MetadotConfig->get_reserved_iid('PORTAL_PARENT'));
    return $object;
}

sub get_basic_tool_parent {
    my $class = shift;
    my $object = Gizmo->restore(MetadotConfig->get_reserved_iid('PARENT_BASIC_TOOL'));
    return $object;
}

sub get_clipboard_parent {
    my $class = shift;
    my $object = Gizmo->restore(MetadotConfig->get_reserved_iid('PARENT_CLIPBOARD'));
    return $object;
}


sub get_gizmo_object {
    my $class = shift;
    my $iid = shift;
    my $object = Gizmo->restore($iid);
    return $object;
}

sub get_utility_object {
    return Utility->new();
}

sub get_site_library {
    return SiteLibrary->get_instance();
}

sub get_authenticator {
    return Metadot::AuthenticatorFactory->get_authenticator();
}


1;
