###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Register;
use Metadotclass;
use strict;
use DBAccess;
use MetadotConfig;


use vars qw(@ISA);
@ISA=qw(Auditable Metadotclass);
use DBAccess;

#CKIM-ADMIN-NOTIFY
use Metadot qw ($PARAMS $USER);

sub is_a {
	my $self = shift;

	return ref($self);
}

sub id {
	my $self = shift;

	return 1;
}

sub save {
	my $self = shift;

	$self->save_permissions;
}

sub get_owner_uid {
	my $self = shift;

	return MetadotConfig->get_reserved_uid('ADMIN');
}

sub parent_id {
	my $self = shift;
       # my $id = $user->parent_id ;
	
	return 0;
}

sub new {
    my $proto = shift;
    my $class = __PACKAGE__;
    my $self = {};

    bless ($self, $class);
    return $self;
}


sub www_register_form {
    die "abstract method, must override\n";
}       

sub www_register {
    die "abstract method, must override\n";
}

sub www_show_params {
    die "abstract method, must override\n";
}

sub www_edit_params {
    die "abstract method, must override\n";
}

sub www_save_params {
    die "abstract method, must override\n";
}

sub save_modify_settings_form {
    die "abstract method, must override\n";
}

sub show_modify_settings_form {
    die "abstract method, must override\n";
}

sub send_approval_message {
    die "abstract method, must override\n";
}

sub send_new_user_notification {
    my $self = shift;
    my $user = shift;

    # don't even bother if this setting is turned off    
    return() unless $PARAMS->{'new_user_notification_enabled'};
    # aju - recurse on sub groups
    my @admin_list = @{Group->get_emails_by_grp_name('SITE_MGR', 1)};
#      my @admin_list = @{Group->get_emails_by_grp_name('ADMIN')};
    
    if (@admin_list) {
            @admin_list = @{Utility->filter_valid_emails(@admin_list)};
            my $to = join ',', @admin_list;
            my $cc = "";
            my $bcc = "";
            my $useremail= $user->email;
            my $fullname = $user->fullname;
            my $nickname = $user->nickname || $user->webname;
            
            my $subject = "New user awaiting approval";
            my $urgent = 0;
            my $mesg = $PARAMS->{'new_user_notification_message'};
        
            my $basename = $PARAMS->{basename};
            my $link_text = "$basename/index.pl?iid=0&amp;isa=User&amp;op=show_users&amp;pending=true";
            $mesg =~ s/<<new_user_link>>/$link_text/g;
            $mesg = $USER->substitute($mesg, $fullname, $useremail, '', $nickname);
            
            #print STDERR "CKIM: Register-> sending new user notifications to $to\n";
            if ($to ne '') {
                my $sm = Sendmail->new($to, $cc, $bcc, $subject, $mesg);
                $sm->sendMail();
            }
    }
}


1;


=head1 NAME

Register - 

=head1 SYNOPSIS

    use Metadot::Register
    $register  = $class->SUPER::new();
    $register->send_new_user_notification();
    $register->www_register_form 
    $register->www_register
    $register->www_show_params
    $register->www_edit_params
    $register->www_save_params
    $register->save_modify_settings_form
    $register->show_modify_settings_form
    $register->send_approval_message
    $register->is_a
    $register->id
    $register->save
    $register->get_owner_uid
    $register->parent_id
    
    
=head1 DESCRIPTION

Metadot::Register is an abstract class.  Subclasses of Register are instantiated and returned
by Metadot::RegisterFactory.  Methods are classified as being part of those required by the
Auditable interface, abstract methods, and the send_new_user_notification method

Required by the auditable interface

    $register->is_a
    $register->id
    $register->save
    $register->get_owner_uid
    $register->parent_id

    
Abstract methods

Show the form that people use to register on the site
    $register->www_register_form 
    
Process the register form

    $register->www_register
    
Show the registration parameters

    $register->www_show_params
    
Form to edit the registration paramaters
    
    $register->www_edit_params
    
Code to process the changed parameters

    $register->www_save_params
    
Code to process the changed registration parameters    

    $register->save_modify_settings_form
    
Form to edit the registration parameters
    
    $register->show_modify_settings_form

Sends an approval message to the new user

    $register->send_approval_message


Sends an email to every member of the admin group, notifying them that a new user has
registered, and is awaiting approval
    $register->sent_new_user_notification

=cut

