###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Register::FlexRegister;
use strict;

use Data::Dumper;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CACHE $ACCESS_BROKER );
use MetadotConfig;
use CGI qw(escape escapeHTML);
use Group;
use Metadot::DataCollection;
use Metadot::UserFactory;
use Metadot::Register::Default;
use Metadot::AuthenticatorFactory;

use vars qw(@ISA);
@ISA=qw(Metadot::Register::Default);


sub www_register_form {
    my $self = shift;

    if (!$USER->is_admin() && !$FORM{admin_mode} ) {    
        $USER->logout();
    }       

    if( $USER->max_number_of_users_quota_exceeded() ) {
  
	$DISPLAY->print_click_back( 'User registration forbidden. Maximum number of users has been reached.' );
	exit;
    }

    if( $PARAMS->{registration_closed} 
        && ! $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) 
        && ! $ACCESS_BROKER->is_allowed_to_do( 'register_users' ) 
        ){

	$DISPLAY->print_click_back( 'User registration forbidden. Only Site Managers and the Admin can register new users.' );
	exit;

    }


    my $u = Metadot::UserFactory->get_user();
    my $config = $u->get_fields_config();
    
    # If the user is being registered directly into a group, and that
    # group is an organization, set the default organization value
    my $enroll_groups_form_field = $FORM{enroll_groups};
    if ($enroll_groups_form_field) {
       my @enroll_group_list;
       if (ref($enroll_groups_form_field) eq 'ARRAY') {
           @enroll_group_list = @$$enroll_groups_form_field;
       } else {
           @enroll_group_list = ($enroll_groups_form_field);
       }
       foreach my $grp_id (@enroll_group_list) {
           Metadotclass->debug_msg(5, "Is group '$grp_id' an org?");           
           if (Group->is_group_an_organization($grp_id)) {
               my $field_name = $u->get_organization_field_name();
               $u->set_default_value($field_name, $grp_id);
               Metadotclass->debug_msg(5, "Yes, it is. Set the default value of '$field_name' to '$grp_id'.");           
           }
       }
    }

    my @sets;
    my $enable_param;
    foreach my $field ( @{ $config->{FIELDS} } ) {	
        if (defined $field->{access_op}) {
            next unless ($ACCESS_BROKER->is_allowed_to_do($field->{access_op}));
        }
        $enable_param = $field->{enable_param};
        next if (defined($enable_param) && !$PARAMS->{$enable_param});
        $sets[$field->{display_set}] .= $self->render_registration_control(
            user    => $u,
            field   => $field,
            value   => $u->get_default_value($field->{field_name}),
        );    
	}

    my $set_num = 0;
    foreach my $set_name ( @{ $config->{DISPLAY_SETS} } ) {
        next unless ($sets[$set_num]);
    	my $set_html = <<EOM
<table width="450">
    <tr>
        <td colspan='2'>
            <h2>$set_name</h2>
        </td>
    </tr>
EOM
;

    	$set_html .= $sets[$set_num];
    	$set_html .= "</table>";
        	  	
    	$sets[$set_num] = $set_html;
    
    	$set_num++;	
    }
    
    my $form = $DISPLAY->popup_window() || '';
    my $refererlink = $FORM{refererlink} || $SESSION->referer() || 'index.pl';
    $refererlink = Utility->get_qualified_url($refererlink);

    my $auth = Metadot::AuthenticatorFactory->get_authenticator();
    ## will set form parameters depending on site's login mode
    my ($form_action, $login_mode, $submessage) = $auth->get_password_form_parameters(login_mode => $FORM{login_mode}, form_type => 'register', refererlink => $refererlink);
    
    $form .= <<EOM
<FORM action="$form_action"  name="profile_form" method=post enctype="multipart/form-data">
    <INPUT type="hidden" name="isa" value="Register">
    <INPUT type="hidden" name="op" value="register">
    <INPUT type="hidden" name="this_op" value="$FORM{'op'}">
    <INPUT type="hidden" name="this_isa" value="$FORM{'isa'}">
    <INPUT type="hidden" name="this_iid" value="$FORM{'iid'}">
    <INPUT type="hidden" name="admin_mode" value="$FORM{'admin_mode'}">
    <INPUT type="hidden" name="refererlink" value="$refererlink">
    <INPUT type="hidden" name="enroll_groups" value="$FORM{enroll_groups}">

<table>
  <tr>
    <td><b>Register as a New User</b><br>
       <font face=\"Arial, Helvetica, sans-serif\" size=\"-1\">A red star
        (<font size=\"+1\"><b><font color=\"#ff0000\">*</font></b></font>)
         in front of a field indicates a mandatory entry.</font>
EOM
;

    if( $PARAMS->{regapproval} ) {
	$form .= <<EOM
    <font color='red'>* This system requires administrator approval for new registrations.
     You will receive notification via email when your registration has been approved.</font>
EOM
;          
    }
    
    my $i = 0;
    foreach my $set_html (@sets) {

	my $frame = new HTML::TableFrame("", "#ffffff");
	$frame->add($set_html);
	$form .= $frame->print;
	$form .= "
    </td>
  </tr>";
    
	unless ( $i == $#sets ) {
	    # we don't add this after the last set
	    $form .= "
  <tr><td><br></td></tr>
  <tr><td>";
	}
	$i++;
    }

    # the administrative user can directly modify the admin properties of a new user
    if( $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $USER ) ){
        my $admin_frame = $self->render_admin_frame($u);
        if ($admin_frame) {
            $form .= "<tr><td><br></td></tr>";	
            $form .= "<tr><td>";
            $form .= $admin_frame;
            $form .= "</td></tr>\n";
        }
    }    

    my $button_name = $login_mode eq 'standard' ? "Register" : "Register Securely";
    $form .= "
  <tr>
    <td align='right'>
      <br><INPUT type=submit name=action value=\"$button_name\">
    </td>
  </tr>
</table>
</FORM>";
    $form .= "<p>" . $submessage;

    $DISPLAY->print_form($form, "Registration");
}


#
# Notw that many of these 
#
sub www_register {
    my $self = shift;
    my %args = @_;

    if( $USER->max_number_of_users_quota_exceeded() ) {
        $DISPLAY->print_click_back( 'User registration forbidden. Maximum number of users has been reached.' );
        exit;
    }
    
    if( $PARAMS->{registration_closed} 
        && ! $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) 
        && ! $ACCESS_BROKER->is_allowed_to_do( 'register_users' ) 
        ){

	$DISPLAY->print_click_back( 'User registration forbidden. Only Administrators can register new users.' );
	exit;

    }

    my $config = $USER->get_fields_config();
    
    unless( $FORM{nickname} ) {
        $FORM{nickname} = $FORM{first_name};	
    }
    
    my $verify = Form::Verify->new();
    
    # verify proper format of form values
    foreach my $field ( @{ $config->{FIELDS} } ) {
        if (defined $field->{access_op}) {
            next unless ($ACCESS_BROKER->is_allowed_to_do($field->{access_op}));
        }
        if( $FORM{ $field->{field_name} } || $field->{is_required} ) {
            # if the field is required or has been filled
            
            my $verify_type = $field->{verify_type} || 'non_empty';
        
            unless( defined $field->{field_type} && $field->{field_type} eq 'password' ) {
                $verify->verify( $verify_type, 
                                 $field->{description},
                                 $FORM{ $field->{field_name} },
                                 $field->{date_format} );
            } else {
                if($PARAMS->{password_checker} == 1){
                    $verify->verify( "password", 
                                     $field->{description}, 
                                     $FORM{ $field->{field_name} }, 
                                     "", $FORM{first_name},
                                     $FORM{nickname} );
                } elsif ($PARAMS->{password_checker} == 5){
                    $verify->verify( "password_long", 
                                     $field->{description}, 
                                     $FORM{ $field->{field_name} }, 
                                     "", $FORM{first_name},
                                     $FORM{nickname} );
                } else {
                    $verify->verify( "non_empty",
                                     $field->{description}, 
                                     $FORM{ $field->{field_name} }, 
                                     "" );		    
                }
                
                if ( $FORM{ $field->{field_name} } ne $FORM{ 'verify_' . $field->{field_name} } ) {
                    $DISPLAY->print_click_back("Password verification error", "Your password fields didn't match!");
                    exit;
                }
            }
        }
    }
    
    if ($verify->errors) {
	my $error_addendum = $self->_get_error_addendum();
#  	print STDERR "***got error  addendum: $error_addendum\n";
        if($PARAMS->{password_checker} > 0) {
            $DISPLAY->print_click_back("Sorry", "The following field(s) were invalid:<br> " . $verify->errors
                .$self->_password_error_addendum($verify->errors). $error_addendum);
            exit;
        }
        $DISPLAY->print_click_back("Sorry", "The following field(s) were invalid:<br> " . $verify->errors
				   . $error_addendum);
        exit;
    }
    
    # Check if User Exists
    my $cnt = $USER->email_already_registered( $FORM{email} );
    
    unless($cnt == 0) {
        $DISPLAY->print_click_back(
            "Error",
            "A user already exists with that email address:<br /><br />"
            . $FORM{email} . "<br /><br />"
            . "If this is your email address then go ahead and \n"
            . "<a href="
            .$self->make_url(Metadot::AuthenticatorFactory->get_authenticator()->get_login_url("email=$FORM{email}"))
            . ">"
            . "Log in now!</a>!"
        );
        exit;
    }
    
    my $user = $self->create_user(%args);
    my $uid = $user->uid();

    # return admins back to user management
    my $refererlink = $FORM{refererlink} || $SESSION->referer();
    
    if (($USER->is_admin() || $ACCESS_BROKER->is_allowed_to_do('register_users')) && $FORM{admin_mode}) {
        
        # first process admin params if the user is an admin
        $self->_process_user_admin_params( $user );
        $user->save();

        return $self->post_admin_user_registration();
    } elsif((!$user->{approval_needed}) || ($user->{approval_needed} eq "accepted")) {
        $SESSION = Session->new();
        my $session_id = $SESSION->add($uid);
        $USER = Metadot::UserFactory->get_user($uid);
        #using new persistence mechanism
        my $session_handler = Metadot::SessionHandlerFactory->get_session_handler();
        $session_handler->persist_session($SESSION);
        #end of using new persistence mechanism
        ## we refresh his session
        $user->last_online(time);
        
        my $name = $user->webname;
        my $hpiid = $user->has_portal;
        
        # send email approval notification
        $self->send_approval_message($user, "auto");
        my $refererlink = $FORM{refererlink} || $SESSION->referer() || 'index.pl';
        
        if( $hpiid ) {
            $DISPLAY->two_choices ("Success, $name, you are now registered.",
                "confirm",
                "Continue",
                Utility->get_qualified_url($refererlink),
                "You can visit your public website",
                Utility->get_qualified_url("index.pl?iid=$hpiid")
            );
            exit;
        } else {
            $DISPLAY->print_choices ("Success, $name, you are now registered.",
                "confirm",
                "Continue",
                Utility->get_qualified_url($refererlink),
            );
            exit;
        }
    } else {
    #CKIM-ADMIN-NOTIFY
        $self->send_new_user_notification($user);
        my $name = $user->webname;
        $DISPLAY->two_choices ("Success, $name, you are now registered.<br>This system requires site manager or admin approval for new registrations. You will receive notification via email when your registration has been approved",
            "readme",
            "Return to the front page",
            Utility->get_qualified_url("index.pl")
        );
        exit;
    }
}

sub post_admin_user_registration {
    my $self = shift;
    
    my $refererlink = $FORM{refererlink} || $SESSION->referer();
    $DISPLAY->two_choices ("You have successfully registered a new user.",
        "confirm",
        "You can register another user",
        "index.pl?isa=Register&amp;op=register_form&amp;admin_mode=true",
        "Finished registering users",
        "$refererlink"
    );
}

sub create_user {
    my $self = shift;
    my %args = @_;

    my $user = Metadot::UserFactory->get_user();
    
    my @groups_to_add_to = ();
    
    my $config = $USER->get_fields_config();
    
    foreach my $field ( @{ $config->{FIELDS} } ) {
        if (defined $field->{access_op}) {
            next unless ($ACCESS_BROKER->is_allowed_to_do($field->{access_op}));
        }
        my $value = ( $field->{can_contain_html} ) ? escapeHTML( $FORM{ $field->{field_name} } ) : $FORM{ $field->{field_name} };
    
        if( lc $field->{field_type} eq 'password' and $PARAMS->{encryption_done} eq 'yes') {
            $value = Portal->encrypt_password( $value );
        }
    
        unless ($field->{field_type} eq 'group' || $field->{field_type} eq 'checkbox_group') {
            $user->set_value( $field->{field_name}, $value );
        } else {
            push(@groups_to_add_to, split(",", $value)) if (defined $value && $value ne "");
        }
    }
    
    # process any custom field input
    $self->input_custom_fields(
        user    => $user,
    );
    
    # get registration approval setting from database
    
    if ( $PARAMS->{regapproval} ) {
        $user->{approval_needed} = "required";
    } else {
        #aju 09-27-01: if approval is not required, mark the new user's status as 'accepted'.
        #This addresses cases where a mail needs to go out to all registered
        #users (as in the Surveys).
        $user->{approval_needed} = "accepted";
    }
    
    ##########################
    
    my $uid = $user->add();
    
    # check if the register form included groups for enrollment
    my $form_enroll_groups = $FORM{enroll_groups};
    if ($form_enroll_groups) {
        if (ref($form_enroll_groups) && ref($form_enroll_groups) eq 'ARRAY') {
            push @groups_to_add_to, @$form_enroll_groups;
        } else {
            push @groups_to_add_to, $form_enroll_groups;
        }
    }


    # vj: add user to groups that are allowed to add to (for security reasons we should not allow users
    # to just add themselfs to arbitrary groups)
    my $allowed_group_collection = $self->get_allowed_group_collection();
    $allowed_group_collection->restart();
    while (my $grp = $allowed_group_collection->get_next()) {

        # ckim: this seems overly cautious? it's interfering with orphan delete function, when
        # creating a new user with enrolled groups
        # $grp->remove_member($uid);

        foreach my $grp_id (@groups_to_add_to) {
            # vj: this block of code is copied from Group::add_remove_group_members
            if ($grp_id == $grp->{grp_id} && !DBAccess->sqlSelect ("grp_id", "grpmembers", "grp_id=$grp_id and userid='$uid'")) {
                Group->_add_a_member_to_a_group($grp_id, $uid);
            }
        }
    }
    
    ##########################
    
    ($USER->create_mypage($uid)) || die "can't create mypage";

    return $user;
}

sub _get_error_addendum {
    my $self = shift;
    return "";
}

sub show_modify_settings_form {
    my $self = shift;
    my ($uid, $showadminlevel)=@_;
    
    my $action;
    my $op = "";
    
    if(!$uid){
    	$DISPLAY->print_click_back("Error", "Sorry I did not get an UID to work on!");
    	exit;
    }
    
    my $u = Metadot::UserFactory->get_user($uid);
    
    my $form_type = '';
    if($showadminlevel){
    	$action="Save User Settings";
    	$op = 'save';
        $form_type = 'profile_other';
    } else {
    	$action="Save Settings";
    	$op = 'save_self';
        $form_type = 'profile_self';
    }
        
    my $config = $u->get_fields_config();
    my @sets;

    my $user_management_is_external = MetadotConfig->get_param_value('user_management_is_external');   
    my $enable_param;
    unless( $user_management_is_external ) {
    	foreach my $field ( @{ $config->{FIELDS} } ) {    	
            if (defined $field->{access_op}) {
                next unless ($ACCESS_BROKER->is_allowed_to_do($field->{access_op}));
            }
            $enable_param = $field->{enable_param};
            next if (defined($enable_param) && !$PARAMS->{$enable_param});
            $sets[$field->{display_set}] .= $self->render_registration_control(
                user    => $u,
                field   => $field,
            );
        }	    
    }
	
    my $refererlink = $FORM{refererlink} || $SESSION->referer() || 'index.pl';
    $refererlink = Utility->get_qualified_url($refererlink);
	
    my $set_num = 0;
	foreach my $set_name ( @{ $config->{DISPLAY_SETS} } ) {
        next unless ($sets[$set_num]);
    	my $set_html = <<EOM
<table width="450">
    <tr>
        <td colspan='2'>
            <h2>$set_name</h2>
        </td>
    </tr>
EOM
;
	    $set_html .= $sets[$set_num];
	    $set_html .= "</table>";
	    
	    $sets[$set_num] = $set_html;
	    
	    $set_num++;	
	}
    
    my $auth = Metadot::AuthenticatorFactory->get_authenticator();
    ## will set form parameters depending on site's login mode
    my ($form_action, $login_mode, $submessage) = $auth->get_password_form_parameters(login_mode => $FORM{login_mode}, form_type => $form_type, refererlink => $refererlink);

    $SESSION->set_value('User_uid_to_modify', $uid);
    
    my  $form = $DISPLAY->popup_window();
    
    $form .= <<EOM
<FORM action="$form_action"  name="profile_form" method=post enctype="multipart/form-data">
    <INPUT type="hidden" name="isa" value="User">
    <INPUT type="hidden" name="op" value="$op">
    <INPUT type="hidden" name="refererlink" value="$refererlink">

<table>
  <tr>
    <td><b>Change Profile</b><br>
       <font face="Arial, Helvetica, sans-serif" size="-1">A red star
        (<font size="+1"><b><font color="#ff0000">*</font></b></font>)
         in front of a field indicates a mandatory entry.</font>
EOM
;    
    my $i = 0;

    unless( $user_management_is_external ) {
    	foreach my $set_html (@sets) {
    	    
    	    my $frame = new HTML::TableFrame("", "#ffffff");
    	    $frame->add($set_html);
    	    $form .= $frame->print;
    	    $form .= "
    </td>
  </tr>";
	    
    	    unless ( $i == $#sets ) {
    		# we don't add this after the last set
    		$form .= "
  <tr><td><br></td></tr>
  <tr><td>";
    	    }
    	    $i++;
    	}
    }
	
    if( $showadminlevel || $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $u ) ){
        my $admin_frame = $self->render_admin_frame($u);
        if ($admin_frame) {
            $form .= "<tr><td><br></td></tr>";	
            $form .= "<tr><td>";
            $form .= $admin_frame;
            $form .= "</td></tr>\n";
        }
    }
	
    my $button_name = $login_mode eq 'standard' ? "Save" : "Save Securely";
    $form .= "
  <tr>
    <td align='right'>
      <br><INPUT type=submit name=action value=\"$button_name\">
    </td>
  </tr>
</table>
</FORM>
";
    $form .= "<p>" . $submessage;
    
    return $form;
#      $DISPLAY->print_form($form, "Registration");
}

sub render_registration_control {
    my $self = shift;
    my %args = @_;
    my $result = '';

    my $field = $args{field};
    my $u  = $args{user} || Metadot::UserFactory->get_user();

    my $field_name = $field->{field_name} || die "must specify field_name";
    my $field_desc = $field->{description} || die "must specify field description";
    my $trailing_text = "<font size=\"-2\"><i>" . ($field->{trailing_text} || '') . "</i></font>";
    my $field_type = $field->{field_type} || 'text';

    my $value = $args{value};
    unless (defined $value) {
        if (defined $FORM{$field_name}) {
            $value = $FORM{$field_name};
        } elsif (defined $u->get_value($field_name)) {
            $value = $u->get_value($field_name);
        }
    }

    $args{value} = $value;

    my $size = $field->{size} || 25;
    my $max_length = $field->{max_length} || 64;

    if( lc $field_type eq 'textarea' ) {
        my $rows = $field->{rows} || 3;
        my $cols = $field->{cols} || 30;
        my $wrap = $field->{wrap} ? 'wrap' : '';
        my $max_length = $field->{max_length} || 255;
    
        my $control = <<EOM
        <$field_type name=$field_name rows=$rows cols=$cols wrap=$wrap maxlength=$max_length>$value</$field_type>
        $trailing_text
EOM
;        
        $result .= $self->_format_register_control( 
            $self->_render_control_label(%args),
            $control
            );
        
    } elsif( lc $field_type eq 'password' ) {
        my $required = ($field->{is_required}) ? "<b><font color=\"#FF0000\">* </font></b>" : '';
        
        # password is not required if this person already has one
        if ($u->password()) {
            $required='';
        }
        $result .= <<EOM
<tr>
    <td>
    $required<b>$field_desc</b>
EOM
;
        if( $PARAMS->{'password_checker'} == 1) {
            $result .= "<br><font color=\"red\">Strong Passwords Required</font><br>Click <a href=\"javascript:popUp('/passwdchecker.html')\">Here</a> for more information."; 
        } elsif( $PARAMS->{'password_checker'} == 5) {
            $result .= "<br><font color=\"red\">Long Passwords Required (6 characters min)</font>"; 
        }

        # include a "verify password" field
        $result .= <<EOM
    </td>
    <td>
        <INPUT type=$field_type name=$field_name size=$size maxlength=$max_length>
        $trailing_text
    </td>
</tr>
<tr>
    <td>
        $required<b>Verify $field_desc: </b>
    </td>
    <td>
        <INPUT type=password name=verify_$field_name size=20>
    </td>
</tr>
EOM
;	    	    	   	    
	} elsif( lc $field_type eq 'radio' ) {
        $result .= $self->_format_register_control( 
            $self->_render_control_label(%args),
            $self->_render_radio_control(%args)
            );

    } elsif( lc $field_type eq 'select' ) {
        my $field_values;
        my $field_values_function = $field->{field_values_function};
        if ($field_values_function && $u->can($field_values_function)) {
            $field_values = $u->$field_values_function();
        } else {
            $field_values = $field->{field_values} || $u->get_modify_settings_form_field_values(field_name => $field_name);
        }
        my $control = '';
        if (!$u->uid ||  !$field->{only_admin_can_edit} || ($field->{only_admin_can_edit} && $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks')) ) {
            $control = <<EOM
    <select name="$field_name">
EOM
;
            foreach my $fval (@$field_values) {
                my $selected = $fval->{form_value} eq $value ? 'selected' : '';
                $control .= "<option value=\"" 
                . $fval->{form_value} 
                . "\" $selected> " 
                . $fval->{label}. "\n";
            }
            $control .= <<EOM
    </select>
    $trailing_text
EOM
;
        } else { ##cannot change value
            foreach my $fval (@$field_values) {
                if ($fval->{form_value} eq $value) {
                    $control = $fval->{label};
                    last;
                }
            }
        }
        $result .= $self->_format_register_control( 
            $self->_render_control_label(%args),
            $control
            );
    } elsif ( lc $field_type =~ /_upload$/ ) {
        my $op = $FORM{op};
        my $uid = $FORM{uid};
        my $return_url = "index.pl?isa=User&amp;op=$op";
        $return_url    .= "&amp;uid=$uid" if defined($uid);
        $return_url = $self->make_url( $return_url );
        my $control = '';
        $control .= $u->get_image_src(field_name => $field_name);
        $control .= $u->get_upload_input_form_line( {
                       isa => $u->is_a(),
                       id => $u->uid(),
                       field_name => $field_name,
                       field_desc => $field_desc,
                       is_required => $field->{is_required},
                       type => $field_type,
                       return_url => $return_url,
                   } );
        $result .= $self->_format_register_control( 
            $self->_render_control_label(%args),
            $control
            );
    } elsif ( lc $field_type eq 'custom' ) {
        $result .= $self->_format_register_control( 
            $self->_render_control_label(%args),
            $self->render_custom_control(%args)
            );

    } else {
            my $control = '';
            unless( ( lc $field_name eq 'first_name' or lc $field_name eq 'last_name' or lc $field_name eq 'initial' ) and $USER->is_default_account( $u->email() ) ) {
                $control .= <<EOM
        <INPUT type=$field_type name=$field_name size=$size value="$value" maxlength=$max_length>
        $trailing_text
EOM
;		
            } else {
                $control .= "<INPUT type=hidden name=$field_name value=\"$value\">";
                if( lc $field_name eq 'first_name' ) {
                $control .= <<EOM
        $value
        $trailing_text
EOM
;				    
                }
            }
            $result .= $self->_format_register_control( 
                $self->_render_control_label(%args),
                $control
                );                      
    }
    return $result;
}

sub render_custom_control {
    my $self = shift;
    my %args = @_;
    # please extend this method in a subclass
}

sub _format_register_control {
    my $self = shift;
    my $label = shift || '';
    my $control = shift || '';
    my $result = <<EOM
<tr>
   <td>
      $label
    </td><td>
      $control
    </td>
</tr>
EOM
;
    return $result;
}

sub _render_control_label {
    my $self = shift;
    my %args = @_;
    my $result = '';

    my $field = $args{field};
    my $required = ($field->{is_required}) ? "<b><font color=\"#FF0000\">* </font></b>" : '';
    my $field_desc = $field->{description} || die "must specify field description";

    $result .= <<EOM
$required<b>$field_desc</b>		
EOM
;
    return $result;
}

sub _render_radio_control {
    my $self = shift;
    my %args = @_;
    my $result = '';

    my $field = $args{field};
    my $u  = $args{user} || Metadot::UserFactory->get_user();
    
    my $field_name = $field->{field_name} || die "must specify field_name";
    my $field_desc = $field->{description} || die "must specify field description";
    my $trailing_text = "<font size=\"-2\"><i>" . ($field->{trailing_text} || '') . "</i></font>";
    
    my $value = $args{value};

    my $field_values;
    my $field_values_function = $field->{field_values_function};
    if ($field_values_function && $u->can($field_values_function)) {
        $field_values = $u->$field_values_function();
    } else {
        $field_values = $field->{field_values} || $u->get_modify_settings_form_field_values(field_name => $field_name);
    }



    #my $field_values = $field->{field_values} || $USER->get_modify_settings_form_field_values(field_name => $field_name);
    my $control='';
    if ($field->{use_line_breaks}) {
        foreach my $fval (@$field_values) {
        my $checked = $fval->{form_value} eq $value ? 'checked' : '';
        $control    .= "<input type=\"radio\" name=\"$field_name\" value=\"" 
                    . $fval->{form_value} 
                    . "\" $checked> " 
                    . $fval->{label} . "<br>\n";
        }              
    } else {
        $control ="<table border=\"0\" cellspacing=\"1\" cellpadding=\"1\">\n<tr>\n";
        foreach my $fval (@$field_values) {
            my $checked = $fval->{form_value} eq $value ? 'checked' : '';
            $control    .= "\n<td><input type=\"radio\" name=\"$field_name\" value=\"" 
                        . $fval->{form_value} 
                        . "\" $checked> " 
                        . $fval->{label} 
                        . "</td>\n";
        }
        $control .= "\n</tr>\n</table>\n";
    }
    $result .= $control;
    $result .= "\n$trailing_text";

    return $result;
}

sub render_admin_frame {
    my $self = shift;
    my $u = shift;
	
    my $not_admin = (! $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $u ) ) ? "checked" : "" ;
	my $is_admin = ( $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $u ) ) ? "checked" : "" ;

	my $admin_switch = "
<td align='right'><b>Person Role:</b></td><td>
<input type='radio' name='adminlevel' value='1' $not_admin> Regular User <br>
<input type='radio' name='adminlevel' value='9' $is_admin> Site Manager</td>
";
    
    my $not_default_size = ($u->{upload_size} ne "-1")? "checked": "";
    my $default_size = ($u->{upload_size} eq "-1")? "checked": "";
    
    my $max_total_upload_size = $PARAMS->{'max_total_upload_size'};
    my $max_total_number_upload = $PARAMS->{'max_total_number_upload'};
    
    my $uploadsize = "<input type='radio' name=upload_size_bn value='-1' $default_size>Default: $max_total_upload_size<br>";
    if($u->{upload_size} ne "-1") {
        $uploadsize .= "<input type='radio' name=upload_size_bn value='1' $not_default_size>Custom: &nbsp;<INPUT type=text name=upload_size size=15 value=\"".$u->{upload_size}."\" maxlength=50>";
    }
    else {
        $uploadsize .= "<input type='radio' name=upload_size_bn value='1' $not_default_size>Custom: &nbsp;<INPUT type=text name=upload_size size=15 value=\"\" maxlength=50>";
    }
    
    $not_default_size = ($u->{upload_size} ne "-1")? "checked": "";
    $default_size = ($u->{upload_size} eq "-1")? "checked": "";
    
    my $not_default_number = ($u->{upload_number} ne "-1")? "checked": "";
    my $default_number = ($u->{upload_number} eq "-1")? "checked": "";
    
    my $uploadnumber = "<input type='radio' name=upload_number_bn value='-1' $default_number>Default: $max_total_number_upload<br>";
    if($u->{upload_number} ne "-1") {
        $uploadnumber .= "<input type='radio' name=upload_number_bn value='1' $not_default_number>Custom:&nbsp;<INPUT type=text name=upload_number size=15 value=\"".$u->{upload_number}."\" maxlength=50>";
    }
    else {
        $uploadnumber .= "<input type='radio' name=upload_number_bn value='1' $not_default_number>Custom:&nbsp;<INPUT type=text name=upload_number size=15 value=\"\" maxlength=50>";
    }
    

    my $adminform = "";        
    $adminform .= "<table width=\"450\" border=\"0\">";
    $adminform .= "<tr><td colspan='2'>";
    $adminform .= "<h2>User Management</h2>";
    $adminform .= "</td></tr>";
    
    $adminform .= "<tr valign=baseline><td align='right'>";
    $adminform .= "<b>Size (MB):</b><br>" .
	"<font face=\"Verdana, Arial, Helvetica, sans-serif\" size=\"-2\">Disk space given to this user to upload their files</font>\n";
    $adminform .= "</td><td>";
    $adminform .= "$uploadsize";
    $adminform .= "</td></tr>";

	$adminform .= "<tr><td>&nbsp;</td></tr>\n";    

    $adminform .= "<tr valign=baseline><td align='right'>";
    $adminform .= "<b>Number:</b><br>" . 
	"<font face=\"Verdana, Arial, Helvetica, sans-serif\" size=\"-2\">Maximum number of uploads this user can perform</font>";
    $adminform .= "</td><td>";
    $adminform .= "$uploadnumber";
    $adminform .= "</td></tr>";
    
    if( $FORM{uid} && ( $FORM{uid} != MetadotConfig->get_reserved_uid('ADMIN') ) ) {
	$adminform .= "<tr valign=baseline>";
	$adminform .= "$admin_switch";
	$adminform .= "</tr>";
    }
    
    $adminform .= "</table>";
    
    
    my $admin_frame = new HTML::TableFrame("", "#ffffff");
    $admin_frame->add( $adminform );
    return $admin_frame->print;
    
}

###############################################################
### save user's settings : 'backend' function for
### User::www_save and User::www_save_self
### the uid being modifyied is in FORM{uid}
### the user modifying is in $USER->{uid}
###
### if the keep_same_adminlevel==1 then the user is modifying
### himself and we don't want to loose his current privilege
### so we will read the db first.
###############################################################
sub save_modify_settings_form {
    my $self = shift;
    my @groups_to_add_to = ();
    
    my ($keep_same_adminlevel) = @_;
    my $basename = $PARAMS->{basename};

    my $user = Metadot::UserFactory->get_user( $SESSION->get_value('User_uid_to_modify') );
    
    my $user_management_is_external = MetadotConfig->get_param_value('user_management_is_external');    
    unless( $user_management_is_external ) {
    	my $config = $USER->get_fields_config();
    
    	unless( $FORM{nickname} ) {	    
    	    $FORM{nickname} = $FORM{first_name};	
    	}
        
    	my $old_email = $user->{email};          
    	my $verify = Form::Verify->new;    
        
    	# verify proper format of form values
    	foreach my $field ( @{ $config->{FIELDS} } ) {
            if (defined $field->{access_op}) {
                next unless ($ACCESS_BROKER->is_allowed_to_do($field->{access_op}));
            }
    	    if( $FORM{ $field->{field_name} } || 
    		( $field->{is_required}
    		  and not ( $USER->is_default_account( $user->email() ) and 
    			    $field->{is_not_required_for_default_users} ) ) ) {
        		
        		# if the field is required or has been filled	    
        		
        		my $verify_type = $field->{verify_type} || 'non_empty';        		
        		unless( $field->{field_type} eq 'password' ) {
        		    
        		    $verify->verify( $verify_type, 
        				     $field->{description},
        				     $FORM{ $field->{field_name} },
        				     $field->{date_format} );
        		} else {
        		    
        		    # only verify password if a new password is provided
        		    if( $FORM{ $field->{field_name} } ) {
            			if ($PARAMS->{password_checker} == 1){
            			    $verify->verify( "password", 
            					     $field->{description}, 
            					     $FORM{ $field->{field_name} }, 
            					     "", $FORM{first_name},
            					     $FORM{nickname} );
            		    
            			} elsif($PARAMS->{password_checker} == 5){
            			    $verify->verify( "password_long", 
            					     $field->{description}, 
            					     $FORM{ $field->{field_name} }, 
            					     "", $FORM{first_name},
            					     $FORM{nickname} );
            			    
            			} else {
            			    
            			    $verify->verify( "non_empty",
            					     $field->{description}, 
            					     $FORM{ $field->{field_name} }, 
            					     "" );		    
            			}
            
            			if ( $FORM{ $field->{field_name} } ne 
            			     $FORM{ 'verify_' . $field->{field_name} } ) {
            			    $DISPLAY->print_click_back("Password verification error", "Your password fields didn't match!");
            			    exit;
            			}
        		    }
        		}
    	    }	
    	}    

    	if ($verify->errors) {
    	    my $error_addendum = $self->_get_error_addendum();
    	    if($PARAMS->{password_checker} > 0) {
    		$DISPLAY->print_click_back("Sorry", "The following field(s) were invalid:<br> " . $verify->errors
    					   .$self->_password_error_addendum($verify->errors). $error_addendum);
    		exit;
    	    }
    
    	    $DISPLAY->print_click_back("Sorry", "The following field(s) were invalid:<br> " . $verify->errors
    				       . $error_addendum);
    	    exit;
    	}
    
    	# check if the email address exist for another user with a different uid
    	unless ( $user->can_use_email_address( $FORM{email} ) ){
    	    my $errormsg .=  "<h3>The email address you entered is already used by another user. Please verify your email address. If this 'another user' is you then try to login or ask your system admin to help you.</h3><br><p>\n";
    	
    	    #####      ERROR     #####
    	    $DISPLAY->print_click_back("Sorry there is a slight problem", "$errormsg<br>");
    	    exit;
    	}
    
    	##### NO ERROR we do the update #####
    	### set privilege to NORMAL USER BY DEFAULT
    
    	foreach my $field ( @{ $config->{FIELDS} } ) {    	
            
            if (defined $field->{access_op}) {
                print STDERR "User is not allowed to save field '$field->{field_name}'\n";
                next unless ($ACCESS_BROKER->is_allowed_to_do($field->{access_op}));
            }
    	    
            my $value = ( $field->{can_contain_html} ) ? escapeHTML( $FORM{ $field->{field_name} } ) :
    	      $FORM{ $field->{field_name} };
    	    
    	    if( $value and lc $field->{field_type} eq 'password' and $PARAMS->{encryption_done} eq 'yes') {
    		
    		$value = Portal->encrypt_password( $value );
    	}
    	    unless ($field->{field_type} eq 'group' || $field->{field_type} eq 'checkbox_group') {
                #print STDERR "Setting user value for '$field->{field_name}' to '$value'\n";
                $user->set_value( $field->{field_name}, $value );
    	    } else {
    		push(@groups_to_add_to, split(",", $value)) if (defined $value && $value ne "");
    	    }
    	}
    
        # process any custom field input
        $self->input_custom_fields(
            user    => $user,
        );
    
        $self->_process_user_admin_params( $user, $keep_same_adminlevel );
    	$user->save();
    	
        my $uid = $user->{uid};
    	
        if (@groups_to_add_to) {
            # vj: add user to groups that are allowed to add to (for security reasons we should not allow users
            # to just add themselfs to arbitrary groups)
            my $allowed_group_collection = $self->get_allowed_group_collection();
            $allowed_group_collection->restart();
            while (my $grp = $allowed_group_collection->get_next()) {
                $grp->remove_member($uid);
                foreach my $grp_id (@groups_to_add_to) {
                    # vj: this block of code is copied from Group::add_remove_group_members
                    if ($grp_id == $grp->{grp_id} && !DBAccess->sqlSelect ("grp_id", "grpmembers", "grp_id=$grp_id and userid='$uid'")) {
                        Group->_add_a_member_to_a_group($grp_id, $uid);
                    }
                }
            }
        }
        
    	# send email 
    	my $to = $user->{email};
    	my $cc = "";
    	my $bcc = "";
    	my $subject = "Your Profile Has Been Changed";
    	my $message = "Your profile has been changed.  If you wish to view your new profile, please click here:  $basename/index.pl?isa=User&amp;op=modify_self";
    	my $sm = Sendmail->new($to, $cc, $bcc, $subject, $message);
    	$sm->sendMail();
        
    	if ($old_email ne $to) {
    	    $sm = Sendmail->new($old_email, $cc, $bcc, $subject, $message);
    	    $sm->sendMail();
    	}
            
    	my $grp_id = DBAccess->sqlSelect("grp_id", "grp", "group_name='ADMIN'");
        
    	#################
        }
    
    my $name = $user->webname();
    
    my ($title);
    
    my $refererlink = $FORM{refererlink} || $SESSION->referer() || 'index.pl';

    if( $SESSION->get_value('User_uid_to_modify') != $USER->{uid}){
    	# an admin is updating a user
    	
    	$title="Success: settings are saved.<br /><br />\n";
    	$DISPLAY->print_choices ($title, "confirm", "Continue", Utility->get_qualified_url($refererlink));
	
    } else {
	
	# self modif
	$title="Success $name, your settings are saved.\n";
        $refererlink = $refererlink =~ /confirm_upload_deletion/ ? 'index.pl' : $refererlink;
	$DISPLAY->two_choices ($title, "confirm", "Continue", Utility->get_qualified_url($refererlink));
    }        
}

sub input_custom_fields {
    my $self = shift;
    my %args = @_;
    
    my $user = $args{user};
    # please extend this method in a subclass
}

sub _process_user_admin_params {
    my $self = shift;
    my $user = shift;
    my $keep_same_adminlevel = shift;

    # CKIM: THIS IS BAD! The instance that this method is modifying IS NOT THE same
    #       instance that the calling method is working with!!!!!
    # my $user = Metadot::UserFactory->get_user( $SESSION->get_value('User_uid_to_modify') );

    # CKIM: make sure the $user that is passed to me is valid
    (defined $user && ref($user) && $user->isa('Metadot::User::FlexUser')) || die("Don't know how to work with user object: $user");

    my $admin_level = 1;
    if( $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) ) {	
    	if(defined $FORM{adminlevel} && $FORM{adminlevel}) {	    
    	    $admin_level = $FORM{adminlevel};	    
    	}
    }
    
    if($FORM{upload_size_bn} && ($FORM{upload_size_bn} >= 0) && $FORM{upload_size}) {	
        $user->upload_size( escapeHTML($FORM{upload_size}) );	
    } else{	
        $user->upload_size( "-1");
    }
    
    if($FORM{upload_number_bn} && ($FORM{upload_number_bn} >= 0) && $FORM{upload_number}){	
    	$user->upload_number( escapeHTML($FORM{upload_number}) );	
    } else{	
        $user->upload_number( "-1");
    }
    
    unless ($keep_same_adminlevel){	
        $user->admin_level( $admin_level );
    }
    
    my $grp_id = DBAccess->sqlSelect("grp_id", "grp", "group_name='SITE_MGR'");
    
    # XXX move all this code to Groups.pm
    if($admin_level == 9){
        ## READ CORY'S COMMENTS ABOVE AS TO WHY THIS BAD
		#Group->_add_a_member_to_a_group($grp_id, $SESSION->get_value('User_uid_to_modify') );	
		Group->_add_a_member_to_a_group($grp_id, $user->uid() );	
    } elsif ( ! $keep_same_adminlevel) { #if not keeping admin level	
        ## READ CORY'S COMMENTS ABOVE AS TO WHY THIS BAD
        #my $uid = $SESSION->get_value('User_uid_to_modify');
        my $uid = $user->uid();
    	
    	if(DBAccess->sqlSelect("userid", "grpmembers", "userid=$uid AND grp_id=$grp_id")){
    	    DBAccess->sqlDelete("grpmembers", "userid=$uid AND grp_id=$grp_id");    	    
    	    $CACHE->set_as_dirty('GROUPS_HASH');
    	}
    }    
}

# should be overridden in subclass (e.g. FlexRegisterWithGroups)
sub get_allowed_group_collection {
    my $class = shift;
    my $result = Metadot::DataCollection->new();
    
    if ($ACCESS_BROKER->is_allowed_to_do('register_users')) {
        foreach my $group (Group->groups_managed_by_user(), Group->groups_owned_by_user()) {
            $result->add($group);
        }
    }
    return $result;
}


1;
