###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::TreeNode;
use strict;

sub new {
	my $proto = shift;
	my %args = @_;

	my $class = ref($proto) || $proto;
	my $self = {};

	$self->{id} = undef;
	$self->{children} = [];
	$self->{parent} = undef;
	$self->{referent} = undef;
	$self->{description} = undef;

	bless($self, $class);

	$self->id($args{id}) if defined($args{id});
	$self->referent($args{referent}) if $args{referent};
	$self->description($args{description}) if $args{description};
	$self->parent($args{parent}) if $args{parent};
	$self->children($args{children}) if $args{children};

	return $self;
}

sub test {
	my $self = shift;

	print STDERR "Testing TREENODE _____________________\n";

	my $root = $self->new();
	my $child = $self->new();
	$root->add_child($child);
	use Data::Dumper;
	print STDERR Dumper $root;
	print STDERR "Is root a leaf? '",$root->is_leaf(),"'\n";
	print STDERR "Is child a leaf? '",$child->is_leaf(),"'\n";
}

sub add_child {
	my $self = shift;
	my $child = shift;

	$child->parent($self);
	$self->set_children($self->get_children(), $child);
}

sub is_leaf {
	my $self = shift;
	return (!defined $self->children() || !defined $self->children()->[0]);
}

sub is_root {
	my $self = shift;
	return (!defined($self->parent()));
}

sub get_children {
	my $self = shift;
	return @{$self->children()};
}

sub set_children {
	my $self = shift;
	my @children = @_;
	$self->children(\@children);
}

sub access {
	my $self = shift;
	my $property = shift || die ("No property name specified\n");
	my $value = shift;

	if (defined $value) {
		$self->{$property} = $value;
	}
	return $self->{$property};
}

sub id {
	my $self = shift;
	my $value = shift;
	$self->access('id',$value);
}

sub parent {
	my $self = shift;
	my $value = shift;
	$self->access('parent',$value);
}

sub children {
	my $self = shift;
	my $value = shift;
	$self->access('children',$value);
}

sub referent {
	my $self = shift;
	my $value = shift;
	$self->access('referent',$value);
}

sub description {
	my $self = shift;
	my $value = shift;
	$self->access('description',$value);
}



1;
