###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::UploadsManager;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CACHE $CGI);
use MetadotConfig;

use ErrorHandling;

use URI::Escape qw(uri_unescape uri_escape);
use Utility;


sub www_download_file {
    my $self = shift;
    my $params = shift;

    my ( $id, $isa, $field_name, $file_type );

    if( $params ) {

	$params->{isa} || die "must include param isa";
	$params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    ( defined $params->{id} ) || die "must include param id";
	$params->{field_name} || die "must include param field_name";
	
	$id = $params->{id};
	$isa = $params->{isa};
	$field_name = $params->{field_name};
	$file_type = $params->{type};

    } else {

	$id = $FORM{id};
	$isa = $FORM{isa};
	$field_name = $FORM{field_name};	
	$file_type = $FORM{file_type};
    }

    $id = undef unless(Utility->is_valid_integer($id));    # secu_200401    
    my ( $file, $stored_at ) = DBAccess->sqlSelect( "file_name, stored_at", 
						    "uploads",
						    "isa=".DBAccess->quote($isa)." and id=$id and field_name=".DBAccess->quote($field_name));  #secu_200401
    

    my $read_file;

    # if field is public, must fetch the file from the public dir
    if( $stored_at eq 'public' ) {

	$read_file = MetadotConfig->get_public_uploads_dir() . "/" . $file;	

    } else {

	my $absolutepathfileuploaddir = $PARAMS->{absolutepathfileuploaddir};
	my $write_dir_namespace = $self->get_write_dir_namespace() || "$absolutepathfileuploaddir/$id";
    
	$read_file = $write_dir_namespace . "/$file";
    #print STDERR "\n\nread_file:$read_file\n\n";
    }

	$self->transmit_file(
        file        => $file,
        read_file   => $read_file,
        file_type   => $file_type,
    );    
}

sub transmit_file {
    my $self = shift;
    my %args = @_;
    my $file = $args{file} || die ("Must supply file");
    my $read_file = $args{read_file} || die("Must supply read_file");
    
    my $file_type = $args{file_type};
    my $mtype;
    my $mimetypes = Metadot::MIME::Types->new();
	if ($file_type){
    	$mtype = $mimetypes->mimeTypeOf($file_type) ;
	} else {
    	$mtype = $mimetypes->mimeTypeOf($file) ;
	}
    unless (open OUT, "<$read_file") {
        my $error_name;
        if ($self->can('name')) {
            $error_name = $self->name();
        } else {
            $error_name = $self;
        }
        ErrorHandling->printError("Gizmo.pm: ". $error_name. " : Couldn't open file $read_file: $!");
        return;
    }
    binmode (OUT);
    
    my %header_values = (
                         '-Connection'          => 'close',
                         '-Accept-ranges'       => 'bytes',
                         '-Content-location'    => $file,
    );
    
    if (defined $mtype && defined $mtype->type) {
        if ($mtype->type eq "application/pdf") {# [vadim] IE+Acrobat workaround
            $header_values{'-Content-disposition'} = "attachment; filename=$file";
        } else {
            $header_values{'-Content-disposition'} = "inline; filename=$file";
        }
        $header_values{type} = $mtype->type;
    } else {
        $header_values{'-Content-disposition'} = "inline; filename=$file";
        $header_values{type} = 'application/octet-stream';
    }
    
    print $CGI->header(%header_values);
    
    {
        local $/ = undef;
        while(<OUT>) {
                print;	    
        }
    }
    close OUT ;
}


sub www_confirm_upload_deletion {
    my $self = shift;
    
    my $id   = $FORM{id};
    my $isa   = $FORM{isa};
    my $field_name = $FORM{field_name};
    my $return_url = $CGI->escape( $FORM{return_url} );

    $id = undef unless(Utility->is_valid_integer($id));    # secu_200401    
    my ( $file_name ) = DBAccess->sqlSelect( "file_name",
					     "uploads",
					     "isa=".DBAccess->quote($isa)." and id=$id and field_name=".DBAccess->quote($field_name));  #secu_200401

    $DISPLAY->two_choices( "Do you want to delete attachment $file_name ?",
			   "caution",
			   "Yes please", $DISPLAY->make_url( "index.pl?id=$id&amp;isa=$isa&amp;field_name=$field_name&amp;op=delete_upload&amp;return_url=$return_url" ),
			   "No!", $DISPLAY->make_url( $FORM{return_url} ) );

}


sub www_delete_upload {
    my $self = shift;

    my $return_url = uri_unescape( $FORM{return_url} );

	$self->_delete_upload(
		id		=> $FORM{id},
		isa   	=> $FORM{isa},
		field_name 	=> $FORM{field_name},
		field_index => $FORM{field_index},
	);

    Utility->md_redirect( $return_url );
}

sub _delete_upload {
	my $self = shift;
	my %args = (@_);

	my $id = $args{id};
    $id = undef unless(Utility->is_valid_integer($id));    # secu_200401    
	die("Must supply id\n") unless defined($id);
	
    my $isa = $args{isa} || die("Must supply isa\n");
	my $field_name = $args{field_name} || die("Must supply isa\n");
	my $field_index = $args{field_index};

	my $where_clause = "isa=".DBAccess->quote($isa)." and id=$id and field_name=".DBAccess->quote($field_name);  #secu_200401
	if (defined $field_index) {
		$where_clause .= " and field_index=".DBAccess->quote($field_index); # secu_200401
	}

    my ( $filename, $stored_at ) = DBAccess->sqlSelect( "file_name, stored_at",
							"uploads",
							$where_clause );

    if( $stored_at eq 'public' ) {

	my $write_file = MetadotConfig->get_public_uploads_dir() . "/" . $filename;

	unlink( $write_file );

	DBAccess->sqlDelete( "uploads", $where_clause );

    } else {

	my $absolutepathfileuploaddir = $PARAMS->{'absolutepathfileuploaddir'};
	my $write_dir_namespace = $self->get_write_dir_namespace() || "$absolutepathfileuploaddir/$id";

	unlink( "$write_dir_namespace/$filename" );

	DBAccess->sqlDelete( "uploads", $where_clause );
    }
	
}

sub get_field_index {
	my $self = shift;
	my %args = @_;

	my $id = $args{id};
    $id = undef unless(Utility->is_valid_integer($id));    # secu_200401    
	die("Must supply id\n") unless defined($id);
	my $isa = $args{isa} || die("Must supply isa\n");
	my $field_name = $args{field_name} || die("Must supply isa\n");
	my $file_name = $args{file_name} || die("Must supply isa\n");

	my $where_clause = "isa=".DBAccess->quote($isa)." AND id=$id AND field_name=".DBAccess->quote($field_name)." AND file_name=".DBAccess->quote($file_name);  #secu_200401
	my ($result) = DBAccess->sqlSelect("field_index","uploads",$where_clause);
	return $result;
}

sub get_upload_input_form_line {
    my $self = shift;
    my $params = shift || die "must include params hash";

    $params->{isa} || die "must include param isa";
    $params->{field_name} || die "must include param field_name";
    $params->{field_desc} || die "must include param field_desc";
    $params->{type} || die "must include param type";
    $params->{return_url} || die "must include param return_url";

    my $filename;

	$params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    if( $params->{id} ) {
	# check if we already have an upload for the passed params
	( $filename ) = DBAccess->sqlSelect( "file_name",
					     "uploads",
					     "isa=".DBAccess->quote($params->{isa})." and id=$params->{id} and field_name=".DBAccess->quote($params->{field_name})." and type=".DBAccess->quote($params->{type}));  #secu_200401
    }

    unless( $filename ) {
	my $public_location = MetadotConfig->get_public_uploads_location();
	my $htm = "<INPUT TYPE=\"file\" NAME=\"$params->{field_name}\" SIZE=30>\n";
	if( $params->{type} eq 'user_selected_upload' ) {
	    $htm .= "          <table width=\"98%\" >
            <tr valign=\"top\">
              <td width=\"5%\">
                <input type=\"radio\" name=\"upload_to_public_$params->{field_name}\" value=\"1\">
              </td>
              <td><b>Fast download</b><br>
                <font face=\"Verdana, Arial, Helvetica, sans-serif\" size=\"-2\">Your
                file will be stored under " .
		$PARAMS->{uri_scheme} . "://". $PARAMS->{rootdomain} . $public_location . ".
		This url
                is publicly accessible.  You can reference the file elsewhere on this site
                as \"$public_location/&lt;your_file&gt;\", 
                where &lt;your_file&gt; is replaced with
                your actual file name.
            </font></td>
            </tr>
            <tr valign=\"top\">
              <td width=\"5%\">
                <input type=\"radio\" name=\"upload_to_public_$params->{field_name}\" value=\"0\" checked>
              </td>
              <td><b>Access-control protected</b><br>
                <font face=\"Verdana, Arial, Helvetica, sans-serif\" size=\"-2\">Your
                file will be under <i>access control</i>.
                This means that only users who have permissions to view this
                content object will be able to access the file.</font></td>
            </tr>
          </table>";

# 	    $htm .= "<br><input type=checkbox name=\"upload_to_public_$params->{field_name}\" value=\"1\"> Upload to public";
	}

	return $htm;
    }

    my $return_url = Utility->get_qualified_url($CGI->escape( $params->{return_url} )); 

    my $delete_label = "delete attachment";

#     print STDERR "isa is: " . $self->is_a() . "\n";
    if ($self->is_a() eq 'ImageItem' || $self->is_a() eq 'Portal') {	# special case label was requested
	# for image item (sigh)
	$delete_label = "delete image";
    }

    # the file exists; return html for deleting attachment
    return "$filename&nbsp;&nbsp;&nbsp;&nbsp;<a class=\"text_button\" href=\"" . $self->make_url( "index.pl?id=$params->{id}&amp;isa=$params->{isa}&amp;op=confirm_upload_deletion&amp;field_name=$params->{field_name}&amp;return_url=$return_url" ) . "\">$delete_label</a>";

}


sub process_upload {
    my $self = shift;
    my $params = shift || die "must include params hash";

    $params->{isa} || die "must include param isa";
	$params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    ( defined $params->{id} ) || die "must include param id";
    $params->{field_name} || die "must include param field_name";
    $params->{field_desc} || die "must include param field_desc";
    $params->{type} || die "must include param type";
    ( defined $params->{is_required} ) || die "must include param is_required";

    if( $FORM{ $params->{field_name} } ) {
	# upload a file

	my $err;

	if( $params->{type} eq 'public_upload' ||
	    ( $params->{type} eq 'user_selected_upload' &&
	      ($FORM{ "upload_to_public_$params->{field_name}"})
	      )
	    ) {
	    # upload to the public dir

	    $err = $self->_upload_public_file( $params );

	} else {

	    # upload to the access controlled dir
	    $err = $self->_upload_access_controlled_file( $params );
	}

	if( $err ) { return $err }

	return ''

    } else {
	# the param for this upload was not submitted
	if( $params->{is_required} ) {
	    # must check if this upload is already there
	    # else we should raise an error

	    my ($cnt) = DBAccess->sqlSelect("count(*)", "uploads",
					    "isa=".DBAccess->quote($params->{isa})." and id=$params->{id} and field_name=".DBAccess->quote($params->{field_name})." and type=".DBAccess->quote($params->{type}));  #secu_200401

	    unless( $cnt ) { return "$params->{field_desc} is required" }

	    return '';
	}
	
	return '';
    }
}


#################################################################
# _upload_public_file
# script heavily inspired from Muhammad A Muquit upload example.
# $filepath: the filename with the path where the file is located
# $fileiid: the object unique id that that file refers to.
#################################################################

sub _upload_public_file {
    my $self = shift;
    my $params = shift;
    my $uid = shift;

	$params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    my $filepath = $FORM{ $params->{field_name} } || die "this must be checked from where _upload_public_file is called";

    $uid = (defined $uid) ? $uid : $USER->uid();

    my $bytes_read = 0;
    my $size = 0;
    my $buff = '';
    my $filename = $self->get_filename_from_path($filepath);
    my $write_file = '';

    $write_file = MetadotConfig->get_public_uploads_dir() . "/" . $filename;

    print STDERR "FILE PATH  IS: $filepath\n";
    print STDERR "WRITE FILE IS: $write_file\n";

    if( -e $write_file ) {
	# file alredy exists, must not allow upload

	return "File $filename already exists in the public access directory. Please rename your file or go to the pre-existing file (if you own it) and update it from there";

    }

    ### moving the file from tmp dir to final location
    if (!open(WFD,">$write_file")) {

	return "Error uploading file $filename, cannot write to disk. Please contact your site administrator."
    }

    while ( $bytes_read = read( $filepath, $buff, 2096 ) ) {

	# while (read($filepath,$buff,2096)){
	$size += $bytes_read;
	binmode WFD;
	print WFD $buff;

    }

    if ($size == 0) {
	close(WFD);

	unlink $write_file;

	return "Error, cannot access specified upload file $filepath, or file is empty. Please recheck the file name.";

    }

    close(WFD);

    my ($upload_size_allowed, $upload_number_allowed) = $USER->get_uploads_space_and_num_files_quota( $uid );

    if($upload_size_allowed == -1){
	$upload_size_allowed = $PARAMS->{max_total_upload_size};
    }

    if($upload_number_allowed == -1){
	$upload_number_allowed = $PARAMS->{max_total_number_upload};
    }

    # multiply by 2^20 to convert to bytes
    $upload_size_allowed *= 1048576;

    my ($used_space, $uploaded_number) = $USER->get_used_uploads_space_and_num_files( $uid );

    my $available_space = $upload_size_allowed - $used_space;

    if ( $size > $available_space ) {

	unlink($write_file);

	return "Error, File upload has exceeded allowed user quota. Your remaining quota is of $available_space bytes and file $filename is of $size bytes. Please request a larger quota to the site administrator or delete some of your files to free up space";
    }

    if ( $uploaded_number >= $upload_number_allowed ) {

	unlink($write_file);

	return "Error, File upload has exceeded allowed number of files user quota. Your number of files quota is of $upload_number_allowed. Please request a larger number of files quota to the site administrator or delete some of your existing files.";
    }

    # check for global quota overrun
    ($used_space, $uploaded_number) = $self->get_global_used_uploads_space_and_num_files();

    $upload_size_allowed = $PARAMS->{global_uploads_quota};

    # multiply by 1M to convert to bytes
    $upload_size_allowed *= 1048576;

    $available_space = $upload_size_allowed - $used_space;

    if ( $size > $available_space ) {

	unlink($write_file);

	return "Error, File upload has exceeded the site-wide allowed disk space quota. The remaining quota is of $available_space bytes and file $filename is of $size bytes. Please request a larger quota to the site administrator or delete some of your files to free up space";
    }


    if ((stat $write_file)[7] <= 0) {

	unlink($write_file);
	return "Could not upload file: $filename. File is empty.";

    }

	my %fields = (
			id => $params->{id},
			userid => $uid,
			isa => $params->{isa},
			field_name => $params->{field_name},
			type => $params->{type},
			stored_at => 'public',
			file_name => $filename,
			file_size => $size
	);

	# allow for possible field_index
	if ($params->{allow_multiple}) {
		my ($index) = DBAccess->sqlSelect(
			"max(field_index)","uploads","id=$params->{id} AND isa=".DBAccess->quote($params->{isa})." AND field_name=".DBAccess->quote($params->{field_name}));  #secu_200401
		$fields{field_index} = $index + 1;
	}

    # insert into bookkeeping table
    DBAccess->sqlInsert("uploads", %fields);

    return '';
}

#################################################################
# get_filename_from_path
# $filepath: the filename with the path where the file is located
#################################################################
sub get_filename_from_path {
	my $self = shift;
	my $filepath = shift;
	my $filename;
    # James Bee" <JamesBee@home.com> reported that from Windows filename
    # such as c:\foo\fille.x saves as c:\foo\file.x, so we've to get the
    # filename out of it
    # look at the last word, hold 1 or more chars before the end of the line
    # that doesn't include / or \, so it will take care of unix path as well
    # if it happens, muquit, Jul-22-1999
    if ($filepath =~ /([^\/\\]+)$/) {
	$filename="$1";
    } else {
	$filename="$filepath";
    }

	return $filename;
}

# alias for backward compatibility
sub _get_filename_from_path {
    my $class = shift;
    return $class->get_filename_from_path(@_);
}

#################################################################
# _upload_access_controlled_file
# script heavily inspired from Muhammad A Muquit upload example.
# $filepath: the filename with the path where the file is located
# $fileiid: the object unique id that that file refers to.
#################################################################

sub _upload_access_controlled_file {
    my $self = shift;
    my $params = shift;
    my $uid = shift || $params->{uid};

	$params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    my $filepath = $FORM{ $params->{field_name} } || die "this must be checked from where _upload_public_file is called";

    $uid = (defined $uid) ? $uid : $USER->uid(); 
    
    my $bytes_read = 0;
    my $size = 0;
    my $buff = '';
    my $filename = '';
    my $write_file = '';

    unless( $PARAMS->{absolutepathfileuploaddir} ) {
	my $msg = "Sorry the parameter 'upload_directory' is not set therefore the upload is just impossible. Please contact the system administrator";
	$self->click_back($msg);	
	exit;
    }

   ### create the upload global directory if not there
    unless( -d $PARAMS->{absolutepathfileuploaddir}) {

	unless( mkdir($PARAMS->{absolutepathfileuploaddir}, 0755) ) {
	    
	    ErrorHandling->printError("cannot create upload file directory: '"
				      . $PARAMS->{absolutepathfileuploaddir}. "': $?");
	    exit;
	}
    }

    # James Bee" <JamesBee@home.com> reported that from Windows filename
    # such as c:\foo\fille.x saves as c:\foo\file.x, so we've to get the
    # filename out of it
    # look at the last word, hold 1 or more chars before the end of the line
    # that doesn't include / or \, so it will take care of unix path as well
    # if it happens, muquit, Jul-22-1999
    if ($filepath =~ /([^\/\\]+)$/) {
	$filename="$1";
    } else {
	$filename="$filepath";
    }

    my $write_to_dir = $self->get_write_dir_namespace() || "$PARAMS->{absolutepathfileuploaddir}/$params->{id}";

    $write_file = "$write_to_dir/$filename";

    ### create the upload directory with name=iid
    unless( -d $write_to_dir ) {
        unless (mkdir( $write_to_dir, 0755 )) {
            $self->print_click_back("Upload Error", "Cannot create directory: $write_to_dir: $?.  If this problem occurs again, please contact the site administrator.");
            exit(0);
        }
    }
       
    if( -e $write_file ) {
	# file alredy exists, must not allow upload
	
	return "File $filename already exists in the public access directory. Please rename your file or go to the pre-existing file (if you own it) and update it from there";
	
    }

    ### moving the file from tmp dir to final location
    if (!open(WFD,">$write_file")) {

	return "Error uploading file $filename, cannot write to disk. Please contact your site administrator."
    }

    while ( $bytes_read = read( $filepath, $buff, 2096 ) ) {

	# while (read($filepath,$buff,2096)){
	$size += $bytes_read;
	binmode WFD;
	print WFD $buff;

    }
    
    if ($size == 0) {

	close(WFD);

	unlink $write_file;

	return "Error, cannot access specified upload file $filepath, or file is empty. Please recheck the file name.";

    }
    
    close(WFD);

    my ($upload_size_allowed, $upload_number_allowed) = $USER->get_uploads_space_and_num_files_quota( $uid );

    if($upload_size_allowed == -1){
	$upload_size_allowed = $PARAMS->{max_total_upload_size};
    }
    
    if($upload_number_allowed == -1){
	$upload_number_allowed = $PARAMS->{max_total_number_upload};
    }
    
    # multiply by 1M to convert to bytes
    $upload_size_allowed *= 1048576;
    
    my ($used_space, $uploaded_number) = $USER->get_used_uploads_space_and_num_files( $uid );

    my $available_space = $upload_size_allowed - $used_space;

#    print STDERR "=====> SOME NUMBERS: $upload_size_allowed, $used_space, $available_space\n";

    if ( $size > $available_space ) {

	unlink($write_file);

	return "Error, File upload has exceeded allowed user quota. Your remaining quota is of $available_space bytes and file $filename is of $size bytes. Please request a larger quota to the site administrator or delete some of your files to free up space";
    }

    if ( $uploaded_number >= $upload_number_allowed ) {

	unlink($write_file);

	return "Error, File upload has exceeded allowed number of files user quota. Your number of files quota is of $upload_number_allowed. Please request a larger number of files quota to the site administrator or delete some of your existing files.";
    }


    # check for global quota overrun
    ($used_space, $uploaded_number) = $self->get_global_used_uploads_space_and_num_files();

    $upload_size_allowed = $PARAMS->{global_uploads_quota};
    my $upload_size_allowed_is_unlimited = (!defined $upload_size_allowed || $upload_size_allowed eq "") ? 1 : 0;

    # multiply by 1M to convert to bytes
    $upload_size_allowed *= 1048576;

    $available_space = $upload_size_allowed - $used_space;

    if ( !$upload_size_allowed_is_unlimited && $size > $available_space ) {

	unlink($write_file);

	return "Error, File upload has exceeded the site-wide allowed disk space quota. The remaining quota is of $available_space bytes and file $filename is of $size bytes. Please request a larger quota to the site administrator or delete some of your files to free up space";
    }

    if ((stat $write_file)[7] <= 0) {

	unlink($write_file);	
	return "Could not upload file: $filename. File is empty.";
	
    } 

    # insert into bookkeeping table
    DBAccess->sqlInsert("uploads",
			id => $params->{id},
			userid => $uid,
			isa => $params->{isa},
			field_name => $params->{field_name},
			type => $params->{type},
			stored_at => 'access_controlled',
			file_name => $filename,
			file_size => $size );    

    return '';
}

sub get_write_dir_namespace {
    my $self = shift;
    return '';
}

# generic method for uploading files. The other upload methods need to be refactored to
# share this code. Returns 0 if successful, otherwise returns an error message
use constant _ERROR_FILE_EXISTS_    => 'Target file exists';
use constant _ERROR_WRITING_FILE_   => 'Error writing file';
use constant _ERROR_FILE_EMPTY_     => 'Source file was empty';

sub upload_file {
    my $class = shift;
    my %args = @_;

    my $source_file = $args{source_file} || die "Must specify arg source_file";
    my $write_dir   = $args{write_dir};
    my $write_file  = $args{write_file};
    my $overwrite   = $args{overwrite} || 0;
    
    $write_dir || $write_file || die "Must specify either arg write_file or write_dir";

    my $filename = $class->get_filename_from_path($source_file);
    if (!$write_file) {
        $write_file = $write_dir.'/'.$filename;
    }


    print STDERR "UploadsManager->upload_file Will read '$source_file' and write '$write_file'. Overwrite=$overwrite\n";


    if( -e $write_file && !$overwrite) {
	# file already exists, must not allow upload
        return _ERROR_FILE_EXISTS_;
    }

    ### moving the file from tmp dir to final location
    if (!open(WFD,">$write_file")) {
        return _ERROR_WRITING_FILE_;
    }

    my $bytes_read;
    my $buff = '';
    while ( $bytes_read = read( $source_file, $buff, 2096 ) ) {
    	binmode WFD;
    	print WFD $buff;
    }
    close(WFD);

    
    # remove the file if it is empty
    if ((stat $write_file)[7] <= 0) {
    	unlink($write_file);
    	return _ERROR_FILE_EMPTY_;
    }

    return $filename;
}



sub space_used {
    my $self = shift;
    my $uid = shift;

    #$uid = DBAccess->quote( $uid );
    $uid = undef unless(Utility->is_valid_integer($uid));    #secu_200401

    my ($c) = DBAccess->sqlSelectMany( "file_size", "uploads", "userid=$uid" );
    my $size = 0;

    while ( my $s = $c->fetchrow()){
	$size += $s;
    }

    my $number = $c->rows;	# this is ok because $c has been fetched
    $c->finish();

    return ($size, $number);
}


sub get_global_used_uploads_space_and_num_files {
    my $self = shift;

    my ($c) = DBAccess->sqlSelectMany( "file_size", "uploads" );
    my $size = 0;

    while ( my $s = $c->fetchrow()){
	$size += $s;
    }

    my $number = $c->rows; # this is ok because $c has been fetched
    $c->finish();

    # add the totals from legacy uploads in Gizmo
    my ($siz, $num) = Gizmo->total_space_used();

    $size += $siz;    
    # add default disk usage
    $size += MetadotConfig->get_initial_disk_usage_mb()*1024*1024;
    $number += $num;
    
    return ($size, $number);
}


# returns the filename, icon src, and url. Does not decorate with any html
sub get_upload_filename_iconsrc_and_url {
    my $self = shift;
    my $params = shift;

    $params->{isa} || die "must include param isa";
	$params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    ( defined $params->{id} ) || die "must include param id";
    $params->{field_name} || die "must include param field_name";

    my ( $filename, $stored_at ) = DBAccess->sqlSelect( "file_name, stored_at",
							"uploads",
							"isa=".DBAccess->quote($params->{isa})." and id=$params->{id} and field_name=".DBAccess->quote($params->{field_name}));  #secu_200401

    unless( $filename ) { return }

    my $icon_src = $DISPLAY->get_icon_src( $filename ) ;
    my ( $attach, $url );

    if( $stored_at eq 'public' ) {
    	my $public_location = MetadotConfig->get_public_uploads_location();   
    	$url = MetadotConfig->get_public_uploads_location() .'/'. &uri_escape($filename);
    } else {
        $url = $self->make_url("index.pl?id=$params->{id}&amp;isa=$params->{isa}&amp;field_name=$params->{field_name}&amp;op=download_file");
    }

    return ( $filename, $icon_src, $url );
}


# returns the filename, icon, and url. _Does_ decorate with html
sub get_upload_filename_icon_and_url {
    my $self = shift;
    my $params = shift;

    $params->{isa} || die "must include param isa";
    ( defined $params->{id} ) || die "must include param id";
    $params->{field_name} || die "must include param field_name";

    my ($filename, $icon_src, $url) = $self->get_upload_filename_iconsrc_and_url($params);
    my $attach;
    if (defined $filename) {
        $attach = "&nbsp;" . "<a href='" . $url . "'>" . "<img class=\"icon\" align='bottom' src=\"" . $icon_src . "\" border=\"0\" align='center' alt='Download Me!' width=\"16\" height=\"16\"></a>" . "&nbsp;" . "<a href='" . $url . "'><span class=\"centerNewsLink\">download</span></a>" . "&nbsp;" ;
    }

    return ( $filename, $attach, $url );
}



sub get_upload_url {
    my $self = shift;
    my $params = shift;

    $params->{isa} || die "must include param isa";
    
    $params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    ( defined $params->{id} ) || die "must include param id";
    $params->{field_name} || die "must include param field_name";

	my $where = "isa=".DBAccess->quote($params->{isa})." and id=$params->{id} and field_name=".DBAccess->quote($params->{field_name});  #secu_200401
	if ($params->{file_name}) {
		$where .= " and file_name='$params->{file_name}'";
	}

    my ( $filename, $stored_at ) = DBAccess->sqlSelect( "file_name, stored_at",
							"uploads",
							$where );


    unless( $filename ) { return '' }

    my $url = '';

    if( $stored_at eq 'public' ) {

	$url = MetadotConfig->get_public_uploads_location() . "\/$filename";

    } else {

	$url = $self->make_url("index.pl?id=$params->{id}&amp;isa=$params->{isa}&amp;field_name=$params->{field_name}&amp;op=download_file");

    }

    return $url;
}

sub delete_all_uploads {
    my $self = shift;
    my $params = shift;

    $params->{isa} || die "must include param isa";
	$params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    ( defined $params->{id} ) || die "must include param id";

    my ($records) = DBAccess->sqlSelectMany("field_name, file_name, stored_at, id", "uploads", "id=$params->{id} and isa=".DBAccess->quote($params->{isa}));  #secu_200401


    while ( my ( $field_name, $file_name, $stored_at, $id ) = $records->fetchrow() ){

	if( $stored_at eq 'public' ) {

	    my $write_file = MetadotConfig->get_public_uploads_dir() . "/" . $file_name;

	    unlink( $write_file );

	    DBAccess->sqlDelete( "uploads", "isa='$params->{isa}' and id='$params->{id}' and field_name='$field_name'" );

	} else {

	    # take care of ac files

	    my $absolutepathfileuploaddir = $PARAMS->{'absolutepathfileuploaddir'};

	    unlink( "$absolutepathfileuploaddir/$id/$file_name" );

	    DBAccess->sqlDelete( "uploads", "isa=".DBAccess->quote($params->{isa})." and id=$params->{id} and field_name=".DBAccess->quote($field_name) );

	}
    }
}

# returns a list of the public uploads, optionally filtering on an 'isa' value.
# the format of the return value is an array of hashes, each containing a 'url' and 'filename' key
sub list_public_uploads {
    my $self = shift;
    my $params = shift;
	my @results = ();

	my $where_clause = "stored_at='public'";
	if ($params->{isa}) {
		# check if isa is a ref to an array
		if (ref($params->{isa}) eq 'ARRAY') {
			# put single quotes around each element
			for (my $i=0; $params->{isa}->[$i]; $i++) {
				$params->{isa}->[$i] = DBAccess->quote($params->{isa}->[$i]);
			}
			$where_clause .= ' and isa in ('.join(',',@{$params->{isa}}).')';
		} else {
			$where_clause .= " and isa=".DBAccess->quote($params->{isa});  #secu_200401
		}
	}
    my ($records) = DBAccess->sqlSelectMany("file_name", "uploads", $where_clause);

    while ( my ($file_name) = $records->fetchrow() ){
		push @results, { 	file_name	=> $file_name,
							url			=> MetadotConfig->get_public_uploads_location() . "\/$file_name",
		}
	}

	return @results;
}


#
# will return the number of bytes in all uploaded files
# found according to search terms
#
sub get_upload_size {
    my $self = shift;
    my $params = shift;
    
	$params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    
    my @where;
    if( $params->{isa} ) {

	push @where, "isa=".DBAccess->quote($params->{isa});  # secu_200401
    }
    if( $params->{id} ) {

	push @where, "id=$params->{id}";
    }
    if( $params->{field_name} ) {

	push @where, "field_name=".DBAccess->quote($params->{field_name});      # secu_200401
    }

    my $where = join( ' and ', @where ); 

    print STDERR "WHERE CLAUSE IS $where\n";

    my ( $records ) = DBAccess->sqlSelectMany( "file_size",
					       "uploads",
					       $where );

    my $file_size;

    while ( my ($size) = $records->fetchrow() ) {
	
	$file_size += $size;
    }

    print STDERR "FOUND SIZE $file_size\n";

    return  $file_size || 0;
}

sub change_uploads_userid {
    my $class = shift;

    my $params = shift;

    $params->{isa} || die "must include param isa";
	
    $params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    ( defined $params->{id} ) || die "must include param id";
    ( defined $params->{new_userid} ) || die "must include param new_userid";

    my $where = "isa=".DBAccess->quote($params->{isa})." and id=$params->{id}";  #secu_200401

    if( $params->{field_name} ) {

	$where .= " and field_name=".DBAccess->quote($params->{field_name});
    }

    DBAccess->sqlUpdate("uploads", $where,
			userid => $params->{new_userid} );

}













1;

__END__

CREATE TABLE uploads (
  uid varchar(20) NOT NULL,
  isa varchar(64) NOT NULL,
  id varchar(64) NOT NULL,
  field_name varchar(64) NOT NULL,
  type varchar(64) NOT NULL,
  stored_at varchar(64) NOT NULL,
  file_name varchar(128) NOT NULL,
  file_size int(11) NOT NULL,
  PRIMARY KEY  (id,isa,field_name)
);
