###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::User::Default;
use strict;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $ACCESS_BROKER $DISPLAY);
use Portal;
use DBAccess;
use Carp;

use MetadotConfig;
use Metadot::UploadsManager;

use Group;
use Gizmo::Category;
use CGI;
use CGI qw(escapeHTML);
use Net::LDAP;
use MyPage;
use Form::Verify;
use HTML::TableFrame;
use ErrorHandling;
use Metadot::Utils::Display;
use Metadot::RegisterFactory;
use Utility;

use vars qw(@ISA);
@ISA=qw(Portal); #inherit from the meta class

my %default_account_names = ( admin => 1, default_look => 1 );

my $AC = {
	  BUNDLES => {
									
                   CREATE => { DESC =>          'Create Account', 
                               OPS =>            [qw(register_form register)], 
                               DEFAULT_LEVEL =>  0,
                               MIN_LEVEL =>      0,
                             },                
                   MOD =>    { DESC =>          'Edit', 
                               OPS =>            [qw(modify_self save_self)], 
                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                             },                
                   SEARCH =>    { DESC =>          'Search', 
                               OPS =>            [qw(show_users_portal)], 
                               DEFAULT_LEVEL =>  0,
                               MIN_LEVEL =>      0,
                             },                
                   DEL =>    { DESC =>          'Delete and Cut', 
                               OPS =>            [qw(delete delete_ok cut delete_group delete_request)], 
                               DEFAULT_LEVEL =>  8,
                               MIN_LEVEL =>      8,
                               PARENT_DEP =>     {cut => 'MOD'},
                             },                
                   DISP =>   { DESC =>          'View', 
                               OPS =>            [qw(require_pager_form process_pager_form require_icq_form process_icq_form)], 

                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                               DEFAULTS_TO =>    'on',
                             },                
                  ADMIN => { DESC =>			'Admin Modify',
							 OPS =>              [qw(show_details show_users modify save accept_registration reject_registration)],
							 DEFAULT_LEVEL => 9,
							 MIN_LEVEL =>	  9,
							 DEFAULTS_TO =>   'on',
							},
                   EDITP =>  { DESC =>          'Change<br>Permissions', 
                               OPS =>            [qw(edit_permissions set_permissions change_owner)], 
                               DEFAULT_LEVEL =>  8,
                               MIN_LEVEL =>      8,
                             },                
                  },
          DISPLAY_ORDER => [qw(DISP CREATE MOD DEL ADMIN EDITP)],
        };
        
sub get_AC {
	return $AC;
}

sub is_a {
	my $self = shift;

	return ref($self);
}

sub id {
	my $self = shift;

	return 1;
}

sub get_owner_uid {
	my $self = shift;

	return MetadotConfig->get_reserved_uid('ADMIN');
}

sub parent_id {
	my $self = shift;
       # my $id = $user->parent_id ;
	
	return 0;
}

####################################################

sub new {
    my $proto = shift;
    my $uid = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    $self->{uid}            = undef;
    $self->{nickname}       = "";
    $self->{fullname}       = "";
    $self->{quote}          = "";
    $self->{email}          = "";
    $self->{approval_needed} = 0;
    $self->{password}       = "";
    $self->{creation_date}   = "";
    $self->{update_date}     = "";
    $self->{pager_number}    = "";
    $self->{icq_number}      = "";
    $self->{icq_urgent}      = "";
    $self->{last_online}     = "0";
    $self->{ip}             = "";
    $self->{admin_level}     = 1;
    $self->{homepage}       = "";
    $self->{debug_level}     = 0;
    $self->{admin_mode}      = 0;
    $self->{password}        = "";
    $self->{link_name_1}      = "";
    $self->{link_name_2}      = "";
    $self->{link_name_3}      = "";
    $self->{link_name_4}      = "";
    $self->{link_name_5}      = "";
    $self->{link_name_6}      = "";
    $self->{link_name_7}      = "";
    $self->{link_name_8}      = "";
    $self->{link_name_9}      = "";
    $self->{link_name_10}     = "";
    $self->{link_name_11}     = "";
    $self->{link_name_12}     = "";
    $self->{link_name_13}     = "";
    $self->{link_name_14}     = "";
    $self->{link_name_15}     = "";
    $self->{link_url_1}       = "";
    $self->{link_url_2}       = "";
    $self->{link_url_3}       = "";
    $self->{link_url_4}       = "";
    $self->{link_url_5}       = "";
    $self->{link_url_6}       = "";
    $self->{link_url_7}       = "";
    $self->{link_url_8}       = "";
    $self->{link_url_9}       = "";
    $self->{link_url_10}      = "";
    $self->{link_url_11}      = "";
    $self->{link_url_12}      = "";
    $self->{link_url_13}      = "";
    $self->{link_url_14}      = "";
    $self->{link_url_15}      = "";
    $self->{ldap_dn}           = "";
    $self->{upload_size}    = "";
    $self->{upload_number}    = "";

    # we don't check if a user is logged_in until
    # is_logged_in is called
    $self->{_logged_in} = undef;

    bless ($self, $class);
        
   # $self->debug_msg(4, "Usernew()");

    if (defined ($uid)) {
	$self->{uid} = $uid;
	$self->_initialize();
    }
 
    return $self;
}

#######################################################################
#  RESTRICTED-USER: THESE FUNCTIONS NEED TO BE REMOVED WHEN
#  PROPER ROLE-BASED ACL IS IN PLACE
#######################################################################
my $restricted_flag = '_RESTRICTED_USER';
my $restricted_attribute = 'link_name_1'; #this is safe because restricted users
                                          #cannot have a mypage.
my $owner_attribute      = 'link_name_2';                                          

sub set_restricted {
    my $self = shift;
    my $value = shift;
    # if no value is sent at all, then default is true
    if (!defined($value)) {$value = 1};
    
    if ($value) {
        # set the restricted_attribute to the flag for restricted users
        $self->{$restricted_attribute} = $restricted_flag;    
    } else {
        # changing the user to non-restricted
        # check to see if the user was restricted to begin with
        if ($self->is_restricted()) {
            $self->{$restricted_attribute} = '';
        }
    }
}

# determines whether the user is restricted
sub is_restricted {
     my $self = shift;
     my $result = (defined($self->{$restricted_attribute}) && ($self->{$restricted_attribute} eq $restricted_flag));
     return $result;
}

# restricted users have owners. this returns the uid of the owner
sub get_restricted_owner {
    my $self = shift;
    my $result;
    if ($self->is_restricted()) {
        $result = $self->{$owner_attribute};
    }
    return $result;
}

# restricted users have owners. this sets the uid of the owner
sub set_restricted_owner {
    my $self = shift;
    my $input = shift;
    if ($self->is_restricted()) {
        $self->{$owner_attribute} = $input;   
    }
}

# a user is orphaned if he does not belong to any groups
sub is_orphan {
    my $self = shift;    
    my $result = 0;

    my $grp_ref = Group->get_groups_where_member($self->uid()) || [];
    unless (ref $grp_ref eq 'ARRAY') {
        $grp_ref = [$grp_ref];
    }
    unless (@$grp_ref) {
        $result = 1;
    }
    
    return $result;
}

# a restricted user is orphaned if he does not belong to any groups.
sub is_restricted_orphan {
    my $self = shift;
    my $result = ($self->is_restricted() && $self->is_orphan());

    return $result;
}

# this will delete a user if he is a restricted orphan
sub delete_if_restricted_orphan {
    my $self = shift;
    if ($self->is_restricted_orphan()) {
        $self->delete();   
    }
}

# this will find the pages that the restricted user can see, and allow him to
# choose from them. If there is only one choice, the system will redirect
sub www_restricted_user_home {
    my $self = shift;
    
    use HTML::Decorator::BulletDecorator;
    my $decorator = 'HTML::Decorator::BulletDecorator';
    
    my @group_names = @{ Group->get_groups_where_member($self->{uid}) };
    my @instances;
    my $page_list = '';
    for my $group_name (@group_names) {
        my $group = Group->get_group_by_name($group_name);
        my $iid = $group->{iid};
        if ($iid) {
            my $instance = Gizmo->restore($iid);
            push @instances, $instance;
            $page_list .= $decorator->new(component=>$instance)->show_summary()."<br>\n";
        }
    }
    
    # if only one instance was found, just redirect them
    if (scalar(@instances) == 1) {
        my $instance = $instances[0];
        my $url = $self->make_url("index.pl?iid=$instance->{id}&amp;isa=$instance->{is_a}");
        Utility->md_redirect($url);
    }
    
    # if the user has no instances at all, send them the error
    if (!@instances) {
        $self->print_click_back( "Error", "Sorry, you are not authorized to view this site.");	   
		exit;
    }
    
    my $title = 'title';
    my $content = <<EOM
    <h1>Welcome, $self->{fullname}!</h1>    
    You have access to the following pages:<br /><br />
    $page_list
EOM
;    
    $self->print_portal($content, $title);
}


#####################################################
#              WWW functions
#####################################################

###############################################################
###
### Analyse the form give in argument and save it in db if no error
###
###############################################################
####################################################################
#  Admin wants to delete one of users
####################################################################
sub www_delete_request {
    my $self = shift;
    
    my $referer = $FORM{referer} || $SESSION->{referer};
    my $escaped_referer = CGI::escape($referer);

    if(!$FORM{uid}){
	$self->print_click_back("Error",
			    "Sorry I didn't get a user uid to delete!");
	return 0;
    }
    
    my $u = Metadot::UserFactory->get_user( $FORM{uid} );
    
    if ( $USER->is_default_account( $u->email ) ) {
	
	$self->print_click_back("Error", "Sorry, system special users cannot be deleted");
	return 0;
    }

    my $fullname = " " . $u->webname;

	
    $self->three_choices( "Delete user: <br><i>$fullname?</i>",
			  "caution",
			  "Yes, delete user but leave his/her content untouched",
			  "index.pl?isa=User&amp;op=delete_ok&amp;uid=$FORM{uid}&amp;referer=$escaped_referer",
			  "Yes, delete user <b>and all his/her content</b>",
			  "index.pl?isa=User&amp;op=delete_ok&amp;uid=$FORM{uid}&amp;delete_user_content=1&amp;referer=$escaped_referer",
			  "Cancel", $referer );

    return 1;

}


###############################################################
### delete a user and all its dependencies..
###############################################################

sub www_delete_ok {
    
    my $self = shift;
    my  $uid = shift || $FORM{uid};
    my $res=0;
    
    my $referer = $FORM{referer} || $SESSION->{referer};

    my $delete_content_as_well = $FORM{delete_user_content};
    
    $self->debug_msg(3, "Metadot::User::Default::www_delete_ok(uid=$uid)");
    if(!$uid){
	$self->print_click_back("Error",
				"Sorry I don't know whom to delete (UID=$uid)");
	return 0;

    } else{

	my $u = Metadot::UserFactory->get_user($uid);

	$res=$u->delete($delete_content_as_well); #backend function :)
	
	if($res==1){
	    $self->debug_msg(4, "Metadot::User::Default::user deleted (uid=$uid)");
        $DISPLAY->print_choices("<Big>User deleted!</big><br>",
			       "confirm",
			       "Continue",
			       $referer
			      );
	}else{
	    $self->debug_msg(4, "Metadot::User::Default::problem: user not deleted (uid=$uid), error code is ($res)");
	    $self->print_click_back("Error deleting user",
				    "Please contact your system admin if this error persists.<br>Error code: $res");
	}
	return 1;
    }
}

my @SHOW_DETAIL_FIELDS = (
    {   field_name  => 'fullname',
        label       => 'Full Name',
        nowrap      => 1,
    },
    {   field_name  => 'nickname',
        label       => 'Nickname / Screenname',
        nowrap      => 1,
    },
    {   field_name  => 'email',
        label       => 'E-mail',
        nowrap      => 1,
    },
    {   field_name  => 'homepage',
        label       => 'Home Page',
    },
    {   field_name  => 'quote',
        label       => 'Quote',
    },
    {   field_name  => 'icq_number',
        label       => 'ICQ Numebr',
    },
    {   field_name  => 'pager_number',
        label       => 'Pager Number',
    },
    {   field_name  => 'uid',
        label       => 'Unique ID',
    },
);

sub _get_show_detail_fields {
    return @SHOW_DETAIL_FIELDS;
}

##################################################
### showUserDtails
### Show user informatoin
##################################################
sub www_show_details {
    my $self = shift;
    my $uid  = shift || $FORM{uid} || $self->uid();
    
    $self->debug_msg(3,"Metadot::User::Default::www_show_details($uid)");
    if(!$uid){
        $self->print_click_back("Error", "Sorry I didn't get a user uid (and I need one!)<br /><br />");
        return 0;
    }
    
  ######################################
  #### build and display the page
  ######################################

#   my $html = $self->show_top();
#   $html .= $self->show_title("User Information");
#   $html .= $self->_get_user_details_display(uid => $uid);
#   $html .= $self->show_bottom();
#   $self->print_http_header();
#   print $html;
    my $u = $USER->new($uid);

    my $content = $PARAMS->{show_user_directory} && $u->can('get_user_details_view') ? $u->get_user_details_view(view => 'businesscard') : $self->_get_user_details_display(uid => $uid);
    $DISPLAY->print_simple_layout({content=>$content, title=>"User Information"});
}

sub _get_user_details_display {
    my $self = shift;
    my %args = @_;

    my $uid = $args{uid};
    $self->debug_msg(3,"Metadot::User::Default::_get_user_details_display($uid)");

    my $s = "";
    my $u = Metadot::UserFactory->get_user($uid);

    if ($u) {
        my @fields = $u->_get_show_detail_fields();
        my %properties = ();
        foreach my $field( @fields) {
            my $field_name = $field->{field_name};
            if ($field_name eq 'email') {
                my $email = $u->email();
                $properties{$field_name} = "<a href='mailto:$email'>$email</a>";
            } elsif ($field_name eq 'homepage') {
                my $homepage = $u->homepage();
                if ($homepage) {
                    $properties{$field_name} = "<a href='".$self->make_url($homepage)."'>$homepage</a>";
                } else {
                    $properties{$field_name} = "&nbsp;";
                }
            } else {
                $properties{$field_name} = $u->$field_name() || '&nbsp;';
            }
        }
        
        $s = "\n\n<!-- USER INFORMATION -->\n\n";
        $s .= "<br /><br /><center><table border=\"1\", cellpadding=\"4\" cellspacing=\"0\" width='%80'></tr>\n\n";
        foreach my $field (@fields) {
            my $nowrap= $field->{nowrap} ? 'nowrap="nowrap"' : '';
            $s .= "<tr><td $nowrap><b>$field->{label}</b></td><td>$properties{$field->{field_name}}</td></tr>\n";
        }
        $s .= "</tr></table></center><br /><br />\n\n";
        $s .= "\n\n<!-- END OF USER INFORMATION -->\n\n";
    }else{
        # the user doesn't exist anymore
        $s = "\n\n<!-- USER INFORMATION -->\n\n";
        $s .= "<br /><br /><CENTER><b>Sorry, this user doesn't exist anymore!</b></CENTER><br /><br />";
        $s .= "\n\n<!-- END OF USER INFORMATION -->\n\n";
    }

    return $s;
}


#######################################################
#  Here user changes his settings
#######################################################

sub www_modify_self {
    my $self = shift;
    
    # calling backend method. 
    # the user will not see the "make admin"
    # radiobuttom because $show_admin_level is equal to zero,
    my $registrator = Metadot::RegisterFactory->get_register();
    
    my $uid = $SESSION->uid();
    my $show_admin_level = 0;

	my $form;
	my $form_name;
	if ($FORM{new_user}) {
		$form = $registrator->show_new_password_form($uid);
		$form_name = 'Creating your password';
	} else {
		$form = $registrator->show_modify_settings_form( $uid, $show_admin_level );
		$form_name = 'Modifying User Settings';
	}


    $DISPLAY->print_form($form, $form_name);
    
    return;
}

########################################################
# Function for admin to be able to modify user's
# settings
########################################################

sub www_modify {
    my $self = shift;

    my $uid = $FORM{uid};

    if( $USER->belongs_to_grp('ADMIN', $uid ) ) {

	unless( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {

	    $DISPLAY->print_click_back("Error: you are not allowed to modify this user's information.");
	    exit;
	}
    }

    # A little trick: if uid ne $SESSION->uid then it's an
    # admin - we should turn on 'show_admin_level'.
    my $registrator = Metadot::RegisterFactory->get_register();
    
    my $show_admin_level = ($uid ne $SESSION->uid() ) ? 1 : 0 ;
    
    my $form = $registrator->show_modify_settings_form( $uid, $show_admin_level );
    
    $DISPLAY->print_form($form, "Modifying User Settings");
    
    return;
}

####################################################################

sub www_save {
    my $self = shift;
    
    my $keep_admin_level = 0;
    
    my $uid = $SESSION->get_value('User_uid_to_modify');

    if( $USER->belongs_to_grp('ADMIN', $uid ) ) {

	unless( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {

	    $DISPLAY->print_click_back("Error: you are not allowed to modify this user's information.");
	    exit;
	}
    }

    my $registrator = Metadot::RegisterFactory->get_register();
    
    $registrator->save_modify_settings_form( $keep_admin_level );
    
    return;
}

####################################################################

sub www_save_self {
    my $self = shift;

    my $keep_admin_level = 1;

    my $registrator = Metadot::RegisterFactory->get_register();
	
	if ($FORM{new_user}) {
		$registrator->save_new_password_form( $keep_admin_level );
	} else {
		$registrator->save_modify_settings_form( $keep_admin_level );
	}

    return;
}

####################################################################
# initializing User object
#####################################################################
sub _initialize{
    my $self = shift;
    my $uid = $self->uid;
    
    if(defined ($uid)){
        #RESTRICTED-USER: using linkname1, linkname2 for restricted user properties    
        my $att = "userid, fullname, nickname, email, homepage, quote, icqnumber, pager, adminmode, debuglevel, lastonline, approvalneeded, password, ldapdn,max_total_upload_size,max_total_numberof_upload, linkname1, linkname2";
        
        (
            $self->{uid}, $self->{fullname}, $self->{nickname}, $self->{email},
            $self->{homepage}, $self->{quote}, $self->{icq_number},
            $self->{pager_number}, $self->{admin_mode},
            $self->{debug_level},
            $self->{last_online},
            $self->{approval_needed},
            $self->{password},
            $self->{ldap_dn},
            $self->{upload_size},
            $self->{upload_number},
            $self->{link_name_1},
            $self->{link_name_2},
        )
        = DBAccess->sqlSelect($att, "users", "userid='$uid'");
        
        my $level = $self->admin_level() || 1;
        # if a user is not an admin then we force debug to zero
        # otherwise s/he may get debug msg without being able
        # to switch them off since s/he is not an admin.
        
        # THIS FEATURE IS NOT EVEN USED ANY MORE

        #unless (defined($USER) && ( $USER->belongs_to_grp('SITE_MGR', $uid) || $USER->belongs_to_grp('ADMIN', $uid) )) {
        #    $self->{debug_level} = 0;
        #}
    }
}

#####################################################

sub debug_level {
    my $s = shift;
    my $level = shift;
    my $uid = $s->uid;
    $s->debug_msg(3, "Metadot::User::Default::debug_level: ". $s->{debug_level});

    (defined ($level))
      ? (DBAccess->sqlUpdate("users", "userid=$uid", (debuglevel => $level)),
	$s->{debug_level} = $level)
	: ($level = $s->{debug_level});
    return $s->{debug_level};
}


#####################################################
# delete user
# delete my page
# delete my portal
# delete subscription
# delete bb, test if bb still work when click on the name
# delete channels +channelitems
sub delete {
    my $self = shift;
    my $delete_content_as_well = shift;

    my $uid;
    $self->debug_msg(3, "Metadot::User::Default::delete(uid='$uid')");
    $uid = $self->uid();
    
    if(!$uid){
	$self->debug_msg(3, "Metadot::User::Default::Error User::delete(): I don't know what uid to delete");
	return 0;
    }

    my ($hpiid)=$self->has_portal();

    if( $delete_content_as_well && ( defined ($hpiid) && $hpiid != 0 ) ){

	print STDERR "DELETING USERS MYWEBSITE " . $self->fullname() . "\n";

	## we need to delete the tree of cat/item/discuss the users has created
	$self->debug_msg(3, "Metadot::User::Default::delete: deleting portal iid=$hpiid");
	
	my $c = Gizmo->new($hpiid);
	$c->delete("YES"); # yes = delete recursively sub-cat
	#Metadot::deleteCategory($hpiid, "YES");	
    }

    DBAccess->sqlDelete ("mypage", "userid=$uid");
    DBAccess->sqlDelete ("subscription", "userid=$uid");
    DBAccess->sqlDelete ("sessions", "userid=$uid");
    
    DBAccess->sqlDelete ("users", "userid=$uid");
    $self->debug_msg(4, "Metadot::User::Default::user deleted (userid=$uid)");
    my $admin_uid = MetadotConfig->get_reserved_uid("ADMIN");
    DBAccess->sqlDelete ("grpmembers", "userid=$uid");
    DBAccess->sqlUpdate ("grp", "userid=$uid", (-userid => $admin_uid));
    
    return 1;    
}

####################################################
sub has_portal{
  my $self = shift;
  my ($uid)=$self->uid;

  $self->debug_msg(5, "Metadot::User::Default::has_portal(uid='" . ($uid||"") . "')");
  if (!defined($uid) || !$uid){
    return 0;
  }

  my $portal_parent = MetadotConfig->get_reserved_iid('PORTAL_PARENT');

  my ($iid) = DBAccess->sqlSelect("iid", "instance", "parentiid=$portal_parent AND userid=$uid");

  if ($iid) {
    return $iid
  }else{
    return 0;
  }
}
####################################################
sub has_mypage{
  my $self = shift;
  my $uid = $self->uid;
  $self->debug_msg(5, "Metadot::User::Default::has_mypage(uid='$uid')");

  return 1 if (!defined($uid)); #we avoid creating zillion mypages

  my ($mp_elements) = DBAccess->sqlSelect("count(*)", "mypage", "userid=$uid");
  return $mp_elements;
}
####################################################
sub fullname{
  my $self=shift;
  my $val = shift;

  if (defined $val){
    $self->{fullname}=$val;
    return $self->{fullname};
  }
  return $self->{fullname} || $self->{nickname};

}
####################################################
sub webname{
  my $self=shift;

  return $self->{nickname} || $self->{fullname};

}
###################################################
sub belongs_to_grp{
    my $self= shift;
    my $grp = shift;
    my $uid = shift;
    
    return undef if (!defined($grp) || !defined($uid));
    
    if (Group->is_member($uid, $grp)) {
	return 1;
    } else {
	return undef;
    }
}

sub belongs_to_grp_id {
    my $self= shift;
    my $grp_id = shift;
    my $uid = shift;
    
    return undef if (!defined($grp_id) || !defined($uid));
    
    if (Group->is_member_of_grp_id($uid, $grp_id)) {
	return 1;
    } else {
	return undef;
    }
}



########################################################
# Returns the largest group (the one with most members)
# this user belongs to
sub largest_group {
  my $self = shift;
  my ($max, $largest);
  my $uid = $USER->{uid};

  my $groups = Group->get_groups_where_member($uid);

  foreach my $grp (@$groups) {   
      my $num = Group->number_of_members($grp);
      $max = $num, $largest = $grp if($num > $max);          
  }
  return $largest;
}

###################################################
sub is_owner {
  my $self= shift;
  my $object=shift || die "No object passed";
  my $uid = $self->uid();
  my $objectuid = $object->get_owner_uid();

  if (!$uid){
     return undef;
  } elsif ($uid == $objectuid) {
     return 1;
  } else {
     return undef;
  }
}
###################################################
sub is_admin{
    my $self= shift;
    
    my $value = $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $self );

    return $value
}


###################################################
sub is_logged_in {
    my $self= shift;
    my $session = shift || $SESSION;

# XXX [claudio]
# should move this sql query to a method in Session.pm
    
    if( defined $self->{_logged_in} ) {
	
	return $self->{_logged_in} 
    }
    
    my $uid = $self->uid();


    
    my $minimum_time = time - $PARAMS->{'sessionduration'};
    
    my $sessid = $session->session_id() || 0;
    
    my ($count) = defined $uid ?
      DBAccess->sqlSelect("count(*)", "sessions", "userid = $uid and (lastonline > $minimum_time) and (sessionid = '$sessid')") : 0;
    
    $self->{_logged_in} = $count;
    
#    if (!$count) {		# aju 02-21-02: if user is not logged in,
#	                        # then erase id info if flag is set.
# 	if( $PARAMS->{sessions_are_deleted_on_logout} 
#	    and not $self->{uid} eq MetadotConfig->get_reserved_uid('ANONYMOUS_USER') )  {
# 	    $session->erase();
# 	}
#    }
    
    return $count;
}

###################################################
sub save{
  my $self=shift;
#    print STDERR "in User::save, with upload_size " . $self->{upload_size} . "\n";
  my $db_now = Utility->db_now();
  DBAccess->sqlUpdate("users","userid='$self->{uid}'",
	    (
	     fullname => $self->{fullname},
	     nickname => $self->{nickname},
	     homepage => $self->{homepage},
	     email    => $self->{email},
	     icqnumber=> $self->{icq_number},
	     pager    => $self->{pager_number},
	     adminlevel=>$self->{admin_level},
	     quote    => $self->{quote},
             max_total_upload_size => $self->{upload_size},
             max_total_numberof_upload => $self->{upload_number},
	     approvalneeded => $self->{approval_needed},
	     updatedate     => $db_now,
	     ldapdn         => $self->{ldap_dn},
             lastonline     => $self->{last_online},
	    )
	   );

}
####################################################

sub save_password{
    my $self = shift;
    my $password= shift;
    
    my $uid = $self->uid();
    
    my $encryption_done = $PARAMS->{'encryption_done'};

    if ($encryption_done eq 'yes') {
	$password = $self->encrypt_password($password);
    }
    
    my $db_now = Utility->db_now();
    DBAccess->sqlUpdate("users","userid=$uid",
			(updatedate=>$db_now, password => $password)
		       );
  
}


#####################################################
sub can_use_email_address{
  my $self = shift;
  my $email = shift;

  my $uid = $self->uid();

  $email = DBAccess->quote( $email );

  my ($cnt)= DBAccess->sqlSelect("count(*)","users","email=$email and userid !=$uid");
  if ($cnt !=0){
    #email address already in use by another user
    return 0;
  }else{
    #email address not taken yet
    return 1;
  }

}
#####################################################
sub email_address_exists {
  my $s = shift;
  my $email = shift;

  $email = DBAccess->quote( $email );

  my ($uid) = DBAccess->sqlSelect("userid", "users", "email=$email");

  if($uid){
    return $uid;
  }
  return 0;
}

#####################################################
sub add{
    my $self = shift;

    $self->{upload_size} = -1;
    $self->{upload_number} =  -1;

    my $db_now = Utility->db_now();
    my $uid = DBAccess->sqlInsert("users",
			fullname => $self->{fullname},
			nickname => $self->{nickname},
			homepage => $self->{homepage},
			email    => $self->{email},
			icqnumber=> $self->{icq_number},
			pager    => $self->{pager_number},
			quote    => $self->{quote},
			password => $self->{password},
			approvalneeded => $self->{approval_needed},
            linkname1   => $self->{link_name_1}, #RESTRICTED-USER
            linkname2   => $self->{link_name_2}, #RESTRICTED-USER
			adminlevel  => 1,
			adminmode   => 0,
			debuglevel  => 0,
			updatedate  =>$db_now,
			creationdate=>$db_now,
			max_total_upload_size => $self->{upload_size},
			max_total_numberof_upload => $self->{upload_number},
			ldapdn  => $self->{ldap_dn}
		       );
    
    $self->uid($uid);
    
    # create default user portal
    my $portal = Portal->new(undef);
    
    #RESTRICTED-USER: added additional condition against creation
    if ($PARAMS->{my_website_creation} && !$self->is_restricted() ) {
	
	my ($hpiid) = $self->create_default_portal( $uid );
	
    }

    return $uid;
}

####################################################
### show the edit/delete button for admins
sub show_panel {
  my $self = shift;
  my $uid = $self->uid();
  my $approval_needed = $self->approval_needed() || "";
  my $str = "";
  $self->debug_msg(4, "Metadot::User::Default::show_panel(uid=$uid)");

  #if (!$SESSION->admin_mode()){ return "";}

  my $imageslocation = $PARAMS->{'imageslocation'};

  if ($SESSION->admin_mode()){
      $str.= "<a href='".$self->make_url("index.pl?isa=User&amp;op=modify&amp;uid=$uid")."'><img src='$imageslocation/obj/edit.gif' width=\"25\" height=\"11\" border=\"0\" align='top' alt='Edit Me!'></a>\n";

      #unless( $USER->is_default_account( $self->email() ) ) {
      # save a query
      #print STDERR "checking if '".$self->fullname()."' is a default account name\n";
      unless( $USER->is_default_account_name( $self->fullname() ) ) {
          $str.= "<a href='".$self->make_url("index.pl?isa=User&amp;op=delete_request&amp;uid=$uid")."'><img src=$imageslocation/obj/del.gif width=\"11\" height=\"11\" border=\"0\" align='top' alt='Delete Me!'></a>\n";
      }
  }

  if ($approval_needed eq "required"){
    $str.= "<a href='".$self->make_url("index.pl?isa=User&amp;op=accept_registration&amp;uid=$uid")."'><img src=$imageslocation/obj/accept.gif width=\"40\" height=\"13\" border=\"0\" align='top' alt='Accept Me!'></a>\n";
    $str.= "<a href='".$self->make_url("index.pl?isa=User&amp;op=reject_registration&amp;uid=$uid")."'><img src=$imageslocation/obj/reject.gif width=\"40\" height=\"13\" border=\"0\" align='top' alt='Reject Me!'></a>\n";
  }

  return $str;
}


####################################################################
#  Admin accepts a new user
####################################################################
sub www_accept_registration {
    my $self = shift;
    
    if(!$FORM{uid}){
	$self->print_click_back("Error",
				"Sorry I didn't get a user uid to accept!");
	return 0;
    }
    
    my $u = Metadot::UserFactory->get_user($FORM{uid});
    my $fullname .= " " . $u->webname;
    my $email .= " " . $u->email;
    $u->approval_needed("accepted");
    $u->save;


    my $registrator = Metadot::RegisterFactory->get_register();    
    $registrator->send_approval_message( $u, 'accepted' );
    
    $self->two_choices("<b><i>$fullname</i></b> has been accepted.",
		       "confirm",
		       "Pending Users List",
		       "index.pl?isa=User&&amp;op=show_users&amp;pending=true",
		       "Home",
		       "index.pl"
		      );
    
    return 1;
}

####################################################################
#  Admin rejects a new user.
####################################################################
sub www_reject_registration {
    my $self = shift;
    
    if(!$FORM{uid}){
	$self->print_click_back("Error", 
				"Sorry I didn't get a user uid to reject!");
	return 0;
    }
	
    my $u = Metadot::UserFactory->get_user($FORM{uid});
    my $fullname .= " " . $u->webname;
    my $email .= " " . $u->email;
    $u->approval_needed("rejected");
    $u->save;
	
    my $registrator = Metadot::RegisterFactory->get_register();    
    $registrator->send_approval_message( $u, 'rejected' );
    
    $self->two_choices("<b><i>$fullname</i></b> has been rejected.", 
		       "confirm",
		       "Pending Users List", 
		       "index.pl?isa=User&&amp;op=show_users&amp;pending=true",
		       "Home", 
		       "index.pl"
		      );
    
    return 1;
}

####################################################################
# Returns the iid of the clipboard category associated with this 
# user. Will create instance record if not already there.
####################################################################
sub get_clipboard_iid {
    my $self = shift;
    
    $self->debug_msg(4, "Metadot::User::Default::get_clipboard()");
    
    my ($uid)=$self->uid;
    
    if (!defined($uid) || !$uid){
	return 0;
    }
    
    # XXX [claudio] note that this requires a sql query each time the clipboard
    # is used (better have a direct pointer to this instance in the user
    # object)  
    
  my $parent_clipboard = MetadotConfig->get_reserved_iid('PARENT_CLIPBOARD');
    my $admin_uid = MetadotConfig->get_reserved_uid('ADMIN');
    
    my ($iid) = DBAccess->sqlSelect("iid", "instance", "parentiid=$parent_clipboard AND userid=$uid");
    
    if ($iid) {
	return $iid
    } else {
	# create a clipboard instance for this user 
	$self->debug_msg(3, "Metadot::User::Default::get_clipboard()::creating for user $uid");
	
	my $cat = Category->new({
				 uid => $uid,
				 parent_id => $parent_clipboard,
				 name => "Clipboard for $USER->{nickname}",
				 description => "Clipboard for $USER->{nickname}", 
				 keywords => "Clipboard $USER->{nickname}",
				 display_look => 'LULI',
				 created_by => $admin_uid,
				});
	my $iid = $cat->add();

	# make sure the new clipboard is not public accsessible
	
	$cat->set_bundle_level( 'DISP', 8 );
	$cat->set_bundle_level( 'MOD', 8 );
	$cat->save();
	
	return $iid;
    }
}


####################################################
sub www_require_pager_form {
  my $self = shift;
  my $referrer = shift; # url for add
  my $target   = shift; # url after add is done

  my $form = "";

  $form .= "<table><tr><td>"
          ."<form action='index.pl' method=post>\n"
          ."<input type='hidden' name='isa' value='User'>\n"
          ."<input type='hidden' name='op' value='process_pager_form'>\n"
          ."<input type='hidden' name='referrer' value='$referrer'>\n"
          ."<input type='hidden' name='target' value='$target'>\n"
          ."</td></tr>\n";

  $form .= "<tr><td>You do not have a Pager address set. Please provide one below.<br>\n"
          ."</td></tr>\n";

  $form .= "<tr><td><input type=text name=pageraddress value='$FORM{pageraddress}'>"
          ."</td></tr>\n";

  $form .= "<tr><td><input type=submit value='Continue'>"
          ."</form>\n"
          ."</td></tr></table>\n"; 

  $DISPLAY->print_form($form, "Add Pager");

}

####################################################
sub www_process_pager_form {
  my $self = shift;
  my $referrer = $FORM{referrer};
  my $target   = $FORM{target};

  my $verify = new Form::Verify;

  $verify->verify('email', 'pager address', $FORM{pageraddress});
  if (my $myerror = $verify->errors){
    $self->print_click_back("Sorry", "$myerror.\n");
    exit;
  } else {
    $self->pager_number($FORM{pageraddress});
    $self->save;
#    $self->pager_accepted($referrer);
    $target = escape($target);
    $self->md_redirect($referrer . "&amp;target=$target");
    exit;
  }
}

####################################################
sub pager_accepted {
  my $self = shift;
  my $referrer = shift;

  my$form = "Your pager address has been accepted.\n"
           ."<table align='right'><tr><td>"
           ."<a href='".$self->make_url("$referrer")."'>Continue</a>\n"
           ."</td></tr></table>\n";

  $DISPLAY->print_form($form, "Pager Accepted");

}
 
####################################################

sub www_require_icq_form {
  my $self = shift;
  my $referrer = shift; # url for add
  my $target   = shift; # url after add is done

  my $form = "";

  $form .= "<table><tr><td>"
          ."<form action='index.pl' method=post>\n"
          ."<input type='hidden' name='isa' value='User'>\n"
          ."<input type='hidden' name='op' value='process_icq_form'>\n"
          ."<input type='hidden' name='referrer' value='$referrer'>\n"
          ."<input type='hidden' name='target' value='$target'>\n"
          ."</td></tr>\n";

  $form .= "<tr><td>You do not have an ICQ number set. Please provide one below.<br>\n"
          ."</td></tr>\n";

  $form .= "<tr><td><input type=text name=icq_number value='$FORM{icq_numbner}'>"
          ."</td></tr>\n";

  $form .= "<tr><td><input type=submit value='Continue'>"
          ."</form>\n"
          ."</td></tr></table>\n";

  $DISPLAY->print_form($form, "Add ICQ");

}

####################################################
sub www_process_icq_form {
  my $self = shift;
  my $referrer = $FORM{referrer};
  my $target   = $FORM{target};

  my $verify = new Form::Verify;

  $verify->verify('icq', 'icq number', $FORM{icq_number});
  my $myerror =  $verify->errors;
  if ($myerror){
    $self->print_click_back("Sorry", "The $myerror is invalid.\n");
    exit;
  } else {
    $self->icq_number($FORM{icq_number});
    $self->save;
#    $self->icq_accepted($referrer);
    $target = escape($target);
    $self->md_redirect($referrer . "&amp;target=$target");
    exit;
  }
}

####################################################
sub icq_accepted {
  my $self = shift;
  my $referrer = shift;

  my$form = "Your icq numner has been accepted.\n"
           ."<table align='right'><tr><td>"
           ."<a href='".$self->make_url("$referrer")."'>Continue</a>\n"
           ."</td></tr></table>\n";

  $DISPLAY->print_form($form, "ICQ Accepted");

}


####################################################
sub DESTROY{
  # EMPTY destroy method to please mod_perl when
  # it complains about a missing DESTROY function.

}


###################################################
#  Show settings form functions
###################################################
sub create_mypage {
    my $self = shift ;
    my $uid = shift ;

    $self->debug_msg(3, "Metadot::User::Default::create_mypage");
    
    $uid = undef unless (Utility->is_valid_integer($uid));   #secu_200401
    return if (!defined $uid);
    
    my $anon_user = MetadotConfig->get_reserved_uid('ANONYMOUS_USER');

    #### Copy channels from user uid=-1 who is the default
    my ($default) = DBAccess->sqlSelectMany("id, isa, col, mypageorder", "mypage", "userid=$anon_user");
    my $db_now = Utility->db_now();
    while (my ($id, $isa, $col, $mypageorder)=$default->fetchrow()) {
	DBAccess->sqlInsert("mypage", userid => $uid, id => $id, isa => $isa, col => $col,
			    mypageorder => $mypageorder, updatedate  => '', creationdate=> $db_now);
    }
    $default->finish();

    #### Copy links from user uid=-1 who is the default
    my @link_array = $self->load_mypage_links($anon_user);

    my $i =1;
    foreach my $link_hash (@link_array) {
	
	$USER->save_mypage_link( $uid, $i, $link_hash->{name}, $link_hash->{value} );
	$i++;
    }
    
    $self->stock_gizmos($uid);
    return 1;
}


sub stock_gizmos { #here is where we fill the new user's gizmo tools with the same gizmotool items that the anonymous user has.
  my $self = shift;
  my $uid = shift;

  $uid = DBAccess->quote( $uid );

  my $anon_user = MetadotConfig->get_reserved_uid('ANONYMOUS_USER');

  my ($anon_mpid, $sth, $sth2, $ciid, $valid_anon_mpid, $anon_isa, $isa, $mpid);
  ($sth) = DBAccess->sqlSelectMany ("mpid, isa" ,"mypage", "userid=$anon_user"); #this gives us all the mpids of the anonymous user
  while (($anon_mpid, $anon_isa) = $sth->fetchrow() ) {
    ($sth2) = DBAccess->sqlSelectMany("mpid, ciid", "gizmotoolitem", "mpid = $anon_mpid"); #we want only the mpids of the anonymous use that are gizmotools.
    while (($mpid, $ciid) = $sth2->fetchrow() ) {
      ($isa) = DBAccess->sqlSelect("isa", "mypage", "mpid = $mpid");  #this gets the isa (name) of the anonymous user's mpids.
      ($mpid) = DBAccess->sqlSelect("mpid", "mypage", "isa = '$isa' and userid=$uid");        #this gets the mpid of the new user that we should put the appropriate ciid with.
      DBAccess->sqlInsert("gizmotoolitem", mpid => $mpid, ciid => $ciid);
    }
  }
}

sub get_uid_from_email {
    my $self = shift;
    my $email = shift;

    $email = DBAccess->quote( $email );

    my ($uid) = DBAccess->sqlSelect("userid", "users", "email=$email");

    return $uid;
}


sub get_uid_from_ldap_uid {
    my $self = shift;
    my $ldap_uid = shift;
    
    my ($uid) = DBAccess->sqlSelect("userid", "users", "ldapdn=".DBAccess->quote($ldap_uid));  # secu_200401

    return $uid;
}


sub get_email_from_uid {
    my $self = shift;
    my $uid = shift;
  
    $uid = DBAccess->quote( $uid );
  
    my $email;
    eval {
	($email) = DBAccess->sqlSelect("email", "users", "userid=$uid");
    };
    if ($@) {
	print STDERR $@ . "\n";
	return '';
    }
    
    return $email;
}


sub get_fullname_from_uid {
    my $self = shift;
    my $uid = shift;

    $uid = DBAccess->quote( $uid );
    
    my $fullname;
    eval {
	($fullname) = DBAccess->sqlSelect("fullname", "users", "userid=$uid");
    };
    if ($@) {
	print STDERR $@ . "\n";
	return '';
    }
    
    return $fullname;
}

sub get_nickname_from_uid {
    my $self = shift;
    my $uid = shift;

    $uid = DBAccess->quote( $uid );
    
    my $nick;
    eval {
	($nick) = DBAccess->sqlSelect("nickname", "users", "userid=$uid");
    };
    if ($@) {
	print STDERR $@ . "\n";
	return '';
    }
    
    return $nick;
}


sub get_fullname_from_email {
    my $self = shift;
    my $email = shift;

    $email = DBAccess->quote( $email );
    
    my ($fullname) = DBAccess->sqlSelect("fullname", "users", "email=$email");
    
    return $fullname;
}


sub get_email_from_fullname {
    my $self = shift;
    my $fullname = shift;
    
    $fullname = DBAccess->quote( $fullname );
    
    my ($email) = DBAccess->sqlSelect("email", "users", "fullname=$fullname");
        
    return $email;
}

sub get_vitals_from_uid {
    my $self = shift;
    my $uid = shift;

    $uid = DBAccess->quote( $uid );

    my ($name, $nickname, $email) = DBAccess->sqlSelect ("fullname, nickname, email", "users", "userid = $uid");
    return ($name, $nickname, $email);
}

sub substitute{
    my $self = shift;
    
    my ($message, $fullname, $email, $password, $nickname) = @_;
    
    $message=~s/<<forgot_pwd_url>>/$PARAMS->{basename}\/index.pl?isa=Session&amp;op=forgot_password/gi;
    $message=~s/<<fullname>>/$fullname/gi;
    $message=~s/<<website_url>>/$PARAMS->{basename}\/index.pl/gi;
    $message=~s/<<email>>/$email/gi;
    $message=~s/<<password>>/$password/gi;
    $message=~s/<<nickname>>/$nickname/gi;
    $message=~s/<<profile_url>>/$PARAMS->{basename}\/index.pl?isa=User&amp;op=modify_self/gi;
    $message=~s/<<subscription_url>>/$PARAMS->{basename}\/userchannel.pl?op=select/gi;

    return $message;
}


# will return a reference to an array containing all users
sub get_all_users {
    my $self = shift;

    my $users = DBAccess->sqlSelectMany ("userid", "users");  
    my @users;
    while(my ($uid) = $users->fetchrow) {
	push @users, Metadot::UserFactory->get_user($uid);
    }

    return \@users;
}

sub get_all_active_users {
    my $self = shift;

    my $users = DBAccess->sqlSelectMany( "userid", "users", "(approvalneeded IS NULL OR NOT(approvalneeded = 'rejected' OR approvalneeded = 'required'))" );
  
    my @users;
    while(my ($uid) = $users->fetchrow) {
	push @users, Metadot::UserFactory->get_user($uid);
    }

    return \@users;
}


sub user_exists {
    my $proto = shift;
    my $uid = shift;

    die "must provide uid" unless defined $uid;

    my $user = Metadot::UserFactory->get_user($uid);


}


sub get_uploads_space_and_num_files_quota {
    my $self = shift;
    my $uid = shift;

    $uid = (defined $uid) ? $uid : $self->uid();
    defined $uid || die "couldn't get a uid\n";

    $uid = DBAccess->quote( $uid );
    
    my ($upload_size_allowed, $upload_number_allowed) = 
      DBAccess->sqlSelect("max_total_upload_size, max_total_numberof_upload", "users", "userid=$uid");

    if($upload_size_allowed == -1){
        $upload_size_allowed = $PARAMS->{max_total_upload_size};
    }

    if($upload_number_allowed == -1){
        $upload_number_allowed = $PARAMS->{max_total_number_upload};
    }

    return ($upload_size_allowed, $upload_number_allowed)
}


sub get_used_uploads_space_and_num_files {
    my $self = shift;
    my $uid = shift;

    $uid = (defined $uid) ? $uid : $self->uid();

    my ( $size, $number ) = Metadot::UploadsManager->space_used( $uid );

    $uid = DBAccess->quote( $uid );

    my ($c) = DBAccess->sqlSelectMany("file1_size", "instance", "userid=$uid and file1_size>=0");

    while ( my $s = $c->fetchrow() ){
	$size += $s;
    }

    $number += $c->rows;	#this is ok because all $c has beenfetched

    $c->finish();

    return ($size, $number);
}


sub is_default_account {
    my $class = shift;
    my $email = shift;
    
    my $fullname = $class->get_fullname_from_email( $email );
    return $class->is_default_account_name($fullname);
}


sub is_default_account_name {
    my $class = shift;
    my $name  = shift;

    # trim white space
    $name =~ s/^\s*//;
    $name =~ s/\s*$//;

    #print STDERR "Checking '$name'\n";
    return $default_account_names{lc $name};
}


sub www_show_users {
    my $self = shift;
    my $param = shift;
    my @external_filters = defined($param) ? @{$param} : ();
    $self->debug_msg(3, "Metadot::User::Default::www_show_users()");

    my @filters = ();
    my $add_tables;

    if ($FORM{name}) {

	my $value = uc(DBAccess->quote( "%$FORM{name}%" ));

	push @filters, "UPPER(fullname) like $value";
    }

    if ($FORM{nickname}) {

	my $value = uc(DBAccess->quote( "%$FORM{nickname}%" ));

        push @filters, "UPPER(nickname) like $value";
    }
    if ($FORM{email}) {

	my $value = uc(DBAccess->quote( "%$FORM{email}%" ));

       push @filters, "UPPER(email) like $value";
    }

    if ($FORM{pending}){
        push @filters, "approvalneeded = 'required'";
    }

    if ($FORM{who_online}) {
	push @filters, "users.userid=sessions.userid and ".$SESSION->get_last_online_clause();
	$add_tables = 'sessions';
    }
    my $filter = join(' and ', @filters);

    #$DISPLAY->print_http_header;
    my $all_url = "";

    # add a 'show all users' link before generating the table if a search filter has been defined.
    my $iid;
    if ($filter ne "") {
    	$iid = $FORM{iid} || 0 ;
    	$all_url = "index.pl?iid=$iid&amp;isa=User&amp;op=show_users&amp;threads=$FORM{threads}&amp;order=$FORM{order}";
    	$all_url = "\n<a href=\"" . $self->make_url($all_url) . "\">Show All Users</a><br>\n";
    }

    #print $self->show_top($FORM{iid}),
    #$self->show_title("User Directory"),
    #$all_url,

    #$self->show_user_list( {filter=>$filter, startfrom=>$FORM{from}, maxitem=>$FORM{nb}, add_tables =>$add_tables}),
    #"\n\n<!-- End of Users -->\n\n",
    #$self->show_bottom;

    my $content = $all_url.$self->show_user_list( {filter=>$filter, startfrom=>$FORM{from}, maxitem=>$FORM{nb}, add_tables =>$add_tables});
    $DISPLAY->print_simple_layout({content=>$content, title=>"User Directory", iid=>$iid});
}

##################################################
### show_user_list
### Show a list of user
### where condition clause as argument
##################################################
sub show_user_list {
    my $self = shift ;
    my ($filter, $startfrom, $maxitem) = @_;
    my $s1 = "";
    my $where;
    my $admin;
    my $c;
    my $oldi=0;
    my ($uid, $fullname, $nickname, $email, $uploadsize, $uploadnumber);
    my $portal="";
    my $hpiid=0;
    $self->debug_msg(3, "Metadot::User::Default::show_user_list(filter=$filter)");

    my $table_clause = "users";

    # Filter is now a vector into an args hash. This maintains backward compatibility.
    my $args;
    if (ref($filter) eq 'HASH') {
	$args 	= $filter;
	$filter 	= $args->{filter};
	$startfrom 	= $args->{startfrom};
	$maxitem 	= $args->{maxitem};
	my $add_tables = $args->{add_tables}; # this must be an array ref

	# add tables to the table clause if needed
	if ($add_tables) {
	    $add_tables = [$add_tables] unless (ref($add_tables) eq 'ARRAY');
	    $table_clause = join(',', ($table_clause, @{$add_tables}));
	}

    }

    if ($USER->is_admin) {
	$where = $filter || "1=1";
    } else {
	$where = "userid>0";
	$where .= " and (approvalneeded IS NULL OR approvalneeded not in ('required', 'rejected'))";
	$where .= " and $filter" if ($filter);
    }

    ### select user's portal iid

    my $message;
    if($FORM{uid}){
        my $u = $USER->new($FORM{uid});
        my $fullname .= " " . $u->webname;
        $message ="$fullname has been accepted and notified via email.";
    }
    #### table
    if($message){
        $s1 .= "<font color='#18bb94'>$message</font><br>";
    }
    my $order;
    my $order_name_value_pair;

    if ($FORM{'order'}) {
	if ($FORM{order} eq "desc") {
	    $order = " DESC";
	    $order_name_value_pair = "&amp;order=asc";
	}
    } else {
	$order = "ASC";
	$order_name_value_pair = "&amp;order=desc";
    }

    my ($name_order_arrow, $nickname_order_arrow, $email_order_arrow) = (undef, undef, undef);

    my $imageslocation = $PARAMS->{'imageslocation'};


    $FORM{'threads'} = 0 unless $FORM{'threads'};

    if ($FORM{'threads'} == 1) {
	if ($FORM{'order'} eq 'desc') {
	    $nickname_order_arrow = "<img src=\"$imageslocation/obj/triangle_down.gif\" border=\"0\" alt=\"Sorted Descending\">";
	} else {
	    $nickname_order_arrow = "<img src=\"$imageslocation/obj/triangle_up.gif\" border=\"0\" alt=\"Sorted Ascending\">";
	}
    } elsif ($FORM{'threads'} == 2) {
	if ($FORM{'order'} eq 'desc') {
	    $email_order_arrow = "<img src=\"$imageslocation/obj/triangle_down.gif\" border=\"0\" alt=\"Sorted Descending\">";
	} else {
	    $email_order_arrow = "<img src=\"$imageslocation/obj/triangle_up.gif\" border=\"0\" alt=\"Sorted Ascending\">";
	}
    } else {
	if (($FORM{'order'} || 0) eq 'desc') {
	    $name_order_arrow = "<img src=\"$imageslocation/obj/triangle_down.gif\" border=\"0\" alt=\"Sorted Descending\">";
	} else {
	    $name_order_arrow = "<img src=\"$imageslocation/obj/triangle_up.gif\" border=\"0\" alt=\"Sorted Ascending\">";
	}
    }

	my $other;
    if ($FORM{threads} == 1){
        $other ="order by users.nickname $order";
    } elsif ($FORM{threads} == 2){
        $other = "order by users.email $order";
    } else {
        $other = "order by users.fullname $order";
    }

    $maxitem = 100 if (!$maxitem);   #PREVIOUS NEXT MAGIC
    $startfrom = 0  if(!$startfrom); #PREVIOUS NEXT MAGIC
    my $from = $startfrom;           #PREVIOUS NEXT MAGIC

    my $i=0;
    
    #aju - add 'distinct' to query to fix online count inconsistency
	($c)=DBAccess->sqlSelectMany("distinct users.userid, users.fullname, users.nickname, users.email, users.max_total_upload_size, users.max_total_numberof_upload", $table_clause, $where, $other);
    # my ($totalrows)=$c->rows;
	# rows function is not reliable for counting the rows retrieved by SELECT
	my ($totalrows) = DBAccess->sqlSelect("count(*)",$table_clause, $where, $other);

    my $max_total_upload_size = $PARAMS->{'max_total_upload_size'};
    my $max_total_number_upload = $PARAMS->{'max_total_number_upload'};

    if($totalrows <= $maxitem){
        $s1 .="<br><font size=-1><I><b>Found $totalrows entries</b></I></font>";
    }

    $email_order_arrow = $email_order_arrow ? $email_order_arrow : "";
    $nickname_order_arrow = $nickname_order_arrow ? $nickname_order_arrow : "";
    $name_order_arrow = $name_order_arrow ? $name_order_arrow : "";
    
    if ($USER->is_admin) {         
            $s1 .= "<center><h2><a href=\"index.pl?isa=Register&amp;op=register_form&amp;admin_mode=true\">Register a new user</a></h2></center>\n";
    }
    
    $s1 .= <<TEXT; 
<center><table border="0" cellpadding = 0 cellspacing = 2>
                <tr>
                        <td colspan = 3>
                                <center><H2>Search for a user</H2></center>
                        </td>
                </tr>
                <tr>
                        <td>
                                Name
                        </td>
                        <td>
                                Nickname / Screenname
                        </td>
                        <td>
                                Email address
                        </td>
                </tr>
                <tr>
                <tr>
                        <td>
                                <form method = post action = index.pl>
                                <input type=hidden name=isa value=User>
                                <input type=hidden name=op value=show_users>
                                <input type=hidden name=iid value = 0>
                                <input type=text name = name>
                        </td>
                        <td>
                                <input type=text name=nickname>
                        </td>
                        <td>
                                <input type=text name=email>
                        </td>
                </tr>
                <tr>
                        <td colspan = 3>
                                <center><input type=submit value = 'Start Search'></center></form>
                        </td>
                </tr>
         </table></center><br><center>
         <table border="1" cellpadding="4" cellspacing="0" width="100%">
            <tr>
              <td valign='middle'><font size=-1><b><a class='prevnextbar' href=
TEXT
;

    my $base_url = $self->make_url("index.pl?iid=0&amp;isa=User&amp;op=show_users&amp;pending=$FORM{pending}&amp;who_online=$FORM{who_online}");
    $s1 .= $base_url."&amp;threads=0$order_name_value_pair" .
    ">$name_order_arrow Name</a> </b> <I><a class='prevnextbar' href=". 
      $base_url."&amp;threads=1$order_name_value_pair" .
     ">(nickname) $nickname_order_arrow</a><I></font></td>
                           <td valign='middle'><font size=-1><b>ID</b></font></td>
              <td><b>Portal</b></td>
                 <td valign='middle'><font size=-1><b><a class='prevnextbar' href=" .
		   $base_url."&amp;threads=2$order_name_value_pair" .
      ">$email_order_arrow Email address</a></b></font></td>     <td valign='middle'><font size=-1>Max. total upload size (MB)<br><font size=-5>(Default: $max_total_upload_size MB)</font></td>                     <td valign='middle'><font size=-1>Max. total number of upload<br><font size=-5>(Default: $max_total_number_upload)</font></td>    </tr>\n";

    
    while((($uid, 
	    $fullname,
	    $nickname, 
	    $email, 
	    $uploadsize,
	    $uploadnumber) = $c->fetchrow()) && ($i < $maxitem)) {

	if(($uploadsize || 0) == -1) {

	    $uploadsize = "Default";
	} elsif (!$uploadsize) {

	    $uploadsize = "";
	}

	if(($uploadnumber || 0) == -1){

	    $uploadnumber = "Default";
	} elsif (!$uploadnumber) {

	    $uploadnumber = "";
	}

	if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC

	$i++;

	# skip hidden users only when user doesn't have server_setup access
	unless( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

	    if ( $USER->is_hidden( $uid ) ) { next }
	}

	if( $USER->belongs_to_grp('SITE_MGR', $uid) || $USER->belongs_to_grp('ADMIN', $uid) ) {
	    
	    $fullname="<b>$fullname</b>";
	}
	### Prepare the user portal link
	my $u = $USER->new($uid);
	($hpiid) = $u->has_portal();
	$portal = "&nbsp;";
	$portal = "<a href=".$self->make_url("index.pl?iid=$hpiid").">Portal</a>\n" if ($hpiid);
	
	if ($nickname) {
	    $nickname = "<I>($nickname)</I>\n";
	} else {
	    $nickname = "";
	}

	my $rejected_string = "";
	if (($u->approval_needed() || "") eq "rejected"){
	    $rejected_string = "<font color=#FF0000>(rejected)</font>";
	}
	
	$email = "<a href=\"mailto:$email\">$email</a>\n" if ($email);
	
	$s1 .= "<font size=-3><tr><td>\n";
	
	if($USER->is_admin){

	    $s1 .= $u->show_panel();	    
	}

	$rejected_string = $rejected_string ? $rejected_string : "";
	
	$s1 .= "<a href='".$self->make_url("index.pl?isa=User&amp;op=show_details&amp;uid=$uid")."'>$fullname</a> $nickname $rejected_string</td>\n";
	
	$s1 .= "<td>$uid</td>\n<td>$portal</td><td>$email</td><td>$uploadsize</td><td>$uploadnumber</td></tr></font>\n";
    }
    
    $c->finish();
    
    my $iid = $FORM{iid} || 0 ;
    my $url = "index.pl?iid=$iid&amp;isa=User&&amp;op=show_users&amp;threads=$FORM{threads}&amp;order=$FORM{order}";
    
    my $switch = "";
    my ($pending_approval) = DBAccess->sqlSelect("count(*)", "users", "approvalneeded='required'");
    if (($USER->is_admin) && ($pending_approval || (($FORM{pending} || "") eq 'true'))){
	my $link = ($FORM{pending} eq 'true') ?
          "<a class='prevnextbar' href='".$self->make_url("$url")."'>All Users</a>":
	    "<a class='prevnextbar' href='".$self->make_url("$url&amp;pending=true")."'>Pending users only</a>";
	$switch = "<table width=\"100%\"><tr><td align='right'>[ $link ] </td></tr></table>";
    }
    
    # display online users
    my $who_online = $SESSION->get_members_online();
    if (($USER->is_admin) && ($who_online || (($FORM{who_online} || "") eq 'true'))){
	my $link = ($FORM{who_online} eq 'true') ?
            "<a class='prevnextbar' href='".$self->make_url("$url")."'>All Users</a>":
	      "<a class='prevnextbar' href='".$self->make_url("$url&amp;who_online=true")."'>Show $who_online online users only</a>";
	$switch .= "<table width=\"100%\"><tr><td align='right'>[ $link ] </td></tr></table>";
    }

  
  #PREVIOUS NEXT MAGIC
    my $pn = "" ;
    $pn = Components->get_previous_next_button($maxitem, $i, $totalrows, $startfrom, $url) if ($totalrows > $maxitem);

    $s1.="</td></td></td></tr></table></center><br>";
    my $page = "$switch\n$pn\n$s1\n$pn\n<br><I>Note: Names in bold are administrators</I>\n\n";
    return $page;
}

sub show_user_object_list {
	# shows a list of users, given a list of user objects	
	my $self = shift;
}

# XXX [claudio] note that this depends on groups table structure
sub get_emails_by_grp_id {
    my $self = shift;
    my $grp_id = shift;
    my $recurse = shift;

    $grp_id = undef unless (Utility->is_valid_integer($grp_id));   #secu_200401
    defined($grp_id) || die("Must provide a group ID to query against.\n");

    my $users = DBAccess->sqlSelectMany ('u.email','users u, grpmembers gm',"gm.grp_id='$grp_id' AND u.userid=gm.userid");
    my @user_list;

    while (my ($email) = $users->fetchrow) {
        push @user_list, $email;
    }

    #aju
    if ($recurse) {
	# see if the group has any subgroups
	my $subgroups =
	    DBAccess->sqlSelectMany ('subgrp_id', 'grpmembers', "grp_id = '$grp_id' and subgrp_id > 0");
	while (my ($sg) = $subgroups->fetchrow) {
	    my $sg_user_list = $self->get_emails_by_grp_id($sg, $recurse);
	    push @user_list, @$sg_user_list;
	}
    }
    return \@user_list;
}

# XXX [claudio] note that this depends on groups table structure
sub get_emails_by_grp_name {
    my $class = shift;
    my $grp_name = shift || die("Must provide a group name to query against.\n");
    my $recurse = shift;

    my $users = DBAccess->sqlSelectMany('u.email','users u, grpmembers gm, grp g',"g.group_name=".DBAccess->quote($grp_name)." AND gm.grp_id=g.grp_id AND u.userid=gm.userid");  #secu_200401

    my @user_list;
    while (my ($email) = $users->fetchrow) {
        push @user_list, $email;
    }

    #aju
    if ($recurse) {
	# get the group id from its name
	my ($grp_id) = DBAccess->sqlSelect("grp_id", "grp", "group_name = '$grp_name'");
	if ($grp_id) {
	    # see if the group has any subgroups
	    my $subgroups =
		DBAccess->sqlSelectMany ('subgrp_id', 'grpmembers', "grp_id = '$grp_id' and subgrp_id > 0");
	    while (my ($sg) = $subgroups->fetchrow) {
		my $sg_user_list = $class->get_emails_by_grp_id($sg, $recurse);
		push @user_list, @$sg_user_list;
	    }
	}
    }
    return \@user_list;
}


# XXX [claudio] note that this depends on groups table structure
sub get_users_in_group {
    my $self = shift;
    my $grp_id = shift;
    my $field = shift;
    my $order = shift;
        
    $grp_id = undef unless (Utility->is_valid_integer($grp_id));   #secu_200401
    defined($grp_id) || die("Must provide a group ID to query against.\n");
    
    ## must use field aliasing otherwise sort will break
    my $rows = DBAccess->sqlSelectMany('users.email as email, users.fullname as fullname, users.nickname as nickname, users.userid as userid, grpmembers.is_manager as is_manager',
				       'users,grpmembers',
				       "grpmembers.grp_id='$grp_id' and grpmembers.userid=users.userid",
				       "ORDER BY $field $order");


    my $result;
    while (my $row = $rows->fetchrow_hashref() ) {
	
	push @$result, $row;	
	
    }

    return $result;
    
}


# XXX [claudio] note that this depends on groups table structure
# returns a reference to an array of hashes containing the data
sub get_groups_data_by_owner {
    my $self = shift;
    my $field = shift;
    my $order = shift;

    my $sth = DBAccess->sqlSelectMany("grp.group_name as group_name,
                                       grp.grp_id as grp_id,
                                       users.email as email,
                                       users.fullname as owner,
                                       users.nickname as nick,
                                       grp.group_description as description",
				      'grp, users',
				      'grp.userid = users.userid',
				      "order by $field $order");

    my $result;
    while ( my $row = $sth->fetchrow_hashref ) {

	push @$result, $row;	
    }

    return $result;
}



sub logout {
    my $self = shift;
    my $session = shift || $SESSION;

    $self->{_logged_in} = 0;

    if( $PARAMS->{sessions_are_deleted_on_logout} 
	and not $self->{uid} eq MetadotConfig->get_reserved_uid('ANONYMOUS_USER') ) {

	$session->erase();
	$USER = Metadot::UserFactory->get_user( MetadotConfig->get_reserved_uid('ANONYMOUS_USER') );
	

    } else {

	$session->set_value('GB_status','0');
	$session->edit_off();
   	
	# just expire the session
	$session->_expire();	
    }
}


sub load_mypage_links {
    my $self = shift ;
    my $uid = shift ;

    $self->debug_msg(4, "Metadot::User::Default::load_mypage_links");
    
    my $maxlinks = $PARAMS->{maxlinks};
    my $fields = join( ", ", map( "linkname$_, linkurl$_", ( 1..$maxlinks )));

    $uid = DBAccess->quote( $uid );

    my @link_values = DBAccess->sqlSelect($fields, "users", "userid=$uid");
    my @link_array;

    for (my $i=1; $i<= $maxlinks; $i++) {
	my %links;
	$links{name} = shift @link_values;
	$links{value} = shift @link_values;
	push @link_array, \%links;
    }
    return @link_array;
}

sub save_mypage_link {
    my $self = shift;
    my $uid = shift;
    my $link_num = shift;
    my $link_name = shift;
    my $link_value = shift;

    $uid = DBAccess->quote( $uid );

    DBAccess->sqlUpdate("users", "userid=$uid", ("linkname$link_num" => $link_name, "linkurl$link_num" => $link_value ));

}


sub get_num_accounts_pending_approval {
    my $self = shift;

    my ($num) = DBAccess->sqlSelect("count(*)", "users", "approvalneeded='required'");

    return $num;
}


sub get_search_user_results {
    my $self = shift;
    
    my $config = shift || die "should provide config hash";
    my $excluded_uids = shift;  #optional reference to a hash containing uids not to return in search results
    my $grp_id = $config->{SKIP_GRP};
    my @user_info;
    my $response;
    my $radio_buttons;
    my $positive_result;
    my $uid;
    my $where = "(approvalneeded IS NULL OR NOT(approvalneeded = 'rejected' OR approvalneeded = 'required'))";
    my $member_present;
    my $image_info;
    my $email =  $config->{EMAIL};
    my $fullname = $config->{FULLNAME};
    my $nickname = $config->{NICKNAME};
    my $select_one = $config->{SELECT_ONE};
    my $s='';
    my $trusted_site = $PARAMS->{trusted_site};
    
    unless( $trusted_site || $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) ) {

        unless (($fullname =~ m/[^\*][^\*]/) || ($nickname =~ m/[^\*][^\*]/) || ($email =~ m/[^\*][^\*]/)) {
            return "<table><tr><td><font color='red'>You must include at least 2 non wildcard characters in your search</font></td></tr></table>";
        }
    }
    
    
    #use Data::Dumper;
    
    #CKIM-SENDABLE: action missing closing bracket and quotes
    #$s .= "<form method=post action =". $config->{ACTION}  if($config->{ACTION});
    $s .= "<form method=post action =\"$config->{ACTION}\">"  if($config->{ACTION});
    $s .= "<table cellspacing=\"0\">
	<tr bgcolor=\"		#bbbbbb\">
		<th>
			&nbsp;
		</th>";
    if( $trusted_site || $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) ) {
	$s .= "<th>
			E-mail address
		</th>"
     }
     $s .= "
		<th>
			Full Name
		</th>
		<th>
			Nickname / Screenname
		</th>
	</tr>";

    if ($email)  {
	# if we got an email address, we'll use that as part of the query

	$email = uc(DBAccess->quote( $email ));

	$where .= " AND UPPER(email) LIKE $email";

	$response = 1;
    }
    if ($fullname) {

	$fullname = uc(DBAccess->quote( $fullname ));

	$where .= " AND UPPER(fullname) LIKE $fullname";
	$response = 1;
    }
    if ($nickname) {

	$nickname = uc(DBAccess->quote( $nickname ));

	$where .= " AND UPPER(nickname) LIKE $nickname";
	$response = 1;
    }

    if ($response) {
	$where =~ s/\*/%/g;
	
	my $rows = DBAccess->sqlSelectMany("userid, nickname, fullname, email", "users", $where, ""); #we should now have back all the possible matches
	my $count = 0;
	
	# support for multiple return types    
	my $return_field;
  
	if ($config->{RETURN_VAL}) {
	    #CKIM-SENDABLE: adding support for multiple return types
	    $return_field = $config->{RETURN_VAL} || 'uid';
	}
	
	# need to sort the results
	my @results;
	while (($uid, $nickname, $fullname, $email) = $rows->fetchrow()) {
	    
	    # skip hidden users only when user doesn't have server_setup access
	    unless( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){
		
		if ( $USER->is_hidden( $uid ) ) { next }
	    }

	    push @results, {
			    uid=>$uid,
			    nickname=>$nickname,
			    fullname=>$fullname,
			    email=>$email,
			   };
	}
	
	my $sort_field = $config->{RETURN_VAL} || 'email';
	@results = sort {lc($a->{$sort_field}) cmp lc($b->{$sort_field})} @results;
	
	for my $result (@results) {
	    ($uid, $nickname, $fullname, $email) = ($result->{uid}, $result->{nickname}, $result->{fullname}, $result->{email});
	    $member_present = 1 if ($$excluded_uids{$uid}); #check to see if the uid being search for is in the list of uids that should not be return
	    
	    if(not $member_present && $uid != MetadotConfig->get_reserved_uid('ANONYMOUS_USER')) {
		$positive_result = 1;
		my $rownum = ($count++ % 2);
		my $color = $rownum ? "#eeeeee" : "#ffffff";
		
		$s .= "<tr bgcolor=\"$color\">
                <td><center>";
		if ($return_field) {
		    my $return_val = $result->{$return_field};
   
                    
		    if ($select_one) {
			$s .="<input type=radio name=\"new_$return_field\" value=\"$return_val\">";
		    } else {
			$s .="<input type=checkbox name=\"new_$return_field\" value=\"$return_val\">";
		    }
		} else {
		    #CKIM-SENDABLE: backwards compatibility
		    if ($select_one) {
			$s .="<input type=radio name=uid value='$uid'>";
		    } else {
			$s .="<input type=checkbox name=user$uid value=1>";
		    }
		}
		
		$s .= "			</center>
                    </td>";
                    if ( $trusted_site || $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) ) {
                    $s .= "<td>
                        $email&nbsp;
                    </td>";
                    }
         $s .= "
                    <td>
                        $fullname&nbsp;
                    </td>
                    <td>
                        $nickname&nbsp;
                    </td>
                </tr>";
	    }
	    $member_present = 0;
	}
	#CKIM-SENDABLE: this HTML was missing some tags      
	$s .= '</table>';
    } else {			# we are here if none of the fields was filled in
	$s = "<table><tr><td><font color='red'>None of the information was filled in.  Please go back and enter some information</font></td></tr></table>";
    }
    if ($response && !$positive_result) { #if we got a response from the user, but that response didn't turn up anything in the groups table
	$s = "<table><tr><td><font color='red'>No users match the criteria who are not already part of the group</font></td></tr></table> ";
	
    }
    if ($positive_result) {	#if we got some information to present to the user, we need to generate a submit button
	if($config->{ACTION}) {
	    foreach my $param (keys %{$config->{PARAMS}}){
		#CKIM-SENDABLE: value=''? why?
		#$s .= "<input type=hidden name='$param' value=''>\n";
		$s .= "<input type=hidden name=\"$param\" value=\"$config->{PARAMS}{$param}\">\n";
	    }      
	    #CKIM-SENDABLE: cleaned up the html (moved </table> up)
	    $s .=  "<input type=submit name='add' value='Add'></form>";
	}
#	print STDERR "CKIM: Components->show_user_search_results got:\n$s\n";
	return $s;
    } else {
	return ( "<center><font color=\"#ff0000\">$s</font></center>" );
    }
}


sub check_password {
    my $self = shift;
    
    my $email = shift;
    my $password = shift;
    
    $email = DBAccess->quote( $email );
    $password = DBAccess->quote($password);
    
    my ( $resuid ) = DBAccess->sqlSelect("userid", "users", "email=$email and password = $password");
    
    return $resuid;
}


sub email_already_registered {
    my $self = shift;
    my $email = shift;

    $email = DBAccess->quote( $email );


    my ($cnt)=DBAccess->sqlSelect( "count(*)", "users", "lower(email)=lower($email)" );    
    
    return $cnt;
}

sub ldap_uid_already_registered {
    my $self = shift;
    my $ldap_uid = shift;

    my ($cnt)=DBAccess->sqlSelect( "count(*)", "users", "lower(ldapdn)=lower('"
				    . $ldap_uid
				    . "')"
				  );
    return $cnt;
}


##################################################
#### create_default_portal
#### create a default portal for a user
#### add some default categories, items and news item
####
##################################################
sub create_default_portal{
    my $self = shift;
    my $uid = shift;

    my $createdby;
    my ($parentiid, $name, $url, $description, $keywords,$owneruid,
	$displaylook,$fullname, $nickname);
    
    $self->debug_msg(3, "Metadot::User::Default::create_default_portal()");
    
    if (!defined $uid){
	# if not uid is passed to the function
	# then we use the current metadot user
	$uid =  $USER->{uid};
    }
    
    # let's load user's information
    my $u = Metadot::UserFactory->get_user( $uid );
    
    # user doesn't exist
    if (!$u->uid) {
	$uid = -1;
	$u = Metadot::UserFactory->get_user( $uid );
    }
    
    # some durty inits
    $uid = $u->uid;
    $owneruid = $u->uid;
    my $webname = $u->webname;
    $fullname = $webname;
    $nickname = $webname;
    
    # check if user already have one
    if ($u->has_portal($uid)){
	$self->debug_msg (3, "Metadot::User::Default User $webname (uid:$uid) has already a portal");
	return 0;
    }
    
    # this is where we attach all users portals
    $parentiid = MetadotConfig->get_reserved_iid('PORTAL_PARENT');
    
    use Gizmo::Category;
    use Gizmo::Newsitem;
    use Gizmo::Item;
    
    # build the top home page
    $description="This is the Public Website for $webname";
    $name="$fullname Website";
    $keywords="home personal page projects my website page public";
    $displaylook="LULI";
    
    my $cat = Category->new({
			     uid => $uid,
			     parent_id => $parentiid,
			     name => $name,
			     description => $description,
			     keywords => $keywords,
			     display_look => $displaylook,
			     created_by => $createdby,
			    });
    
    $self->debug_msg(4, "User/Default created new category $cat");

    $cat->set_bundle_level('MOD', 8);
    $cat->set_bundle_level('DEL', 9);
    $cat->set_bundle_level('EDITP', 8);
    
    my $iid = $cat->add();
    
    $self->debug_msg(5, "Metadot::User::Default::createDefaultPortal: created home for user $owneruid w/ iid=$iid<br>");
    
    # add news channel to the category
    # aju 02-10-03: off by default now.
#     my $chan = Channel->new();
#     my $cid = $chan->create($iid, "Internal", "$fullname Website News", "$fullname Website News");
    
    
    ##################### add an example of news
    $name="Welcome to my Public Website News";
    $description="I will post here important news I want to tell the world about! Come back often to check it out.";
    $keywords="news";
    my $showfrom="2000-01-25 00:00:00";
    $url="";
    my $longdescription="";
    
    #  print STDERR "owneruid: $owneruid, parent_id: $iid, show_from: $showfrom, url: $url, name: $name, description: $description, long_description: $longdescription, keywords: $keywords";
    my $ni = Newsitem->new({
			    uid => $owneruid,
			    parent_id => $iid,
			    show_from => $showfrom,
			    url => $url,
			    name => $name,
			    description => $description,
			    long_description => $longdescription,
			    keywords => $keywords,
			   });
    
    $ni->add();
    
    ##################### add an example category
    $name="My Category";
    $description="Here is publically available information about me and about my work.";
    $keywords="home personal project portal";
    $displaylook="LULI";
    
    my $newcat = Category->new({
				uid => $uid,
				parent_id => $iid,
				name => $name,
				description => $description,
				keywords => $keywords,
				display_look => $displaylook,
				created_by => $createdby,
			       });
    my $newiid = $newcat->add();
    #my $newchan = Channel->new();
    #$cid = $newchan->create($newiid, "Internal", "$fullname project news", "$fullname project news");
    
    ##################### add another example of news
    $name="My Website news ";
    $description="I'll will post here all important news about my projects! Stay tuned!";
    $keywords="news";
    $showfrom="2000-01-25 00:00:00";
    $url="";
    $longdescription="";
    
    $ni = Newsitem->new({
			 uid => $owneruid,
			 parent_id => $iid,
			 show_from => $showfrom,
			 url => $url,
			 name => $name,
			 description => $description,
			 long_description => $longdescription,
			 keywords => $keywords,
			});
    $ni->add();
    
    
    ##################### add an example of item
    $name="Google";
    $description="You can edit this item when *YOU* -the owner of this page- are in <b>Edit mode.</b> Do you see the <b>EDIT</b> button?";
    $keywords="no keyword";
    $url="http://www.google.com";
    my $file1="";
    my $cool=1;
    my $newitem = Item->new({uid => $uid,
			     parent_id => $iid,
			     name => $name,
			     description => $description,
			     keywords => $keywords,
			     url => $url,
			     file1 => $file1,
			     cool => $cool,
			     created_by => $createdby,
			    });
    $newitem->add();
    
    return $iid;
}



####################################################

sub www_show_users_portal {
    my $self = shift;
    my $user = $FORM{user};
    my $basename = $PARAMS->{basename};
    my @valid_iids;
    my $sth = DBAccess->sqlSelectMany("iid, users.fullname", "instance, users", "(users.fullname = '$user' or users.nickname = '$user') and users.userid = instance.userid and instance.parentiid = -1000 and instance.iid > 1000 and users.userid > 0 order by users.fullname");
    while (my ($iid, $fullname) = $sth->fetchrow_array()) {
        my $category = Gizmo->new($iid);
        if ($category->is_allowed_to_do('show', $USER)) {
            push (@valid_iids, $fullname);
            push (@valid_iids, $iid);
        }
    }
    
    if (scalar(@valid_iids) == 2) {
        $self->print_http_header();
        $self->md_redirect("$basename/index.pl?op=show&amp;iid=$valid_iids[1]");
    } elsif (scalar(@valid_iids) > 1 ) {
        my @array = ("More than one match was found", "Caution");
        while (my $fullname = shift(@valid_iids)) {
            push (@array, "The portal of $fullname");
            push (@array, ("$basename/index.pl?op=show&amp;isa=Category&amp;iid=" . shift(@valid_iids)));
        }
        $self->print_choices(@array);
        
    } else {
        $self->print_choices("No match was found", "Caution", "Go to the home page", "$basename/index.pl");
    }
}


sub is_hidden {
    my $self = shift;
    my $uid = shift;

    unless(defined $uid) { $uid = $self->uid() }

    if( $USER->belongs_to_grp('ADMIN', $uid) ||
	$USER->get_fullname_from_uid( $uid ) eq 'default_look' ) {
	return 1;
    }

    return '';
}


sub get_number_of_active_users {
   
    my ( $total ) = DBAccess->sqlSelect("count(*)", "users");

    # super-admin users should not be counted:
    $total -= Group->number_of_members('ADMIN');
    
    # default_look user should not be counted:
    $total -= 1;

    return $total;
}


sub max_number_of_users_quota_exceeded {
    my $self = shift;

    # when $PARAMS->{max_number_of_users} is false we have unlimited users
    
    if( ( $USER->get_number_of_active_users() >= $PARAMS->{max_number_of_users} ) &&
	$PARAMS->{max_number_of_users} ) {
	
	return 1;	
    }	
    
    return undef;
}

sub get_user_defined_homepage {
    my $self = shift;
    return undef;
}

sub get_display_name {
    my $self = shift;

    my $fullname = $self->fullname();
    my $authorname = $self->{nickname} || $fullname;
    $authorname .=  " ($fullname)" if defined($fullname) && $fullname ne '';

   return $authorname;
}
####################################################

1;


__END__

=head1 NAME

Metadot::User::Default - Handles user data storage, user profile editing and user profile display. Can be subclassed to provide customized behavior.

=head1 SYNOPSIS


    User::new ($uid);
    $AC = $user->get_AC [**use newer AC scheme? **];
    $isa = $user->is_a ;
    $id = $user->id ;
    $user->save ;
    $uid = $user->get_owner_uid ;
    $id = $user->parent_id ;
    $user->not_logged_in_callback ;
    $user->www_register_form ;
    $user->www_register ;
    $boolean = $user->www_delete_request ;
    $boolean = $user-> www_delete_ok ([$uid]);
    $user->www_show_details ([$uid]);
    $user->www_modify_self ;
    $user->www_modify ([$uid]) ;
    $user->www_save ;
    $user->www_save_self ;
    $debug_level = $user->debug_level ([$level]);
    $boolean = $user->delete ;
    $boolean = $user->has_portal;
    $count = $user->has_mypage;
    $fullname = $user->fullname ([$val]);
    $webname = $user->webname;
    $boolean = $USER->belongs_to_grp ($grp, $user->uid());
    $boolean = $USER->belongs_to_grp_id ($grp_id, $user->uid());
    $largest = $user->largest_group ;
    $boolean = $user->is_owner ($object);
    $admin = $user->is_admin;
    $count = $user->is_logged_in ;
    $user->save;
    $user->save_password ($password);
    $boolean = $user->can_use_email_address ($email);
    $boolean = $user->email_address_exists ($email);
    $uid = $user->add;
    $html = $user->show_panel ;
    $user->www_accept_registration ;
    $user->www_reject_registration ;
    $iid = $user->get_clipboard_iid ;
    $html = &popup_window
    $user->www_require_pager_form ($referrer, $target);
    $user->www_process_pager_form ;
    $user->pager_accepted ($referrer);
    $user->www_require_icq_form ($referrer, $target);
    $user->www_process_icq_form ;
    $user->icq_accepted ($referrer);
    $user->DESTROY;
    $user->create_mypage ($uid);
    $user->stock_gizmos  ;
    @link_array = $user->load_mypage_links ($uid);
    $uid = $user->get_uid_from_email ($email);
    $email = $user->get_email_from_uid ($uid);
    $fullname = $user->get_fullname_from_uid ($uid);
    $message = $user->substitute;
    $listref = &get_all_users ;
    $user->user_exists ($uid);
    $USER->is_default_account( $email )
    $user->ldap_uid_already_registered( $ldap_uid )  
    $user->email_already_registered( $email )
    $user->check_password( $email, $password )
    $user->get_search_user_results( $config )
    $user->get_num_accounts_pending_approval()
    $user->save_mypage_link( $uid, $link_num, $link_name, $link_value )
    $user->load_mypage_links()
    $user->logout()
    $user->get_groups_data_by_owner( $field, $order )
    $user->get_users_in_group( $group_id, $field, $order )
    $user->get_emails_by_grp_name( $grp_name )
    $user->get_emails_by_grp_id( $grp_id )
    $user->show_user_list($filter, $startfrom, $maxitem )
    $user->www_show_users()
    $iid = $user->create_default_portal ([$uid]);

=head1 DESCRIPTION

User handles all additions, deletions, and alterations of user accounts.

=head1 Methods

    $AC = $user->get_AC [**use newer AC scheme? **];

Returns AC for user:

    $isa = $user->is_a ;

Returns the name of the module currently being used:

    $id = $user->id ;

Returns the ID of the current user:

    $user->save ;

Saves permissions for user:

    $uid = $user->get_owner_uid ;

Returns uid for ADMIN:

    $id = $user->parent_id ;

Returns parent_id:

    $user->not_logged_in_callback ;

Redirects user to login:

    $boolean = $user->www_delete_request ;

Called when requesting to delete a user. Verifies that a uid is passed and that the user is not ADMIN. Presents delete verification screen.

    $boolean = $user-> www_delete_ok ([$uid]);

Deletes user and presents notification screen that user has been deleted:

    $user->www_show_details ([$uid]);

Displays uneditable user information from shifted in uid:

    $user->www_modify_self ;

Calls backend function for user to alter their profile:

    $user->www_modify ([$uid]) ;

Calls backend function for admin to alter user profiles:

    $user->www_save ;

Called when user profile is being altered by someone other than the user (i.e. ADMIN) backend function that verifies form data and saves user profile:

    $user->www_save_self ;

Called when user is altering their own profile, ensures that users priveleges are kept. Verifies form data and saves user profile:

    $debug_level = $user->debug_level ([$level]);

If $level is passed in it updated debug_leve in the user table, else debug_level remains the same. Returns debug_level:

    $boolean = $user->delete ;

Deletes user information from the following tables: user, MyPage, Subscription, Session, grpmembers and updates grp:

    $boolean = $user->has_portal;


Returns iid from the instance table is user has a portal, else returns 0:

    $count = $user->has_mypage;

Performs a count(*) on mypage table for entries with user's uid. Returns count number:

    $fullname = $user->fullname ([$val]);

Returns users fullname, if no fullname returns nickname:

    $webname = $user->webname;

Returns users nickname, if no nickname returns fullname:

    $boolean = $USER->belongs_to_grp ($grp, $uid);

Pass in group name, returns 1 if user belongs to group, else returns undef:


    $boolean = $USER->belongs_to_grp_id ($grp_id, $uid);

Pass in group id, returns 1 if user belongs to group, else returns undef:

    $largest = $user->largest_group ;


Returns larget group that the user belongs to:

    $boolean = $user->is_owner ($object);

Returns 1 is user is owner, else returns undef:

    $admin = $user->is_admin;

Returns 1 if user is an admin, else returns undef:

    $count = $user->is_logged_in ;

Verifies if user is logged in:

    $user->save;

Inserts/Updates user permissions:

    $user->save_password ($password);

Encrypts password and saves to user table:

    $boolean = $user->can_use_email_address ($email);

Returns 1 if email address is not taken, else returns 0:

    $boolean = $user->email_address_exists ($email);

Returns uid of user with mataching email, else returns 0;

    $uid = $user->add;

Inserts user information into user table and create default portal:

    $html = $user->show_panel ;

Shows edit/delete buttons for admin:

    $user->www_accept_registration ;

Used when admin acceptance is needed for user registration. Updates user record as accepted, sends email, displays notifications screen that user has been accepted:

    $user->www_reject_registration ;

Used when admin acceptance is needed for user registration. Updates user record as rejected, sends email, displays notifications screen that user has been rejected:

    $iid = $user->get_clipboard_iid ;

Returns clipboard iid, creates clipboard if one does not exist:

    $html = &popup_window

JavaScript popup window:

    $user->www_require_pager_form ($referrer, $target);

Prompts for user pager number:

    $user->www_process_pager_form ;

Verifies and saves user pager information:

    $user->pager_accepted ($referrer);

Notifies that pager information has been accepted:

    $user->www_require_icq_form ($referrer, $target);

Prompts for user ICQ information:

    $user->www_process_icq_form ;

Verifies and saves user pager information:

    $user->icq_accepted ($referrer);

Notifies that ICQ information has been accepted:

    $user->DESTROY;

EMPTY destroy method to please mod_perl when it complains about a missing DESTROY function.

    $user->create_mypage ($uid);

Create default MyPage from anonymous user data. Returns 1:

    $user->stock_gizmos  ;

Create default gizmo tools for MyPage from anonymous user data:

    @link_array = $user->load_mypage_links($uid);

Returns array of users links:

    $uid = $user->get_uid_from_email ($email);

Returns users uid from email:

    $email = $user->get_email_from_uid ($uid);

Returns email from uid:

    $fullname = $user->get_fullname_from_uid ($uid);

Returns fullname from uid:

    $message = $user->substitute;

Substitutes tags in customized email:

    $listref = &get_all_users ;

Returns array of all uids:

    $user->user_exists ($uid);

Returns true if user exists.

    $USER->is_default_account( $email )

Will check if email belongs to a default account, eg. Admin or default_look. Default accounts are not verified against the LDAP server when LDAP authentication is used.

    $user->ldap_uid_already_registered( $ldap_uid )

Will return true if a user is already registered with the ldap_uid supplied. Useful from Register modules that require storing LDAP uids.

    $user->email_already_registered( $email )

Will return true if a user is already registered with the email supplied. Useful from Register modules..

    $user->check_password( $email, $password )

Will return true when the password corresponds with the supplied email.

    $user->get_search_user_results( $config )

Will return HTML with a list of users, filtered by the params passed to it in the $config hash. 

  An example such hash would be:

my $config = { EMAIL => 'foo@bar.com',
	       FULLNAME => 'Foo*',
	       NICKNAME => 'Foo*',
	       SELECT_ONE => 1 };

The SELECT_ONE option will make the HTML have a radio button by each user entry for user selection.


    $user->get_num_accounts_pending_approval()

Will return the number of users pending administrator approval.

    $user->save_mypage_link( $uid, $link_num, $link_name, $link_value )

Will save link information for the MyPage save links feature.

    $user->load_mypage_links()

Will return an array of hashes containing the url links a user has saved under her save links feature in MyPage.

    $user->logout()

Will logout a user. 

    $user->get_groups_data_by_owner( $field, $order )

Will return an array of hashes, one for each existing group in the system, containg group description information in the following fields: email, owner, nick and description.  Results will be sorted by $field in ascending or descending order according to $order.

    $user->get_users_in_group( $group_id, $field, $order )

Will return an array of hashes containing descriptions of all users in a group. Results will be sorted by $field in ascending or descending order according to $order.

    $user->get_emails_by_grp_name( $grp_name )

Will return an array containing email addresses of members of a group.

    $user->get_emails_by_grp_id( $grp_id )

Will return an array containing email addresses of members of a group.

    $user->show_user_list($filter, $startfrom, $maxitem )

Will return HTML showing a list of users. $filter should contain a where clause for the user table. $startfrom and $maxitem are used to select ranges of users within the results.

    $user->www_show_users()

Will output a list of users (the result of invoking show_user_list) to the users browser.

    $iid = $user->create_default_portal([$uid]);

Creates the default portal that new users are given. Adds some default items also.


=cut
