###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::User::FlexUser;
use strict;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $ACCESS_BROKER $CGI);

use Metadot::User::Default;

use vars qw(@ISA);
use Group;

@ISA=qw(Metadot::User::Default Metadot::UploadsManager);


my $AC = Metadot::User::Default->get_AC();

push( @{ $AC->{BUNDLES}{ADMIN}{OPS} }, 'show_users_groups' );
push( @{ $AC->{BUNDLES}{MOD}{OPS} }, 'confirm_upload_deletion' );
push( @{ $AC->{BUNDLES}{MOD}{OPS} }, 'delete_upload' );
push( @{ $AC->{BUNDLES}{SEARCH}{OPS} }, 'download_file' );
push( @{ $AC->{BUNDLES}{SEARCH}{OPS} }, 'show_user_directory' );
push( @{ $AC->{BUNDLES}{SEARCH}{OPS} }, 'show_user_details' );

sub get_AC {
	return $AC;
}

my $ORG_KEY = '__ORG__';

my $FIELDS_CONFIG = {
		     DISPLAY_SETS => [ 'Login Information', 'My start page after login', 'Personal Information' ],
		     FIELDS => [
				{
				 field_name => 'first_name',
				 description => 'First Name',
				 max_length => 64,
				 size => 20,
				 is_required => 1,
				 display_set => 0,
				 storage_type => 'secondary',
				 store_at_column => "f1",
				 verify_type => 'non_empty',
				},
				{
				 field_name => 'initial',
				 description => 'Middle Initial',
				 max_length => 1,
				 size => 1,
				 is_required => 0,
				 is_not_required_for_default_users => 1,
				 display_set => 0,
				 storage_type => 'secondary',
				 store_at_column => "f2",				 
				},
				{
				 field_name => 'last_name',
				 description => 'Last Name / Family Name',
				 max_length => 64,
				 size => 20,
				 is_required => 1,
				 is_not_required_for_default_users => 1,
				 display_set => 0,
				 storage_type => 'secondary',
				 store_at_column => "f3",				 
				},
				{
				 field_name => 'nickname',
				 description => 'Nickname / Screename',
				 max_length => 64,
				 size => 20,
				 is_required => 0,
				 display_set => 0,
				 storage_type => 'primary',
				},
                {
                 field_name => &get_organization_field_name,
                 description => 'Organization',
                 field_type => 'select',
                 is_required => 0,
                 display_set => 0,
                 #field_values => &get_organizations(),
                 field_values_function => 'get_organizations',
                 storage_type => 'secondary',
                 store_at_column => "f16",
                 enable_param => "allow_organization_registration",
                 only_admin_can_edit => 1, ## currently does not apply to all field types
                },
                {
                 field_name => &get_flex_homepage_field_name(),
                 description => '&nbsp;',
                 field_type => 'radio',
                 is_required => 0,
                 display_set => 1,
                 field_values_function => 'get_homepage_values',
                 storage_type => 'secondary',
                 store_at_column => "f17",
                 use_line_breaks => 1,
                },
				{
				 field_name => 'email',
				 description => 'E-mail Address',
				 max_length => 64,
				 size => 20,
				 is_required => 1,
				 display_set => 0,
				 storage_type => 'primary',
				},
				{
				 field_name => 'password',
				 description => 'Password',
				 max_length => 64,
				 size => 20,
				 is_required => 1,
				 display_set => 0,
				 storage_type => 'primary',
				 field_type => 'password',
				},
				{
				 field_name => &get_picture_field_name(),
				 description => 'Picture',
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'user_upload',
                                 field_type => 'user_upload',
                 #enable_param => 'show_user_directory',
				},
				{
				 field_name => 'address_line_1',
				 description => 'Address Line 1',
				 max_length => 64,
				 size => 36,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "f4",				 
				},
				{
				 field_name => 'address_line_2',
				 description => 'Address Line 2',
				 max_length => 64,
				 size => 36,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "f5",				 
				},
				{
				 field_name => 'city',
				 description => 'City',
				 max_length => 64,
				 size => 30,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "f6",
				 
				},
				{
				 field_name => 'state',
				 description => 'State',
				 max_length => 64,
				 size => 30,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "f7",				 
				},
				{
				 field_name => 'zip_code',
				 description => 'Zip Code',
				 max_length => 64,
				 size => 30,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "f8",				 
				},

				{
				 field_name => 'country',
				 description => 'Country',
				 max_length => 64,
				 size => 30,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "f9",				 
				},

				{
				 field_name => 'telephone',
				 description => 'Telephone',
				 max_length => 64,
				 size => 30,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "f10",				 
				},

				{
				 field_name => 'fax',
				 description => 'Fax',
				 max_length => 64,
				 size => 30,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "f11",				 
				},

				{
				 field_name => 'mobile_phone',
				 description => 'Mobile Phone',
				 max_length => 64,
				 size => 30,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "f12",				 
				},

				{
				 field_name => 'pager_phone',
				 description => 'Pager Phone',
				 max_length => 64,
				 size => 30,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "f13",				 
				},

				{
				 field_name => 'pager_number',
				 description => 'Pager Address',
				 max_length => 64,
				 size => 30,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'primary',
				},

				{
				 field_name => 'icq_number',
				 description => 'ICQ Number',
				 max_length => 64,
				 size => 20,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'primary',
				},

				{
				 field_name => 'homepage',
				 description => 'Homepage URL',
				 max_length => 64,
				 size => 30,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'primary',
				},

				{
				 field_name => 'job_title',
				 description => 'Job Title',
				 field_type => 'textarea',
				 rows => 3,
				 cols => 30,
				 wrap => 1,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "t1",				 
				},

				{
				 field_name => 'schedule',
				 description => 'Schedule',
				 field_type => 'textarea',
				 rows => 3,
				 cols => 30,
				 wrap => 1,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "t2",				 
				},

				{
				 field_name => 'notes',
				 description => 'Notes',
				 field_type => 'textarea',
				 rows => 3,
				 cols => 30,
				 wrap => 1,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "t3",				 
				},

				{
				 field_name => 'quotes',
				 description => 'Quotes',
				 field_type => 'textarea',
				 rows => 3,
				 cols => 30,
				 wrap => 1,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "t4",				 
				},

				{
				 field_name => 'home_telephone',
				 description => 'Home Telephone',
				 max_length => 64,
				 size => 30,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "f14",				 
				},

				{
				 field_name => 'home_fax',
				 description => 'Home Fax',
				 max_length => 64,
				 size => 30,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "f15",				 
				},

				{
				 field_name => 'home_address',
				 description => 'Home Address',
				 field_type => 'textarea',
				 rows => 3,
				 cols => 30,
				 wrap => 1,
				 is_required => 0,
				 display_set => 2,
				 storage_type => 'secondary',
				 store_at_column => "t5",				 
				},
			       ],
		    };


my $check_for_required_fields_done;


# required for backward compatibility
sub fullname{
    my $self=shift;
    my $val = shift;
    
    if (defined $val) {
	
	my ($first_word, $second_word, $third_word, $remainder ) = ('', '', '', '');
	
	($first_word, $second_word, $remainder ) = split( /\s+/, $val, 3 );
		
	($first_word) || print STDERR "user has no defined name";
	
	my $first_name = $first_word || '';
	my ($last_name, $initial) = ('', '');
	
	if( $second_word && $remainder && ( length $second_word == 1 ) ) {
	    
	    $initial = $second_word || '';
	    $last_name = $remainder || '';
	    
	} else {

	    $initial = '';
	    $last_name = 
	      ( $second_word ) ? ( ( $remainder ) ? "$second_word $remainder" : $second_word ) : '';

	}	    
	
	$self->set_value( 'first_name', $first_name );
	$self->set_value( 'last_name', $last_name );
	$self->set_value( 'initial', $initial );
	
	$self->{fullname} = $val;
	return $self->{fullname};	
    }

    my $fullname = $self->{first_name};
    
    $fullname .= ($self->{initial}) ?  
      ' ' . $self->{initial} . ' ' : ' ';
    
    $fullname .= $self->{last_name};
    
    return $fullname || $self->{nickname};
    
}


## This method must be overwritten in child classes. See Sample below.
## Sample:
### must override SUPER because $FIELDS_CONFIG is a class variable
#sub get_fields_config {
#    my $self = shift;
#    return $self->SUPER::get_fields_config($FIELDS_CONFIG);
#}
## Copy and paste above method in child class

sub get_fields_config {
    my $self = shift;
    my $fields_config = shift || $self->{_fields_config} || $FIELDS_CONFIG;

    my $required_fields = [ qw( first_name last_name email password ) ];
    
    # we do this to trap any missing required fields from the 
    # fields data structure
    unless( $check_for_required_fields_done ) {
        $check_for_required_fields_done++;
    
        foreach my $req_field ( @{ $required_fields } ) {
            my $found;
            foreach my $field ( @{ $fields_config->{FIELDS} } ) {
                $found++ if ( $field->{field_name} eq $req_field );
            }
            unless( $found == 1 ) {
                die "fields configuration data structure does not contain field $req_field (or contains it more than once)";
            }
        }
    }
    
    return $fields_config;
}

sub get_config_for_field {
    my $self = shift;
    my $field_name = shift || die("Must specify a field");
    my $fields_config = $self->get_fields_config();
    foreach my $field (@ { $fields_config->{FIELDS} }) {
        if ($field->{field_name} eq $field_name) {
            return $field;
        }
    }
}

sub get_value {
    my $self = shift;
    my $field_name = shift || die "must include field_name";
    
    my $result = $self->{$field_name};

    return $result;
}

sub set_value {
    my $self = shift;
    my $field_name = shift || die "must include field_name";
    my $value = shift;
    
    $self->{$field_name} = $value;
}

sub get_default_value {
    my $self = shift;
    my $field_name = shift || die("Must supply a field name");
    my $result;

    my $field_config = $self->get_config_for_field($field_name);
    if ($field_config) {
        $result = $field_config->{default_value};
    }
    return $result;
}

sub set_default_value {
    my $self = shift;
    my $field_name = shift || die("Must supply a field name");
    my $value = shift;

    my $field_config = $self->get_config_for_field($field_name);
    $self->debug_msg(5, "Setting default value of $field_name ($field_config) to '$value'");

    if ($field_config) {
        $field_config->{default_value} = $value;
    }
}

sub add {
    my $self = shift;

    $self->{upload_size} = -1;
    $self->{upload_number} =  -1;

    # make fullname from subparts
    my $fullname = $self->{first_name};

    $fullname .= ($self->{initial}) ?
      ' ' . $self->{initial} . ' ' : ' ';

    $fullname .= $self->{last_name};

    my $db_now = Utility->db_now();
    # primary storage
    my $uid = DBAccess->sqlInsert("users",
			fullname => $fullname,
			nickname => $self->{nickname},
			homepage => $self->{homepage},
			email    => $self->{email},
			icqnumber=> $self->{icq_number},
			pager    => $self->{pager_number},
			quote    => $self->{quote},
			password => $self->{password},
			approvalneeded => $self->{approval_needed},
            linkname1   => $self->{link_name_1}, #RESTRICTED-USER
            linkname2   => $self->{link_name_2}, #RESTRICTED-USER
			adminlevel  => 1,
			adminmode   => 0,
			debuglevel  => 0,
			updatedate  =>$db_now,
			creationdate=>$db_now,
			max_total_upload_size => $self->{upload_size},
			max_total_numberof_upload => $self->{upload_number},
			ldapdn  => $self->{ldap_dn}
		       );
    # secondary storage
    $self->_insert_secondary_fields( $uid );

    $self->uid($uid);
    
    # create default user portal
    my $portal = Portal->new(undef);
    
    #RESTRICTED-USER: added additional condition against creation
    if ($PARAMS->{my_website_creation} && !$self->is_restricted() ) {
    
    my ($hpiid) = $self->create_default_portal( $uid );
    
    }
    $self->add_user_to_primary_organization();
    $self->_process_uploads();

    return $uid;
}

sub _insert_secondary_fields {
    my $self = shift;
    my $uid = shift;

    my $config = $self->get_fields_config();

    my %values;
    my $enable_param = '';
    foreach my $field ( @{ $config->{FIELDS} } ) {
	if( lc $field->{storage_type} eq 'secondary' ) {
        $enable_param = $field->{enable_param};
        if (defined($enable_param) && !$PARAMS->{$enable_param}) {
            next;
        }
	    
	    my $store_at = $field->{store_at_column};
	    
	    ($store_at) || die "field $field->{field_name} does not contain store_at_column";

	    my $value = $self->get_value( $field->{field_name} );
	    
	    $values{ $store_at } = $value;	      
	}
    }

    DBAccess->sqlInsert(
			"extended_user",
			userid => $uid,
			%values
		       );   

}


##################################################
sub save {
    my $self=shift;

    # build fullname from its parts
    my $fullname = $self->{first_name};
    
    $fullname .= ($self->{initial}) ?  
      ' ' . $self->{initial} . ' ' : ' ';
    
    $fullname .= $self->{last_name};

    my $db_now = Utility->db_now();
    # also update password if the password field has been set
    if( $self->{password} ) {
	DBAccess->sqlUpdate("users","userid='$self->{uid}'",
			    (
			     fullname => $fullname,
			     password => $self->{password},
			     nickname => $self->{nickname},
			     homepage => $self->{homepage},
			     email    => $self->{email},
			     icqnumber=> $self->{icq_number},
			     pager    => $self->{pager_number},
			     adminlevel=>$self->{admin_level},
			     quote    => $self->{quote},
			     max_total_upload_size => $self->{upload_size},
			     max_total_numberof_upload => $self->{upload_number},
			     approvalneeded => $self->{approval_needed},
			     updatedate     =>$db_now,
			     ldapdn         => $self->{ldap_dn},
                             lastonline     => $self->{last_online},
			    )
			   );
    } else {
	DBAccess->sqlUpdate("users","userid='$self->{uid}'",
			    (
			     fullname => $fullname,
			     nickname => $self->{nickname},
			     homepage => $self->{homepage},
			     email    => $self->{email},
			     icqnumber=> $self->{icq_number},
			     pager    => $self->{pager_number},
			     adminlevel=>$self->{admin_level},
			     quote    => $self->{quote},
			     max_total_upload_size => $self->{upload_size},
			     max_total_numberof_upload => $self->{upload_number},
			     approvalneeded => $self->{approval_needed},
			     updatedate =>$db_now,
			     ldapdn   => $self->{ldap_dn}
			    )
			   );	
    }
 
    $self->_update_secondary_fields();   
    if ($ACCESS_BROKER->is_allowed_to_do( 'admin_tasks')) {
        $self->add_user_to_primary_organization();
    }
    $self->_process_uploads();
}


sub _update_secondary_fields {
    my $self = shift;
    
    my $config = $self->get_fields_config();
    
    my %values;
    my $enable_param = '';
    foreach my $field ( @{ $config->{FIELDS} } ) {
	if( lc $field->{storage_type} eq 'secondary' ) {
        $enable_param = $field->{enable_param};
        if (defined($enable_param) && !$PARAMS->{$enable_param}) {
            next;
        } elsif ($field->{only_admin_can_edit} && !$ACCESS_BROKER->is_allowed_to_do( 'admin_tasks')) {
            next;
        }
	    
	    my $store_at = $field->{store_at_column};
	    
	    ($store_at) || die "field $field->{field_name} does not contain store_at_column";
	    
	    my $value = $self->get_value( $field->{field_name} );
	    
	    $values{ $store_at } = $value;	      
	   
	}
    }

    #print STDERR "_update_secondary_fields using values\n",Dumper \%values;

    # learn if a record exists already for this user 
    my ($already_there) = 
      DBAccess->sqlSelect("count(*)",
			  "extended_user",
			  "userid = '$self->{uid}'");

    if( $already_there ) {
    
	DBAccess->sqlUpdate(
			    "extended_user",
			    "userid='$self->{uid}'",
			    ( %values ) 
			   );		
    } else {
	
	DBAccess->sqlInsert(
			    "extended_user",
			    userid => $self->{uid},
			    ( %values )
			   );   
    }
}


sub _process_uploads {
    my $self = shift;
    
    my $config = $self->get_fields_config();
    my $isa = $self->is_a();
    
    my $err = '';
    foreach my $field ( @{ $config->{FIELDS} } ) {
	if( lc $field->{storage_type} =~ /_upload$/ ) {
            my $field_name = $field->{field_name} || die "must specify field_name";
            my $field_desc = $field->{description} || die "must specify field description";
            my $field_type = $field->{field_type} || die "must specify field description";

            ## Uploads Manager
            my $result = $self->process_upload( {
                isa => $isa,
                id => $self->uid(),
                uid => $self->uid(),
                field_name => $field_name,
                field_desc => $field_desc,
                type => $field_type,
                is_required => $field->{is_required}, } );
            if($result) { $err .= "$result<BR>" }
        }
    }

    return $err;
}

sub get_write_dir_namespace {
    my $self = shift;

    my $uid = $self->uid();
    my $file_isa = $self->is_a();
    $file_isa =~ s/\:/_/g;

    unless (defined($uid) && $file_isa) {
        $self->print_click_back( "Upload Error", "User id not defined.  If this problem occurs again, please contact the site administrator." ) ;
        exit(0);
    }

    return $PARAMS->{absolutepathfileuploaddir} . "/" . $file_isa . "_" . $uid;
}

sub new {
    my $class = shift;

    my $self = $class->SUPER::new(@_);
    
    $self->initialize(@_);
    return $self;
}

sub initialize {
    my $self = shift;
    # this allows the class variable to appear polymorphic
    unless (defined $self->{_fields_config}) {
        $self->{_fields_config} = $FIELDS_CONFIG;
    }
    
        $self->_load_secondary_fields();
}

sub _load_secondary_fields {
    my $self = shift;
    
    if (defined $self->{uid}) {
        my $config = $self->get_fields_config();
        
        my $fields_string;
        my @field_names;
        
        my $enable_param = '';
        foreach my $field ( @{ $config->{FIELDS} } ) {
        	if( lc $field->{storage_type} eq 'secondary' ) {
                $enable_param = $field->{enable_param};
                if (defined($enable_param) && !$PARAMS->{$enable_param}) {
                    next;
                }
        	    my $db_field_name = $field->{store_at_column};
        	    $fields_string .= (defined $fields_string) ? ", $db_field_name" : $db_field_name;
        	    
        	    push @field_names, $field->{field_name};
        	}
        }
        
        my @results = DBAccess->sqlSelect(
            $fields_string, 
            'extended_user',
            "userid='$self->{uid}'"
        );
    
        foreach my $value (@results) {
        	my $field_name = shift @field_names;    	
        	$self->set_value( $field_name, $value );
        }    
    }
}

sub www_show_users {
    my $self = shift;

    $self->debug_msg(3, "Metadot::User::FlexUser::www_show_users()");
    $USER->www_show_users_groups();

    exit;
    ### no longer used

	my @filters = ();
	my $add_tables = "";

    if ($FORM{first_name}) {

	my $first_name_pos = $self->_get_storage_position( 'first_name' );

	my $value = DBAccess->quote( uc("%$FORM{first_name}%") );

	push @filters, "upper(extended_user.$first_name_pos) like $value";
    }
    
    if ($FORM{last_name}) {

	my $last_name_pos = $self->_get_storage_position( 'last_name' );

	my $value = DBAccess->quote( uc("%$FORM{last_name}%") );
	
	push @filters, "upper(extended_user.$last_name_pos) like $value";
    }
    
    if ($FORM{nickname}) {

	my $value = DBAccess->quote( uc("%$FORM{nickname}%") );

        push @filters, "upper(users.nickname) like $value";
    }

    if ($FORM{email}) {

	my $value = DBAccess->quote( uc("%$FORM{email}%") );
	
	push @filters, "upper(users.email) like $value";
    }

    if ($FORM{pending}) {
        push @filters, "users.approvalneeded = 'required'";
    }

    if ($FORM{who_online}) {
    	push @filters, "users.userid=sessions.userid and ".$SESSION->get_last_online_clause();
    	$add_tables = 'sessions';
    }
	my $filter = join(' AND ', @filters);

#    my $all_url = "";
    
    # add a 'show all users' link before generating the table if a search filter has been defined.
#    if ($filter ne "") {
#	my $iid = $FORM{iid} || 0 ;
#	$all_url = "index.pl?iid=$iid&amp;isa=User&amp;op=show_users&amp;sort_by=$FORM{sort_by}&amp;order=$FORM{order}";
#	$all_url = "\n<a href=\"" . $self->make_url($all_url) . "\">Show All Users</a><br>\n";
#    }

#    $DISPLAY->print_http_header;
      
#     print $self->show_top($FORM{iid}),
#     $self->show_title("User Directory"),
# #    $all_url,
#     #$self->show_user_list($filter, $FORM{from}, $FORM{nb}),
#     $self->show_user_list( {filter=>$filter, startfrom=>$FORM{from}, maxitem=>$FORM{nb}, add_tables =>$add_tables}),
#
#     "\n\n",
#     $self->show_bottom;

    my $content = $self->show_user_list( {filter=>$filter, startfrom=>$FORM{from}, maxitem=>$FORM{nb}, add_tables =>$add_tables});
    $DISPLAY->print_simple_layout({content=>$content, title=>"User Directory"});
}

##################################################
### show_user_list
### Show a list of user
### where condition clause as argument
##################################################
sub show_user_list {
    my $self = shift ;
    my ($filter, $startfrom, $maxitem) = @_;
    my $s1 = "";
    my $where;
    my $c;
    my $oldi=0;

    my ($uid, $fullname, $nickname, $email, $uploadsize, $uploadnumber);
    my $portal="";
    my $hpiid=0;

    $self->debug_msg(3, "Metadot::User::FlexUser::show_user_list(filter=$filter)");


    my $table_clause = "users, extended_user";
    
    # Filter is now a vector into an args hash. This maintains backward compatibility.
    my $args;
    if (ref($filter) eq 'HASH') {
	$args 	= $filter;
	$filter 	= $args->{filter};
	$startfrom 	= $args->{startfrom};
	$maxitem 	= $args->{maxitem};
	my $add_tables = $args->{add_tables}; # this can be an array ref

	# add tables to the table clause if needed
		if ($add_tables) {
		    $add_tables = [$add_tables] unless (ref($add_tables) eq 'ARRAY');
		    $table_clause = join(',', ($table_clause, @{$add_tables}));		
		}
    }
    
    
    if ($USER->is_admin()) {

	$where = ($filter) ? "extended_user.userid = users.userid and $filter" :  "users.userid = extended_user.userid";
	
    } else {
	
	$where = "users.userid>0 and extended_users.userid = users.userid";
	$where .= " and users.approvalneeded not in ('required', 'rejected')";
	$where .= " and $filter" if ($filter);
    }
    
    ### select user's portal iid
    
    my $message;
    if($FORM{uid}){
        my $u = $USER->new($FORM{uid});
        my $fullname .= " " . $u->webname;
        $message ="$fullname has been accepted and notified via email.";
    }
    #### table
    if($message){
        $s1 .= "<font color='#18bb94'>$message</font><br>";
    }

    my $order;
    
    if ( $FORM{order} ) {
	if ($FORM{order} eq "desc") {
	    $order = " DESC";
	} else {
	    $order = "ASC";
	}
    } else {
	$order = "ASC";
    }
    
    my $imageslocation = $PARAMS->{'imageslocation'};
        
    $FORM{sort_by} = 'last_name' unless $FORM{'sort_by'};

    my $first_name_pos = $self->_get_storage_position( 'first_name' );
    my $initial_pos = $self->_get_storage_position( 'initial' );
    my $last_name_pos = $self->_get_storage_position( 'last_name' );

	my $other;
    if( $FORM{sort_by} eq 'last_name' ) {
        $other = "order by extended_user.$last_name_pos $order";
    } elsif( $FORM{sort_by} eq 'nickname' ) {
        $other = "order by users.nickname $order";
    } elsif($FORM{sort_by} eq 'email' ) {
        $other = "order by users.email $order";
    } elsif( $FORM{sort_by} eq 'first_name' ) {
        $other = "order by extended_user.$first_name_pos $order";
    }
    # aju - make distinct
    ($c) = DBAccess->sqlSelectMany("distinct users.userid, extended_user.$first_name_pos, extended_user.$initial_pos, extended_user.$last_name_pos, users.nickname, users.email, users.max_total_upload_size, users.max_total_numberof_upload", $table_clause, $where, $other);

    $maxitem = 100 if (!$maxitem);   #PREVIOUS NEXT MAGIC
    $startfrom = 0  if(!$startfrom); #PREVIOUS NEXT MAGIC
    my $from = $startfrom;           #PREVIOUS NEXT MAGIC

    my $i=0;
    # my ($totalrows)=$c->rows;
	# rows function is not reliable for counting the rows retrieved by SELECT
	my ($totalrows) = DBAccess->sqlSelect("count(*)",$table_clause, $where, $other);

    my $max_total_upload_size = $PARAMS->{'max_total_upload_size'};
    my $max_total_number_upload = $PARAMS->{'max_total_number_upload'};

    if ($USER->is_admin) {         

	unless( MetadotConfig->get_param_value('user_management_is_external') ) {

	    $s1 .= "<center><h2><a href=\"index.pl?isa=Register&amp;op=register_form&amp;admin_mode=true\">Register a new user</a></h2></center>\n";
	}
    }

    $s1 .= "
<center><table border=\"0\" cellpadding = 0 cellspacing = 2>
                <tr>
                        <td colspan = 4>
                                <center><H2>Search for a user</H2></center>
                        </td>
                </tr>
                <tr>
                        <td>
                                First Name
                        </td>
                        <td>
                                Last Name / Family Name
                        </td>
                        <td>
                                Nickname / Screenname
                        </td>
                        <td>
                                Email address
                        </td>
                </tr>
                <tr>
                <tr>
                        <td>
                                <form method = post action = index.pl>
                                <input type=hidden name=isa value=User>
                                <input type=hidden name=op value=show_users>
                                <input type=hidden name=iid value = 0>
                                <input type=text size=15 name=first_name value=\"". $FORM{first_name} . "\">
                        </td>
                        <td>
                                <input type=text size=15 name=last_name value=\"". $FORM{last_name} . "\">
                        </td>
                        <td>
                                <input type=text size=15 name=nickname value=\"". $FORM{nickname} . "\">
                        </td>
                        <td>
                                <input type=text size=15 name=email value=\"". $FORM{email} . "\">
                        </td>
                </tr>
                <tr>
                        <td colspan = 4>
                                <center><input type=submit value = 'Start Search'></center></form>
                        </td>
                </tr>
         </table>";
         $s1 .= "</center><br><center>
         <table border=\"1\" cellpadding=\"4\" cellspacing=\"0\" width=\"100%\">
            <tr>
              <td valign='middle'><font size=-1><b><a class='prevnextbar' href=";
   

    my %order_arrows = ( first_name => '', last_name => '', nickname => '', email => '' );
    my %sort_direction = ( first_name => '', last_name => '', nickname => '', email => '' );
    my ( $sort_direction, $sort_image );

    if( $FORM{order} eq 'desc' ) {

	$order_arrows{ $FORM{sort_by} } = # secu_200401: needed to prevent bad input on iid, uid, etc.

	  "<img src=\"$imageslocation/obj/triangle_down.gif\" border=\"0\" alt=\"Sorted Descending\">";

	$sort_direction{ $FORM{sort_by} } = "&amp;order=asc";

    } else {

	$order_arrows{ $FORM{sort_by} } = 
	  "<img src=\"$imageslocation/obj/triangle_up.gif\" border=\"0\" alt=\"Sorted Ascending\">";

	$sort_direction{ $FORM{sort_by} } = "&amp;order=desc";
     
    }

	my $base_url = $self->make_url("index.pl?iid=0&amp;isa=User&amp;op=show_users" .
				       "&amp;first_name=$FORM{first_name}&amp;last_name=$FORM{last_name}" .
				       "&amp;email=$FORM{email}&amp;nickname=$FORM{nickname}" .
				       "&amp;pending=$FORM{pending}&amp;who_online=$FORM{who_online}");
    $s1 .= $base_url."&amp;sort_by=first_name$sort_direction{first_name}" . ">First Name $order_arrows{first_name}</a>&nbsp; </b> <I><a class='prevnextbar' href="; 

    $s1 .= $base_url."&amp;sort_by=last_name$sort_direction{last_name}" . ">Last Name / Family Name $order_arrows{last_name}</a>&nbsp; </b> <I><a class='prevnextbar' href="; 

    $s1 .= $base_url."&amp;sort_by=nickname$sort_direction{nickname}" . ">(nickname) $order_arrows{nickname}"; 

    $s1 .= "</a><I></font></td>
              <td valign='middle'><font size=-1><b>ID</b></font></td>
              <td><b>Portal</b></td>
                 <td valign='middle'><font size=-1><b><a class='prevnextbar' href=" .
     $base_url."&amp;sort_by=email$sort_direction{email}" .
      ">Email address $order_arrows{email}</a></b></font></td>     <td valign='middle'><font size=-1>Max. total upload size (MB)<br><font size=-5>(Default: $max_total_upload_size MB)</font></td>                     <td valign='middle'><font size=-1>Max. total number of upload<br><font size=-5>(Default: $max_total_number_upload)</font></td>    </tr>\n";

    my $hidden_users_count;
    my $current_user_is_admin = $USER->is_admin();
    my $is_allowed_to_see_hidden_users = $ACCESS_BROKER->is_allowed_to_do('server_setup');

    while(( my ($uid, 
		$first_name,
		$initial,
		$last_name,
		$nickname, 
		$email, 
		$uploadsize,
		$uploadnumber) = $c->fetchrow()) ) {
	
	# skip hidden users only when user doesn't have server_setup access
	my $user_can_see_fullname = $USER->belongs_to_grp('ADMIN', $uid) || $USER->belongs_to_grp('SITE_MGR', $uid);
	unless($is_allowed_to_see_hidden_users){
	    if ( $USER->is_hidden( $uid ) ) { 
		$hidden_users_count++;			    
		next 
	    }
	}
		
	if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC

	# skip the ones after the window
	if($i < $maxitem) {
	    $i++;
	} else {
	    next
	}
	
	# make fullname from subparts
	my $fullname = $first_name;
	
	$fullname .= ($initial) ?  
	  ' ' . $initial . ' ' : ' ';
	
	$fullname .= $last_name;
	
	if(($uploadsize || 0) == -1) {
	    $uploadsize = "Default";
	} elsif (!$uploadsize) {
	    $uploadsize = "";
	}

	if(($uploadnumber || 0) == -1){
	    $uploadnumber = "Default";
	} elsif (!$uploadnumber) {
	    $uploadnumber = "";
	}

	if( $user_can_see_fullname ){

	    $fullname = "<b>$fullname</b>";
	}

	### Prepare the user portal link
	my $u = $USER->new($uid);
	($hpiid) = $u->has_portal();
	$portal = "&nbsp;";
	$portal = "<a href=".$self->make_url("index.pl?iid=$hpiid").">Portal</a>\n" if ($hpiid);
	
	if ($nickname) {
	    $nickname = "<I>($nickname)</I>\n";
	} else {
	    $nickname = "";
	}
	
	my $rejected_string = "";
	if (($u->approval_needed() || "") eq "rejected"){
	    $rejected_string = "<font color=#FF0000>(rejected)</font>";
	}
	
	$email = "<a href=\"mailto:$email\">$email</a>\n" if ($email);
	
	$s1 .= "<font size=-3><tr><td>\n";

	if($current_user_is_admin){
	    $s1 .= $u->show_panel();
	}
	
	$rejected_string = $rejected_string ? $rejected_string : "";
	
	$s1 .= "<a href='".$self->make_url("index.pl?isa=User&amp;op=show_details&amp;uid=$uid")."'>$fullname</a> $nickname $rejected_string</td>\n";

	$s1 .= "<td>$uid</td>\n<td>$portal</td><td>$email</td><td>$uploadsize</td><td>$uploadnumber</td></tr></font>\n";
    }
    
    $c->finish();
    
    my $iid = $FORM{iid} || 0 ;

    my $url = "index.pl?iid=$iid&amp;isa=User&&amp;op=show_users&amp;threads=$FORM{threads}&amp;order=$FORM{order}&amp;sort_by=$FORM{sort_by}";
    # aju 05-28-02 filter info wasn't being passed in to the next/prev button.
    my $nurl = $url .
	    "&amp;first_name=$FORM{first_name}&amp;last_name=$FORM{last_name}" .
            "&amp;email=$FORM{email}&amp;nickname=$FORM{nickname}" ;

    my $switch = "";

    my ($pending_approval) = DBAccess->sqlSelect("count(*)", "users", "approvalneeded='required'");

   # add All Users link
    if( $FORM{pending} eq 'true' or $FORM{who_online} eq 'true' or $filter ) {
	my $link = "<a class='prevnextbar' href='". $self->make_url("$url") . "'>All Users</a>";

	$switch .= "<table width=\"100%\"><tr><td align='right'>[ $link ] </td></tr></table>";
    }

    if (($USER->is_admin) && ($pending_approval || (($FORM{pending} || "") eq 'true'))){
	unless ($FORM{pending} eq 'true') {
	    my $link = 
	      "<a class='prevnextbar' href='".$self->make_url("$url&amp;pending=true")."'>Pending users only</a>";
	    $switch .= "<table width=\"100%\"><tr><td align='right'>[ $link ] </td></tr></table>";
	}
    }
    
    # display online users
    my $who_online = $SESSION->get_members_online();
    if (($USER->is_admin) && ($who_online || (($FORM{who_online} || "") eq 'true'))){

	unless( $FORM{who_online} eq 'true' ) {
	    my $link = 
	      "<a class='prevnextbar' href='".$self->make_url("$url&amp;who_online=true")."'>Show $who_online online users only</a>";
	    $switch .= "<table width=\"100%\"><tr><td align='right'>[ $link ] </td></tr></table>";
	}
    }

    #PREVIOUS NEXT MAGIC
    my $pn = "" ;
    
    my $total_displayed = $totalrows - $hidden_users_count;
    
    if ($total_displayed > $maxitem) {
	$pn = Components->get_previous_next_button($maxitem, $i, $total_displayed, $startfrom, $nurl) 
    } else {
	
	$pn="\n\n<center><table border=\"0\" cellpadding=\"5\" cellspacing=\"5\" WIDTH='100%'><tr><td align='left'><font size=-1><I><b>Found $total_displayed entries</b></I></font></td></tr></table></center>";
	
    }

    $s1.="</td></td></td></tr></table></center><br>";
    my $page = "$switch\n$pn\n$s1\n$pn\n<br><I>Note: Names in bold are administrators</I>\n\n";
    return $page;
}

#aju
sub www_show_users_groups {
    my $self = shift;

    #print STDERR "in www_show_users_groups\n";
    $self->debug_msg(3, "Metadot::User::FlexUser::www_show_users_groups()");

    my @filters = (); my $groups_filter;
    my $add_tables = "";

    if ($FORM{first_name}) {

	my $first_name_pos = $self->_get_storage_position( 'first_name' );
	push @filters, "upper(extended_user.$first_name_pos) like upper('%$FORM{first_name}%')";
    }
    
    if ($FORM{last_name}) {

		my $last_name_pos = $self->_get_storage_position( 'last_name' );
		push @filters, "upper(extended_user.$last_name_pos) like upper('%$FORM{last_name}%')";
    }
    
    if ($FORM{nickname}) {
        push @filters, "upper(users.nickname) like upper('%$FORM{nickname}%')";
    }

    if ($FORM{email}) {
       push @filters, "upper(users.email) like upper('%$FORM{email}%')";
    }

    if ($FORM{groups}) {
	$groups_filter = $FORM{groups};
    }
    if ($FORM{pending}) {
        push @filters, "users.approvalneeded = 'required'";
    }
    if ($FORM{who_online}) {
    	push @filters, "users.userid=sessions.userid and ".$SESSION->get_last_online_clause();
    	$add_tables = 'sessions';
    }
    my $filter = join(' AND ', @filters);

    my $all_url = "";
    my $manage_users_url = "\n<a href=\"" . $self->make_url("index.pl?isa=User&amp;op=show_users") .
	"\">Go to 'Manage Users' Table</a><br>";

    # add a 'show all users' link before generating the table if a search filter has been defined.
    my $iid;
    if ($filter ne "") {
    	$iid = $FORM{iid} || 0 ;
    	$all_url = "index.pl?iid=$iid&amp;isa=User&amp;op=show_users_groups&amp;sort_by=$FORM{sort_by}&amp;order=$FORM{order}";
    	$all_url = "\n<a href=\"" . $self->make_url($all_url) . "\">Show All Users' Groups</a><br>\n";
    }

#     $DISPLAY->print_http_header;
#
#     print $self->show_top($FORM{iid}),
#     $self->show_title("User Directory"),
#    #aju - added for mbt
# #      "<A HREF='/metadot/index.pl?isa=DownloadUserExcel&amp;op=show_user_groups_list'>Download User-Groups Info as Excel Sheet</a><br>",
# #    $all_url,
#     #$manage_users_url,
#     $self->show_users_groups( {filter=>$filter, groups_filter => $groups_filter,
#                    startfrom=>$FORM{from}, maxitem=>$FORM{nb}, add_tables =>$add_tables}),
#
#     "\n\n",
#     $self->show_bottom;

    my $content = $self->show_users_groups( {filter=>$filter, groups_filter => $groups_filter,
			       startfrom=>$FORM{from}, maxitem=>$FORM{nb}, add_tables =>$add_tables});
    $DISPLAY->print_simple_layout({content=>$content, title=>"User Directory", iid=>$iid});

}


#aju
sub show_users_groups {
    my $self = shift ;
    my ($filter, $startfrom, $maxitem) = @_;
    my $s1 = "";
    my $c;
    my $oldi=0;

    #print STDERR "in show_users_groups\n";
    my ($uid, $fullname, $nickname, $email, $uploadsize, $uploadnumber);
    my $portal="";
    my $hpiid=0;

    $self->debug_msg(3, "Metadot::User::FlexUser::show_users_groups(filter=$filter)");

    # Filter is now a vector into an args hash. This maintains backward compatibility.
    my $args; my $groups_filter;
	if (ref($filter) eq 'HASH') {
		$args 	= $filter;
		$filter 	= $args->{filter};
		$startfrom 	= $args->{startfrom};
		$maxitem 	= $args->{maxitem};
		$groups_filter = $args->{groups_filter};

	}


    ### select user's portal iid
    
    my $message;
    if($FORM{uid}){
        my $u = $USER->new($FORM{uid});
        my $fullname .= " " . $u->webname;
        $message ="$fullname has been accepted and notified via email.";
    }
    #### table
    if($message){
        $s1 .= "<font color='#18bb94'>$message</font><br>";
    }

    my $imageslocation = $PARAMS->{'imageslocation'};
        
    $FORM{sort_by} = 'last_name' unless $FORM{'sort_by'};

    ## get the user result set
    $c = $self->get_show_users_groups_result_set(@_);
    
    $maxitem = 100 if (!$maxitem);   #PREVIOUS NEXT MAGIC
    $startfrom = 0  if(!$startfrom); #PREVIOUS NEXT MAGIC
    my $from = $startfrom;           #PREVIOUS NEXT MAGIC
    
    my $i=0;
    my ($totalrows) = $c->rows;
    
    my $max_total_upload_size = $PARAMS->{'max_total_upload_size'};
    my $max_total_number_upload = $PARAMS->{'max_total_number_upload'};
    
    #if($totalrows <= $maxitem){
    #    $s1 .="<br><font size=-1><I><b>Found $totalrows entries</b></I></font>";
    #}

    $s1 .= "
<center>
          <form method = post action = index.pl>
            <input type=hidden name=isa value=User>
            <input type=hidden name=op value=show_users_groups>
            <input type=hidden name=iid value = 0>
          <center><H2>Search for a user</H2></center>
          <table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" align=\"center\" bgcolor=\"#EEEEEE\"><tr><td>
              <table border=\"0\" cellpadding = 3 cellspacing = 3 width=\"100%\">
                <tr bgcolor=\"#FFFFFF\" align=center>
                        <td>
			    <b>First Name</b><br><input type=text size=15 name=first_name value=\"" .  $FORM{first_name} . "\">
                        </td>
                        <td>
			    <b>Last Name / Family Name</b><br><input type=text size=15 name=last_name value=\"". $FORM{last_name} . "\">
                        </td>
                        <td>
			    <b>Nickname / Screenname</b><br><input type=text size=15 name=nickname value=\"". $FORM{nickname} . "\">
                        </td>
                        <td>
			    <b>Email address</b><br><input type=text size=15 name=email value=\"". $FORM{email} . "\">
                        </td>
                        <td>
			    <b>Groups</b><br><input type=text size=15 name=groups value=\"" . $FORM{groups} . "\">
                        </td>
                </tr>
                <tr bgcolor=\"#FFFFFF\">
                        <td colspan = 5>
                                <center><input type=submit value = 'Start Search'></center></form>
                        </td>
                </tr>
              </table>
          </td></tr></table>\n";
    $s1 .= "</center><br><center>
         <table border=\"1\" cellpadding=\"4\" cellspacing=\"0\" width=\"98%\">
            <tr>
              <td valign='middle'><font size=-1><b><a class='prevnextbar' href=";
   

    my %order_arrows = ( first_name => '', last_name => '', nickname => '', email => '' );
    my %sort_direction = ( first_name => '', last_name => '', nickname => '', email => '' );
    my ( $sort_direction, $sort_image );

    if( $FORM{order} eq 'desc' ) {
	$order_arrows{ $FORM{sort_by} } = 
	  "<img src=\"$imageslocation/obj/triangle_down.gif\" border=\"0\" alt=\"Sorted Descending\">";
	$sort_direction{ $FORM{sort_by} } = "&amp;order=asc";
    } else {
	$order_arrows{ $FORM{sort_by} } = 
	  "<img src=\"$imageslocation/obj/triangle_up.gif\" border=\"0\" alt=\"Sorted Ascending\">";
	$sort_direction{ $FORM{sort_by} } = "&amp;order=desc";
    }

    my $base_url = $self->make_url("index.pl?iid=0&amp;isa=User&amp;op=show_users_groups" .
				   "&amp;first_name=$FORM{first_name}&amp;last_name=$FORM{last_name}" .
				   "&amp;email=$FORM{email}&amp;nickname=$FORM{nickname}" .
				   "&amp;groups=$FORM{groups}" .
				   "&amp;pending=$FORM{pending}&amp;who_online=$FORM{who_online}");
    $s1 .= $base_url."&amp;sort_by=first_name$sort_direction{first_name}" . ">First Name $order_arrows{first_name}</a>&nbsp; </b> <I><a class='prevnextbar' href="; 

    $s1 .= $base_url."&amp;sort_by=last_name$sort_direction{last_name}" . ">Last Name $order_arrows{last_name}</a>&nbsp; </b> <I><a class='prevnextbar' href="; 

    $s1 .= $base_url."&amp;sort_by=nickname$sort_direction{nickname}" . ">(nickname) $order_arrows{nickname}"; 

    $s1 .= "</a><I></font></td>
              <td valign='middle'><font size=-1><b>ID</b></font></td>" .
#                "<td><b>Portal</b></td>" .
                 "<td valign='middle'><font size=-1><b><a class='prevnextbar' href=" .
     $base_url."&amp;sort_by=email$sort_direction{email}" .
      ">Email address $order_arrows{email}</a></b></font></td>";
#       my $upload_cells = "<td valign='middle'><font size=-1>Max. total upload size (MB)<br><font size=-5>(Default: $max_total_upload_size MB)</font></td>                     <td valign='middle'><font size=-1>Max. total number of upload<br><font size=-5>(Default: $max_total_number_upload)</font></td>";
    if ($PARAMS->{allow_organization_registration}) {
        $s1 .= "<td><b>Organization</b></td>";
    }
    $s1 .= "<td><b>Groups</b></td>";
    $s1 .= "</tr>\n";

#     print STDERR "using groups filter: $groups_filter\n";

    my $hidden_users_count;
    my $is_allowed_to_see_hidden_users = $USER->is_admin();
    my $current_user_is_admin = 0;
    
# 	my $user_can_see_fullname = $USER->belongs_to_grp('ADMIN', $uid);
    while(( my ($uid, $fullname,
		$first_name,
		$initial,
		$last_name,
		$nickname, 
		$email, 
		$uploadsize,
		$uploadnumber,
        $organization_grp_id) = $c->fetchrow()) && ($i < $maxitem)) {


        ## mike - not needed
        ## skip hidden users only when user doesn't have server_setup access
    	#unless( $is_allowed_to_see_hidden_users ){
    	#    if ( $USER->is_hidden( $uid ) ) { 
    	#	$hidden_users_count++;			    
    	#	next;
    	#    }
    	#}
        #my $user_can_see_fullname = $USER->belongs_to_grp('ADMIN', $uid) || $USER->belongs_to_grp('SITE_MGR', $uid);
    	
    	
    	my $u = $USER->new($uid);
        $current_user_is_admin = $u->is_admin();

    	my $user_groups; my @user_groups_sort = ();
    	# see if the groups filter string is in the groups list
    	if ($uid) {		#  this excludes root, uid 0, also
      	    #print STDERR "getting user groups for $uid with name $fullname\n";
    	    $user_groups = Group->get_groups_where_member($uid);
      	    #print STDERR "uid $uid ($first_name $last_name) is in groups: ", Dumper($user_groups), "\n";
    	    #sort the group names
    	    @user_groups_sort = sort(@{$user_groups});
    	}
    	#format the user's group info to go into the table cell
    #  	my @user_groups_links = ();
    #  	foreach my $ug (@user_groups_sort) {
    #  	    push @user_groups_links, "<a href=\"index.pl\">$ug</a>";
    #  	}
      	my $user_groups_fmt = join(', ', @user_groups_sort);
    	if (!$user_groups_fmt || ($user_groups_fmt eq '')) {
    	    $user_groups_fmt = "&nbsp;";
    	}
    
    	if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
    
    	$i++;
    	if($current_user_is_admin){
    	    $fullname = "<b>$fullname</b>";
    	}
    	
    	if ($nickname) {
    	    $nickname = "<I>($nickname)</I>\n";
    	} else {
    		$nickname = "";
    	    }
    
    	my $rejected_string = "";
    	if (($u->approval_needed() || "") eq "rejected"){
    	    $rejected_string = "<font color=#FF0000>(rejected)</font>";
    	}
    
    	$email = "<a href=\"mailto:$email\">$email</a>\n" if ($email);
    	$s1 .= "<font size=-3><tr><td>\n";
        if($is_allowed_to_see_hidden_users){
    	    $s1 .= $u->show_panel('user_groups');
    	}
    	$rejected_string = $rejected_string ? $rejected_string : "";
    
    	$s1 .= "$fullname $nickname $rejected_string [ <a href='".
            $self->make_url("index.pl?isa=User&amp;op=show_details&amp;uid=$uid") . "'>view profile</a> ]";

        $s1 .= "</td>\n";
    
    	$s1 .= "<td>$uid</td>\n" .
    		"<td>$email</td>";
    		    
        if ($PARAMS->{allow_organization_registration}) {
            $s1 .= "<td>" . ($u->get_organization_name() || "&nbsp;") . "</td>";
        }
        $s1 .= "<td>$user_groups_fmt</td>" .
    			"</tr></font>\n";
  }

    $c->finish();
  
    my $iid = $FORM{iid} || 0 ;
    my $url = "index.pl?iid=$iid&amp;isa=User&&amp;op=show_users_groups&amp;threads=$FORM{threads}&amp;order=$FORM{order}&amp;sort_by=$FORM{sort_by}";

    # include filter info in next/prev links
    my $nurl = $url .
	"&amp;first_name=$FORM{first_name}&amp;last_name=$FORM{last_name}" .
        "&amp;email=$FORM{email}&amp;nickname=$FORM{nickname}" .
	"&amp;groups=$FORM{groups}" ;
    my $switch = "";

    my ($pending_approval) = DBAccess->sqlSelect("count(*)", "users", "approvalneeded='required'");

   # add All Users link
    if( $FORM{pending} eq 'true' or $FORM{who_online} eq 'true' or $filter ) {
	my $link = "<a class='prevnextbar' href='". $self->make_url("$url") . "'>All Users</a>";

	$switch .= "[ $link ] ";
    }

    if (($USER->is_admin) && ($pending_approval || (($FORM{pending} || "") eq 'true'))){
	unless ($FORM{pending} eq 'true') {
	    my $link = 
	      "<a class='prevnextbar' href='".$self->make_url("$url&amp;pending=true")."'>Pending users only</a>";
	    $switch .= "[ $link ] ";
	}
    }
    
    # display online users
    my $who_online = $SESSION->get_members_online();
    if (($USER->is_admin) && ($who_online || (($FORM{who_online} || "") eq 'true'))){

	unless( $FORM{who_online} eq 'true' ) {
	    my $link = 
	      "<a class='prevnextbar' href='".$self->make_url("$url&amp;who_online=true")."'>Show $who_online online users only</a>";
	    $switch .= "[ $link ] ";
	}
    }

    if ($USER->is_admin) {         

	unless( MetadotConfig->get_param_value('user_management_is_external') ) {
	    #$s1 .= "<center><h2><a href=\"index.pl?isa=Register&amp;op=register_form&amp;admin_mode=true\">Register a new user</a></h2></center>\n";
	    my $link = 
	      "<a class='prevnextbar' href='".$self->make_url("index.pl?isa=Register&amp;op=register_form&amp;admin_mode=true")."'>Register a new user</a>";
	    $switch .= "[ $link ] ";
	}
    }
  
  
    #PREVIOUS NEXT MAGIC
    my $pn = "" ;

    my $total_displayed = $totalrows - $hidden_users_count;
    
    if ($total_displayed > $maxitem) {
	$pn = Components->get_previous_next_button($maxitem, $i, $total_displayed, $startfrom, $nurl);
    } else {
	
	$pn="\n\n<center><table border=\"0\" cellpadding=\"5\" cellspacing=\"5\" WIDTH='100%'><tr><td align='left'><font size=-1><I><b>Found $total_displayed entries</b></I></font></td></tr></table></center>";
	
    }

#    $pn = Components->get_previous_next_button($maxitem, $i, $totalrows, $startfrom, $nurl) if ($totalrows > $maxitem);
    
    $s1.="</td></td></td></tr></table></center><br>";
    my $page = "$switch\n$pn\n$s1\n$pn\n<br><I>Note: Names in bold are administrators</I>\n\n";
    return $page;
}

sub get_show_users_groups_result_set {
    my $self = shift ;
    my ($filter, $startfrom, $maxitem) = @_;
    $self->debug_msg(3, "Metadot::User::FlexUser::get_show_users_groups_result_set(filter=$filter)");
    my $where = '';
    my $c;

    my $table_clause = "users, extended_user";

    # Filter is now a vector into an args hash. This maintains backward compatibility.
    my $args; my $groups_filter;
	if (ref($filter) eq 'HASH') {
		$args 	= $filter;
		$filter 	= $args->{filter};
		$startfrom 	= $args->{startfrom};
		$maxitem 	= $args->{maxitem};
		my $add_tables = $args->{add_tables}; # this can be an array ref
		$groups_filter = $args->{groups_filter};

		# add tables to the table clause if needed
		if ($add_tables) {
			$add_tables = [$add_tables] unless (ref($add_tables) eq 'ARRAY');
			$table_clause = join(',', ($table_clause, @{$add_tables}));		
		}
	}


    if ($USER->is_admin()) {

	$where = ($filter) ? "extended_user.userid = users.userid and $filter" :  "users.userid = extended_user.userid";

    } else {

	$where = "users.userid >0 and extended_user.userid = users.userid";
	$where .= " and users.approvalneeded not in ('required', 'rejected')";
	$where .= " and $filter" if ($filter);
    }

    if ($groups_filter && ($groups_filter ne '')) {
	$table_clause .= ", grpmembers, grp";
	$where .= " and grp.group_name like '%$groups_filter%' and grp.grp_id = grpmembers.grp_id and grpmembers.userid = users.userid";
    }

    my $order;
    
    if ( $FORM{order} ) {
	if ($FORM{order} eq "desc") {
	    $order = " DESC";
	} else {
	    $order = "ASC";
	}
    } else {
	$order = "ASC";
    }
        
    $FORM{sort_by} = 'last_name' unless $FORM{'sort_by'};

    my $first_name_pos = $self->_get_storage_position( 'first_name' );
    my $initial_pos = $self->_get_storage_position( 'initial' );
    my $last_name_pos = $self->_get_storage_position( 'last_name' );
    my $organization_pos = $self->_get_storage_position( $self->get_organization_field_name );

    my $main_clause = "distinct users.userid, users.fullname, extended_user.$first_name_pos, extended_user.$initial_pos, extended_user.$last_name_pos, users.nickname, users.email, users.max_total_upload_size, users.max_total_numberof_upload, extended_user.$organization_pos";
    if( $FORM{sort_by} eq 'last_name' ) {
	
        ($c) = DBAccess->sqlSelectMany($main_clause, $table_clause, $where, "order by extended_user.$last_name_pos $order");

    } elsif( $FORM{sort_by} eq 'nickname' ) {

        ($c) = DBAccess->sqlSelectMany($main_clause, $table_clause, $where, "order by users.nickname $order");

    } elsif($FORM{sort_by} eq 'email' ) {

        ($c) = DBAccess->sqlSelectMany($main_clause, $table_clause, $where, "order by users.email $order");

    } elsif( $FORM{sort_by} eq 'first_name' ) {

        ($c) = DBAccess->sqlSelectMany($main_clause, $table_clause, $where, "order by extended_user.$first_name_pos $order");
    }

    return $c;
}

sub www_show_user_directory {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Metadot::User::FlexUser::www_show_user_directory()");
    my $str = "";

    unless ($PARAMS->{show_user_directory}){
        $self->print_click_back("Error", "Sorry, you cannot view user details.<br /><br />");
        exit;
    }

    # XXX [claudio] $maxitem is not used
    my $thingtosearch = $args{search_words};
    my $iid           = $args{iid} || 0;
    my $object        = $args{object} || $self;

    if ($thingtosearch eq "") {
        $str = "<B>&nbsp; Please, enter a search word!<\/B>";
    } else {
        $str = $self->get_user_directory_results(@_);
        if ($str eq "") {
            $str= "&nbsp; No results matched your query.";
        }
    }

    # if we are in template view mode we call the template
    # get the template that is not the main template
    my $look = $PARAMS->{globallook};
    if ($look eq "usetemplate") {
        $object->print_http_header(); #CKIM: ensure http header is sent
        print  $DISPLAY->simple_expand($iid, $object, $object->show_template(-1), $str);
    } else {
        # normal view mode
        #print "normal view mode";
        $object->print_portal($str,"Search Result: $FORM{filter}",$iid);
    }
}

sub get_user_directory_results {
    my $self = shift;
    my %args = @_;
    my $search_words = $args{search_words};
    my $from         = $args{from};
    my $original_from = $from;
    my $nb           = $args{nb}  || 100;
    my $iid          = $args{iid} || 0;
   
    $self->debug_msg(3, "Metadot::User::FlexUser::get_user_directory_results()");
    my $filter = $self->get_search_where_for_all_user_fields(search_words => $search_words);
    $filter = "($filter)" if $filter ne '';
    ## get the user result set
    my $c = $self->get_show_users_groups_result_set( {filter=> $filter, startfrom=> $from, maxitem=>$nb } );
    #return $self->show_users_groups( {filter=> $filter, startfrom=>$from, maxitem=>$nb } );
    my $i=0;
    my $hidden_users_count;
    my $is_allowed_to_see_hidden_users = $USER->is_logged_in() && $ACCESS_BROKER->is_allowed_to_do('server_setup');
    my $s = '';
    my $user;
    while(( my ($uid, $fullname,
		$first_name,
		$initial,
		$last_name,
		$nickname, 
		$email, 
		$uploadsize,
		$uploadnumber,
        $organization_grp_id) = $c->fetchrow()) && ($i < $nb)) {
        # skip hidden users only when user doesn't have server_setup access
    	unless( $is_allowed_to_see_hidden_users ){
    	    if ( $USER->is_hidden( $uid ) ) { 
    		$hidden_users_count++;			    
    		next;
    	    }
    	}
    	if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC

        $user =  $USER->new($uid);
        #### CONTENT
        
        $s .= $user->get_user_details_view();
        $s .= "<hr size=0>\n";

        #### END CONTENT

    	$i++;
     }

    #PREVIOUS NEXT MAGIC
    my $pn = "" ;

    my ($totalrows) = $c->rows;
    my $total_displayed = $totalrows - $hidden_users_count;
    
#    if ($total_displayed > $nb) {
        my $url = "index.pl?iid=$iid&amp;isa=Portal&amp;op=search&amp;filter=$FORM{filter}&amp;search_choice=$FORM{search_choice}";
	$pn = Components->get_previous_next_button($nb, $i, $total_displayed, $original_from, $url);
#    } else {
#	$pn="\n\n<center><table border=0 cellpadding=5 cellspacing=5 WIDTH='100%'><tr><td align='left'><font size=-1><I><B>Found $total_displayed entries</b></I></font></td></tr></table></center>";
#    }

    return $s ne '' ? "\n$pn\n$s\n$pn\n\n\n" : '';
}

sub get_user_details_view {
    my $self = shift;
    my %args = @_;

    my $view = $args{view} || 'brief';
    my $uid  = $args{uid}  || $self->uid();

    if ($view eq 'businesscard') { ## same
        $view = 'medium';
    }

    my $user = $USER->new($uid);

    my $show_medium_details_link = "index.pl?isa=User&amp;op=show_user_details&amp;view=medium&amp;uid=" . $user->uid();
    my $show_full_details_link = "index.pl?isa=User&amp;op=show_user_details&amp;view=full&amp;uid=" . $user->uid();
    my $show_image_details_link = "index.pl?isa=User&amp;op=show_user_details&amp;view=imagezoom&amp;uid=" . $user->uid();

    my $full_width = $view eq 'imagezoom' ? 1 : 0;
    my $image_src = $user->get_image_src(full_width => $full_width);
    my $zoom_image = '';
    #if ($image_src) {
        if ($view eq 'brief' || $view eq 'imagezoom') {
            $image_src = "<a href=\"$show_medium_details_link\">$image_src</a>";
        #} elsif ($view eq 'medium') {
        #    $image_src = "<a href=\"$show_full_details_link\">$image_src</a>";
        }
        $zoom_image = "<a href=\"$show_image_details_link\"><img src=\"" . $PARAMS->{'imageslocation'} . "/zoom.gif\" border = 0 valign=top></a>&nbsp;";
    #} else {
    #    $image_src = $no_image_src;
    #}



    my $s = '';

    if ($view eq 'brief') {
        $s .= "<table border=0 cellspacing=2 cellpadding=2>\n";
        $s .= "  <tr><td valign=top>$image_src</td>\n";
        $s .= "      <td valign=top>";
        $s .= "<b><a href=\"$show_medium_details_link\">" . $user->{fullname} . "</a></b>";
        $s .= ", " . $user->{job_title} if $user->{job_title};
        #$s .= "  <font class=\"smaller\">";     
        $s .= "  <br> <a href=\"mailto:" . $user->{email} . "\">" . $user->{email}  . "</a>";
        $s .= "  <br><b>Phone:</b> " . $user->{telephone} if $user->{telephone};
        $s .= "  <br><b>Mobile Phone:</b> " . $user->{mobile_phone} if $user->{mobile_phone};
        $s .= "  <br /><br /><a class=\"smaller\" href=\"$show_medium_details_link\">business card</a>";
        if ($PARAMS->{my_website_creation}) {
            my $portal_id = $user->has_portal();
            $s .= "<font class=\"smaller\">,</font> <a class=\"smaller\" href=\"" .$user->make_url("index.pl?iid=$portal_id") . "\">my website</a>\n" if ($portal_id);
        }
        #$s .= "</font>";
        $s .= "      </td>\n";
        $s .= "  </tr>\n";
        $s .= "</table>\n";

    } else {
        $s .= "<div style=\"width:315px;padding:2em;margin:4em;border:thin solid black\">\n";
        $s .= "<table border=0 cellspacing=2 cellpadding=2 width=300>\n";
        $s .= "  <tr><td valign=top align=center>\n";
        $s .= "        $image_src\n";
        if ($view ne 'imagezoom') {
            $s .= "<br>\n";
            $s .= $zoom_image;
            if ($PARAMS->{my_website_creation}) {
                my $portal_id = $user->has_portal();
                $s .= "<a href=\"" .$user->make_url("index.pl?iid=$portal_id") . "\">my website</a>\n" if ($portal_id);
            }
            $s .= "      </td>\n";
            $s .= "      <td valign=top>";
            $s .= "<b>" . $user->{fullname} . "</b>";
            $s .= "<br><a href=\"mailto:" . $user->{email} . "\">" . $user->{email}  . "</a>";
            $s .= "<br>" . $user->{job_title} if $user->{job_title};
            $s .= "<p>";
            $s .= "<br>" . $user->{address_line_1} if $user->{address_line_1};
            $s .= "<br>" . $user->{address_line_2} if $user->{address_line_2};
            $s .= "<br>" . $user->{city} . "," if $user->{city};
            $s .= "&nbsp;" . $user->{state} if $user->{state};
            $s .= "&nbsp;" . $user->{zip_code} if $user->{zip_code};
            $s .= "<br><b>Phone:</b> " . $user->{telephone} if $user->{telephone};
            $s .= "<br><b>Mobile Phone:</b> " . $user->{mobile_phone} if $user->{mobile_phone};
            $s .= "<br><b>Fax:</b> " . $user->{fax} if $user->{fax};
        }

        if ($view eq 'full') {
            $s .= "<br><b>Home Telephone:</b> " . $user->{home_phone} if $user->{home_phone};
            $s .= "<br><b>Home Fax:</b> " . $user->{home_fax} if $user->{home_fax};
            $s .= "<br><b>Home Address:</b> " . $user->{home_address} if $user->{home_address};
            $s .= "<br><b>Pager Phone:</b> " . $user->{pager_phone} if $user->{pager_phone};
            $s .= "<br><b>Pager Number:</b> " . $user->{pager_number} if $user->{pager_number};
            $s .= "<br><b>ICQ Number:</b> " . $user->{icq_number} if $user->{icq_number};
            $s .= "<br><b>Nickname / Screename:</b> " . $user->{nickname} if $user->{nickname};
            $s .= "<br><b>Country:</b> " . $user->{country} if $user->{country};
            $s .= "<br><b>Homepage URL:</b> " . $user->{homepage} if $user->{homepage};
            $s .= "<br><b>Schedule:</b> " . $user->{schedule} if $user->{schedule};
            $s .= "<br><b>Notes:</b> " . $user->{notes} if $user->{notes};
            $s .= "<br><b>Quotes:</b> " . $user->{quotes} if $user->{quotes};

            $s .= "<p><a href=\"$show_medium_details_link\">Less detail</a>";
        }

        if ($view eq 'medium') {
            $s .= "<p><a href=\"$show_full_details_link\">More detail</a>";
        }

        $s .= "      </td>\n";
        $s .= "  </tr>\n";
        $s .= "</table>\n";

        $s .= "</div>\n";
    }
 


        

    return $s;
}


sub www_show_user_details {
    my $self = shift;
    $self->debug_msg(3, "Metadot::User::FlexUser::www_show_user_details()");

    unless ($PARAMS->{show_user_directory}){
        $self->print_click_back("Error", "Sorry, you cannot view user details.<br /><br />");
        exit;
    }

    my $s = $self->get_user_details_view(uid => $FORM{uid}, view => $FORM{view});
    my $look = $PARAMS->{globallook};
    if ($look eq "usetemplate") {
        print  $DISPLAY->simple_expand(0, $self, $self->show_template(-1), $s);
    } else {
        # this is compatible with the new template mode
        $DISPLAY->print_simple_layout({content=>$s,title=>"User Information"});
    }
}

sub get_modify_settings_form_field_values {
    my $self = shift;
    die "method get_modify_settings_form_field_values should always be overidden in default classes: $!";
}
 


my $fields_to_storage;

sub get_fields_to_storage_hash {
    my $self = shift;
    unless ( defined $fields_to_storage ) {

	my $config = $self->get_fields_config();
	
	foreach my $field ( @{ $config->{FIELDS} } ) {
	    if( lc $field->{storage_type} eq 'secondary' ) {
		
		my $field_name = $field->{field_name};
		my $position = $field->{store_at_column};
		
		$fields_to_storage->{ $field_name } = $position;
		
	    }
	}
    }

    return $fields_to_storage;
}

sub _get_storage_position {
    my $self = shift;
    my $field_name = shift;

    my $fts = $self->get_fields_to_storage_hash();
    return $fts->{$field_name};
}

sub get_search_where_for_all_user_fields {
    my $self = shift;
    my %args = @_;

    my $search_words = $args{search_words};
    return '' unless $search_words;
    $search_words = uc($search_words);
    $search_words = DBAccess->quote("%$search_words%");

    my $fts = $self->get_fields_to_storage_hash();

    return "upper(users.nickname) like $search_words or upper(users.email) like $search_words or " . join(' or ', map("upper(extended_user.$_) like $search_words", values(%$fts)));
}

sub get_organizations {
    my $self = shift;

    my $groups = Group->get_organization_groups();

    my @groups_array = ( { form_value => '', label => '- none -'} );

    foreach my $grp_id ( sort {$groups->{$a} cmp $groups->{$b}} keys (%$groups) ) {
        push @groups_array, { form_value => $grp_id, label => $groups->{$grp_id} };
    }
    return \@groups_array;
}

sub get_homepage_values {
    my $self = shift;

    my $field_name = $self->get_flex_homepage_field_name();
    my $value = $self->get_value($field_name);

    my @homepages= ();
    my $radio_index = 0;
    if ($PARAMS->{allow_organization_registration}) {
        push @homepages, { form_value => $ORG_KEY, label => 'My Organization Homepage' };
        $radio_index++;
    }
    my $referer = $SESSION->referer();
    my $referer_label = '';
    if (length($referer) > 70) {
        $referer_label = substr($referer, 0, 70) . "...";
    } else {
        $referer_label = $referer;
    }
    push @homepages, { form_value => $referer, label => "Use recent: $referer_label" };
    $radio_index++;

    my $url = '';
    if ($value ne '' && $value ne $ORG_KEY) {##then a url
        $url = $value;
    }
    
    # unless a default value is specified, both the "default" and "url" options will be checked.
    $url ||= "Type in a URL";
    my $url_text_box  = <<EOM
    <input type="text" name="__url_text__" value="$url" onBlur="document.profile_form.$field_name\[$radio_index\].value=this.value">
EOM
;
    push @homepages, { form_value => $url, label => "Use URL $url_text_box" };
    $radio_index++;

    push @homepages, { form_value => '', label => "Portal Default" };
    $radio_index++;

    return \@homepages
}

sub get_user_defined_homepage {
    my $self = shift;

    my $field_name = $self->get_flex_homepage_field_name();
    my $value = $self->get_value($field_name);

    Metadotclass->debug_msg(5, "get_user_defined_homepage starts with value of '$value'");
    
    my $url = undef;
    #print STDERR "\n\nvalue:$value\n\n";
    if ($value eq $ORG_KEY) {
        Metadotclass->debug_msg(5, "get_user_defined_homepage has detected that this is the organization homepage");
        if ($PARAMS->{allow_organization_registration}) {
            my $organization_field_name = $self->get_organization_field_name();
            my $organization_grp_id = $self->get_value($organization_field_name);
            #print STDERR "\n\norganization_grp_id:$organization_grp_id\n\n";
            Metadotclass->debug_msg(5, "get_user_defined_homepage found user's organization as '$organization_grp_id', using property '$organization_field_name'");
            if ($organization_grp_id) {
                my $group = Group->new($organization_grp_id);
                $url = $group->{organization_homepage};
                #print STDERR "\n\ngroup url:$url\n\n";
                Metadotclass->debug_msg(5, "get_user_defined_homepage uses organization url of '$url'");
            }
        }
    } elsif ($value ne '') {
        $url = $value;
    }
    #print STDERR "\n\nurl:$url\n\n";

    Metadotclass->debug_msg(5, "get_user_defined_homepage returns '$url'");
    return $url;
}

## returns user's organization name
sub get_organization_name {
    my $self = shift;
    my $organization_grp_id = $self->get_value($self->get_organization_field_name());
    #print STDERR "\n\norganization_grp_id:'$organization_grp_id'\n\n";
    return '' unless $organization_grp_id;
    my ($name, $is_organization) = Group->get_organization_info($organization_grp_id);
    #print STDERR "\n\nname:'$name' is_organization:'$is_organization'\n\n";

    return $is_organization ? $name : '';
}

sub add_user_to_primary_organization {
    my $self = shift;
    my $uid  = $self->uid();
    my $organization_grp_id = $self->get_value($self->get_organization_field_name());

    return unless $uid > 0 && $organization_grp_id > 0 ;

    ## organization registration must be turned on
    return unless ($PARAMS->{allow_organization_registration});
    ## must be accepted
    return unless (!$self->{approval_needed}) || ($self->{approval_needed} eq "accepted");
    ## group must be an organization
    return unless Group->is_group_an_organization($organization_grp_id);

    if (!DBAccess->sqlSelect ("grp_id", "grpmembers", "grp_id=$organization_grp_id and userid='$uid'")) {
        Group->_add_a_member_to_a_group($organization_grp_id, $uid);
    }    
}

## removes user's primary organization choice if it
## matches grp_id
sub remove_user_primary_organization_choice {
    my $self = shift;
    my $grp_id = shift;

    my $organization_grp_id = $self->get_value($self->get_organization_field_name());
    #print STDERR "\n\ngrp_id:$grp_id o_grp_id:$organization_grp_id\n\n";
    if ($grp_id && $organization_grp_id && $grp_id == $organization_grp_id) {
        $self->set_value($self->get_organization_field_name(), '');
        $self->save();
    }
}

sub get_organization_field_name {
    my $self = shift;
    return "organization_grp_id";
}

sub get_flex_homepage_field_name {
    my $self = shift;

    return "flex_homepage";
}

sub get_upload_filename_icon_and_url {
    my $self = shift;
    my %args = @_;

    my $field_name = $args{field_name};
    my $uid        = $args{uid} || $self->uid();
    return $self->SUPER::get_upload_filename_icon_and_url( {
        id => $uid,
        isa => $self->is_a(),
        field_name => $field_name,
    });
}

sub get_picture_field_name {
    my $self = shift;
    return 'picture';
}

sub get_image_src {
    my $self = shift;
    my %args = @_;

    my $field_name = $args{field_name} || $self->get_picture_field_name();
    my $uid        = $args{uid} || $self->uid();
    my $full_width = $args{full_width} || 0;

    return '' unless $uid;

    #print STDERR "\n\nfield_name:$field_name\n\n";
    my ( $file, $file_icon, $file_url ) = $self->get_upload_filename_icon_and_url(field_name => $field_name, uid => $uid);
    #print STDERR "file:$file\n\nfile_icon:$file_icon\n\nfile_url:$file_url\n\n";

    my $width = $full_width ? '' : 'width="68"';
    return $file_url ne '' ? "<img src=\"$file_url\" border='0' $width alt='User photograph'/>" : $self->no_image_src();
}

sub no_image_src {
    return "<img src=\"" . $PARAMS->{'imageslocation'} . "/nopic.jpg\" border=0 width=68>";
}

sub www_download_file {
    my $self = shift;

### mike: not restricting to user directory
#    return '' unless $PARAMS->{show_user_directory};

    my $field_name = $FORM{field_name};
    my $config = $self->get_config_for_field($field_name);
    my $field_type = $config->{field_type};
    return $self->SUPER::www_download_file( {
        id => $self->uid(),
        isa => $self->is_a(),
        field_name => $field_name,
        type => $field_type,
    });
}

1;

__END__

CREATE TABLE extended_user (
  uid varchar(20) NOT NULL,			
  f1 varchar(255),
  f2 varchar(255),
  f3 varchar(255),
  f4 varchar(255),
  f5 varchar(255),
  f6 varchar(255),
  f7 varchar(255),
  f8 varchar(255),
  f9 varchar(255),
  f10 varchar(255),
  f11 varchar(255),
  f12 varchar(255),
  f13 varchar(255),
  f14 varchar(255),
  f15 varchar(255),
  f16 varchar(255),
  f17 varchar(255),
  f18 varchar(255),
  f19 varchar(255),
  f20 varchar(255),
  t1 text,
  t2 text,
  t3 text,
  t4 text,
  t5 text,
  t6 text,
  t7 text,
  t8 text,
  t9 text,
  t10 text,
  PRIMARY KEY (uid)
);
