###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Revision$
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
# $Revision$

# TransitionalDisplay helps to migrate from the old style UI to the newer
# template-driven system. Until all the display code is extracted from the 
# model classes, this class can be used to partially migrate to templates.

package Metadot::Utils::TransitionalDisplay;

use strict;
use Data::Dumper;
use Metadot qw(%FORM $USER $SESSION %REQUEST_VARS );
use Metadot::PortalFactory;
use Metadot::Utils::Display;
use vars qw(@ISA);
@ISA=qw(Metadot::Utils::Display);

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my @args = @_;

    my $self = $class->SUPER::new(@args);
    return bless($self, $class);
}

sub print_portal {
    my ($class, $args) = @_;

    my $content = $args->{content};
    my $title   = $args->{title};
    my $iid     = $args->{iid} || 0;
    my $subject = $args->{subject} || Gizmo->restore($iid);

    my $controller = Metadot::PortalFactory->get_controller();
    my $context = $class->build_context({subject=>$subject});
    $REQUEST_VARS{portal_content} = $content;
    $REQUEST_VARS{portal_title} = $title;

    my $result = $controller->view({
        context     =>$context,
        template    =>'transitional_portal.tt2',
    });    

    if ($result) {
        #print STDERR "Got result.\n";
        $class->print_http_header();
        print $result;
    } else {
        #print STDERR "Calling super\n";
        return $class->SUPER::print_portal($args);
    }
}

######################################################################3

# prints content with header and footer only
sub print_simple_layout {
    my ($class, $args) = @_;

    my $content = $args->{content};
    my $title   = $args->{title};
    my $iid     = $args->{iid};
    my $subject = $args->{subject};

    if (defined $iid && !defined $subject) {
        $subject = Gizmo->restore($iid);
    }

    my $controller = Metadot::PortalFactory->get_controller();
    my $context = $class->build_context({subject=>$subject});
    $REQUEST_VARS{portal_content} = $content;
    $REQUEST_VARS{portal_title} = $title;

    my $result = $controller->view({
        context     =>$context,
        template    =>'transitional_simple.tt2',
    });    

    if ($result) {
        #print STDERR "Got result.\n";
        $class->print_http_header();
        print $result;
    } else {
        #print STDERR "Calling super\n";
        return $class->SUPER::print_portal($args);
    }
}

######################################################################3


sub print_form {
    my $class = shift ;
    my $content = shift;
    my $title = shift;
    my $iid = shift;

    my $controller = Metadot::PortalFactory->get_controller();
    my $context = $class->build_context({iid=>$iid});
    $REQUEST_VARS{form_content} = $content;
    $REQUEST_VARS{form_title} = $title;

    $class->print_http_header();
    print $controller->view({
        context     =>$context,
        template    =>'transitional_form.tt2',
    });    
}

sub build_context {
    my $class = shift;
    my $args = shift;

    my $subject = $args->{subject};
    my $op      = $args->{op} || $FORM{op};
    
    unless ($subject) {
        my $iid;
        if (defined $args->{iid}) {
            $iid = $args->{iid};
        } else {
            $iid = $FORM{iid} || $FORM{id};
        }
        my $isa;
        if (defined $args->{isa}) {
            $isa = $args->{isa};
        } else {
            $isa = $FORM{isa};
        }

        if ($isa && $isa->can('new')) {
            $subject = $isa->new($iid); 
        } elsif(defined $iid) {
            $subject = Gizmo->restore($iid);
        }
    }

    my $context = Metadot::PortalFactory->build_context({
        subject => $subject,
        op      => $op,
        user    => $USER,
        form    => \%FORM,
        session => $SESSION,
    });

    return $context;
}

1;
