###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package MetadotConfig;

use strict;

use Metadot qw( $PARAMS );

use DBI;
use FileHandle;
#use XML::Parser;
use Data::Dumper;
use Metadot::Config;
#use Metadot::ConfigParser;
use Cwd qw( abs_path );

my $reserved_ids;

# reserved gizmo iids
$reserved_ids->{RESERVED_IIDS}->{HIDDEN_CATEGORY} = -1;
$reserved_ids->{RESERVED_IIDS}->{PARENT_CLIPBOARD} = -2;
$reserved_ids->{RESERVED_IIDS}->{PARENT_OF_ROOT} = -9999;
$reserved_ids->{RESERVED_IIDS}->{PORTAL_PARENT} = -1000;
$reserved_ids->{RESERVED_IIDS}->{ROOT_CATEGORY} = 0;
$reserved_ids->{RESERVED_IIDS}->{PARENT_BASIC_TOOL} = 1040;

# reserved user ids
$reserved_ids->{RESERVED_UIDS}->{ANONYMOUS_USER} = -1;
$reserved_ids->{RESERVED_UIDS}->{ADMIN} = 8;

my $version = '6.1.6';
my $conf_dir = "../etc/";
my $conf_file = "metadot.conf";
my $selected_server = '';
my $defaultsite_dir = "/home/sharem/defaultsite";
#my $defaultsite_dir = "C:/Projects/sharem_defaultsite";
my $sql_script_file = "sampledata_sharem.sql";
my $mysql_path = "mysql";
# the following parameter goes before database name in the metadot.conf file
my $datasource_prefix = "dbi:mysql:";
# the following parameter is appended to uploads path to store public content of the sites
my $public_dir_suffix = "public";
my $skin_dir_suffix = "skins";
my $main_site_name_prefix = "smart";
my $site_name_suffix = ".metadot.com";
my $db_name_prefix = "sharemdb";
my $initial_disk_usage_mb = 1;

my $conf_file_path = abs_path($conf_dir)."/".$conf_file;
sub get_conf_file_path {
    return $conf_file_path;
}

sub get_defaultsite_dir {
    return $defaultsite_dir;
}

sub get_sql_file_path {
    my $class = shift;
    return abs_path($class->get_defaultsite_dir())."/".$sql_script_file;
}

sub get_mysql_path {
    return $mysql_path;
}

sub get_datasource_prefix {
    return $datasource_prefix;
}

sub get_public_dir_suffix {
    return $public_dir_suffix;
}

sub get_skin_dir_suffix {
    return $skin_dir_suffix;
}

sub get_main_site_name_prefix {
    return $main_site_name_prefix;
}

sub get_site_name_suffix {
    return $site_name_suffix;
}

sub get_db_name_prefix {
    return $db_name_prefix;
}

sub get_initial_disk_usage_mb {
    return $initial_disk_usage_mb;
}

sub _get_server_name {

    unless ( $ENV{HTTP_HOST} ) {

        if ($selected_server){
            return $selected_server;
        } else {
	    return 'localhost';
        }
    } else {

	return $ENV{HTTP_HOST};
    }
}

{
    my $config_cache;
    my $config_timestamp;
    
    sub _get_config {
	my $class = shift;

	my $server_name = $class->_get_server_name();

	# make sure the cache is up to date
	$class->_update_cache();

	if( $config_cache->{$server_name} ) {

	    return $config_cache->{$server_name}
	}

	$config_cache->{$server_name} = Metadot::Config->new($server_name);

	return $config_cache->{$server_name}

    }

    # clears the cache if it is outdated
    sub _update_cache {
	my $class = shift;
	my $current_timestamp = $class->_get_config_timestamp();
	if (!defined $config_timestamp || $config_timestamp > $current_timestamp) {
	    $config_cache = {};
	    $config_timestamp = $current_timestamp;
	    #print STDERR "MetadotConfig cache was updated\n";
	}
    }
    
    sub clear_config_cache {
	
	$config_cache = {};
    }
}

# returns the timestamp of the config
sub _get_config_timestamp {
	my $result;
    my $class = shift;

	# currently this method uses the age of the conf file as the timestamp
	# NOTE: this number decreases if the file is newer
	my $timestamp = -M $class->get_conf_file_path();
	return $timestamp;
}

# XXX change all uses of version to use this call
sub get_version {
    my $class = shift;

    return $version;
}

sub get_reserved_iid {
    my $class = shift;
    my $name = shift;

    return $reserved_ids->{'RESERVED_IIDS'}->{$name};
}

sub get_reserved_uid {
    my $class = shift;
    my $name = shift;

    return $reserved_ids->{'RESERVED_UIDS'}->{$name};
}

sub get_featureset_tag {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name());

    return $dbconfig->{'featureset'};
}

sub get_sessionhandler_type {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name());

    return $dbconfig->{'session_handler_type'} || 'CookieSessionHandler';
}

sub get_access_broker_type {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name());

    return $dbconfig->{'access_broker_type'} || 'Default';
}

sub get_implementation {
    my $class = shift;
	my $implementation_base = shift;
	my $default = shift;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name());

    return $dbconfig->{$implementation_base} || $default;
}

sub get_keep_inactive_session_for_days {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name());

    return $dbconfig->{'keep_inactive_session_for_days'} || 365;
}

sub get_select_registration_modules_from_GUI {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name());

    if ( $dbconfig->{disable_selecting_registration_modules_from_GUI} ) {

	return 0;
    }

    return 1;
}


sub get_user_type {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name());

    if ( MetadotConfig->get_select_registration_modules_from_GUI() ) {

	#return ( $PARAMS->{if_ldap} ) ? 'Default' : 'FlexUser';
	if ( $PARAMS->{if_ldap} && $PARAMS->{managed_ldap} ) {
            return 'ManagedLDAPUser';
        } elsif ( $PARAMS->{if_ldap} ) {
            return 'Default';
        } else {
            return 'FlexUser';
        }

    } else {
    
    	return $dbconfig->{'user_type'} || 'FlexUser';
    }
}


sub get_registration_type {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name());

    if ( MetadotConfig->get_select_registration_modules_from_GUI() ) {

	#return ( $PARAMS->{if_ldap} ) ? 'LDAP' : 'FlexRegister';
	if ( $PARAMS->{if_ldap} && $PARAMS->{managed_ldap} ) {
            return 'ManagedLDAPRegister';
        } elsif ( $PARAMS->{if_ldap} ) {
            return 'LDAP';
        } else {
            return 'FlexRegister';
        }

    } else {

	return $dbconfig->{'registration_type'} || 'FlexRegister';

    }
}

sub get_authenticator_type {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name());

    if ( MetadotConfig->get_select_registration_modules_from_GUI() ) {
	
	#return ( $PARAMS->{if_ldap} ) ? 'LDAPAuthenticator' : 'UserPassAuthenticator';
	if ( $PARAMS->{if_ldap} && $PARAMS->{managed_ldap} ) {
            return 'ManagedLDAPAuthenticator';
        } elsif ( $PARAMS->{if_ldap} ) {
            return 'LDAPAuthenticator';
        } else {
            return 'UserPassAuthenticator';
        }

    } else {

	return $dbconfig->{'authenticator_type'} || 'UserPassAuthenticator';
	
    }
}

# for shareM
sub get_site_in_trial_mode {
    my $class = shift ;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name()) ;
    return $dbconfig->{'site_in_trial_mode'} || "false";
}

# for shareM
sub get_site_is_suspended{
    my $class = shift ;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name()) ;
    return $dbconfig->{'site_is_suspended'} || "false";
}

# for shareM
sub get_is_managed_site {
    my $class = shift ;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name()) ;
    return $dbconfig->{'is_managed_site'} || "false";
}

sub get_use_template_mgmt {
    my $class = shift ;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name()) ;

    return $dbconfig->{'use_template_mgmt'} || "false";
}

sub get_jsform_type {
    my $class = shift ;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name()) ;

    return $dbconfig->{'jsform_type'} || "Default";
}

sub get_jsform_filter {
    my $class = shift ;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name()) ;

    return $dbconfig->{'jsform_filter'} || "Default";
}

sub get_ldap_subsystem_type {
    my $class = shift ;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name()) ;

    return $dbconfig->{'ldap_subsystem_type'} || "Default";
}


sub get_dbname {
    my $class = shift;
    # aju - now allow optional server name to be passed in. 
    my $server_name = shift;

    my $dbconfig;
    if ($server_name) {
	print STDERR "in get_dbname, passed in server_name: $server_name\n";
	$dbconfig = $class->_get_config->get_db_info($server_name);
    }
    else {
#  	print STDERR "in get_dbname, server name is: " . $class->_get_server_name() . "\n";
	$dbconfig = $class->_get_config->get_db_info($class->_get_server_name());
    }

#      print STDERR "get_dbname returns " . $dbconfig->{'name'} . "\n";
    return $dbconfig->{'name'};
}

sub get_dbname_sid {
    my $class = shift;
    my $server_name = shift;

    my $dbconfig;
    if ($server_name) {
	print STDERR "in get_dbname, passed in server_name: $server_name\n";
	$dbconfig = $class->_get_config->get_db_info($server_name);
    }
    else {
	$dbconfig = $class->_get_config->get_db_info($class->_get_server_name());
    }

    if ($dbconfig->{'sid'}) {
	return $dbconfig->{'name'} . ":" . $dbconfig->{'sid'};
    }
    else {
	return $dbconfig->{'name'};
    }
}

sub get_dbusername {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_db_info($class->_get_server_name());
    
    return $dbconfig->{'user'};
}

sub get_dbpassword {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_db_info($class->_get_server_name());
    
    return $dbconfig->{'pass'};
}

sub get_dbtype {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_db_info($class->_get_server_name());

    return $dbconfig->{'dbtype'};
}

sub get_dbhost {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_db_info($class->_get_server_name());

    return $dbconfig->{'host'};
}
sub get_dbsid {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_db_info($class->_get_server_name());

    return $dbconfig->{'sid'};
}

sub get_debug_log {
    my $class = shift;

    my $log_config = $class->_get_config->get_log_file($class->_get_server_name());

    return $log_config;
}


sub get_section_name {
    my $class = shift;
    my $server = shift;

    my $section = $class->_get_config->get_section_name($server);
    return $section;
}

sub set_server_name {
    my ($class, $serv) = @_;

    $selected_server = $serv;

    $selected_server || die "must provide server name";
    
}


sub get_server_name {
    my $class = shift;

    return $class->_get_server_name();

}


sub get_all_server_names {
    my $class = shift;

#    my @tlist = Metadot::Config->get_all_server_names();
#    print STDERR "in MetadotConfig get_all_server_names: ", Dumper(\@tlist), "\n";

    return Metadot::Config->get_all_server_names();    
}

sub get_all_config_sections {
    my $class = shift;

    return Metadot::Config->get_all_config_sections();

}


sub get_public_uploads_dir {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name());

    return $dbconfig->{'public_uploads_dir'} || '/home/metadot/public';
}


sub get_public_uploads_location {
    my $class = shift;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name());

    return $dbconfig->{'public_uploads_location'} || '/public';
}

sub get_min_session_duration {
    my $class = shift ;
    
    return 300;
}

sub get_min_users_number {
    my $class = shift ;
    
    return 5;
}

sub get_skin_template_dir {
    my $class = shift;
    return $class->get_param_value('skin_template_dir');
}

sub get_param_value {
    my $class = shift;
    my $param_name = shift;
    my $default = shift;

    my $dbconfig = $class->_get_config->get_section_info($class->_get_server_name());

    my $result = $dbconfig->{$param_name};

    if (defined $result) {
        # trim whitespace before and after
        $result =~ s/^\s*//;
        $result =~ s/\s*$//;
        $result = $class->_parse_boolean_value($result);
    } else {
        $result = $default;
    }

    return $result;
}


sub _parse_boolean_value {
    my $class = shift;
    my $value = shift;
    my $result;

    if (!defined($value) || (lc($value) eq 'false')) {
        #$result = (0);
        $result = undef;
    } else {
        $result = $value;
    }
    return $result;
}



1;


__END__

=head1 NAME

MetadotConfig - A class containing reserved database ids and accessors to configuration info.

=head1 SYNOPSIS

    use MetadotConfig;

    $version = MetadotConfig->get_version ;
    $reserved_iid = MetadotConfig->get_reserved_iid ($name);
    $reserved_uid = MetadotConfig->get_reserved_uid ($name);
    $dbname = MetadotConfig->get_dbname ;
    $dbusername = MetadotConfig->get_dbusername ;
    $dbpassword = MetadotConfig->get_dbpassword ;
    $logname = MetadotConfig->get_debug_log ;
    MetadotConfig->set_server_name ($server);
    $server = MetadotConfig->get_server_name ;
    $server = MetadotConfig->get_all_server_names ;

=head1 DESCRIPTION

MetadotConfig contains a hash of reserved id values for the Metadot database. It also contains accessors for specific data from the config files. The config file data is gathered by calling Metadot::Config. The accessors is Metadot::Config are wrapped here for backwards compatibility.

=head2 Methods

There is no constructor.

Get the current version of the Metadot software:

	$version = MetadotConfig->get_version;

Get the reserved instance id for a specified object (these are special objects installed with the base system; not user-created objects).
$name is the name of one of the system generated objects:

	$reserved_iid = MetadotConfig->get_reserved_iid($name);

Get the reserved user id for a specified user (these are special users that are created with the base system.):
$name is the name of one of the system generated users:

	$reserved_uid = MetadotConfig->get_reserved_uid($name);

Get the name of the database (this method wraps the get_db_info() method in Metadot::Config):

	$dbname  = MetadotConfig->get_dbname;

Get the username for connecting to the database (this method wraps the get_db_info() method in Metadot::Config):

	$dbusername = MetadotConfig->get_dbusername;

Get the password for connecting to the database (this method wraps the get_db_info() method in Metadot::Config):

	$dbpassword = MetadotConfig->get_dbpassword;

Get the name of the log file for debug information (this method wraps the get_log_info() method in Metadot::Config):

	$logname = MetadotConfig->get_debug_log;

Set the name of the server running this instance of Metadot. This server name is used to specify which server variables in the config file apply to this server:
$server is the name of the server section of the configuration script this installation uses:

	MetadotConfig->set_server_name($server);

Get the name of the server running this instance of Metadot. This grabs the name from the environment the server is running in, or the hardcoded name at the top of the file:

	$server = MetadotConfig->get_server_name;

Get all of the server names available. Grabs all the server names in the config file and reports them. (This method is currently unavailable):
	
	$server = MetadotConfig->get_all_server_names;

=cut
