###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################


# this is a handy base class
package MetadotObject;

use strict;
use Data::Dumper;


# send an argument "properties" with a hash ref of desired properties
# of the object.
sub new {
    my ($proto, %args) = @_;
    #print STDERR "Beginning ".__PACKAGE__." new() with args\n", Dumper \%args;
    my $class = ref($proto) || $proto;
    my $self = {};
    bless $self, $class;

    $self->_initialize(%args);
    return $self;
}

# Initialize the object. IMPORTANT: Always called by new()
sub _initialize {
    my ($self, %args) = @_;
    #print STDERR __PACKAGE__."->_initialize begins, with args\n",Dumper \%args;
    my $properties = $args{properties};
    if ($properties) {
        $self->set_properties($properties);
    }

    #print STDERR __PACKAGE__."->_initialize is complete. self=\n",Dumper $self;
    return $self;
}


# returns a hashtable of properties that are accepted in the
# constuctor. these hashtable should contain the following structure:
#   $property => { value => $value }
# where the $property is the name of the property, $value is the default value
sub get_constructor_properties {
    return {};
}

# Returns a new object with the same properties as the current object.
# If any property is a reference, the same reference is used. Be careful of that!
sub clone {
    my $self = shift;
    my $clone = $self->new();
    foreach my $key (keys(%{$self})) {
        $clone->{$key} = $self->{$key};
    }
    return $clone;
}

# Base accessor
sub access_property {
    my $self        = shift;
    defined(my $property = shift) || die("No property to get");
    my $default     = shift;
    my $result      = $self->{$property};
    unless (defined($result)) {
        $result = $default;
    }
    return $result;
}

# Base mutator
sub mutate_property {
    my $self        = shift;
    defined(my $property = shift) || die("No property to get");
    my $value       = shift;
    $self->{$property} = $value;
}

# sets all the properties at once
sub set_properties {
    my ($self, $properties) = @_;
    
    # initialize each property, even if no arg was sent.
    my $constructor_properties = $self->get_constructor_properties();
    foreach my $property ( keys %$constructor_properties ) {
        unless ($self->exists_property($property)) {
            my $value;
            if (exists $properties->{$property}) {
                $value = $properties->{$property};
            } else {
                $value = $constructor_properties->{$property}->{value};
            }
            $self->mutate_property($property, $value);
        }
    }
}

# determines if a property has been set
sub exists_property {
    my $self = shift;
    my $property    = shift || die("No property to get");
    my $result = exists($self->{$property});
    return $result;
}


1;
