#!/usr/bin/perl -w
#
###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################


package Sendmail;
use strict;

use Metadotclass;
use Metadot qw($PARAMS);
use MetadotConfig;
use ErrorHandling;
use Utility;
use Net::SMTP;
#  use Mail::Sendmail;
use Data::Dumper;
use POSIX;

sub new {
  my $proto = shift;
  my $to  = shift || die "must provide 'to' information";
  my $cc = shift; 
  my $bcc = shift;
  my $subject   = shift; 
  my $body   = shift; 
  
  if (!($subject || $body)) {
      die "must provide either 'subject' or body information";
  }
#    print STDERR "in sendmail new: subj is $subject and body is $body\n";
  my $class = ref($proto) || $proto;
  my $self = {};

  $self->{to} = $to;
  if ($subject) {
      $self->{subject} = $subject;
  }
  if ($body) {
      $self->{body} = $body;
  }
  $self->{cc} = $cc if ($cc);
  $self->{bcc} = $bcc if ($bcc);
  
  bless ($self, $class);
  
  return $self;
}

sub set_from {
    my $self = shift;
    my $from = shift;
    $self->{from} = $from;
}

sub set_to {
    my $self = shift;
    my $to = shift;
    $self->{to} = $to;
}

sub set_body {
    my $self = shift;
    my $body = shift;
    $self->{body} = $body;
}

sub set_reply_to {
    my $self = shift;
    my $rt = shift;
    $self->{reply_to} = $rt;
}

sub set_message_id {
    my $self = shift;
    my $mid = shift;
    $self->{msg_id} = $mid;
}

sub set_urgent {
    my $self = shift;
    my $urgent = shift;
    $self->{urgent} = $urgent;
}

sub set_encoding {   
    my $self = shift;
    my $enc = shift;
    $self->{encoding} = $enc;
}

sub set_debug {
    my $self  = shift;
    my $value = shift;
    $self->{debug} = $value;
}

sub sendMail{    
    my $self = shift;
    my $body = $self->{body};
    my $subject = $self->{subject};
    my $to = $self->{to};
    my $cc = $self->{cc};
    my $bcc = $self->{bcc};
    my $message_id = $self->{msg_id};
    my $sender = $self->{from};
    my $reply_to = $self->{reply_to};
    my $encoding = $self->{encoding};

    my $result = 0;
    my $retries = MetadotConfig->get_param_value('sendmail_attempts') || 1; # try this many times
    my $retry_interval = MetadotConfig->get_param_value('sendmail_retry_interval') || 20; # wait this many seconds between attempts
    my $attempts = 0; # track the number of attempts

    my $debug = $self->{debug};
    
    my $suppress_emails = MetadotConfig->get_param_value('suppress_emails') || 0;  # set to 1 to suppress emails, instead print diag information

    if ($debug) {
        print STDERR "in sub sendMail\n";
        print STDERR "Current properties are:\n",Dumper $self;
        print STDERR "Suppress emails? ".($suppress_emails? 'yes' : 'no')."\n";
    }

    my ($mailsenderaddress) = DBAccess->sqlSelect("value", "params", "name = 'mailsenderaddress'");
    my ($mailserveraddress) =  DBAccess->sqlSelect("value", "params", "name = 'mailserveraddress'");
    
    if (!$sender){
        $sender = $mailsenderaddress;
    } else {			# if a sender is specified
        #check if full email is given
        my @senderemail = split(/\@/, $sender);
        if ($senderemail[1]) {
            # then keep sender as is
        }
        else {			#  use mailsenderaddress' domain
            #my @domain = split(/\@/, $mailsenderaddress);
            #$sender = $sender . "@" . $domain[1];
            $sender = $sender . "@" . $mailserveraddress;
        }
    }

    if ($suppress_emails) {
        print STDERR "Sendmail would have sent email:\n", Dumper $self;
        $result = 1;
    } else {
        while (!$result && ($attempts < $retries)) {
            if ($attempts++ > 0) {
                if ($debug) {
                    print STDERR "Attempt #$attempts, sleeping $retry_interval seconds...\n";
                }
                sleep($retry_interval);
            }                                                             
            my $errormsg;
            eval { ###try to send mail and allow error catching

                my @recipients = split (",", $to);
                if ($cc) {
                    push (@recipients, split (",", $cc));
                }
                if ($bcc) {
                    push (@recipients, split (",", $bcc));
                }


                @recipients = @{ Utility->uniquify_array(@recipients) };


                if ($debug) {
                    print STDERR "Sending to recipients: @recipients\n";
                    print STDERR "connecting to '$mailserveraddress'\n";
                }

                my $smtp = Net::SMTP->new($mailserveraddress, Hello=>$mailserveraddress, Timeout => 30); # connect to an SMTP server

                # do null checking
                if (defined $smtp) {
                    if ($debug) {
                        print STDERR "Established SMTP connection\n";
                    }
                    #$smtp->mail( $sender );     # use the sender's address here
                    # 03-27-03 - no, adding the sender's address apparently causes problems for some sendmail configs
                    # with some of the ways metadot addresses mail.
                    $smtp->mail();
                    if ($debug) {
                        print STDERR "Initiated mail transaction\n";
                    }

                    # XXX propagate error if any 
                    my $to_result = $smtp->to(@recipients, { SkipBad => 1 } );   # recipient's address

                    $smtp->data();              # Start the mail

                    # Send the header.
                    if ($encoding eq 'html') {
                         $smtp->datasend("MIME-Version: 1.0\r\n");
                         $smtp->datasend("Content-type: text/html; charset=iso-8859-1\r\n");
                    } 
                    $smtp->datasend("To: $to\n");

                    $smtp->datasend("From: $sender\n");
                    if ($reply_to) {
                        $smtp->datasend("Reply-to: $reply_to\n");
                    }
                    $smtp->datasend("CC: $cc\n") if ($cc);
                    $smtp->datasend("BCC: $bcc\n") if ($bcc);
                    $smtp->datasend("Date: ".strftime("%a, %d %b %Y %H:%M:%S %z", localtime)."\n");
                    $smtp->datasend("Subject: $subject\n");
                    if ($message_id) {
                        $smtp->datasend("Message-Id: $message_id\n");
                    }
                    $smtp->datasend("\n");

                    # Send the body.
                    $smtp->datasend("$body");
                    $smtp->dataend();           # Finish sending the mail
                    $smtp->quit;                # Close the SMTP connection

                    if ($debug) {
                        print STDERR "Completed mail transaction\n";
                    }
                } else {
                    $errormsg = "Could not establish connection with mail server '$mailserveraddress'.\n";
                }
            }; # EVAL


            ### CATCH
            if ($@) {
                $errormsg = "Mail server problem\n\n$@\n";
                $errormsg .= "Error occured while trying to send a mail " .
                    "(server=$mailserveraddress, to=$to, cc=$cc, bcc=$bcc, subject=$subject, " .
                    "message id $message_id)";
            }

            if ($errormsg) {
                # only send the HTML error message if we're on the web
                if (Metadotclass->on_the_web() && ($attempts >= $retries)) {
                    ErrorHandling->printError($errormsg);
                }
                if ($debug || !Metadotclass->on_the_web()) {
                    print STDERR "Error occured during transaction:\n$errormsg\n\n";
                }
                $result = 0;
            } else {
                $result = 1;
            }
        }
    }

    return $result;
}

1;


__END__

=head1 NAME

Sendmail - 

=head1 SYNOPSIS

    use Sendmail;

    $sm = Sendmail->new($to, $cc, $bcc, $subject, $msg);
    $sm->set_from($from);
    $sm->set_reply_to($rt);
    $sm->set_message_id($msg_id);
    $sm->sendMail ();
    


=head1 DESCRIPTION

B<Sendmail::sendMail> is a utility fcn that uses Net::SMTP to send email
messages.  

Constructor: The $to parameter, and either the $subject or $msg (body)
parameters are required to be set.  The sender ('from' field) is not required; 
$params->{mailsenderaddress} will be used for the sender if none
is provided.  Empty strings may be used for the $cc and $bcc fields, in which case
they will not be included in the header.

    $sm = Sendmail->new($to, $cc, $bcc, $subject, $msg);

Set the 'from' field for the message:

    $sm->set_from($from);

Set the 'reply-to' field for the message:

    $sm->set_reply_to($rt);

Set the message-id:

    $sm->set_message_id($msg_id);

Call the sendMail method.  Will utilize the current values for all the header fields.  
Call any necessary 'set' methods above before calling C<sendMail>.

    $sm->sendMail ();


=cut
