###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Subscription;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);

use DBAccess;
use Carp;

use Portal;

use vars qw(@ISA);
@ISA=qw(Portal);

my $AC = {
       BUNDLES => {
                   MOD =>    { DESC =>          'Edit', 
                               OPS =>            [qw(add)], 
                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                             },                
                   DEL =>    { DESC => 'Delete and Cut',
                               OPS => [qw(delete)],
                               DEFAULT_LEVEL => 8,
                               MIN_LEVEL => 8,
		   #aju-- needed to change the above levels to 2 to get things to work
		   #properly-- why?  not checking in the change yet..
#                                 DEFAULT_LEVEL => 2,
#                                 MIN_LEVEL => 2,
		                         },
                   EDITP =>  { DESC =>          'Change<br>Permissions', 
                               OPS =>            [qw(edit_permissions set_permissions change_owner)], 
                               DEFAULT_LEVEL =>  8,
                               MIN_LEVEL =>      8,
                             },                
                  },
          DISPLAY_ORDER => [qw(DEL MOD EDITP)],
        };

sub get_AC {
	return $AC;
}

sub is_a {
	my $self = shift;

	return ref($self);
}

#sub name {
#	my $self = shift;
#
#	return ref($self);
#}

sub id {
	my $self = shift;

	return 1;
}

sub save {
	my $self = shift;

	$self->save_permissions;
}

sub get_owner_uid {
	my $self = shift;
#  	return MetadotConfig->get_reserved_uid('ADMIN');
	#aju: the above was not right-- that way only admin could delete subscriptions.
	my $owner_uid = $USER->{uid};
#  	print STDERR "in Subscription:get_owner_uid: $owner_uid\n";
	return $owner_uid;
}

sub parent_id {
	my $self = shift;
	
	return 0;
}

####################################################

sub new {
  my $proto = shift;

  my $class = ref($proto) || $proto;
  my $self = {};
  
  bless ($self, $class);

  return $self;
}

################################################################
####                  OPERATION HANDLER                    #####
################################################################

sub www_add {
  my $self = shift ;
  $self->debug_msg(3, "Subscription::www_add");
  
  $FORM{id} = undef unless (Utility->is_valid_integer($FORM{id}));    # secu_200401
  
  my $cid = $self->defined_or_exit($FORM{id},"id is missing.");
  my $freq = $self->defined_or_exit($FORM{freq},"freq is missing.");
  my $medium = $FORM{medium} || "email";
  my $uid = $self->defined_or_exit($USER->uid(),"uid is missing.");
  #url to perform the add
  my $add_url = "userchannel.pl?id=$cid&amp;isa=Subscription&amp;freq=$freq&amp;medium=$medium&amp;op=add";
  #url to go to after add
  my $target = $FORM{target} || $SESSION->{referer} || "userchannel.pl?isa=MyPage&amp;op=select&amp;type=Selection";
  
  if ((!$USER->pager_number()) && ($medium eq "pager")){
    $USER->www_require_pager_form($add_url,$target);
    exit();
  }
  if ((!$USER->icq_number()) && ($medium eq "icq")){
    $USER->www_require_icq_form($add_url,$target);
    exit();
  }


#  DBAccess->sqlDelete("subscription","userid=$uid and cid=$cid and frequency='$freq' and medium='$medium'");
  DBAccess->sqlDelete("subscription","userid=$uid and cid=$cid and medium='$medium'");

  my $db_now = Utility->db_now();
  DBAccess->sqlInsert("subscription",(userid => $uid, 
			    cid => $cid, 
			    frequency => $freq, 
			    medium => $medium, 
			    creationdate => $db_now,
			    processdate => $db_now));
  # if url = userchannel.pl we come from a search POST form in select
  $target = "userchannel.pl?isa=MyPage&amp;op=select&amp;type=Selection" if ($target =~ m/userchannel\.pl$/);
  $self->md_redirect($target);
}

sub www_delete {
  my $self = shift ;
  $self->debug_msg(3, "Subscription::www_delete");
  
  $FORM{id} = undef unless (Utility->is_valid_integer($FORM{id}));    # secu_200401
  
  my $cid = $self->defined_or_exit($FORM{id},"id is missing.");
  my $freq = $self->defined_or_exit($FORM{freq},"freq is missing.");
  my $uid = $self->defined_or_exit($USER->uid(),"uid is missing.");
  my $medium = $FORM{medium} || "email";

  DBAccess->sqlDelete("subscription","userid=$uid and cid=$cid and frequency='$freq' and medium='$medium'");
  my $url = $SESSION->{referer};
  # if url = userchannel.pl we come from a search POST form in select
  $url = "userchannel.pl?isa=MyPage&amp;op=select&amp;type=Selection" if ($url =~ m/userchannel\.pl$/);
  $self->md_redirect($url);
}

1;

__END__

=head1 NAME

Subscription - 

=head1 SYNOPSIS

    use Subscription;
    $sub = Subscription->new ();

    $sub->www_add;
    $sub->www_delete;

    $AC = $sub->get_AC ;
    $isa = $sub->is_a ;
    $id = $sub->id ;
    $sub->save ;
    $uid = $sub->get_owner_uid ;
    $uid = $sub->parent_id ;
    $sub->not_logged_in_callback ;

=head1 DESCRIPTION

This module provides an interface to storage of subscriptions information. Subscriptions allow users to receive email/pager/icq updates whenever a channel changes its content (see Channel.pm documentation). Subscriptions are registered via the www_method, which is invoked from the user interface provided bt the MyPage module from userchannel.pl.

=head2 Methods

www_add

Will add a subscription for a user. It is called from the user interface generated by the www_select method in MyPage.pm.

www_delete

Will remove a subscription for a user. It is called from the user interface generated by the www_select method in MyPage.pm.

$AC = $sub->get_AC ;
$isa = $sub->is_a ;
$id = $sub->id ;
$sub->save ;
$uid = $sub->get_owner_uid ;
$uid = $sub->parent_id ;
$sub->not_logged_in_callback ;

Auditable.pm required methods. See Auditable documentation.


=cut
