###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Utility;
use strict;

use HTTP::Request;
use LWP::UserAgent;
use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CGI);
use Metadotclass;
use Metadot::Utils::ModuleMgmt;
use Time::Local;
use CGI qw(escapeHTML);
use HTML::Parser;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    return bless({},$class);
}

sub read_modules_in_directory {
    my $class = shift;
    my $dir = shift;

    return Metadot::Utils::ModuleMgmt->read_modules_in_directory($dir);
}


sub is_module_available {
    my $class = shift;
    my $module_name = shift;
    
    return Metadot::Utils::ModuleMgmt->is_module_available($module_name);
}


my $ua;
sub fetch_url {
    my $self = shift;
    my $curl = shift;
    my $timeout = shift || 30;

    my $proxy_host = $PARAMS->{'proxyhost'};
    my $proxy_port = $PARAMS->{'proxyport'};

    my $proxy_msg = ($proxy_host) ? "proxy: $proxy_host:$proxy_port/" : '';
    
    my $request; my $response;
    
    unless( $ua ) {
	$ua = LWP::UserAgent->new( );    
    }
    
    $ua->timeout( $timeout );

    if ($proxy_host){
	my $proxy_host = 'http://'.$proxy_host.':'.$proxy_port.'/';
	$ua->proxy(http  => $proxy_host);
    }
    
    if ($curl){ ### fetch on the web
	$request = new HTTP::Request (GET => $curl);
	$response = $ua->request($request);
    }else{
	return 0;
    }
    
    return $response;
}


sub md_redirect {
    my $class = shift;
    my $location = shift;
    
    if ($location =~ m/index.pl\Z/) {    
        $location .= '?id=0';
    } elsif ($location =~ m/userchannel.pl\Z/) {
        $location .= '?op=show';
    }
    
    # if the header has been sent, i have to send a redirect as HTML
    if ($HTTP_HEADER_SENT) {
        $location ||= Utility->get_qualified_url('index.pl');
    	my $out = <<EOM
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
  <head>
    <meta http-equiv="refresh" content="0;url=$location" />
    <title>Loading</title>
  </head>
  <body onload="document.location='$location'">
    <!-- the following block was used to allow Firefox to redirect, since
         it has a bug with http-equiv. This is now being fixed with the javascript entry above.
    -->
    <!--
    <div style="margin: 30px; padding: 1em; border: 1px solid #ccc; background-color: #eee;">
      You are being redirected to the portal server.
      <a href="$location">Click here if you have not been redirected.</a>
    </div>
    -->
  </body>
</html>
EOM
;
    	print $out;
    } else {
        print $CGI->redirect($DISPLAY->make_url($location));
    }
    
  exit 0;
  
}


# Returns an instance object based on an iid, and optionally, an isa
sub get_instance_object {
    my ($self, $iid, $isa) = @_;
    my $result;
    
    $iid = undef unless (Utility->is_valid_integer($iid));    # secu_200401
    
    unless ($isa) {
        ($isa) = DBAccess->sqlSelect('IsA', 'instance', "IID='$iid'");   
    }
    if ($isa) {
        $result = $isa->restore($iid);
    }
    return $result;
}

# Determines whether a scalar is contained within an array.
sub array_contains {
    my ($self, $scalar, $array) = @_;
    my $result = 0;
    
    # coerce the array
    if (ref($array) ne 'ARRAY') {
        $array = [$array];   
    }

    for (my $i=0; !$result && defined($array->[$i]); $i++) {
        if ($scalar eq $array->[$i]) {
            $result = 1;
        }
    }
    return $result;
}

sub uniquify_array {
    my ($self, @array) = @_;
    my %keys;
    my @out;

    # in case I got a ref to an array instead of an actual array
    if (@array && ref($array[0]) eq "ARRAY") {
        @array = @{ $array[0] };
    }

    for my $val (@array) {
        unless ($keys{$val}) {
            $keys{$val} = 1;
            push @out, $val;
        }
    }
    if (wantarray) {
        return @out;
    } else {
    return \@out;
}
}

# For a given instance, finds the structural ancestor of that instance.
# If one or more classes are specified, then it tries to find a
# structural ancestor with that class, otherwise it will return null
# If a class is specified and the given instance is already of that 
# class, then it will return the instance.
sub get_structural_ancestor {
    my ($self, $instance, $ancestor_class) = @_;
    my $result;

    #print STDERR "CKIM: Utility->get_structural_ancestor for instance=$instance, looking for $ancestor_class\n";
    # make sure I got an instance object
    ref($instance) || return undef;
        
    # trivial case: no ancestor class provided
    if (!$ancestor_class) {
        return Utility->get_instance_object($instance->parent_id());
    }
    
    # trivial case: the instance matches the structural ancestor
    if (Utility->array_contains(ref($instance), $ancestor_class)) {
        return $instance;   
    }
    
    # recursive case
    return Utility->get_structural_ancestor(Utility->get_instance_object($instance->parent_id()));
}

# returns an image containing a spacer, with a given width and height
sub get_spacer {
    my ($self, $width, $height, $make_background) = @_;
    my $result;
    
    my $imageslocation = $PARAMS->{'imageslocation'};        

    if ($make_background) {
        $result = "background = \"$imageslocation/spacer.gif\" width=\"$width\" height=\"$height\"";    
    } else {
        $result = "<img border=\"0\" src=\"$imageslocation/spacer.gif\" width=\"$width\" height=\"$height\" alt=\"\" />";    
   }
    
    return $result;
}

# returns only those emails that fit the correct form name@domain.tld
sub filter_valid_emails {
    my ($self, @array) = @_;
    my @out;

    # in case I got a ref to an array instead of an actual array
    if (@array && ref($array[0]) eq "ARRAY") {
        @array = @{ $array[0] };
    }

    for my $email (@array) {
        if ($self->is_valid_email($email)) {   
            push @out, $email;
        }
    }
    return \@out;
}


# returns true if the value is a valid number. does not accept comma separators.
sub is_valid_number {
    my ($class, $value) = @_;
    my $result = ( defined($value) && $value =~ /^-?(?:\d+(?:\.\d*)?|\.\d+)$/ );
    return $result;
}

# returns true if the value is a valid integer. does not accept comma separators
# secu_200401: needed to prevent bad input on iid, uid, etc.
sub is_valid_integer {
    my ($class, $value) = @_;
    my $result = (defined($value) && $value =~ /^-?\d+$/ );
    return $result;
}


# returns true if the value conforms to name@domain.tld
sub is_valid_email {
    my ($self, $email) = @_;

    if ($email =~ m/^\S+@\S+\.\w+$/) {   
        return 1;
    } else {
        return 0;
    }
}

sub is_valid_delimited_email_string {
    my ($self, $email_string, $delimiter) = @_;

    for my $email (split(/$delimiter/, $email_string)) {
        unless ($self->is_valid_email($email)) {
            return 0;
        }
    }
    return 1;
}

# removes white space from either end of a string
sub trim {
	
	my $class = shift;
	my $str = shift || '';

	$str =~ s/^\s+//;
	$str =~ s/\s+$//;
	
	return $str;
}

# truncates a string, but only at a word boundary
sub trim_clean {
    my $self = shift;
    my $input = shift || '';
    my $length = shift || 0;
    my $result;
    
    if (length($input) > $length) {
	$input =~ m/^([\w\W]{0,$length}\W)/;
	$result = $1;

	# no word boundary could be found within the range, so I return the
	# string with a dirty truncation
	($result) || ($result = substr($input, 0, $length));
	return $result;
    }
    else {
	return $input;
    }
}

#remove html tags
sub remove_tags {
    my $class = shift;
    my $input = shift;
    my $remove_template_tags = shift || 0;
    $input =~ s/<.*?>//g;
    if ($remove_template_tags) {
        $input =~ s/\[\%.*\%\]//g;
    }
    return $input;
}



# returns the content with a border around it
sub draw_border {
    my $self = shift;
    my %args = @_;
    
    my $content = $args{content} || die("Must specify content to draw\n");
    my $color = $args{color} || '#000000';
    my $thickness = $args{thickness} || 1;
    
    my $result = <<EOM
    <table border="0" bgcolor="$color" cellspacing="0" cellpadding="$thickness">
      <tr><td>$content</td></tr>
    </table>
EOM
;    
    return $result;
}

# creates a textarea input that allows you to submit text or upload a file
sub render_text_or_file_selector {
    my $self = shift;
    my %args = @_;
    my $control_basename = $args{control_basename} || die("Must specify control basename\n");
    my $description = $args{description} || '';
    my $rows = $args{rows} || 12;
    my $cols = $args{cols} || 84;
    my $default = $args{default};
    my $result='';
    
    my $control_radio = $control_basename.'_radio';
    my $control_file  = $control_basename.'_file';
    my $control_text  = $control_basename.'_text';

    my $control_script_file = "\"setRadio(this.form, '$control_radio', 'file')\"";
    my $control_events_file = "onClick=$control_script_file onFocus=$control_script_file";
    my $control_script_text = "\"setRadio(this.form, '$control_radio', 'text')\"";
    my $control_events_text = "onClick=$control_script_text onFocus=$control_script_text";

	#escape the html in $default to prevent further parsing (problem discovered when using Mozilla)
	$default = escapeHTML($default);
    
    if (defined $description) {
        $result .= "$description<br>\n";
    }
    $result .= <<EOM
    <table border="0" cellspacing="0" cellpadding="0"><tr valign=baseline>
    <td align='left' nowrap="nowrap">
      <input type=radio name="$control_radio" checked value="text">Use the text below
    </td>
    <td width="25">&nbsp;&nbsp;&nbsp;&nbsp;</td>
    <td align='left'>
      <input type=radio name="$control_radio" value="file">&nbsp;Upload a file:&nbsp;<input type=FILE class='button' name="$control_file" $control_events_file>
    </td>
    </tr></table>
    <textarea name="$control_text" rows="$rows" cols="$cols" WRAP="VIRTUAL" $control_events_text>$default</textarea><br>
EOM
;
    return $result;    
}

# reads the input from the form element created using the above method
sub input_text_or_file_selector {
    my $self = shift;
    my $control_basename = shift || die("Must specify control basename\n");
 
    my $result='';

    my $control_radio = $control_basename.'_radio';
    my $control_file  = $control_basename.'_file';
    my $control_text  = $control_basename.'_text';
    
    if ($FORM{$control_radio} eq 'file') {
        # read from a file
        if ($FORM{$control_file}) {
            # confirm that a file was specified
            my $filepath = $FORM{$control_file};
            my $buff;
            while (read($filepath,$buff,2096)){
                $result .= $buff;
            }
        } else {
            # no file was specified, use the control_text instead
            $result = $FORM{$control_text};
        }
    } elsif ($FORM{$control_radio} eq 'text') {
        $result = $FORM{$control_text};
    }
    
    return $result;
}

sub upload_image {
	my $self = shift;
	my $filepath=shift;

	my $filename;
	if ($filepath =~ /([^\/\\]+)$/) {
		$filename = "$1";
	} else {
		$filename="$filepath";
	}

	my $path = $PARAMS->{imageslocation};
	my @pieces = split("/", $PARAMS->{absolutepathfileuploaddir});
	pop(@pieces);
	my $beginning = join("/", @pieces);

	my $full_file_path = $beginning ."/html". $path."/".$filename;

	if (!open(WFD,">$full_file_path")) {
		die "Error opening file for writing: $full_file_path";
	}
	
	my $start_time = time();
	my $bytes_read=0;
	my $buff='';
	my $size = 0;
	while ($bytes_read=read($filepath, $buff, 2096)) {
		$size += $bytes_read;
		binmode WFD;
		print WFD $buff;
	}
	if ($size == 0) {
		Portal->print_click_back("Error", "Cannot access specified upload file.\n");
		exit(0);
	}

	close(WFD);

    return $filename;
}

sub get_image_size_string {
    my ($self, $istring)  = @_;

    use Image::Size;
#     print STDERR "in get_image_size_string, image content string: $istring\n";
    my ($width, $height) = imgsize(\$istring);
    #print STDERR "in get_image_size_string, got image width: $width\n";
    my %output;
    $output{width} = $width;
    $output{height} = $height;
    return \%output;
}

sub get_image_size {
	my $self = shift;
	my $file = shift;

	use Image::Size;

	my $path = $PARAMS->{imageslocation};
	my @pieces = split("/", $PARAMS->{absolutepathfileuploaddir});
	pop(@pieces);
	my $beginning = join("/", @pieces);

	my $filepath = $beginning."/html".$path."/"."$file";

#	if (!open(FILE, "<$filepath")) {
#		die "Error opening file for reading: $filepath";
#	}

	my ($width, $height) = imgsize("$filepath");

	my %output;
	$output{width} = $width;
	$output{height} = $height;

	return \%output;
}

# this returns a qualified URL for the server if 'http' or 'https' not present
sub get_qualified_url {
    my $self = shift;
    my $page = shift || 'index.pl';
    if ($page =~ m/\Ahttps?\:/) {
        return $page;
    } else {
        return $PARAMS->{basename} . "/$page";
    }
}

sub get_secure_url {
    my $self = shift;
    my $page = shift || 'index.pl';
    return $self->_process_url(page => $page, is_secure => 1);
}

sub get_unsecure_url {
    my $self = shift;
    my $page = shift || 'index.pl';
    return $self->_process_url(page => $page, is_secure => 0);
}

sub _process_url {
    my $self = shift;
    my %args = @_;
    my $page = $args{page} || 'index.pl';
    my $is_secure = $args{is_secure} || 0;

    my $basename = $PARAMS->{basename};
    $basename =~ s/\Ahttps?\:\/\///;
    my $uri = $is_secure ? 'https' : 'http';
    return "$uri://" . $basename . "/$page";
}

# returns 'foo','bar',...'n'
sub convert_list_to_sql_list {
	my $self = shift;
	my @list = @_;
	foreach my $elem (@list) {
		$elem = "'$elem'";
	}
	return join ',', @list;
}

sub db_time_format {
    # takes a unix timestamp and returns a date formatted for entry into the database
    # the format it returns is YYYY-MM-DD hh24:mi:ss
    # the code in the method has been benchmarked, and is faster than two possible alternatives:
    # putting the results of localtime($timestamp) into an array, and useing the Time::localtime module
    my $class = shift;
    my $timestamp = shift;
    my ($seconds, $minutes, $hours, $day_of_month, $month, $year) = localtime($timestamp);
    my $db_time = sprintf("%04d-%02d-%02d %02d:%02d:%02d", $year + 1900, $month + 1, $day_of_month, $hours, $minutes, $seconds);
    return $db_time;
}

sub db_now {
    my $class = shift;
    my $timestamp = time;
    my $db_time = Utility->db_time_format($timestamp);
    return $db_time;
}

sub unix_time_format {
    my $class = shift;
    my $db_time = shift || '';

    return undef if !defined($db_time) or $db_time eq '';

    $db_time =~ m/(\d\d\d\d)-(\d\d)-(\d\d) (\d\d):(\d\d):(\d\d)/;
    # [vadim] -1 was added because the December (12th month) gives the error in this place
    my $unix_time = eval{timelocal($6, $5, $4, $3, ($2-1), $1)};
    if (@_) { 
        return undef;
    } else {
        return $unix_time;
    }
}

sub format_datetime {
    my $self  = shift;
    my $datetime  = shift;
    my $format = shift;

    return $self->format_unix_timestamp($self->unix_time_format($datetime), $format);
}


## formats a unix timestamp
## M  - month (January)
## b  - month abv (Jan)
## m  - month 2-digit (07)
## e  - day   (1-31)
## Y  - year  (YYYY)
## y  - year  (YY)
## T  - time  (1:00 AM)
## r  - time  (1:00:00 PM)
## l  - hour  (1-12)
## W  - WeekDay name (Sunday, Monday...)
## a  - WeekDay name abbreviated (Sun, Mon...)
sub format_unix_timestamp {
    my $self  = shift;
    my $timestamp  = shift || 0;
    my $format = shift || 'M e, Y T';

    return unless defined($timestamp) && $timestamp > 0;

    my ($sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst) = localtime($timestamp);
    $year+=1900;
    my $date = '';
    my @DAYS_OF_WEEK = qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday);
    my @DAYS_OF_WEEK_AB = qw(Sun Mon Tue Wed Thur Fri Sat);
    my @MONTHS = qw(January February March April May June July August September October November December);

    my @format_array = split(//, $format);

    foreach my $char (@format_array) {
        if ($char eq 'W') {
            $date .= $DAYS_OF_WEEK[$wday];
        } elsif ($char eq 'a') {
            $date .= $DAYS_OF_WEEK_AB[$wday];
        } elsif ($char eq 'M') {
            $date .= $MONTHS[$month];
        } elsif ($char eq 'b') {
            $date .= substr($MONTHS[$month], 0, 3);
        } elsif ($char eq 'm') {
            my $m = $month + 1;
            $date .= $m < 10 ? "0$m" : $m;
        } elsif ($char eq 'e') {
            $date .= $day;
        } elsif ($char eq 'd') {
            $date .= $day < 10 ? "0$day" : $day;
        } elsif ($char eq 'Y') {
            $date .= $year;
        } elsif ($char eq 'y') {
            $date .= substr($year, -2);
        } elsif ($char eq 'T') {
            $date .= $self->_format_time($timestamp);
        } elsif ($char eq 'r') {
            $date .= $self->_format_time($timestamp, 1);
        } elsif ($char eq 'l') {
            $date .= $self->_format_hour($hour);
        } else {
            $date .= $char;
        }
    }

    return $date;
}

sub same_day {
    my $self = shift;
    my $datetime1 = shift;
    my $datetime2 = shift;

    my ($sec1, $min1, $hour1, $day1, $month1, $year1) = localtime($datetime1);
    my ($sec2, $min2, $hour2, $day2, $month2, $year2) = localtime($datetime2);

    if ($day1 == $day2 && $month1 == $month2 && $year1 == $year2) {
        return 1;
    } else {
        return 0;
    }
}

sub is_today {
    my $self = shift;
    my $datetime1 = shift;
    return $self->same_day($datetime1, time);
}

## formats hour portion of a unix time stamp (i.e. 1:00 pm)
sub _format_time {
    my $self  = shift;
    my $time  = shift;
    my $include_seconds = shift || 0;

    my ($hour, $min, $seconds) = (localtime($time))[2, 1, 0];
    my $ampm = $hour > 11 ? 'PM': 'AM';
    $min = $min < 10 ? "0$min" : $min;

    my $format_time = $self->_format_hour($hour) . ":$min";
    $format_time .= ":$seconds" if $include_seconds;
    $format_time .= " $ampm";

    return $format_time;
}

## formats hour (returns 1-12)
sub _format_hour {
    my $self  = shift;
    my $hour  = shift;

    my $display_hour = '';

    if ($hour == 0) {
        $display_hour = '12';
    } elsif ($hour > 12) {
        $display_hour = $hour - 12;
    } else {
        $display_hour = $hour;
    }

    $display_hour+=0;

    return $display_hour;
}

#check for membership of a string in a list (does string eq compare only)
#returns position not offset
sub member {
    my ($class, $item, @mlist) = @_;

    my $elt;
    my $position = 1;
    foreach $elt (@mlist) {
	if ($item eq $elt) {
	    return $position ;
	}
        $position++;
    }
    return 0;
}

## returns offset of item in mlist if it exists
sub member_offset {
    my ($class, $item, @mlist) = @_;

    my $position = $class->member($item, @mlist);

    if ($position) {
        return $position - 1;
    } else {
        return undef;
    }
}

sub arrayify {
    my ($self, $list) = @_;
    if (ref($list) ne 'ARRAY') {
        $list = [$list];
    }
    return $list;
}

sub hashify {
    my ($self, $list) = @_;

    $list = $self->arrayify($list);

    my %hash = ();
    foreach my $key (@$list) {
        $hash{$key}++;
    }

    return \%hash;
}


# returns true if a given HTML tag can be used for cross-site-scripting
use constant _CANDIDATE_XSS_TAGS_ => [ 'img', 'iframe', 'script' ];
sub tag_has_xss_potential {
    my $class = shift;
    my $tag   = shift;
    my $result;

    if (defined $tag) {
        $result = Utility->array_contains(lc($tag), &_CANDIDATE_XSS_TAGS_);
    }

    return $result;
}

# strips possible cross-site scripting (XSS)
sub strip_xss {
    my $self = shift;
    my $content = shift;  

    my $not_ok;
    my $tag_open_sub = sub {
    
        my ($tagname, $attr) = @_;        
        #print STDERR "strip_xss Checking against tag: $tagname\n";
        if(Utility->tag_has_xss_potential($tagname)) {
            #print STDERR "Found potential xss tag: $tagname\n";
            # special processing for image tags - they are only vulnerable
            # if they contain certain features
            if( $tagname eq 'img' && $attr->{src}) {
                $not_ok = Portal->img_src_url_check($attr->{src});
                #print STDERR "image tag ok? '$not_ok'\n";
            } else {
                $not_ok = 1;
            }
        }
    
    };
    
    my ($parser) = new HTML::Parser (
        api_version   => 3,
        start_h => [ $tag_open_sub,  'tagname,attr,offset,length' ],
    );
    $parser->parse($content);
    
    my $safe = (!$not_ok);

    return $content, $safe;
}


# this is deprecated, replaced by strip_xss
sub strip_html {
    my $self = shift;
    return $self->strip_xss(@_);
}


# converts HTML to plaintext
sub html2text {
    my $self = shift;
    my $content = shift;  
    my $result = '';

    my ($parser) = new HTML::Parser (
        api_version => 3,
        default_h      => [ sub {$result .= shift;}, "dtext"],
    );
    
    $parser->parse($content);
    use Data::Dumper;
    print STDERR "html2text gets:\n",Dumper $result,"content=\n$content\n\n";

    return $result;
}





1;
__END__

=head1 NAME

Utility - 

=head1 SYNOPSIS

    use Utility;

    Utility->read_modules_in_directory ($dir);
    Utility->fetch_url ;

=head1 DESCRIPTION

=cut

