#!/usr/bin/perl
#
###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2004
# 
# This file is part of Metadot Portal Server software $Name: Release_6_1_6 $
# 
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
############################################################################ 
#                                                                          #
# usage:  metadotd.pl [-options ...]                                #
# ./metadotd.pl -h for help.
############################################################################

# aju 08-22-01: updating to work with MDK.  Requires addition of 'mid' column, of
# type int(11), to channelitem db.

use strict;

# some implementation (SPE) require setting the ORACLE environment
my $oracle_home = MetadotConfig->get_param_value('oracle_home');
if ($oracle_home) {
    print STDERR "Setting ORACLE_HOME to '$oracle_home'\n";
    $ENV{ORACLE_HOME} = $oracle_home;
}


use Fcntl ':flock';		#import flock + constants
use LWP::UserAgent;
use HTTP::Request;
use XML::RSS;
    
#  use Data::Dumper;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);
use GizmoTool;
use GizmoBuilder;
use Portal;
# use ParamsHash;
use Sendmail;
use Digest::MD5;
use Message;
use CGI qw(escape unescapeHTML);


BEGIN {
    use MetadotConfig;

    $ENV{'SERVER_NAME'} = "localhost" ;

}

my $HORIZONTAL_RULE = '-' x 72;

# we need to call this to initialize global vars
# such as $USER, 
my $mdot = Portal->new(1);

# determine if execution must be split to 
# avoid death from memory leaks
foreach my $arg_i ( 0 ..$#ARGV ) {

    my $arg = $ARGV[ $arg_i ];

    if( $arg =~ /^-split/ ) { 

	my $split_n = $ARGV[ $arg_i + 1];
	unless( $split_n > 0 ) {

	    &usage("missing argument for -split");
	}
       
	my @args; 

	# prepare arguments for child processes, don't include split arg
	foreach my $arg_j ( 0 ..$#ARGV ) {	    	    
	    unless( $arg_j == $arg_i or $arg_j == $arg_i + 1 ) {
		push( @args, $ARGV[ $arg_j ] );
	    }
	}

	my @all_servers = MetadotConfig->get_all_server_names();
	
	my $num_iterations = ( $#all_servers / $split_n );

	if( $#all_servers % $split_n ) { $num_iterations++ }

	foreach( 0 .. ( $num_iterations - 1 ) ) {

	    my $start_i = $split_n * $_;
	    my $end_i = $start_i + ( $split_n - 1 );

	    print STDERR "LAUNCHING CHILD $start_i - $end_i\n";

	    use Cwd;
	    my $dir = cwd();
	    system( "$dir/$0", ( @args, '-work_on_servers', $start_i, $end_i ) );
	}

	exit(0);
    }
}



my $ua = LWP::UserAgent->new( timeout => 10 );
my $DEBUG_LEVEL = 0;

    
sub debug_msg{
    my $level = shift;
    my $msg = shift;

    return if (!defined $level);
    return if (!defined $msg);
    my $eol="\n";
    my $head = "";
 
#     if ($level <= $SESSION->{debug_level}){
    if ($level <= $DEBUG_LEVEL) {
	print STDERR" $head debug: $msg$eol" ;
	    
    }    
}


my %time_counters;
my $now = time;
my $dbaccess;
my $db_now = Utility->db_now();

my $logdir    ;
my $logfile;
my $tmpdir;
my $tempfile ;
my $lockfile ;
my $channeldir;
my $overridelockfile; 
my $badXML;
# the following flag controls whether or not to pop replies to discussion
# msgs from an external acct.
my $fetchflag = 0;
# if the following variable is set, do more verbose logging to metadot logfile.
my $logging = 0;


#$|=1;  # do not buffer output so we can see the progress in realtime.

my %http_return_code =
    ( 100   , "Continue",
      101   , "Switching Protocols",
      200   , "OK",
      201   , "Created",
      202   , "Accepted",
      203   , "Non-Authoritative Information",
      204   , "No Content",
      205   , "Reset Content",
      206   , "Partial Content",
      300   , "Multiple Choices",
      301   , "Moved Permanently",
      302   , "Moved Temporarily",
      303   , "See Other",
      304   , "Not Modified",
      305   , "Use Proxy",
      400   , "Bad Request",
      401   , "Unauthorized",
      402   , "Payment Required",
      403   , "Forbidden",
      404   , "Not Found",
      405   , "Method Not Allowed",
      406   , "Not Acceptable",
      407   , "Proxy Authentication Required",
      408   , "Request Time-out",
      409   , "Conflict",
      410   , "Gone",
      411   , "Length Required",
      412   , "Precondition Failed",
      413   , "Request Entity Too Large",
      414   , "Request-URI Too Large",
      415   , "Unsupported Media Type",
      500   , "Internal Server Error",
      501   , "Not Implemented",
      502   , "Bad Gateway",
      503   , "Service Unavailable",
      504   , "Gateway Time-out",
      505   , "HTTP Version not supported"
      );

my %weekdays = ('Sun' => 1, 
		'Mon' => 2, 
		'Tue' => 3, 
		'Wed' => 4, 
		'Thu' => 5, 
		'Fri' => 6, 
		'Sat' => 7);
    
 ##### Global variables

 
 #################################################
 ## WARNING: DO NOT CHANGE 
 ($logdir)    =  DBAccess->sqlSelect("value", "params", "name = 'logdirectory'");

# XXX [claudio] we keep the ShareM version way 
# $logfile   = "$logdir/$0.log";
# $logfile   = "$0.log";
 $logfile   = MetadotConfig->get_param_value('metadotd_logfile') || "$0.log";


 $tmpdir    = "tmp";
 $tempfile  = "$tmpdir/$0.$$.tmp";
 $lockfile  = "$0.lock";
 $channeldir= "channels";

 ################################## 
 # dosubscription: =1 do notification
 #

 my $dosubscription=0;

 ################################## 
 # fetch: =1 fetch external channel
 #

 my $externalchannel=0;

 ##################################

 ## FOr debug output

 my $eol="\n";             

 # cmd line option: show debug information up to level you set here (0 to 5)
 #
 #

 my $show=0;               

 # set to 1 tell this program to force the lock of the execution.
 #
 #

 $overridelockfile = 0;  

 ##################################################
 # noprocess XML to DB: if =1 then the system
 # just gather information then quit without process
 # ing the XML.

 my $noprocess=0;

 $badXML=0;
 my $daemon=0;
 my $forceflags="";
 my $reset = 0;

 ##################################################
 # channelid

my $channelid=0;
my $userid=0;

# 
my $use_server_list;
my %server_list;

my $work_on_servers_start;
my $work_on_servers_end;

#### read argument and set variables
# $SESSION->{debug_level} = 0;

while ( $_ = $ARGV[0]) {
    shift;
    last if /^--$/;

    if    (/^-m/)   { $daemon = 1; $dosubscription=1	} 
    elsif (/^-o/)  	{ $overridelockfile = 1; debug_msg(3,"Overriding lockfile"); }
    elsif (/^-l/)  	{ $lockfile = &getOption("-l"); debug_msg(3,"Using lockfile: '$lockfile"); }
    elsif (/^-n/)  	{ $noprocess=1;debug_msg(3,"Won't process XML => DB"); }
#     elsif (/^-d/) 	{ $SESSION->{debug_level} = &getOption("-debug"); }
    elsif (/^-d/) 	{ $DEBUG_LEVEL = &getOption("-debug"); }
    elsif (/^-i/)  	{ $channelid = &getOption("-id");  debug_msg(3,"Doing only channel#$channelid"); }
    elsif (/^-up/)	{ 
	$use_server_list = 1;
	my $servers = lc &getOption("-update_only");
	my @servers = split /\s+/, $servers;
	%server_list =  map {$_ => 1} @servers;
    } elsif (/^-u/)  	{ $userid = &getOption("-uid"); $dosubscription=1; debug_msg(3,"Doing subscriptions only for user#$userid"); }
    elsif (/^-e/)  	{ debug_msg(3,"Will fetch external channels"); $externalchannel=1;   }
    elsif (/^-s/)  	{ 


	debug_msg (3, "Will do subscription based notification");
	$dosubscription=1;
    }
    elsif (/^-fo/)  	{ $forceflags = &getOption("-force"); $daemon = 1; }
    elsif (/^-fetch/)  	{
    	$fetchflag = 1;
    	debug_msg (3,  "Will be fetching discussion messages from pop account");
    }
    elsif (/^-r/)  	{ $reset = 1; }
    elsif (/^-log/) { $logging = 1; }
    elsif (/^-h/)	{ &usage("HELP REQUESTED..."); }
    elsif (/^-work_on_servers/)	{
	$work_on_servers_start = &getOption('-work_on_servers');
	$work_on_servers_end = &getOption('-work_on_servers');

#	print STDERR "WORK ON SERVERS: $work_on_servers_start - $work_on_servers_end\n";	
    }
    else { 
	&usage("unknown argument: $_");
    }
}
    
debug_msg (3, "========= START LOOP =========");

my $lastdb = undef;

my $SECTION_NAME;

#  my @tlist = MetadotConfig->get_all_server_names();
#  print STDERR "all servers: " , Dumper(\@tlist), "\n";

&ignition;

if( $fetchflag ) {
    use Metadot::EmailToDiscussion;
    my $email2disc = Metadot::EmailToDiscussion->new();
    my $email2disc_result = $email2disc->execute();
    my $email2disc_message = $email2disc->message();
    debug_msg(3, "email2disc_message:$email2disc_message");
}


my $work_on_servers_counter = 0;
foreach my $SERVER_NAME (MetadotConfig->get_all_server_names) {
        
    if( defined $work_on_servers_start ) {

	if( $work_on_servers_counter < $work_on_servers_start or
	    $work_on_servers_counter > $work_on_servers_end ) {
	    $work_on_servers_counter++;
	    next
	}

	$work_on_servers_counter++;
    }
    
    next if($use_server_list && not $server_list{lc $SERVER_NAME}); 

    $SECTION_NAME = MetadotConfig->get_section_name($SERVER_NAME);

    print STDERR "working on section: $SECTION_NAME\n";

    debug_msg (3,  "======= $SERVER_NAME: section $SECTION_NAME");

    $ENV{'SERVER_NAME'} = $SERVER_NAME;    

    # aju -- passing in server name to fix issue w/ v servs; env
    # var apparently not doing what was intended.
    MetadotConfig->set_server_name($SERVER_NAME);
    my $thisdb = MetadotConfig->get_dbname_sid($SERVER_NAME);
    debug_msg (3, "\n******db name is: $thisdb");

    if ($lastdb) {
	debug_msg (3, "\n\nthisdb = $thisdb\nlastdb = $lastdb");
    }

    if ($lastdb && ($lastdb eq $thisdb)){
	next;
    }
    else {
	$lastdb = $thisdb;
    };


    #####################################################################
    #
    #  MAIN
    #
    #####################################################################
    # do not call access method because it tries to modify DB
        
    debug_msg(2,"All systems are go, starting $0");

    my $fetchdone=0;
    
    my $time_update = 0 ;
    %time_counters = (
		      h => $now, #hour
		      d => $now, #day
		      w => $now, #week
		      );
    
    if ($reset) {
	# force time_counters update
	$time_update = 1 ;
    } else { 
	# read time_counters from database
	

	my ($time_counters_string) = DBAccess->sqlSelect("value", "params", "name = 'timecounters'");
	if ($time_counters_string){
	    my @info = split("-", $time_counters_string) ;
	    foreach my $pair (@info) {
		my ($key, $field) = split "=", $pair;
		$time_counters{$key} = $field =~ /^\d+$/ ? $field : $now ;
		debug_msg(5, "counter $pair : $key => $field");
	    }
	} else {
	    debug_msg(1, "Looks like it is the first time we run Pal.");
	    $time_update = 1 ;
	}
    }
    
    #end fetch time counters
    ####
    
    if ($externalchannel == 1) {
	&fetchAllChannels($channelid);
	$fetchdone=1;
    }
    
    debug_msg(2, "Before schedule...");
    debug_msg(2, "Now: ".localtime($now));
    debug_msg(2, "Hourly jobs are scheduled: ".localtime($time_counters{h}));
    debug_msg(2, "Daily jobs are scheduled:  ".localtime($time_counters{d}));
    debug_msg(2, "Weekly jobs are scheduled: ".localtime($time_counters{w}));
    
    
    if($daemon == 1) {
	
	## 
	debug_msg (1, "starting $0 daemon...\n");
	
	##################################################
	# Do hourly stuff
	##################################################
	if (($now >= $time_counters{h})||($forceflags =~ m/h/)){
	    debug_msg(1, "Doing hourly duty..");
	    debug_msg(1, "forceflags: $forceflags..");
	    &fetchAllChannels($channelid) if (!$fetchdone);;
	    $fetchdone=1;
#  	    print STDERR "finished fetchAllChannels, checking re:  update_hourly with forceflags $forceflags\n";
	    if ($forceflags !~ m/h/) {
#  		print STDERR "calling update_hourly with forceflags $forceflags\n";
		&update_hourly();
	    }
#  	    &update_hourly if ($forceflags !~ m/h/);
	}
	
	# have to do channels here before sending subscription
	&doChannels($channelid);

	##################################################
	# Do daily stuff
	##################################################
	if (($now >= $time_counters{d})||($forceflags =~ m/d/)){
	    debug_msg(1, "Doing daily duty..");
	    &doSubscription("onceaday",$userid) if ($dosubscription);
	    &update_daily if ($forceflags !~ m/d/);
	}
	##################################################
	# Do weekly stuff
	##################################################
	if (($now >= $time_counters{w})||($forceflags =~ m/w/)){
	    debug_msg(1, "Doing weekly duty..");
	    &doSubscription("onceaweek",$userid) if ($dosubscription);
	    # remove old & not-displayed channelitems
            ## unix time for seven days ago
            my $datetime = time - ( 86400 * 7 );
            $datetime = Utility->db_time_format($datetime);
            debug_msg(2, "Deleting old news where creationdate <= '$datetime'");
	    DBAccess->sqlDelete("channelitem","urgent='oldnews' and creationdate <= '$datetime'");
	    &update_weekly if ($forceflags !~ m/w/);
	}

	### Each time duties
	&doSubscription("all",$userid) if ($dosubscription);
	### do Gizmo Tools
	&doGizmoToolScheduler($channelid);

    } else {

	&doChannels($channelid);

	&doSubscription("all",$userid) if ($dosubscription);
    }

    # we do this to free memory during execution
    MetadotConfig->clear_config_cache();    
    
    # In some implementations (SPE) this causes the script to stop
    unless (MetadotConfig->get_param_value('do_not_close_db_object_in_metadotd')) {
        DBAccess->close_db_object();
    }
}


&safeLanding;			#cleanup;



exit(0);

######################################################################
#
# END OF MAIN
#
#####################################################################

# aju ------------------------------

#######################################################################


# aju ------------------------------

sub update_counters {
  debug_msg(1, "Updating counters");
  debug_msg(2, "Next hourly jobs: ".localtime($time_counters{h}));
  debug_msg(2, "Next daily jobs:  ".localtime($time_counters{d}));
  debug_msg(2, "Next weekly jobs: ".localtime($time_counters{w}));
  my $new_timecounters = "h=$time_counters{h}-d=$time_counters{d}-w=$time_counters{w}";
#    DBAccess->sqlInsert("params", ('name' => 'timecounters', 'value' => "$new_timecounters"));
  DBAccess->sqlUpdate("params", "name = 'timecounters'", (value => $new_timecounters));
}

sub update_weekly {
  localtime($time_counters{w}) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
  my $hour = $4;
  my $day = $1;
  localtime($now) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
  $time_counters{w} = $now + ($weekdays{$day}-$weekdays{$1})*24*60*60 + ($hour-$4)*60*60 - $5*60 -$6;
  $time_counters{w} += 7*24*60*60 if ($time_counters{w}<$now);
  &update_counters();
}

sub update_daily {
  localtime($time_counters{d}) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
  my $hour = $4;
  localtime($now) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
  $time_counters{d} = $now + ($hour-$4)*60*60 - $5*60 -$6;
  $time_counters{d} += 24*60*60 if ($time_counters{d}<$now);
  &update_counters();
}

sub update_hourly {
  localtime($now) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
  $time_counters{h} = $now + 60*60 - $5*60 -$6;
  &update_counters();
}



######################################################################
#
# doSubscription
# Notify users of new messages based on their subscriptions
# All notifications are fist prepare into a big array then send
# all at the same time at the end of the routine.
#####################################################################

sub doSubscription{
  my $frequency = shift ;
  my $userid = shift ;
  $frequency = "all" if $frequency !~ /^(all|onceaday|onceaweek)$/;

  debug_msg (3, "------------at do Subscription");
  if ($userid) {
      debug_msg STDERR "have userid $userid\n";
  }

  debug_msg(4, "Processing '$frequency' subscriptions...\n");
  my $nb=0;			# number of subscription we sent

  # select channels that have subscriptions with specific frequency
  # CKIM: this was ignoring the -id parameter
  my $where_clause = "channel.cid=subscription.cid and frequency='$frequency'";
  if ($channelid) {
      $where_clause = "channel.cid=$channelid AND ".$where_clause;
  }
  my ($a) = DBAccess->sqlSelectMany(
      "distinct channel.cid, channel.name, channel.iid, min(subscription.processdate) as lasttime", 
      "subscription, channel", 
      "$where_clause group by channel.cid, channel.name, channel.iid");

  while( my ($cid, $channelname, $iid, $lasttime)=$a->fetchrow()) {
#    while( my ($cid, $channelname, $lasttime)=$a->fetchrow()) 
    # For each channel we build a message gathering all new channelitems
      debug_msg (4, "in doSubscription: Processing channel cid=$cid - name=$channelname - lasttime=$lasttime with iid $iid");

    my $message = "";
    my $topics = "";
      debug_msg (4, "---doing db query to find items added after $lasttime:\n" . "select ciid, name, description, url, creationdate, mid from channelitem where cid=$cid and creationdate > '$lasttime' order by creationdate DESC;");

    my ($b)= DBAccess->sqlSelectMany("ciid, name, description, url, creationdate, mid", "channelitem", "cid=$cid and creationdate > '$lasttime'", "order by mid ASC, creationdate DESC");

    my $count = 1;
    my $authorname = '';
    my $authoremail = '';
    my $date = '';
    my ($isa) = DBAccess->sqlSelect("isa", "instance", "iid=$iid") if($iid);

    while( my ($ciid, $title, $description, $url, $creationdate, $mid, $newsiid )=$b->fetchrow()) { 
      # For each channelitem we add the information to the message
      debug_msg (4, "in doSubscription, Processing new channelitem ciid=$ciid - name=$title - creationdate=$creationdate mid=$mid");

      $title        = "" if (!$title);
      $description  = "" if (!$description);
      $creationdate = "" if (!$creationdate);
      $url          = "" if (!$url);

      $url = "http://$url" if ($url !~ /^http/);
      
      if ($isa && ($isa eq "Discussion")) {
          my $mid_where = '';
          if ($mid) {
              $mid_where = $mid > 0 ? "and mid=$mid" : "";
          }
          ($mid, $authorname, $date, $authoremail) = DBAccess->sqlSelect("mid, authorname, creationdate, authoremail","message","iid=$iid $mid_where");
      }

      if ($frequency ne 'all') {
     	  # add the current channelitem to message
    	  $topics .= $count . ") $title\n";
    	  $message .= "\n\nMessage ".$count++. ": $title\n";
          $message .= "Author:  $authorname\n";
          if ($date) {
              $message .= "Date:    $date\n";
          }
          $message .= $HORIZONTAL_RULE."\n";
    	  $message .= $description . "\n\n" if $description;
    	  $message .= "To view this message, click here:\n    $url\n" if $url;
          $message .= "\n";
          
          if ($isa eq 'Discussion') {
              my $cgi_title = escape($title);
              my $server_name = $ENV{'SERVER_NAME'} || '';
              my $message_id2 = "+" . $server_name . "+" . $iid . "+" . $mid;
              
              # see if the discussion has an email alias.
              my $sender_email;
              if ($PARAMS->{allow_discussion_email_alias}) {
                  my $discussion = $isa->restore($iid);
                  $sender_email = $discussion->get_email_alias(full=>1);
              }
              unless ($sender_email) {
                  ($sender_email) = DBAccess->sqlSelect("value", "params", "name = 'mailsenderaddress'");
              }

              # had to fix the quotes
              my $email = "\"+$server_name+$iid+$mid\" <$sender_email>";
              my $email_escape = escape($email);
#CKIM: the line break above the "If the link above..." line is INTENTIONAL, please nobody
#      remove it.         
              $message .= <<EOM

To reply to this message, click here:
mailto:$email_escape?subject=$cgi_title

If the link above is not clickable, please send an email to the following address: $email  

EOM
;
          }
      } else {
    	  my $uctitle = "";	# aju 09-9-01: why was this needed??
    	  if ($description ne '') {
    	      $description .= "\n";
    	  }

          #CKIM: cleaned up some ugly code, and restored message headers
          if ($isa && ($isa eq "Discussion")) {
              $message = "\n$description";
          } else {
              $message = " $uctitle\n$description\n$url\n$creationdate\n\n";
          }
          $nb += sendToUsers($iid, $newsiid, $ciid, $cid, $channelname, $frequency, $message, $lasttime, $userid, $title, $mid, $authorname, $date, $authoremail);
      }
    }
    $b->finish();

    if ($frequency ne 'all' && $message ne '') {
        $message = $topics . $message;
        $nb += sendToUsers($iid, 0, '',$cid, $channelname, $frequency, $message, $lasttime, $userid);
    }

    # update timestamp last subscription proccessing for all user subscriptions of that frequency
    debug_msg(3, "End of processing channel $cid, recording processing timestamp\n");
    # newsitems handle their own
    DBAccess->sqlUpdate("subscription", "cid=$cid and frequency='$frequency'", (processdate => $db_now));

  }
  $a->finish();

  debug_msg(3, "Sent $nb notification(s)");

  debug_msg(5, "End of doSubscription");

}


###########################################
# aju-- modified version of discussion script fcn

sub sendToUsers{
    my ($iid, $newsiid, $ciid, $cid, $channelname, $frequency, $message, $lasttime, $userid, $title, $mid, $authorname, $date, $authoremail)=@_;
    
    # [vadim] get server_name
    my $server_name = $ENV{'SERVER_NAME'} || "";
    
    my $whereclause = $userid ? " and userid=$userid" : "";
    my $nb=0;
    my $isa = '';
    my $newsisa = '';
    my $object;
    my $user;
    debug_msg (3, "in sendToUsers with:\n iid $iid, ciid $cid, lasttime $lasttime \n uid $userid \n title $title \n ciid $ciid\n$message\n\n ");

    my $sender_email = "";
    # use the 'discussion account' email if available, else just use the default.
    my ($sender_login) = DBAccess->sqlSelect("value", "params", "name = 'subscriptionsender'");
    my ($sender_host) = DBAccess->sqlSelect("value", "params", "name = 'subscriptionhost'");
    my ($show_discussion_channel_in_subject) = DBAccess->sqlSelect("value", "params", "name = 'show_discussion_channel_in_subject'");
    
    # some implementations (SPE) prefer to use the global mailsenderaddress    
    if ($sender_login && $sender_host && $fetchflag && !MetadotConfig->get_param_value('metadotd_always_uses_global_mailsenderaddress')) {
        $sender_email = "$sender_login\@$sender_host";
    }
    else {
        ($sender_email) = DBAccess->sqlSelect("value", "params", "name = 'mailsenderaddress'");
    }

    if($message ne "") {
	my ($c) = DBAccess->sqlSelectMany("userid, medium", "subscription", "frequency='$frequency' and cid=$cid and processdate>='$lasttime' $whereclause");
	($isa) = DBAccess->sqlSelect("isa", "instance", "iid=$iid") if($iid);
	($newsisa) = DBAccess->sqlSelect("isa", "instance", "iid=$newsiid") if($newsiid);
        debug_msg (4, "isa($iid)='$isa'; newsisa($newsiid)='$newsisa'");
	while( my ($uid, $medium)=$c->fetchrow()) {
	    # We sent the message to each user who have subscribed to this channel
	    debug_msg (4, "Processing user uid=$uid - medium=$medium");

      ## do permission checks
      #CKIM: the permission checks were only being performed for newsiid, not iid
	    if (defined($newsisa) && $newsisa ne '') {
		require "Gizmo/$newsisa.pm";
		$object = $newsisa->new($newsiid);
            } elsif (defined($isa) && $isa ne '') {
		require "Gizmo/$isa.pm";
		$object = $isa->new($iid);
            }
            if ($object) {
		$user = Metadot::UserFactory->get_user($uid);
                debug_msg(4, "Checking permission for userid: $uid, email: $user->{email}");
		## consider user logged in if accepted
        $user->{approval_needed} ||= '';  #prevent a perl warning
		$user->{_logged_in} = ($user->{approval_needed} ne 'rejected' || $user->{approval_needed} ne 'required') ? 1 : 0;
		next unless $object->is_allowed_to_do('show', $user);
                debug_msg(4, "user is allowed to view this message");
            }

            # see if the object has an email alias.
            if ($object && $object->can('get_email_alias') && $PARAMS->{allow_discussion_email_alias}) {
                my $alias = $object->get_email_alias(full=>1);
                if ($alias) {
                    $sender_email = $alias;
                }
            }


	    ##############################
	    # send the message to the user
	    ##############################
	    $nb++;
	    if ($medium eq "email") {
                if($isa && ($isa eq "Discussion")) {
		    my  $subject;
		    
                    # CKIM: this is confusing, and conflicts with the email header
#                    $subject = "[$channelname]";		    
#                    $subject = $title if ($frequency eq "all");
#                    unless ($subject =~ m/\[$channelname\]/) {
#                        $subject = "[$channelname] $subject";
#                    }

# CKIM: clarified this code, because it was ugly
                    if ($frequency eq 'onceaday') {
                        $subject = "[$channelname] - Daily Digest";
                    } elsif ($frequency eq 'onceaweek') {
                        $subject = "[$channelname] - Weekly Digest";
                    } elsif ($show_discussion_channel_in_subject && $subject !~ m/\[$channelname\]/) {
                        $subject = "[$channelname] $title";
                    } else {
                        $subject = $title;
                    }

		    my($email, $password) = DBAccess->sqlSelect("email, password", "users", "userid=$uid");
		    # XXX [claudio] password var here is cruft (wasit intended to be sent as a 
		    # token for user authentication??
		    
            # CKIM: as noted, this is now cruft. I'm commenting this out because it
            #       is no longer used.
            #$password = Digest::MD5::md5_base64($password.$uid);

		    my $msg ="";
		    
		    my ($basename) = DBAccess->sqlSelect("value", "params", "name = 'basename'");
		    my ($uri_scheme) = DBAccess->sqlSelect("value", "params", "name = 'uri_scheme'");
		    $basename = $uri_scheme . "://" . $basename;
            
# CKIM: restored this, because it's needed by users.            
            $msg = <<EOM
From:    $channelname discussion <$sender_email>
Discussion: $channelname
EOM
;
            
            unless ($frequency ne 'all') {
                $msg .= <<EOM
Author:  $authorname
Date:    $date
Subject: $subject
EOM
;
            }

		    $msg .= <<EMAIL_FOOTER

$message 

$HORIZONTAL_RULE
$HORIZONTAL_RULE
To view this discussion, click here:
   $basename/index.pl?iid=$iid

To unsubscribe or modify your subscriptions details, go to:
   $basename/userchannel.pl?id=$cid&isa=NewsChannel&op=preview

$HORIZONTAL_RULE
EMAIL_FOOTER
;
		    my $result;
		    my $sm = Sendmail->new($email, '', '',$subject, $msg);
		    $sm->set_debug($DEBUG_LEVEL);
		    
		    if ($fetchflag) {
                        my $message_id2 = "+" . $server_name . "+" . $iid . "+" . $mid;
                        #my $addl_params = {from_name => "$channelname", from_sender => "$sender_email"};
                        $sm->set_from("$channelname discussion <$sender_email>");
                        $sm->set_reply_to("\"$message_id2\" <$sender_email>");
                        $sm->set_message_id($message_id2);
                        ($result) = $sm->sendMail();
                        unless ($result) {
                            $c->finish();
                            &sendMailFailed();
                        }
                    } else {
                        $sm->set_from("$sender_email");
                        ($result) = $sm->sendMail();
                        unless ($result) {
                            $c->finish();
                            &sendMailFailed();
                        }
		    }					    
		}else{
		    # email subscription
		    &notifyEmail ($uid, $channelname, $message, $frequency, "No", $title, $cid);
                }
		# aju I think the following line is in error.
		#                &notifyEmail ($uid, $channelname, $message, $frequency, "No", $title) if(! $iid);
		debug_msg(4, "Sending email subscription (frequency=$frequency) to uid=$uid");
		debug_msg(5, $message);
	    } elsif ($medium eq "pager"){
		#send to pager email
		&notifyPager ($uid, $channelname, $message, $frequency, "No", $title);
		debug_msg(4, "Sending pager subscription (frequency=$frequency) to uid=$uid");
		debug_msg(5, $message);
	    } elsif ($medium eq "icq"){
		#send message to icq
		debug_msg(4, "Sending ICQ subscription (frequency=$frequency) to uid=$uid");
		&notifyICQ ($uid, $channelname, $message, $frequency, "No", $title);
	    }	    
	}
	$c->finish();
    }
    return $nb;
}

###########################################

sub notifyICQ{
  my ($uid, $channelname, $message, $frequency, $urg_flag, $title)=@_;

  if (!$uid || !$message){
    debug_msg(3, "notifyICQ: there is a problem: uid=$uid or message is null???!!");
    return 0;
  }

  # get user info.
  my($fullname, $nickname, $icqnumber)=DBAccess->sqlSelect("fullname, nickname, icqnumber", "users", "userid=$uid");

  # check for errors in user info.
  $fullname = $nickname || $fullname;
  if (!$icqnumber){
    debug_msg(3, "notifyICQ: there is a problem user: uid=$uid $fullname -$nickname- doesn't appear to have an ICQ numner  -$icqnumber-!");
    return 0;
  }

  # build icq message



  my $to = "$icqnumber\@pager.icq.com";
# my $subject = $channelname;
  my $subject = $title;
#  my ($from) = DBAccess->sqlSelect("value", "params", "name = 'mailsenderaddress'");
  debug_msg(4, "notifyICQ: orig_message=$message");

  ## remove links otherwise ICQ will not send
  $message =~ s/(http:\/\/.*)\s//g;
  $message .= "\n$channelname";
  $message .= "\nVisit your MetaDot site for more details.";
  $message .= "\n-- MetaDot subscription.";
  debug_msg(3, "notifyICQ: sent_message=$message\nto:$to subject:$subject");
  
  my $sm = Sendmail->new($to, '', '',$subject, $message);
  $sm->set_debug($DEBUG_LEVEL);  
  unless ($sm->sendMail()) {
      &sendMailFailed();
  }

#  $ua->agent("AgentName/0.1 " . $ua->agent);
#  my $url = "http://wwp.icq.com/scripts/WWPMsg.dll";
#  my $parameters = "to=$to&from=$from&fromemail=$from&subject=$subject&body=$message";
#  my $req = new HTTP::Request(POST => $url);
#     $req->content_type('application/x-www-form-urlencoded');
#     $req->content($parameters);
#  my ($result) = $ua->request($req);


}


###########################################
sub notifyPager{
  my ($uid, $channelname, $message, $frequency, $urg_flag, $title)=@_;

  if (!$uid || !$message){
    debug_msg(3, "notifyEmail: there is a problem: uid=$uid or message is null???!!");
    return 0;
  }

  # get user info.
  my($fullname, $nickname, $pager)=DBAccess->sqlSelect("fullname, nickname, pager", "users", "userid=$uid");

  # check for errors in user info.
  $fullname = $nickname || $fullname;
  if (!$pager){
    debug_msg(3, "notifyEmail: there is a problem user: uid=$uid $fullname -$nickname- doesn't appear to have a pager address -$pager-!");
    return 0;
  }

  # build pager message
  my $to = $pager;
  my $cc="";
  my $bcc="";
  my $urgent = "No" if (!$urg_flag);
  my $subject = $channelname;
  
  $message .= "\n-- MetaDot subscription.";

  my $sm = Sendmail->new($to, $cc, $bcc,$subject, $message);
  unless ($sm->sendMail()) {
      &sendMailFailed();
  }
#    my($result)=Sendmail::sendMail($to, $cc, $bcc, $subject, $message, $urgent);

}

###########################################
sub notifyEmail{
  my ($uid, $channelname, $message, $frequency, $urg_flag, $title, $cid)=@_;


  if (!$uid || !$message){
    debug_msg(3, "notifyEmail: there is a problem: uid=$uid or message is null???!!");
    return 0;
  }

  #
  # select user information
  #
  my($fullname, $nickname, $email, $password)=DBAccess->sqlSelect("fullname, nickname, email, password", "users", "userid=$uid");

  #
  # check if user is alright...
  #
  $fullname ||= $nickname;
  $nickname ||= $fullname;
  if (!$email){
    debug_msg(3, "notifyEmail: there is a problem user: uid=$uid $fullname -$nickname- doesn't appear to have an email address -$email-!");
    return 0;
  }

  #
  # build email
  #
  my $to=$email;
  my $cc="";
  my $bcc="";
  my $urgent = "No" if(!$urg_flag);
  my $subject= $channelname;
  $subject .= ":" . $title if ($frequency eq "all");
  $subject .= " - Daily Digest" if ($frequency eq "onceaday");
  $subject .= " - Weekly Digest" if ($frequency eq "onceaweek");

  my ($basename) = DBAccess->sqlSelect("value", "params", "name = 'basename'");
  my ($uri_scheme) = DBAccess->sqlSelect("value", "params", "name = 'uri_scheme'");
  $basename = $uri_scheme . "://" . $basename;


  my ($m) = DBAccess->sqlSelect("value", "params", "name = 'subsmessage'");
        $m=~s/<<forgot_pwd_url>>/$basename\/index.pl?isa=Session&op=forgot_password/gi;
        $m=~s/<<website_url>>/$basename\/index.pl/gi;
        $m=~s/<<email>>/$email/gi;
        $m=~s/<<fullname>>/$fullname/gi;
        $m=~s/<<nickname>>/$nickname/gi;
        $m=~s/<<password>>/$password/gi;
        $m=~s/<<profile_url>>/$basename\/index.pl?isa=User&op=modify_self/gi;
        $m=~s/<<subscription_url>>/$basename\/userchannel.pl?id=$cid&isa=NewsChannel&op=preview/gi;
  $message .= <<EMAIL_FOOTER;
$m
EMAIL_FOOTER

  ## mike: NEWSITEMS' EMAILS ARE SENT AS HTML SINCE ABSTRACTS ARE HTML
  $message =~ s/\n/<br>/g;
  my $sm = Sendmail->new($to, $cc, $bcc, $subject, $message);
  $sm->set_encoding('html');
  unless ($sm->sendMail()) {
      &sendMailFailed();
  }
}

###############################################
#
# fetchAllChannels: fetch all external channels
#
###############################################
sub fetchAllChannels{
  my ($channelid)=@_;
  my $whereclause="";

  $whereclause = " and cid=$channelid" if ($channelid);

  debug_msg(3,"fetchAllChannels(filter=$whereclause)");


  my ($c)=DBAccess->sqlSelectMany("cid, name, url, iid, isa, version", "channel", "isa='External' $whereclause", "order by cid desc");
  while(my ($cid, $name,$url, $iid, $isa, $newsversion)=$c->fetchrow()) {

    fetchOneChannel($url, $cid);

  }

}

################################################
#
# doExternalChannel: read file <cid>.xml and
# parse it, cached it and store it into the db
#
################################################
my $parsed_rss_instances;
sub doExternalChannel  {
    my ($newsversion, $cid, $url)=@_;
    my $html_cache="";
    my $nb=0;
    my $httpstatus=0;

    debug_msg(3, "in doExternalChannel, cid=$cid, newsversion:$newsversion\n");
    
    my $content="";

    my $url_digest = Digest::MD5::md5_hex( $url );
    print STDERR "in reading file, using $url_digest for $url\n";
    debug_msg(4, "Reading from file $channeldir/$url_digest.xml\n");
    
    if (open FILE, "<$channeldir/$url_digest.xml"){
	# we can read the file
	close FILE;
    } else {
      warn "Cannot open $channeldir/$url_digest.xml\n";
      return ("",0);
    }
    debug_msg(4, "About to parse $channeldir/$url_digest.xml\n");

    eval{			### TRY
	my $rss_instance;
	if( $parsed_rss_instances->{$url_digest} ) {

	    $rss_instance = $parsed_rss_instances->{$url_digest};

	} else {  

	    $rss_instance = XML::RSS->new();	    
	    $rss_instance->parsefile("$channeldir/$url_digest.xml");	    
	    $parsed_rss_instances->{$url_digest} = $rss_instance;
	}

	my $updatedb = 1;		#we put result in DB
        ($html_cache, $nb) = &cacheHTMLChannel($cid, $newsversion, $rss_instance, $updatedb);
    };

    ### CATCH
    if ($@) {
	my ($name, $url)=DBAccess->sqlSelect ("name, url", "channel", "cid=$cid");
	warn "\n\n###WARNING: '$name' Channel file contains incorrect RSS content cid=$cid:\n\n$@\n\nThis is not an error but a warning explaining that Metadot Portal Server fetched a file that is not a correct RSS file. If this error persists please contact the creator of this channel file or the channel webmaster, i.e. the guy running the website at $url\n\n\n";
    }
    
    debug_msg(4, "doExternalChannel: end  cid=$cid\n");
    return($html_cache, $nb);
    
}


##########################################################
sub doChannels{
    my ($channelid) = @_;
    debug_msg(3, "doChannels");
    my @gizmo_tools_array = GizmoTool->lists_tools();
    my %gizmo_tools;
    for (@gizmo_tools_array) {
	$gizmo_tools{$_}++;
    }

    my ($httpstatus, $OK);
    my $where;

    if (defined $channelid && "$channelid" ne "0" && "$channelid" ne "") {

        $where = "cid=$channelid" ;

    } else {

        $where = '1=1';
    }

    my $maxitem=10;
    
    my ($c) = DBAccess->sqlSelectMany( "cid, name, url, iid, isa, version", "channel", "$where", "order by cid desc" );
    
    my $s="";
    my $nb=0;
    # Do all channels
    while(my ($cid, $name,$url, $iid, $isa, $newsversion)=$c->fetchrow()) {

	next unless $isa;

	my $html_cache = "";
	$newsversion++;		# we do the next version here!
	
    #CKIM: save the new version number. This used to happen much later in the subroutine. The
    #      problem is that if this process is killed for any reason, between the increment
    #      and the update, then any messages that were scanned in the previous run will
    #      BE RESENT!! 
    DBAccess->sqlUpdate("channel", "cid=$cid",
          (
           version => $newsversion,
           )
          );

    
	debug_msg(3, "=> Channel $name: cid=$cid with version $newsversion");
	$OK=0;
    	if ( defined $isa && ($isa eq "External")){
	    ($html_cache, $nb)= &doExternalChannel($newsversion, $cid, $url);
	    if ($nb>0) {
		$OK=1;
	    } else {
		# nothing in it!
	    }
	} else {
	    $OK=1;
	    
	    if ( defined $isa && ($isa eq 'Internal')){
                # internal: do all msg of this channel
                #the html is now generated in my page
                # the iid is stored in the url field

                # We take the news from these internal channel
                # and we store them into the channelitem table
                # replace either update or insert automatically

                my ($cisa, $isa) = DBAccess->sqlSelect("parentiid, isa", "instance", "iid=$iid");

                debug_msg(5, "ChannelID $cid is pointing to a ".($cisa || '').": ".($iid || ''));

                #### This part is good for category news item
                #### may need to do another query for other stuff

                ##mike - first we update all showfrom dates that are empty
                ## special thanks to christophe for standing by while
                ##  I solved this problem
                debug_msg(5, "ChannelID $cid: updating all showfrom=0 to $db_now");
                # aju-- oracle compat. stuff
                # 		print STDERR "0 update\n";
                
                # CKIM: These queries not only cause problems in Oracle, they are a big waste of time!
                #       This is now handled directly in the Gizmo class, not here!
                # DBAccess->sqlUpdate("instance", "parentiid=$iid and isa='Newsitem' and showfrom = '0000-00-00 00:00:00'", (showfrom => substr($db_now, 0, 10)));
                #  		print STDERR "null update\n";
                
                # CKIM: this one is unnecessary altogether. We can just query for NULL also
                # my $sdate = substr($db_now, 0, 10);
                # DBAccess->sqlSimple("update instance set showfrom = '$sdate 00:00:00' where isa = 'Newsitem' and showfrom is NULL ");
                ##               DBAccess->sqlUpdate("instance", "parentiid=$iid and isa='Newsitem' and showfrom is NULL", (showfrom => substr($db_now, 0, 10)));

                my ($d)=DBAccess->sqlSelectMany("iid, name, url, description", "instance", "parentiid=$iid and isa='Newsitem' and (showfrom IS NULL OR showfrom <= '$db_now')", "order by showfrom desc, iid desc");

                $nb=0;

                #########################################################

                while((my ($ciid, $cname,$curl, $cdescription)=$d->fetchrow()) && ($nb <$maxitem)) {

		    if ( $cdescription =~ s/\A<!-- form type is text -->//) {
			# we need to call this to initialize global vars
			$cdescription = GizmoBuilder->untranslate_white_space( $cdescription );
			$cdescription = unescapeHTML( $cdescription );
		    }

		    my ($basename) = DBAccess->sqlSelect("value", "params", "name = 'basename'");
                    my ($uri_scheme) = DBAccess->sqlSelect("value", "params", "name = 'uri_scheme'");
                    $basename = $uri_scheme . "://" . $basename;


                    $nb++;
                    debug_msg(4, "$nb: item $cname: ciid=$ciid");
                    $curl ||="$basename/index.pl?id=$ciid&isa=Newsitem&op=show";
                    $html_cache .= "<li><a target='_NEWSITEM' href='$curl'><font size=\"-3\">$cname</font></a><br>\n";
                #########################################################

                    debug_msg (5, "Adding channel item $cid, $cdescription, with version $newsversion");
                    &addChannelItem($cid, $cname, $curl, $cdescription, $newsversion, 0, $ciid);

                }

                $d->finish();

	    } elsif ($isa eq 'Discussion') {
		debug_msg(5, "ChannelID $cid is pointing to a $isa: $iid\n");
		
        #CKIM: I really wonder why this was written such that it checks every single message, without regard to
        #      creationdate and updatedate? Seems like bad code to me, but I'm reluctant to modify it...
		
        # Okay, it makes a *little* sense. nb is set to 10. It will only process the most recent 10 messages.
        # It still doesn't make a lot of sense, because it will cause some valid messages to go missing.
                
                my ($d)=DBAccess->sqlSelectMany("mid, name, description", "message", "iid=$iid and status='OK'", "order by mid desc");
		$nb=0;
		my $murl;

		while((my ($mmid, $mname, $mdescription)=$d->fetchrow()) && ($nb <$maxitem)) {
		    $nb++;
		    my ($basename) = DBAccess->sqlSelect("value", "params", "name = 'basename'");
                    my ($uri_scheme) = DBAccess->sqlSelect("value", "params", "name = 'uri_scheme'");
                    $basename = $uri_scheme . "://" . $basename;

		    debug_msg(5, "$nb: item $mname: mid=$mmid\n");
		    $murl = "$basename/index.pl?mid=$mmid&isa=Message&op=show";
		    $html_cache .= "<li><a target='_NEWSITEM' href='$murl'><font size=\"-3\">$mname</font></a><br>\n";
		    #################################################################
		    &addChannelItem($cid, $mname, $murl, $mdescription, $newsversion, $mmid);
		}
		$d->finish();
	    }
	    debug_msg(4, "      found $nb item(s).\n");
	}
    	### here we log the success
    	### save nb items, inc version and update lastfetched
    	if ($OK == 1){
	    DBAccess->sqlUpdate("channel", "cid=$cid",
		      (
		       nbitems => $nb,
		       version => $newsversion,
		       html    => $html_cache,
		       lastfetched => $db_now
		       )
		      );
	    ### now we need to delete the duplicate items
	    DBAccess->sqlDelete("channelitem", "cid = $cid and urgent = 'duplicate'");
	    ### and then mark the non-duplicate old items
	    ### (to be able to delete them when too old for subscriptions.)
	    ### However, we don't want to mark gizmo tool items as oldnews
            unless (defined $isa && $gizmo_tools{$isa}) {
                debug_msg (4, "marking channelitems as old news: cid='$cid', newsversion != '$newsversion'");
		DBAccess->sqlUpdate("channelitem", "cid = $cid and newsversion != $newsversion", (urgent => 'oldnews'));
	    } else {
		debug_msg (5,  "matched on a gizmo tool $isa for channel $cid, won't set to oldnews");
	    }
	}
    }
    $c->finish();
    
    debug_msg(3, "Channel update completed.");
    
}


# CKIM notes: I'm trying to debug this, and here's what I can determine:
# - the system has retrieved all messages for a discussion's channel before calling this sub.
# - for each message, it calls addChannelItem.
# In addChannelItem:
# - the system tries to find a channelitem with matching cid and mid, but NON-matching newsversion
#   (there should be one if the message has previously been processed)
# - if one was not found, a new channelitem will be inserted with creationdate set to NOW
# - if one WAS found, a new channelitem will be inserted with the creationdate of the older channelitem
#   AND, the older channelitem will be updated with the urgent field set to 'duplicate'
# - finally, the channel's updatedate is set to NOW

##########################################################
sub addChannelItem{
    my($cid, $title, $url, $description, $version, $mid, $iid)=@_;

#      print STDERR "in addChannelItem with mid $mid and version $version\n";
    # check if this is a new item
    # in order to do this we check if there is already the same
    # item with the same name and description from the previous
    # version of the channel
    # if we find a previous item then we get the creation date and use it
    # to save it again in the newer version. This avoids having the notification
    # system send the same channelitem many time.
    my $xtitle       = DBAccess->quote ($title) || "";
    my $xurl         = DBAccess->quote ($url) || "";
    my $xdescription = DBAccess->quote ($description) || "";

    $mid ||= 0; # prevent inserting a NULL

    my $firstseen;
    if ($mid) {
#  	print STDERR "sql query is: select creationdate from channelitem where cid=$cid and mid=$mid and newsversion!=$version and name=$xtitle\n";	
	($firstseen) = DBAccess->sqlSelect("creationdate", "channelitem", "cid=$cid and mid=$mid and newsversion!=$version and name=$xtitle");
#  	print STDERR "got firstseen $firstseen for cid $cid and mid $mid and newsversion != $version\n";
    #} elsif ($iid > 0) {
#	($firstseen) = DBAccess->sqlSelect("creationdate", "channelitem", "cid=$cid and iid=$iid and newsversion!=$version and name=$xtitle");
    } else {
	($firstseen) = DBAccess->sqlSelect("creationdate", "channelitem", "cid=$cid and newsversion!=$version and name=$xtitle");
#  	print STDERR "got firstseen $firstseen for cid $cid and newsversion != $version\n";
    }
    if ($mid) {
	debug_msg (5, "firstseen is ".($firstseen || ''));
    }
    if ($firstseen) {
	debug_msg(4, ">|---;> $title: appeared 1st on $firstseen\n");
 	
    debug_msg(4,"***[1]inserting into channelitem with mid $mid and cid $cid");
	DBAccess->sqlInsert("channelitem",
		   (newsversion => $version,
		    cid	         => $cid,
		    name         => $title,
		    url          => $url,
		    description  => $description,
            mid          => $mid,
		    creationdate => $firstseen)
		   );
	#print STDERR "channelitem was inserted\n";

    if ($mid) {
	    # 09-7-01
	    DBAccess->sqlUpdate ("channelitem", "cid=$cid and newsversion!=$version and mid=$mid and name=$xtitle",
				 ( urgent => 'duplicate'));
	}
	else {
	    DBAccess->sqlUpdate ("channelitem", "cid=$cid and newsversion!=$version and name=$xtitle", ( urgent => 'duplicate'));
	}
    }
    else{
	debug_msg(4, ">|+++;> $title: new stuff!\n");
	debug_msg(4, "found no match for cid=$cid and newsversion=$version-1 and \nname=$xtitle at \nurl=$xurl\n");
	#aju  urggg, why was insertion of the mid removed????
	if ($mid) {
 	    #print STDERR "***[2]inserting into channelitem with mid $mid and cid $cid\n";
	    DBAccess->sqlInsert("channelitem",
		   (newsversion => $version,
		    cid	         => $cid,
		    mid	         => $mid,
		    name         => $title,
		    url          => $url,
		    description  => $description,
                    iid          => $iid,
		    creationdate => $db_now)
		   );
	}
	else {
	    DBAccess->sqlInsert("channelitem",
		   (newsversion => $version,
		    cid	         => $cid,
		    name         => $title,
		    url          => $url,
		    description  => $description,
                    iid          => $iid,
		    creationdate => $db_now)
		   );
	}
	DBAccess->sqlUpdate ("channel", "cid=$cid", (updatedate => $db_now));
    }
}

####################################################################################
sub cacheHTMLChannel {
    my $cid = shift;
    my $newsversion = shift;
    my $rss = shift;
    my $updatedb = shift;


    use Unicode::String;
    
    my $i=0;
    
    debug_msg (4, "### chacheHTMLChannel($cid)");
    return 0 if (!$cid);
    my $updatedate = "";
    ($updatedate) = DBAccess->sqlSelect("updatedate", "channel", "cid=$cid") if($updatedb);
    
    # cache channel image
    my $image = "";
    my $image_url = Unicode::String::utf8( $rss->{'image'}->{'url'} )->latin1;
    if ($image_url) {
      my $image_link = Unicode::String::utf8( $rss->{'image'}->{'link'} || "" )->latin1;
      my $image_title = Unicode::String::utf8( $rss->{'image'}->{'title'} || "")->latin1;
      my $width = Unicode::String::utf8( $rss->{'image'}->{'width'} || "" )->latin1;
      my $height = Unicode::String::utf8( $rss->{'image'}->{'height'} || "" )->latin1;
      $width = "width='$width'" if ($width);
      $height = "height='$height'" if ($height);
      $image = "<img src='$image_url' alt='$image_title' border='0' $width  $height>";
      $image = "<a target='_NEWSITEM' href='$image_link'><font size=\"-3\">$image</font></a>" if ($image_link);
    }

    # cache html channel news + save into channelitem table (for subscription) 
    my $html_content = ($image) ? "<center>$image</center><br>\n" : "";

    foreach my $item (@{$rss->{'items'}}) {

	next unless defined($item->{'title'}) && defined($item->{'link'});

	my $title = Unicode::String::utf8( $item->{'title'} )->latin1;

	my $url = Unicode::String::utf8( $item->{'link'} )->latin1;

	my $description;
	if (defined ($item->{'description'})) {

	    $description = Unicode::String::utf8( $item->{'description'} )->latin1 
	}

	# add item to the html cache
	$html_content .= "<li><a target='_NEWSITEM' href='$url'><font size=\"-3\">$title</font></a><br>\n";

	# save the item in the database
	&addChannelItem($cid, $title, $url, $description, $newsversion) if($updatedb);
	$i++;
    }

    # if there's a textinput element
    my $input_title = Unicode::String::utf8($rss->{'textinput'}->{'title'})->latin1;
    if ($input_title) {
      my $input_link = Unicode::String::utf8( $rss->{'textinput'}->{'link'} )->latin1;
      my $input_description = Unicode::String::utf8($rss->{'textinput'}->{'description'})->latin1;
      my $input_name = Unicode::String::utf8($rss->{'textinput'}->{'name'})->latin1;
      $html_content .= "<p><form method='get' action='$input_link'>\n";
      $html_content .= "$input_description<br>\n";
      $html_content .= "<input type='text' name='$input_name'><br>";
      $html_content .= "<input type='submit' value='$input_title'></form>";
    }

    # if there's a copyright element


    # XXX [claudio] we keep the ShareM version way 
    #    my $copyright = Unicode::String::utf8($rss->{'channel'}->{'copyright'})->latin1;
    my $copyright = undef;
    if ($copyright) {
	$html_content .= "<p><sub>$copyright</sub></p>";
      }

    my $channel_title = Unicode::String::utf8($rss->{'channel'}->{'title'})->latin1;
    $channel_title =~ s/^\s*//;

    DBAccess->sqlUpdate("channel", "cid=$cid",
	      (
	       name        => $channel_title,
	       homeurl     => Unicode::String::utf8( $rss->{'channel'}->{'link'} )->latin1,
	       description => Unicode::String::utf8($rss->{'channel'}->{'description'})->latin1,
	       nbitems     => $i
	      ))if($updatedb);
    debug_msg(4, "found $i items");

    return ($html_content, $i);

}
#########################################################
#
# fetchURL("http://hereyougo.com/index.html")
# fetch a URL and return everything into $response
#
#########################################################
sub fetchURL{
    my ($curl)=@_;


    my ($proxy_host) = DBAccess->sqlSelect("value", "params", "name = 'proxyhost'");
    my ($proxy_port) = DBAccess->sqlSelect("value", "params", "name = 'proxyport'");
    my $proxy_msg = ($proxy_host) ? "proxy: $proxy_host:$proxy_port/" : '';

    debug_msg(4, "fetchURL: $curl  $proxy_msg\n");
    my $request; my $response;

    if ($proxy_host){
          my $proxy_host = 'http://'.$proxy_host.':'.$proxy_port.'/';
	  $ua->proxy(http  => $proxy_host);

    }

    my $content="";
    if ($curl){ ### fetch on the web
	$request = new HTTP::Request (GET => $curl);

	$response = $ua->request($request);
    } else {
	return 0;
    }

    # if success then we store it in a file $cid.xml
    return $response;
}
####################################################
sub safeLanding{

  debug_msg(1 , "Safe landing initiated $0");

  DBAccess->close_db_object();
  &removeLockFile;

  &logMessage ("End of program: $0");
  debug_msg  (1, "End of program: $0");  
}

#####################################################
sub ignition{
  debug_msg(1, "Ignition sequence $0");

  ######################################################
  # LOCK FILE BUSINESS
  # check lock file exists
  # if it exists it means than another instance of this 
  # prog is already running or than the file is an old lock file 
  # from a process that already died.
  #
  
  if ($overridelockfile==1){
    debug_msg(1, "Removing lock file $lockfile\n");
    &removeLockFile;
  }

  if  (-e "$lockfile") { 	 	
    ### Here the lockfile exist. 
    ### Check if the file is older than one day. if yes 
    ### we remove it since this is probably due to a interrupted 
    ### process resulting from an LDAP crash or a server restart.
    
    if ((-M $lockfile) >= 1){ 
      
      #&removeLockFile;
    } else { 
      debug_msg(2,  "Lock file exists ($lockfile), use -o option to override. Exiting now."); 
      exit(-1);
    } 
  } else { 
    debug_msg(4, "Lock file $lockfile doesn't exist"); 
  }

  ## check if site is suspended
  if (MetadotConfig->get_site_is_suspended() eq 'true') {
      debug_msg(1, "SITE IS SUSPENDED!! WILL SKIP!!!"); 
      exit(-1);
  } else {
      debug_msg(5, "Site not suspended."); 
  }
  
  ######
  ### we create a lock file and put the date in it
  ### as well as the PID#
  
  open (LOCKFILE, ">$lockfile")
    or warn "Failed to open: '$lockfile',  $?";
  my ($sec, $min, $hour, $mday, $mon, $year) = localtime(time);
  print LOCKFILE "$year-$mon-$mday $hour:$min:$sec : starting process $$,  ";
#  my $date = &timeStamp;
#  print LOCKFILE "$date: starting process $$,  ";
  close LOCKFILE;
  
  if (! -d "$channeldir" ) {
    mkdir($channeldir, 0755) || warn ("cannot create directory: $channeldir: $?");
  }
  
  if (! -d "$logdir" ) {
    mkdir($logdir, 0755) || warn ("cannot create directory: $logdir: $?");
  }
  debug_msg(3, "Starting $0");
  &logMessage("### starting $0\n");
  
  if ( -d "$tmpdir" ) {
    debug_msg(4, "Cleaning tmpdir directory: $tmpdir");
    unlink ("$tmpdir/*.tmp");   
  }
  else{
    mkdir($tmpdir,0755) || warn "cannot create directory: $tmpdir: $?";
  }
  
  &catch_signals;		### allow graceful interrupts
}

###################################################
sub getOption {
  &usage("missing argument for $_[0]") if ($#ARGV==-1) ;
  my $result = $ARGV[0];
  shift @ARGV;	 
  return $result;
}
###################################################
sub usage {
  my ($mess) = @_;
  
  print "$0: $mess\n\n";
  
  print <<"_EOF_";
  
usage: $0 [-options ...] 

where options include:
  -debug <debuglevel>  <1 to 5> (1=small amount, 5=ou la la!)     
  -overridelockfile    run $0 even if the lock file exists
  -l                   specifies the filename of the lock file
  -m                   run $0 as a daemon <= USE THIS FOR NORMAL OPERATIONS!
  -externalchannels    fetch external channels
  -subscription        do  subscription-based notifications
  -noprocess           do not process XML to database
  -id <cid>            process only this channel specific id.
  -uid <uid>           process subscriptions only for this user specific id.
  -force <h|w|d>       force to run hourly/daily/weekly jobs
  -fetch               pop messages from the discussion email account
  -reset               reset time counters to present time
  -logging             enable more verbose logging
  -update_only "blank separated server list"    operate on a subset of available virtual servers

All options (except -force, -fetch and -update only) can be abbreviated up to one letter.

Examples:
### fetch external and override lockfile if exists
$0 -debug 3 -e -o

_EOF_
  
  &removeLockFile;
  exit -1;
}


################################################
sub catch_signals {
  $SIG{'INT'}     = 'SIGNAL_CLEANUP';
  $SIG{'QUIT'}    = 'SIGNAL_CLEANUP';
  $SIG{'PIPE'}    = 'SIGNAL_CLEANUP';
  $SIG{'ALRM'}    = 'SIGNAL_CLEANUP';
  #$SIG{'__DIE__'} = 'SIGNAL_CLEANUP';   ###__WARN__ is also available
}

################################################
sub BAD_XML{
  
  $badXML=1;
  print "badXML-int\n";
  
}
#################################################
sub SIGNAL_CLEANUP {
  
  #### quit gracefully
  
  unlink("$tmpdir/*.$$.tmp"); 
  &removeLockFile; 
  print "$0 was interrupted, lock file ($lockfile) removed!\n";
  die @_;
}
#
#
# CleanString: remove heading spaces and tabs, replace multiple spaces and tabs by one between words 
sub CleanString{
  my ($str) = @_;
  #print "<br> cleaning str:'$str'<br>";
  $str =~ s/^\s+//;		# remove leading white chars
  #### $str =~ s/^[ \t]+//;
  $str =~ s/\s+$//;		# remove trailing white chars
  $str =~ s/[ \t]+/ /;		# replace n white chars by only 1 in the middle of the string
  #print " cleaned  str:'$str'<br>";

  return $str;
}
#########################################
sub removeLockFile {
  unlink "$lockfile";
  print "'$lockfile' has been removed.\n";
}
########################################
sub logMessage
  {
    #some implementations (SPE) want to pipe the log file externally
    unless (MetadotConfig->get_param_value('metadotd_pipe_log_file_externally')) {
        open ( LOG, ">>$logfile" ) || warn "Cannot open $logfile\n";
        print LOG localtime()."\t",join("\t",@_),"\n";	
        close LOG;
    }
    print localtime()."\t",join("\t",@_),"\n";	
  }

#########################################################
# 
# fetchOneChannel: fetch URL and store it in file
# 
#########################################################

my $fetched_urls;
my $unavailable_urls;
sub fetchOneChannel{
  my ($url, $cid)=@_;
  my $response;
  my $httpstatus=0;    
  my $msg = "";

  debug_msg(3, "fetchOneChannel($url, cid=$cid)\n");

  if($url && $cid){

      unless( $fetched_urls->{$url} || $unavailable_urls->{$url} ) {

	  $response = &fetchURL($url);
	  $httpstatus = $response->code;
	  if ($httpstatus != 200){
	      debug_msg(3, "httpstatus=$httpstatus: $http_return_code{$httpstatus}");
	      #should log the problem
	      $msg =  "### error retrieving cid=$cid, $url" . "http status=$httpstatus" ." $http_return_code{$httpstatus}";
	      logMessage ("$msg");
	      #        print STDERR $msg;
	      warn ($msg);

	      $unavailable_urls->{$url} = 1;
	  } else {
	      ### Store the xml into a file
	      my $url_digest = Digest::MD5::md5_hex( $url );
	      print STDERR "in creating file, using $url_digest for $url\n";
	      open FILE, ">$channeldir/$url_digest.xml" || warn "Cannot open $channeldir/$url_digest.xml\n";
	      flock (FILE,LOCK_EX); 
	      seek(FILE, 0, 2);
	      print FILE $response->content;
	      flock (FILE, LOCK_UN);
	      close FILE;
	      
	      $fetched_urls->{$url} = 1;
	  }
      }

      #### We finished one channel
      #### here we log the attempt
      DBAccess->sqlUpdate("channel", "cid=$cid", 
			  (lastfetched   => $db_now,
			   httpstatus     => $httpstatus)
			 );		

      DBAccess->sqlUpdate("channel", "cid=$cid", 
			  (lastprocessed   => $db_now)
			 );

      return 1;

  } else{

      return 0;
  }
}

## GizmoTool Scheduler 
sub doGizmoToolScheduler {
  my $only_cid = shift ;
  my @gizmo_tools = GizmoTool->lists_tools();
  debug_msg(3, "### Gizmo Tool Scheduler ###");

  my $where_clause = join(" or ",map("isa = '$_'", @gizmo_tools));
  $where_clause = "($where_clause)" if ($where_clause);
  $where_clause .= " and '$db_now' > lastprocessed";
  $where_clause .= " and cid=$only_cid" if ($only_cid);
  debug_msg(3, "where: $where_clause");
  my ($tool_channels) = DBAccess->sqlSelectMany("cid, isa","channel",$where_clause,"order by cid");
  while (my ($cid, $isa) = $tool_channels->fetchrow) {
    my $tool = $isa->new($cid);
    debug_msg(3, "$isa : on_schedule($cid)");
    $tool->on_schedule($cid);
    debug_msg(5, "$isa : on_schedule finished");
    DBAccess->sqlUpdate("channel","cid=$cid",(lastprocessed => $db_now));
  }

}


## Gracefully fail if sendmail connection cannot be established
sub sendMailFailed {
    DBAccess->close_db_object();
    &removeLockFile;

    my $abort_msg = "\n\nErrors occurred while attempting to send mail. Aborting $0\n\n";
    &logMessage ($abort_msg);
    debug_msg  (1, $abort_msg);  
    exit(-1);
}




1;
